/* **********************************************************
 * Copyright (c) 2011-2015 Google, Inc.  All rights reserved.
 * **********************************************************/

/* Dr. Memory: the memory debugger
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation;
 * version 2.1 of the License, and no later version.

 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Library General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

/* Many syscall wrappers in user32, gdi32, and kernel32 are not exported.
 * Rather than requiring the user to retrieve symbols, or complicating
 * startup by having the front-end check for symbols, we use a table.
 * Xref i#388.
 */

/* Numbers are for these platforms:
 * USER32(name, w2k, xpx86, w2003, xpwow, vistax86, vistawow,
 *        w7x86, w7wow, w8x86, w8wow, w8x64, w81x86, w81wow, w81x64,
 *        w10x86, w10wow, w10x64)
 * No 64-bit support yet (though should match wow), and no NT support.
 *
 * Data source: win32lore/ syscall lists generated by winsysnums
 * DR client operating on dlls+pdbs.
 *
 * To turn into this format, run tools/sysnums.pl on the proper order
 * of "syscalls" files.  That script will auto-collapse the duplicate
 * names, taking the union of the numbers among them.
 *
 * I did make some manual changes: win2K has NtUserEndTouchOperation in gdi32 but I
 * put its number in the user32 entry (b/c no duplicates allowed): currently
 * we don't care which dll it comes from.
 * Ditto for 7 wrappers in imm32 which on all other platforms duplicate the
 * user32 wrapper of the same name: but on win2k there is no user32 wrapper.
 * These are:
 *   NtUserCallOneParam
 *   NtUserGetImeHotKey
 *   NtUserGetThreadState
 *   NtUserNotifyIMEStatus
 *   NtUserQueryWindow
 *   NtUserUpdateInputContext
 *   NtUserValidateHandleSecure
 * Pre-Win8, NtUserSelectPalette is in gdi32, but I put it in user32 under the
 * assumption that future versions will be only in user32 like Win8 is.
 *
 * XXX: imm32 on x64 seems to have a complete set of user32 wrappers!
 * for now not looking at x64 imm32 but may need to eventually do automated
 * cross-dll de-duplication.
 * Also, recent platforms seem to have NtGdi wrappers in user32.dll, and NtUser wrappers
 * in gdi32.dll.
 *
 * Note that some of these are exported without the NtUser or NtGdi prefix.
 */

USER32(NtBindCompositionSurface                          ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE, 0x111f,   NONE,       NONE, 0x1120, 0x13cd, 0x00001121, 0x1121)
USER32(NtCompositionInputThread                          ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE, 0x1121, 0x13e5, 0x00001122, 0x1122)
USER32(NtCreateCompositionInputSink                      ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE, 0x1122, 0x13de, 0x00001123, 0x1123)
USER32(NtCreateCompositionSurfaceHandle                  ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE, 0x1120,   NONE,       NONE, 0x1123, 0x13cb, 0x00001124, 0x1124)
USER32(NtCreateImplicitCompositionInputSink              ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE, 0x13df, 0x00001125, 0x1125)
USER32(NtDCompositionAddCrossDeviceVisualChild           ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE, 0x1121,   NONE,       NONE, 0x1124, 0x140e, 0x00001126, 0x1126)
USER32(NtDCompositionAddVisualChild                      ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE, 0x1122,   NONE,       NONE, 0x1125, 0x140f, 0x00001127, 0x1127)
USER32(NtDCompositionBeginFrame                          ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE, 0x1123,   NONE,       NONE, 0x1126, 0x13f1, 0x00001128, 0x1128)
USER32(NtDCompositionCapturePointer                      ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE, 0x142a, 0x00001129, 0x1129)
USER32(NtDCompositionCommitChannel                       ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE, 0x1124,   NONE,       NONE, 0x1127, 0x1400, 0x0000112a, 0x112a)
USER32(NtDCompositionConfirmFrame                        ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE, 0x1125,   NONE,       NONE, 0x1128, 0x13f2, 0x0000112b, 0x112b)
USER32(NtDCompositionConnectPipe                         ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE, 0x1126,   NONE,       NONE, 0x1129, 0x1423, 0x0000112c, 0x112c)
USER32(NtDCompositionCreateAndBindSharedSection          ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE, 0x112a, 0x1420, 0x0000112d, 0x112d)
USER32(NtDCompositionCreateChannel                       ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE, 0x1127,   NONE,       NONE, 0x112b, 0x141c, 0x0000112e, 0x112e)
USER32(NtDCompositionCreateConnection                    ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE, 0x112c, 0x13fc, 0x0000112f, 0x112f)
USER32(NtDCompositionCreateConnectionContext             ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE, 0x1128,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE)
USER32(NtDCompositionCreateDwmChannel                    ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE, 0x1129,   NONE,       NONE, 0x112d, 0x141b, 0x00001130, 0x1130)
USER32(NtDCompositionCreateResource                      ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE, 0x112a,   NONE,       NONE, 0x112e, 0x1418, 0x00001131, 0x1131)
USER32(NtDCompositionCurrentBatchId                      ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE, 0x112b,   NONE,       NONE, 0x112f, 0x140a, 0x00001132, 0x1132)
USER32(NtDCompositionDestroyChannel                      ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE, 0x112c,   NONE,       NONE, 0x1130, 0x13fd, 0x00001133, 0x1133)
USER32(NtDCompositionDestroyConnection                   ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE, 0x1131, 0x13f7, 0x00001134, 0x1134)
USER32(NtDCompositionDestroyConnectionContext            ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE, 0x112d,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE)
USER32(NtDCompositionDiscardFrame                        ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE, 0x112e,   NONE,       NONE, 0x1132, 0x13f4, 0x00001135, 0x1135)
USER32(NtDCompositionDuplicateHandleToProcess            ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE, 0x1133, 0x1426, 0x00001136, 0x1136)
USER32(NtDCompositionDuplicateSwapchainHandleToDwm       ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE, 0x142d, 0x00001137, 0x1137)
USER32(NtDCompositionDwmSyncFlush                        ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE, 0x112f,   NONE,       NONE, 0x1134,   NONE,       NONE,   NONE)
USER32(NtDCompositionEnableDDASupport                    ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE, 0x13dd, 0x00001138, 0x1138)
USER32(NtDCompositionEnableMMCSS                         ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE, 0x13db, 0x00001139, 0x1139)
USER32(NtDCompositionGetAnimationTime                    ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE, 0x1428, 0x0000113a, 0x113a)
USER32(NtDCompositionGetChannels                         ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE, 0x1130,   NONE,       NONE, 0x1135, 0x1422, 0x0000113b, 0x113b)
USER32(NtDCompositionGetConnectionBatch                  ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE, 0x1136, 0x13f8, 0x0000113c, 0x113c)
USER32(NtDCompositionGetConnectionContextBatch           ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE, 0x1131,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE)
USER32(NtDCompositionGetDeletedResources                 ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE, 0x1132,   NONE,       NONE, 0x1137, 0x13fa, 0x0000113d, 0x113d)
USER32(NtDCompositionGetFrameLegacyTokens                ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE, 0x1133,   NONE,       NONE, 0x1138, 0x13f6, 0x0000113e, 0x113e)
USER32(NtDCompositionGetFrameStatistics                  ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE, 0x1134,   NONE,       NONE, 0x1139, 0x13f9, 0x0000113f, 0x113f)
USER32(NtDCompositionGetFrameSurfaceUpdates              ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE, 0x1135,   NONE,       NONE, 0x113a, 0x13f5, 0x00001140, 0x1140)
USER32(NtDCompositionOpenSharedResource                  ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE, 0x113b, 0x1419, 0x00001141, 0x1141)
USER32(NtDCompositionOpenSharedResourceHandle            ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE, 0x113c, 0x141a, 0x00001142, 0x1142)
USER32(NtDCompositionReferenceSharedResourceOnDwmChannel ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE, 0x113d, 0x141e, 0x00001143, 0x1143)
USER32(NtDCompositionRegisterThumbnailVisual             ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE, 0x113e, 0x1424, 0x00001144, 0x1144)
USER32(NtDCompositionRegisterVirtualDesktopVisual        ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE, 0x1425, 0x00001145, 0x1145)
USER32(NtDCompositionReleaseAllResources                 ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE, 0x1136,   NONE,       NONE, 0x113f, 0x13fe, 0x00001146, 0x1146)
USER32(NtDCompositionReleaseResource                     ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE, 0x1137,   NONE,       NONE, 0x1140, 0x140b, 0x00001147, 0x1147)
USER32(NtDCompositionRemoveCrossDeviceVisualChild        ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE, 0x1138,   NONE,       NONE, 0x1141, 0x140c, 0x00001148, 0x1148)
USER32(NtDCompositionRemoveVisualChild                   ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE, 0x1139,   NONE,       NONE, 0x1142, 0x140d, 0x00001149, 0x1149)
USER32(NtDCompositionReplaceVisualChildren               ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE, 0x113a,   NONE,       NONE, 0x1143, 0x1410, 0x0000114a, 0x114a)
USER32(NtDCompositionRetireFrame                         ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE, 0x113b,   NONE,       NONE, 0x1144, 0x13f3, 0x0000114b, 0x114b)
USER32(NtDCompositionSetChannelCallbackId                ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE, 0x142b, 0x0000114c, 0x114c)
USER32(NtDCompositionSetChannelCommitCompletionEvent     ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE, 0x113c,   NONE,       NONE, 0x1145, 0x1402, 0x0000114d, 0x114d)
USER32(NtDCompositionSetDebugCounter                     ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE, 0x1146, 0x1421, 0x0000114e, 0x114e)
USER32(NtDCompositionSetResourceAnimationProperty        ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE, 0x113d,   NONE,       NONE, 0x1147, 0x1411, 0x0000114f, 0x114f)
USER32(NtDCompositionSetResourceBufferProperty           ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE, 0x113e,   NONE,       NONE, 0x1148, 0x1414, 0x00001150, 0x1150)
USER32(NtDCompositionSetResourceCallbackId               ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE, 0x142c, 0x00001151, 0x1151)
USER32(NtDCompositionSetResourceDeletedNotificationTag   ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE, 0x113f,   NONE,       NONE, 0x1149, 0x13fb, 0x00001152, 0x1152)
USER32(NtDCompositionSetResourceFloatProperty            ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE, 0x1140,   NONE,       NONE, 0x114a, 0x1416, 0x00001153, 0x1153)
USER32(NtDCompositionSetResourceHandleProperty           ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE, 0x114b, 0x1417, 0x00001154, 0x1154)
USER32(NtDCompositionSetResourceIntegerProperty          ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE, 0x1141,   NONE,       NONE, 0x114c, 0x1415, 0x00001155, 0x1155)
USER32(NtDCompositionSetResourceReferenceArrayProperty   ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE, 0x1142,   NONE,       NONE, 0x114d, 0x1412, 0x00001156, 0x1156)
USER32(NtDCompositionSetResourceReferenceProperty        ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE, 0x1143,   NONE,       NONE, 0x114e, 0x1413, 0x00001157, 0x1157)
USER32(NtDCompositionSetVisualInputSink                  ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE, 0x1427, 0x00001158, 0x1158)
USER32(NtDCompositionSignalGpuFence                      ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE, 0x1144,   NONE,       NONE, 0x114f, 0x141f, 0x00001159, 0x1159)
USER32(NtDCompositionSubmitDWMBatch                      ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE, 0x1145,   NONE,       NONE, 0x1150, 0x13ff, 0x0000115a, 0x115a)
USER32(NtDCompositionSynchronize                         ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE, 0x1146,   NONE,       NONE, 0x1151, 0x141d, 0x0000115b, 0x115b)
USER32(NtDCompositionTelemetryAnimationScenarioBegin     ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE, 0x1152, 0x1407, 0x0000115c, 0x115c)
USER32(NtDCompositionTelemetryAnimationScenarioReference ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE, 0x1153, 0x1408, 0x0000115d, 0x115d)
USER32(NtDCompositionTelemetryAnimationScenarioUnreference,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE, 0x1154, 0x140e, 0x0000115e, 0x1159)
USER32(NtDCompositionTelemetrySetApplicationId           ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE, 0x1155, 0x1406, 0x0000115f, 0x115f)
USER32(NtDCompositionTelemetryTouchInteractionBegin      ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE, 0x1147,   NONE,       NONE, 0x1156, 0x1403, 0x00001160, 0x1160)
USER32(NtDCompositionTelemetryTouchInteractionEnd        ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE, 0x1148,   NONE,       NONE, 0x1157, 0x1405, 0x00001161, 0x1161)
USER32(NtDCompositionTelemetryTouchInteractionUpdate     ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE, 0x1149,   NONE,       NONE, 0x1158, 0x1404, 0x00001162, 0x1162)
USER32(NtDCompositionUpdatePointerCapture                ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE, 0x1429, 0x00001163, 0x1163)
USER32(NtDCompositionValidateAndReferenceSystemVisualForHwndTarget, NONE, NONE, NONE, NONE,  NONE,   NONE,   NONE,   NONE,   NONE,       NONE, 0x114a,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE)
USER32(NtDCompositionWaitForChannel                      ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE, 0x114b,   NONE,       NONE, 0x1159, 0x1401, 0x00001164, 0x1164)
USER32(NtDesktopCaptureBits                              ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE, 0x13da, 0x00001165, 0x1165)
USER32(NtDuplicateCompositionInputSink                   ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE, 0x115a, 0x13e0, 0x00001166, 0x1166)
USER32(NtHWCursorUpdatePointer                           ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE, 0x1467, 0x00001320, 0x1320)
USER32(NtNotifyPresentToCompositionSurface               ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE, 0x12ce,   NONE,       NONE, 0x12e7, 0x13d0, 0x00001321, 0x1321)
USER32(NtOpenCompositionSurfaceDirtyRegion               ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE, 0x12cf,   NONE,       NONE, 0x12e8, 0x13d5, 0x00001322, 0x1322)
USER32(NtOpenCompositionSurfaceSectionInfo               ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE, 0x12d0,   NONE,       NONE, 0x12e9, 0x13d2, 0x00001323, 0x1323)
USER32(NtOpenCompositionSurfaceSwapChainHandleInfo       ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE, 0x12d1,   NONE,       NONE, 0x12ea, 0x13d3, 0x00001324, 0x1324)
USER32(NtQueryCompositionInputIsImplicit                 ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE, 0x13e7, 0x00001325, 0x1325)
USER32(NtQueryCompositionInputQueueAndTransform          ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE, 0x13e6, 0x00001326, 0x1326)
USER32(NtQueryCompositionInputSink                       ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE, 0x12eb, 0x13e1, 0x00001327, 0x1327)
USER32(NtQueryCompositionInputSinkLuid                   ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE, 0x12ec, 0x13e2, 0x00001328, 0x1328)
USER32(NtQueryCompositionInputSinkViewId                 ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE, 0x13e3, 0x00001329, 0x1329)
USER32(NtQueryCompositionSurfaceBinding                  ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE, 0x12d2,   NONE,       NONE, 0x12ed, 0x13cf, 0x0000132a, 0x132a)
USER32(NtQueryCompositionSurfaceRenderingRealization     ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE, 0x12d3,   NONE,       NONE, 0x12ee, 0x13d4, 0x0000132b, 0x132b)
USER32(NtQueryCompositionSurfaceStatistics               ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE, 0x12d4,   NONE,       NONE, 0x12ef, 0x13d1, 0x0000132c, 0x132c)
USER32(NtRIMAddInputObserver                             ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE, 0x145f, 0x0000132d, 0x132d)
USER32(NtRIMGetDevicePreparsedDataLockfree               ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE, 0x1463, 0x0000132e, 0x132e)
USER32(NtRIMObserveNextInput                             ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE, 0x1462, 0x0000132f, 0x132f)
USER32(NtRIMRemoveInputObserver                          ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE, 0x1460, 0x00001330, 0x1330)
USER32(NtRIMUpdateInputObserverRegistration              ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE, 0x1461, 0x00001331, 0x1331)
USER32(NtSetCompositionSurfaceAnalogExclusive            ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE, 0x13f0, 0x00001332, 0x1332)
USER32(NtSetCompositionSurfaceBufferCompositionMode      ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE, 0x12f0, 0x13d8, 0x00001333, 0x1333)
USER32(NtSetCompositionSurfaceIndependentFlipInfo        ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE, 0x12f1, 0x13d9, 0x00001334, 0x1334)
USER32(NtSetCompositionSurfaceOutOfFrameDirectFlipNotification, NONE, NONE,  NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE, 0x12d5,   NONE,       NONE, 0x12f2, 0x13d6, 0x00001335, 0x1335)
USER32(NtSetCompositionSurfaceStatistics                 ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE, 0x12d6,   NONE,       NONE, 0x12f3, 0x13d7, 0x00001336, 0x1336)
USER32(NtTokenManagerConfirmOutstandingAnalogToken       ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE, 0x13ef, 0x00001337, 0x1337)
USER32(NtTokenManagerCreateCompositionTokenHandle        ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE, 0x12f4, 0x13ec, 0x00001338, 0x1338)
USER32(NtTokenManagerDeleteOutstandingDirectFlipTokens   ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE, 0x12f5, 0x13eb, 0x00001339, 0x1339)
USER32(NtTokenManagerGetAnalogExclusiveSurfaceUpdates    ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE, 0x13ee, 0x0000133a, 0x133a)
USER32(NtTokenManagerGetAnalogExclusiveTokenEvent        ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE, 0x13ed, 0x0000133b, 0x133b)
USER32(NtTokenManagerGetOutOfFrameDirectFlipSurfaceUpdates,  NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE, 0x12d7,   NONE,       NONE, 0x12f6, 0x13ea, 0x0000133c, 0x133c)
USER32(NtTokenManagerOpenEvent                           ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE, 0x12d8,   NONE,       NONE, 0x12f7,   NONE,       NONE,   NONE)
USER32(NtTokenManagerOpenSection                         ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE, 0x12f8,   NONE,       NONE,   NONE)
USER32(NtTokenManagerOpenSectionAndEvents                ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE, 0x13e8, 0x0000133d, 0x133d)
USER32(NtTokenManagerThread                              ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE, 0x12d9,   NONE,       NONE, 0x12f9, 0x13e9, 0x0000133e, 0x133e)
USER32(NtUnBindCompositionSurface                        ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE, 0x12da,   NONE,       NONE, 0x12fa, 0x13ce, 0x0000133f, 0x133f)
USER32(NtUpdateInputSinkTransforms                       ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE, 0x12fb, 0x13e4, 0x00001340, 0x1340)
USER32(NtUserAcquireIAMKey                               ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE, 0x13c7, 0x000012db, 0x12db, 0x13ee, 0x000012fc, 0x12fc, 0x1439, 0x00001341, 0x1341)
USER32(NtUserActivateKeyboardLayout                      , 0x1126, 0x1130, 0x112f, 0x1126, 0x1139, 0x1127, 0x113a, 0x111e, 0x1171, 0x0007111e, 0x111e, 0x1173, 0x0007111f, 0x111f, 0x1174, 0x00071120, 0x1120)
USER32(NtUserAddClipboardFormatListener                  ,   NONE,   NONE,   NONE,   NONE, 0x113a, 0x1274, 0x113b, 0x1281, 0x1170, 0x000012dc, 0x12dc, 0x1172, 0x000012fd, 0x12fd, 0x1173, 0x00001342, 0x1342)
USER32(NtUserAlterWindowStyle                            ,   NONE, 0x1131, 0x1130, 0x10d7, 0x113b, 0x10d8, 0x113c, 0x10d3, 0x116f, 0x000c10d3, 0x10d3, 0x1171, 0x000c10d4, 0x10d4, 0x1172, 0x000c10d5, 0x10d5)
USER32(NtUserAttachThreadInput                           , 0x1129, 0x1133, 0x1132, 0x10f0, 0x113d, 0x10f1, 0x113e, 0x10ec, 0x116d, 0x000a10ec, 0x10ec, 0x116f, 0x000a10ed, 0x10ed, 0x1170, 0x000a10ee, 0x10ee)
USER32(NtUserAutoPromoteMouseInPointer                   ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE, 0x13d1, 0x000012de, 0x12de, 0x13fa, 0x000012ff, 0x12ff, 0x1447, 0x00001344, 0x1344)
USER32(NtUserAutoRotateScreen                            ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE, 0x12df,   NONE,       NONE, 0x1300, 0x1438, 0x00001345, 0x1345)
USER32(NtUserBeginPaint                                  , 0x112a, 0x1134, 0x1133, 0x1016, 0x113e, 0x1017, 0x113f, 0x1017, 0x116c, 0x00001018, 0x1018, 0x116e, 0x00001019, 0x1019, 0x116f, 0x0000101a, 0x101a)
USER32(NtUserBitBltSysBmp                                ,   NONE, 0x1135, 0x1134, 0x10cf, 0x113f, 0x10d0, 0x1140, 0x10cb, 0x116b, 0x000010cb, 0x10cb, 0x116d, 0x000010cc, 0x10cc, 0x116e, 0x000010cd, 0x10cd)
USER32(NtUserBlockInput                                  , 0x112c, 0x1136, 0x1135, 0x1226, 0x1140, 0x1276, 0x1141, 0x1283, 0x116a, 0x000012e0, 0x12e0, 0x116c, 0x00001301, 0x1301, 0x116d, 0x00001346, 0x1346)
USER32(NtUserBuildHwndList                               ,   NONE, 0x1138, 0x1137, 0x101b, 0x1142, 0x101c, 0x1143, 0x101c, 0x1168, 0x0000101d, 0x101d, 0x116a, 0x0000101e, 0x101e, 0x116b, 0x0000101f, 0x101f)
USER32(NtUserBuildNameList                               ,   NONE, 0x1139, 0x1138, 0x10b2, 0x1143, 0x10b3, 0x1144, 0x10ae, 0x1167, 0x000010ae, 0x10ae, 0x1169, 0x000010af, 0x10af, 0x116a, 0x000010b0, 0x10b0)
USER32(NtUserBuildPropList                               ,   NONE, 0x113a, 0x1139, 0x1228, 0x1144, 0x1278, 0x1145, 0x1285, 0x1166, 0x000012e2, 0x12e2, 0x1168, 0x00001303, 0x1303, 0x1169, 0x00001348, 0x1348)
USER32(NtUserCalcMenuBar                                 ,   NONE, 0x1236, 0x1232, 0x1098, 0x124e, 0x1099, 0x125b, 0x1097, 0x1219, 0x00001097, 0x1097, 0x121c, 0x00001098, 0x1098, 0x121f, 0x00001099, 0x1099)
USER32(NtUserCalculatePopupWindowPosition                ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE, 0x125a, 0x1286, 0x121a, 0x000012e3, 0x12e3, 0x121d, 0x00001304, 0x1304, 0x1220, 0x00001349, 0x1349)
USER32(NtUserCallHwnd                                    ,   NONE, 0x113b, 0x113a, 0x1112, 0x1145, 0x1113, 0x1146, 0x110c, 0x1165, 0x0000110c, 0x110c, 0x1167, 0x0000110d, 0x110d, 0x1168, 0x0000110e, 0x110e)
USER32(NtUserCallHwndLock                                ,   NONE, 0x113c, 0x113b, 0x1020, 0x1146, 0x1021, 0x1147, 0x1021, 0x1164, 0x00001022, 0x1022, 0x1166, 0x00001023, 0x1023, 0x1167, 0x00001024, 0x1024)
USER32(NtUserCallHwndOpt                                 ,   NONE, 0x113d, 0x113c, 0x1229, 0x1147, 0x1279, 0x1148, 0x1287, 0x1163, 0x000012e4, 0x12e4, 0x1165, 0x00001305, 0x1305, 0x1166, 0x0000134a, 0x134a)
USER32(NtUserCallHwndParam                               ,   NONE, 0x113e, 0x113d, 0x109f, 0x1148, 0x10a0, 0x1149, 0x109e, 0x1162, 0x0000109e, 0x109e, 0x1164, 0x0000109f, 0x109f, 0x1165, 0x000010a0, 0x10a0)
USER32(NtUserCallHwndParamLock                           ,   NONE, 0x113f, 0x113e, 0x1026, 0x1149, 0x1027, 0x114a, 0x1027, 0x1161, 0x00001028, 0x1028, 0x1163, 0x00001029, 0x1029, 0x1164, 0x0000102a, 0x102a)
USER32(NtUserCallMsgFilter                               ,   NONE, 0x1140, 0x113f, 0x1014, 0x114a, 0x1015, 0x114b, 0x1015, 0x1160, 0x00001016, 0x1016, 0x1162, 0x00001017, 0x1017, 0x1163, 0x00001018, 0x1018)
USER32(NtUserCallNextHookEx                              ,   NONE, 0x1141, 0x1140, 0x101d, 0x114b, 0x101e, 0x114c, 0x101e, 0x115f, 0x0000101f, 0x101f, 0x1161, 0x00001020, 0x1020, 0x1162, 0x00001021, 0x1021)
USER32(NtUserCallNoParam                                 ,   NONE, 0x1142, 0x1141, 0x1005, 0x114c, 0x1005, 0x114d, 0x1005, 0x115e, 0x00001006, 0x1006, 0x1160, 0x00001007, 0x1007, 0x1161, 0x00001008, 0x1008)
USER32(NtUserCallOneParam                                , 0x1139, 0x1143, 0x1142, 0x1002, 0x114d, 0x1002, 0x114e, 0x1002, 0x115d, 0x00001003, 0x1003, 0x115f, 0x00001004, 0x1004, 0x1160, 0x00001005, 0x1005)
USER32(NtUserCallTwoParam                                ,   NONE, 0x1144, 0x1143, 0x1029, 0x114e, 0x102a, 0x114f, 0x102a, 0x115c, 0x0000102b, 0x102b, 0x115e, 0x0000102c, 0x102c, 0x115f, 0x0000102d, 0x102d)
USER32(NtUserCallUserpExitWindowsEx                      ,   NONE,   NONE,   NONE, 0x2057,   NONE, 0x205d,   NONE, 0x205b,   NONE, 0x00002000,   NONE,   NONE, 0x00002001,   NONE,   NONE, 0x00002001,   NONE)
USER32(NtUserCallUserpRegisterLogonProcess               ,   NONE,   NONE,   NONE, 0x205a,   NONE, 0x2060,   NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE)
USER32(NtUserCanBrokerForceForeground                    ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE, 0x1140, 0x000012e5, 0x12e5, 0x1142, 0x00001306, 0x1306, 0x1143, 0x0000134b, 0x134b)
USER32(NtUserChangeClipboardChain                        , 0x113b, 0x1145, 0x1144, 0x111f, 0x114f, 0x1120, 0x1150, 0x1119, 0x115b, 0x00081119, 0x1119, 0x115d, 0x0008111a, 0x111a, 0x115e, 0x0008111b, 0x111b)
USER32(NtUserChangeDisplaySettings                       ,   NONE, 0x1146, 0x1145, 0x122a, 0x1150, 0x127a, 0x1151, 0x1288, 0x115a, 0x000012e6, 0x12e6, 0x115c, 0x00001307, 0x1307, 0x115d, 0x0000134c, 0x134c)
USER32(NtUserChangeWindowMessageFilterEx                 ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE, 0x1291, 0x1289, 0x12d9, 0x000012e7, 0x12e7, 0x12e2, 0x00001308, 0x1308, 0x12ee, 0x0000134d, 0x134d)
USER32(NtUserCheckAccessForIntegrityLevel                ,   NONE,   NONE,   NONE,   NONE, 0x1151, 0x127b, 0x1157, 0x128a, 0x1154, 0x000012e8, 0x12e8, 0x1156, 0x00001309, 0x1309, 0x1157, 0x0000134e, 0x134e)
USER32(NtUserCheckDesktopByThreadId                      ,   NONE,   NONE,   NONE,   NONE, 0x1152, 0x127c, 0x1158, 0x128b,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE)
USER32(NtUserCheckMenuItem                               ,   NONE, 0x1148, 0x1147, 0x1108, 0x1155, 0x1109, 0x115a, 0x1103, 0x1151, 0x000a1103, 0x1103, 0x1153, 0x000a1104, 0x1104, 0x1154, 0x000a1105, 0x1105)
USER32(NtUserCheckProcessForClipboardAccess              ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE, 0x13d5, 0x000012e9, 0x12e9, 0x13fe, 0x0000130a, 0x130a, 0x144b, 0x0000134f, 0x134f)
USER32(NtUserCheckProcessSession                         ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE, 0x1280, 0x000012ea, 0x12ea, 0x1283, 0x0000130b, 0x130b, 0x128d, 0x00001350, 0x1350)
USER32(NtUserCheckWindowThreadDesktop                    ,   NONE,   NONE,   NONE,   NONE, 0x1153, 0x2064, 0x1159, 0x128c, 0x1152, 0x000012eb, 0x12eb, 0x1154, 0x0000130c, 0x130c, 0x1155, 0x00001351, 0x1351)
USER32(NtUserChildWindowFromPointEx                      , 0x113f, 0x1149, 0x1148, 0x122b, 0x1156, 0x127e, 0x115b, 0x128d, 0x1150, 0x000012ec, 0x12ec, 0x1152, 0x0000130d, 0x130d, 0x1153, 0x00001352, 0x1352)
USER32(NtUserClearForeground                             ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE,   NONE, 0x13f3, 0x0000130e, 0x130e, 0x143e, 0x00001353, 0x1353)
USER32(NtUserClipCursor                                  , 0x1140, 0x114a, 0x1149, 0x122c, 0x1157, 0x127f, 0x115c, 0x128e, 0x114f, 0x000412ed, 0x12ed, 0x1151, 0x0004130f, 0x130f, 0x1152, 0x00041354, 0x1354)
USER32(NtUserCloseClipboard                              , 0x1141, 0x114b, 0x114a, 0x10d2, 0x1158, 0x10d3, 0x115d, 0x10ce, 0x114e, 0x000110ce, 0x10ce, 0x1150, 0x000110cf, 0x10cf, 0x1151, 0x000110d0, 0x10d0)
USER32(NtUserCloseDesktop                                , 0x1142, 0x114c, 0x114b, 0x10aa, 0x1159, 0x10ab, 0x115e, 0x10a6, 0x114d, 0x000410a6, 0x10a6, 0x114f, 0x000410a7, 0x10a7, 0x1150, 0x000410a8, 0x10a8)
USER32(NtUserCloseWindowStation                          , 0x1143, 0x114d, 0x114c, 0x10b9, 0x115a, 0x10ba, 0x115f, 0x10b5, 0x114c, 0x000410b5, 0x10b5, 0x114e, 0x000410b6, 0x10b6, 0x114f, 0x000410b7, 0x10b7)
USER32(NtUserCompositionInputSinkLuidFromPoint           ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE,   NONE, 0x1401, 0x00001310, 0x1310, 0x144e, 0x00001355, 0x1355)
USER32(NtUserConsoleControl                              ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE, 0x1160, 0x10df, 0x114b, 0x000010df, 0x10df, 0x114d, 0x000010e0, 0x10e0, 0x114e, 0x000010e1, 0x10e1)
USER32(NtUserConvertMemHandle                            ,   NONE, 0x114f, 0x114e, 0x1102, 0x115c, 0x1103, 0x1161, 0x10fd, 0x114a, 0x000510fd, 0x10fd, 0x114c, 0x000510fe, 0x10fe, 0x114d, 0x000510ff, 0x10ff)
USER32(NtUserCopyAcceleratorTable                        , 0x1146, 0x1150, 0x114f, 0x102b, 0x115d, 0x102c, 0x1162, 0x102c, 0x1149, 0x000a102d, 0x102d, 0x114b, 0x000a102e, 0x102e, 0x114c, 0x000a102f, 0x102f)
USER32(NtUserCountClipboardFormats                       , 0x1147, 0x1151, 0x1150, 0x1115, 0x115e, 0x1116, 0x1163, 0x110f, 0x1148, 0x0001110f, 0x110f, 0x114a, 0x00011110, 0x1110, 0x114b, 0x00011111, 0x1111)
USER32(NtUserCreateAcceleratorTable                      , 0x1148, 0x1152, 0x1151, 0x10f5, 0x115f, 0x10f6, 0x1164, 0x10f1, 0x1147, 0x000910f1, 0x10f1, 0x1149, 0x000910f2, 0x10f2, 0x114a, 0x000910f3, 0x10f3)
USER32(NtUserCreateCaret                                 , 0x1149, 0x1153, 0x1152, 0x1031, 0x1160, 0x1032, 0x1165, 0x1032, 0x1146, 0x00141033, 0x1033, 0x1148, 0x00141034, 0x1034, 0x1149, 0x00141035, 0x1035)
USER32(NtUserCreateDCompositionHwndTarget                ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE, 0x13bc, 0x000012ee, 0x12ee, 0x13e4, 0x00001311, 0x1311, 0x142f, 0x00001356, 0x1356)
USER32(NtUserCreateDesktop                               ,   NONE, 0x1154, 0x1153, 0x122d,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE)
USER32(NtUserCreateDesktopEx                             ,   NONE,   NONE,   NONE,   NONE, 0x1161, 0x1280, 0x1166, 0x128f, 0x1145, 0x000012ef, 0x12ef, 0x1147, 0x00001312, 0x1312, 0x1148, 0x00001357, 0x1357)
USER32(NtUserCreateLocalMemHandle                        ,   NONE, 0x1156, 0x1155, 0x10ef, 0x1163, 0x10f0, 0x1168, 0x10eb, 0x1143, 0x001410eb, 0x10eb, 0x1145, 0x001410ec, 0x10ec, 0x1146, 0x001410ed, 0x10ed)
USER32(NtUserCreateWindowEx                              ,   NONE, 0x1157, 0x1156, 0x1077, 0x1164, 0x1078, 0x1169, 0x1076, 0x1142, 0x00001076, 0x1076, 0x1144, 0x00001077, 0x1077, 0x1145, 0x00001078, 0x1078)
USER32(NtUserCreateWindowStation                         ,   NONE, 0x1158, 0x1157, 0x122f, 0x1165, 0x1282, 0x116a, 0x1291, 0x1141, 0x000012f1, 0x12f1, 0x1143, 0x00001314, 0x1314, 0x1144, 0x00001359, 0x1359)
USER32(NtUserCtxDisplayIOCtl                             ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE, 0x12f2,   NONE,       NONE, 0x1315, 0x1290, 0x0000135a, 0x135a)
USER32(NtUserDdeGetQualityOfService                      , 0x114f, 0x1159, 0x1158, 0x1231,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE)
USER32(NtUserDdeInitialize                               ,   NONE, 0x115a, 0x1159, 0x1113, 0x1166, 0x1114, 0x116b, 0x110d, 0x113f, 0x0000110d, 0x110d, 0x1141, 0x0000110e, 0x110e, 0x1142, 0x0000110f, 0x110f)
USER32(NtUserDdeSetQualityOfService                      , 0x1151, 0x115b, 0x115a, 0x1232,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE)
USER32(NtUserDefSetText                                  ,   NONE, 0x115d, 0x115c, 0x1080, 0x1168, 0x1081, 0x116d, 0x107f, 0x113d, 0x0000107f, 0x107f, 0x113f, 0x00001080, 0x1080, 0x1140, 0x00001081, 0x1081)
USER32(NtUserDeferWindowPos                              , 0x1152, 0x115c, 0x115b, 0x1052, 0x1167, 0x1053, 0x116c, 0x1053,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE)
USER32(NtUserDeferWindowPosAndBand                       ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE, 0x113e, 0x000012f3, 0x12f3, 0x1140, 0x00001316, 0x1316, 0x1141, 0x0000135b, 0x135b)
USER32(NtUserDelegateCapturePointers                     ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE, 0x1172, 0x000012f4, 0x12f4, 0x1174, 0x00001317, 0x1317, 0x1175, 0x0000135c, 0x135c)
USER32(NtUserDelegateInput                               ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE, 0x1173, 0x000012f5, 0x12f5, 0x1175, 0x00001318, 0x1318, 0x1176, 0x0000135d, 0x135d)
USER32(NtUserDeleteMenu                                  , 0x1154, 0x115e, 0x115d, 0x10c0, 0x1169, 0x10c1, 0x116e, 0x10bc, 0x117c, 0x000a10bc, 0x10bc, 0x117e, 0x000a10bd, 0x10bd, 0x117f, 0x000a10be, 0x10be)
USER32(NtUserDesktopHasWatermarkText                     , 0x1170,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE)
USER32(NtUserDestroyAcceleratorTable                     ,   NONE, 0x115f, 0x115e, 0x1103, 0x116a, 0x1104, 0x116f, 0x10fe, 0x117b, 0x000410fe, 0x10fe, 0x117d, 0x000410ff, 0x10ff, 0x117e, 0x00041100, 0x1100)
USER32(NtUserDestroyCursor                               ,   NONE, 0x1160, 0x115f, 0x109d, 0x116b, 0x109e, 0x1170, 0x109c, 0x117a, 0x0005109c, 0x109c, 0x117c, 0x0005109d, 0x109d, 0x117d, 0x0005109e, 0x109e)
USER32(NtUserDestroyDCompositionHwndTarget               ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE, 0x13bb, 0x000012f6, 0x12f6, 0x13e3, 0x00001319, 0x1319, 0x142e, 0x0000135e, 0x135e)
USER32(NtUserDestroyMenu                                 , 0x1158, 0x1162, 0x1161, 0x10e1, 0x116d, 0x10e2, 0x1172, 0x10dd, 0x1178, 0x000410dd, 0x10dd, 0x117a, 0x000410de, 0x10de, 0x117b, 0x000410df, 0x10df)
USER32(NtUserDestroyWindow                               , 0x1159, 0x1163, 0x1162, 0x109e, 0x116e, 0x109f, 0x1173, 0x109d, 0x1177, 0x0003109d, 0x109d, 0x1179, 0x0003109e, 0x109e, 0x117a, 0x0003109f, 0x109f)
USER32(NtUserDeviceEventWorker                           ,   NONE,   NONE,   NONE, 0x205d,   NONE, 0x2065,   NONE, 0x205e,   NONE, 0x00002003,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE)
USER32(NtUserDisableImmersiveOwner                       ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE, 0x13ca, 0x000012f8, 0x12f8, 0x13f2, 0x0000131b, 0x131b, 0x143d, 0x00001360, 0x1360)
USER32(NtUserDisableProcessWindowFiltering               ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE, 0x1175, 0x000012f9, 0x12f9, 0x1177, 0x0000131c, 0x131c, 0x1178, 0x00001361, 0x1361)
USER32(NtUserDisableThreadIme                            ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE, 0x12fa,   NONE,       NONE, 0x131d, 0x1179, 0x00041362, 0x1362)
USER32(NtUserDiscardPointerFrameMessages                 ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE, 0x12cf, 0x000012fb, 0x12fb, 0x12d7, 0x0000131e, 0x131e, 0x12e2, 0x00001363, 0x1363)
USER32(NtUserDispatchMessage                             ,   NONE, 0x1165, 0x1164, 0x1035, 0x1170, 0x1036, 0x1175, 0x1036, 0x1174, 0x00001037, 0x1037, 0x1176, 0x00001038, 0x1038, 0x1177, 0x00001039, 0x1039)
USER32(NtUserDisplayConfigGetDeviceInfo                  ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE, 0x1155, 0x1295, 0x1156, 0x000012fc, 0x12fc, 0x1158, 0x0000131f, 0x131f, 0x1159, 0x00001364, 0x1364)
USER32(NtUserDisplayConfigSetDeviceInfo                  ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE, 0x1156, 0x1296, 0x1155, 0x000012fd, 0x12fd, 0x1157, 0x00001320, 0x1320, 0x1158, 0x00001365, 0x1365)
USER32(NtUserDoSoundConnect                              ,   NONE,   NONE,   NONE,   NONE, 0x1171, 0x1286, 0x1176, 0x1297, 0x117e, 0x000012fe, 0x12fe, 0x1180, 0x00001321, 0x1321, 0x1181, 0x00001366, 0x1366)
USER32(NtUserDoSoundDisconnect                           ,   NONE,   NONE,   NONE,   NONE, 0x1172, 0x1287, 0x1177, 0x1298, 0x117d, 0x000012ff, 0x12ff, 0x117f, 0x00001322, 0x1322, 0x1180, 0x00001367, 0x1367)
USER32(NtUserDragDetect                                  , 0x115c, 0x1166, 0x1165, 0x1235, 0x1173, 0x1288, 0x1178, 0x1299, 0x11df, 0x00001300, 0x1300, 0x11e0, 0x00001323, 0x1323, 0x11e2, 0x00001368, 0x1368)
USER32(NtUserDragObject                                  , 0x115d, 0x1167, 0x1166, 0x1236, 0x1174, 0x1289, 0x1179, 0x129a, 0x11de, 0x00001301, 0x1301, 0x11df, 0x00001324, 0x1324, 0x11e1, 0x00001369, 0x1369)
USER32(NtUserDrawAnimatedRects                           , 0x115e, 0x1168, 0x1167, 0x1237, 0x1175, 0x128a, 0x117a, 0x129b, 0x11dd, 0x00141302, 0x1302, 0x11de, 0x00141325, 0x1325, 0x11e0, 0x0014136a, 0x136a)
USER32(NtUserDrawCaption                                 ,   NONE, 0x1169, 0x1168, 0x1238, 0x1176, 0x128b, 0x117b, 0x129c, 0x11dc, 0x00121303, 0x1303, 0x11dd, 0x00121326, 0x1326, 0x11df, 0x0012136b, 0x136b)
USER32(NtUserDrawCaptionTemp                             ,   NONE, 0x116a, 0x1169, 0x1239, 0x1177, 0x128c, 0x117c, 0x129d, 0x11db, 0x00001304, 0x1304, 0x11dc, 0x00001327, 0x1327, 0x11de, 0x0000136c, 0x136c)
USER32(NtUserDrawIconEx                                  ,   NONE, 0x116b, 0x116a, 0x105f, 0x1178, 0x1060, 0x117d, 0x1060, 0x11da, 0x00001060, 0x1060, 0x11db, 0x00001061, 0x1061, 0x11dd, 0x00001062, 0x1062)
USER32(NtUserDrawMenuBarTemp                             ,   NONE, 0x116c, 0x116b, 0x123a, 0x1179, 0x128d, 0x117e, 0x129e, 0x11d9, 0x00001305, 0x1305, 0x11da, 0x00001328, 0x1328, 0x11dc, 0x0000136d, 0x136d)
USER32(NtUserDwmGetDxRgn                                 ,   NONE,   NONE,   NONE,   NONE, 0x1276, 0x128e,   NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE)
USER32(NtUserDwmGetRemoteSessionOcclusionEvent           ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE, 0x127d, 0x00001306, 0x1306, 0x1280, 0x00001329, 0x1329, 0x1288, 0x0000136e, 0x136e)
USER32(NtUserDwmGetRemoteSessionOcclusionState           ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE, 0x127e, 0x00001307, 0x1307, 0x1281, 0x0000132a, 0x132a, 0x1289, 0x0000136f, 0x136f)
USER32(NtUserDwmHintDxUpdate                             ,   NONE,   NONE,   NONE,   NONE, 0x1275, 0x128f,   NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE)
USER32(NtUserDwmKernelShutdown                           ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE, 0x128a, 0x00001370, 0x1370)
USER32(NtUserDwmKernelStartup                            ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE, 0x128b, 0x00001371, 0x1371)
USER32(NtUserDwmStartRedirection                         ,   NONE,   NONE,   NONE,   NONE, 0x1273, 0x1290, 0x127f, 0x129f, 0x127c, 0x00001308, 0x1308, 0x127f, 0x0000132b, 0x132b,   NONE,       NONE,   NONE)
USER32(NtUserDwmStopRedirection                          ,   NONE,   NONE,   NONE,   NONE, 0x1274, 0x1291, 0x1280, 0x12a0, 0x127b, 0x00001309, 0x1309, 0x127e, 0x0000132c, 0x132c,   NONE,       NONE,   NONE)
USER32(NtUserDwmValidateWindow                           ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE, 0x1153, 0x0000130a, 0x130a, 0x1155, 0x0000132d, 0x132d, 0x1156, 0x00001372, 0x1372)
USER32(NtUserEmptyClipboard                              , 0x1163, 0x116d, 0x116c, 0x10fc, 0x117a, 0x10fd, 0x117f, 0x10f8, 0x11d8, 0x000110f8, 0x10f8, 0x11d9, 0x000110f9, 0x10f9, 0x11db, 0x000110fa, 0x10fa)
USER32(NtUserEnableChildWindowDpiMessage                 ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE, 0x1245, 0x00001373, 0x1373)
USER32(NtUserEnableIAMAccess                             ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE, 0x13cb, 0x0000130b, 0x130b, 0x13f4, 0x0000132e, 0x132e, 0x143f, 0x00001374, 0x1374)
USER32(NtUserEnableMenuItem                              ,   NONE, 0x116e, 0x116d, 0x10d6, 0x117b, 0x10d7, 0x1180, 0x10d2, 0x11d7, 0x000a10d2, 0x10d2, 0x11d8, 0x000a10d3, 0x10d3, 0x11da, 0x000a10d4, 0x10d4)
USER32(NtUserEnableMouseInPointer                        ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE, 0x13ce, 0x0000130c, 0x130c, 0x13f7, 0x0000132f, 0x132f, 0x1444, 0x00001375, 0x1375)
USER32(NtUserEnableMouseInputForCursorSuppression        ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE, 0x13d2, 0x0000130d, 0x130d, 0x13fb, 0x00001330, 0x1330, 0x1448, 0x00001376, 0x1376)
USER32(NtUserEnableScrollBar                             , 0x1165, 0x116f, 0x116e, 0x10bb, 0x117c, 0x10bc, 0x1181, 0x10b7, 0x11d6, 0x000c10b7, 0x10b7, 0x11d7, 0x000c10b8, 0x10b8, 0x11d9, 0x000c10b9, 0x10b9)
USER32(NtUserEnableTouchPad                              ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE,   NONE, 0x12d3, 0x00001331, 0x1331, 0x12de, 0x00001377, 0x1377)
USER32(NtUserEndDeferWindowPosEx                         ,   NONE, 0x1170, 0x116f, 0x1025, 0x117d, 0x1026, 0x1182, 0x1026, 0x11d5, 0x00071027, 0x1027, 0x11d6, 0x00071028, 0x1028, 0x11d8, 0x00071029, 0x1029)
USER32(NtUserEndMenu                                     , 0x1167, 0x1171, 0x1170, 0x123b, 0x117e, 0x1292, 0x1183, 0x12a1, 0x11d4, 0x0001130e, 0x130e, 0x11d5, 0x00011332, 0x1332, 0x11d7, 0x00011378, 0x1378)
USER32(NtUserEndPaint                                    , 0x1168, 0x1172, 0x1171, 0x1018, 0x117f, 0x1019, 0x1184, 0x1019, 0x11d3, 0x0000101a, 0x101a, 0x11d4, 0x0000101b, 0x101b, 0x11d6, 0x0000101c, 0x101c)
USER32(NtUserEndTask                                     ,   NONE,   NONE,   NONE, 0x2058,   NONE, 0x205e,   NONE, 0x205c,   NONE, 0x00002001,   NONE,   NONE, 0x00002002,   NONE,   NONE, 0x00002002,   NONE)
USER32(NtUserEndTouchOperation                           , 0x1084,   NONE,   NONE,   NONE,   NONE,   NONE, 0x128f, 0x12a2,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE)
USER32(NtUserEnumDisplayDevices                          ,   NONE, 0x1173, 0x1172, 0x10fb, 0x1180, 0x10fc, 0x1185, 0x10f7, 0x11d2, 0x000010f7, 0x10f7, 0x11d3, 0x000010f8, 0x10f8, 0x11d5, 0x000010f9, 0x10f9)
USER32(NtUserEnumDisplayMonitors                         , 0x116a, 0x1174, 0x1173, 0x1049, 0x1181, 0x104a, 0x1186, 0x104a, 0x11d1, 0x0000104b, 0x104b, 0x11d2, 0x0000104c, 0x104c, 0x11d4, 0x0000104d, 0x104d)
USER32(NtUserEnumDisplaySettings                         ,   NONE, 0x1175, 0x1174, 0x111a, 0x1182, 0x111b, 0x1187, 0x1114, 0x11d0, 0x00001114, 0x1114, 0x11d1, 0x00001115, 0x1115, 0x11d3, 0x00001116, 0x1116)
USER32(NtUserEvent                                       ,   NONE, 0x1176, 0x1175, 0x123c, 0x1183, 0x1293, 0x1188, 0x12a3, 0x11cf, 0x0004130f, 0x130f, 0x11d0, 0x00041333, 0x1333, 0x11d2, 0x00041379, 0x1379)
USER32(NtUserExcludeUpdateRgn                            , 0x116d, 0x1177, 0x1176, 0x104f, 0x1184, 0x1050, 0x1189, 0x1050, 0x11ce, 0x00081051, 0x1051, 0x11cf, 0x00081052, 0x1052, 0x11d1, 0x00081053, 0x1053)
USER32(NtUserFillWindow                                  ,   NONE, 0x1178, 0x1177, 0x108a, 0x1185, 0x108b, 0x118a, 0x1089, 0x11cd, 0x00171089, 0x1089, 0x11ce, 0x0017108a, 0x108a, 0x11d0, 0x0017108b, 0x108b)
USER32(NtUserFindExistingCursorIcon                      ,   NONE, 0x1179, 0x1178, 0x103d, 0x1186, 0x103e, 0x118b, 0x103e, 0x11cc, 0x0000103f, 0x103f, 0x11cd, 0x00001040, 0x1040, 0x11cf, 0x00001041, 0x1041)
USER32(NtUserFindWindowEx                                ,   NONE, 0x117a, 0x1179, 0x106e, 0x1187, 0x106f, 0x118c, 0x106e, 0x11cb, 0x0000106e, 0x106e, 0x11cc, 0x0000106f, 0x106f, 0x11ce, 0x00001070, 0x1070)
USER32(NtUserFlashWindowEx                               , 0x1171, 0x117b, 0x117a, 0x123d, 0x1188, 0x1294, 0x118d, 0x12a4, 0x11ca, 0x00001310, 0x1310, 0x11cb, 0x00001334, 0x1334, 0x11cd, 0x0000137a, 0x137a)
USER32(NtUserFrostCrashedWindow                          ,   NONE,   NONE,   NONE,   NONE, 0x1189, 0x1295, 0x118e, 0x12a5, 0x11c9, 0x00001311, 0x1311, 0x11ca, 0x00001335, 0x1335, 0x11cc, 0x0000137b, 0x137b)
USER32(NtUserGetAltTabInfo                               ,   NONE, 0x117c, 0x117b, 0x10f7, 0x118a, 0x10f8, 0x118f, 0x10f3, 0x11c8, 0x000010f3, 0x10f3, 0x11c9, 0x000010f4, 0x10f4, 0x11cb, 0x000010f5, 0x10f5)
USER32(NtUserGetAncestor                                 , 0x1173, 0x117d, 0x117c, 0x10b6, 0x118b, 0x10b7, 0x1190, 0x10b2, 0x11c7, 0x000710b2, 0x10b2, 0x11c8, 0x000710b3, 0x10b3, 0x11ca, 0x000710b4, 0x10b4)
USER32(NtUserGetAsyncKeyState                            ,   NONE, 0x117f, 0x117e, 0x1043, 0x118d, 0x1044, 0x1192, 0x1044, 0x11c5, 0x00031045, 0x1045, 0x11c6, 0x00031046, 0x1046, 0x11c8, 0x00031047, 0x1047)
USER32(NtUserGetAtomName                                 ,   NONE, 0x1180, 0x117f, 0x10ad, 0x118e, 0x10ae, 0x1193, 0x10a9, 0x11c4, 0x000010a9, 0x10a9, 0x11c5, 0x000010aa, 0x10aa, 0x11c7, 0x000010ab, 0x10ab)
USER32(NtUserGetAutoRotationState                        ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE, 0x13c4, 0x00001313, 0x1313, 0x13ec, 0x00001337, 0x1337, 0x1437, 0x0000137d, 0x137d)
USER32(NtUserGetCIMSSM                                   ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE, 0x11b5, 0x00001314, 0x1314, 0x11b6, 0x00001338, 0x1338, 0x11b8, 0x0000137e, 0x137e)
USER32(NtUserGetCPD                                      ,   NONE, 0x118e, 0x118d, 0x1044, 0x119c, 0x1045, 0x11a1, 0x1045, 0x11b6, 0x00001046, 0x1046, 0x11b7, 0x00001047, 0x1047, 0x11b9, 0x00001048, 0x1048)
USER32(NtUserGetCaretBlinkTime                           , 0x1176, 0x1181, 0x1180, 0x10f8, 0x118f, 0x10f9, 0x1194, 0x10f4, 0x11c3, 0x000110f4, 0x10f4, 0x11c4, 0x000110f5, 0x10f5, 0x11c6, 0x000110f6, 0x10f6)
USER32(NtUserGetCaretPos                                 , 0x1177, 0x1182, 0x1181, 0x123f, 0x1190, 0x1297, 0x1195, 0x12a7, 0x11c2, 0x00041315, 0x1315, 0x11c3, 0x00041339, 0x1339, 0x11c5, 0x0004137f, 0x137f)
USER32(NtUserGetClassInfo                                ,   NONE, 0x1183,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE)
USER32(NtUserGetClassInfoEx                              ,   NONE,   NONE, 0x1182, 0x10bd, 0x1191, 0x10be, 0x1196, 0x10b9, 0x11c1, 0x000010b9, 0x10b9, 0x11c2, 0x000010ba, 0x10ba, 0x11c4, 0x000010bb, 0x10bb)
USER32(NtUserGetClassName                                ,   NONE, 0x1184, 0x1183, 0x107c, 0x1192, 0x107d, 0x1197, 0x107b, 0x11c0, 0x0000107b, 0x107b, 0x11c1, 0x0000107c, 0x107c, 0x11c3, 0x0000107d, 0x107d)
USER32(NtUserGetClipCursor                               , 0x117f, 0x118a, 0x1189, 0x1240, 0x1198, 0x1298, 0x119d, 0x12a8, 0x11ba, 0x00041316, 0x1316, 0x11bb, 0x0004133a, 0x133a, 0x11bd, 0x00041380, 0x1380)
USER32(NtUserGetClipboardAccessToken                     ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE, 0x13d6, 0x00001317, 0x1317, 0x13ff, 0x0000133b, 0x133b, 0x144c, 0x00001381, 0x1381)
USER32(NtUserGetClipboardData                            ,   NONE, 0x1185, 0x1184, 0x10fd, 0x1193, 0x10fe, 0x1198, 0x10f9, 0x11bf, 0x000010f9, 0x10f9, 0x11c0, 0x000010fa, 0x10fa, 0x11c2, 0x000010fb, 0x10fb)
USER32(NtUserGetClipboardFormatName                      ,   NONE, 0x1186, 0x1185, 0x10ed, 0x1194, 0x10ee, 0x1199, 0x10e9, 0x11be, 0x000a10e9, 0x10e9, 0x11bf, 0x000a10ea, 0x10ea, 0x11c1, 0x000a10eb, 0x10eb)
USER32(NtUserGetClipboardOwner                           , 0x117c, 0x1187, 0x1186, 0x10cd, 0x1195, 0x10ce, 0x119a, 0x10c9, 0x11bd, 0x000110c9, 0x10c9, 0x11be, 0x000110ca, 0x10ca, 0x11c0, 0x000110cb, 0x10cb)
USER32(NtUserGetClipboardSequenceNumber                  , 0x117d, 0x1188, 0x1187, 0x1055, 0x1196, 0x1056, 0x119b, 0x1056, 0x11bc, 0x00011056, 0x1056, 0x11bd, 0x00011057, 0x1057, 0x11bf, 0x00011058, 0x1058)
USER32(NtUserGetClipboardViewer                          , 0x117e, 0x1189, 0x1188, 0x1241, 0x1197, 0x1299, 0x119c, 0x12a9, 0x11bb, 0x00011318, 0x1318, 0x11bc, 0x0001133c, 0x133c, 0x11be, 0x00011382, 0x1382)
USER32(NtUserGetComboBoxInfo                             , 0x1180, 0x118b, 0x118a, 0x1242, 0x1199, 0x129a, 0x119e, 0x12aa, 0x11b9, 0x00001319, 0x1319, 0x11ba, 0x0000133d, 0x133d, 0x11bc, 0x00001383, 0x1383)
USER32(NtUserGetControlBrush                             ,   NONE, 0x118c, 0x118b, 0x107b, 0x119a, 0x107c, 0x119f, 0x107a, 0x11b8, 0x000e107a, 0x107a, 0x11b9, 0x000e107b, 0x107b, 0x11bb, 0x000e107c, 0x107c)
USER32(NtUserGetControlColor                             ,   NONE, 0x118d, 0x118c, 0x10e7, 0x119b, 0x10e8, 0x11a0, 0x10e3, 0x11b7, 0x001710e3, 0x10e3, 0x11b8, 0x001710e4, 0x10e4, 0x11ba, 0x001710e5, 0x10e5)
USER32(NtUserGetCurrentInputMessageSource                ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE, 0x11b4, 0x0000131a, 0x131a, 0x11b5, 0x0000133e, 0x133e, 0x11b7, 0x00001384, 0x1384)
USER32(NtUserGetCursorDims                               ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE,   NONE, 0x140a, 0x0000133f, 0x133f, 0x1457, 0x00001385, 0x1385)
USER32(NtUserGetCursorFrameInfo                          ,   NONE, 0x118f, 0x118e, 0x10f6, 0x119d, 0x10f7, 0x11a2, 0x10f2, 0x11b3, 0x001110f2, 0x10f2, 0x11b4, 0x001110f3, 0x10f3, 0x11b6, 0x001110f4, 0x10f4)
USER32(NtUserGetCursorInfo                               , 0x1185, 0x1190, 0x118f, 0x1243, 0x119e, 0x129b, 0x11a3, 0x12ab, 0x11b2, 0x0000131b, 0x131b, 0x11b3, 0x00001340, 0x1340, 0x11b5, 0x00001386, 0x1386)
USER32(NtUserGetDC                                       , 0x1186, 0x1191, 0x1190, 0x100a, 0x119f, 0x100a, 0x11a4, 0x100a, 0x11b1, 0x0003100b, 0x100b, 0x11b2, 0x0003100c, 0x100c, 0x11b4, 0x0003100d, 0x100d)
USER32(NtUserGetDCEx                                     , 0x1187, 0x1192, 0x1191, 0x1093, 0x11a0, 0x1094, 0x11a5, 0x1092, 0x11b0, 0x000c1092, 0x1092, 0x11b1, 0x000c1093, 0x1093, 0x11b3, 0x000c1094, 0x1094)
USER32(NtUserGetDManipHookInitFunction                   ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE, 0x127e, 0x00001387, 0x1387)
USER32(NtUserGetDesktopID                                ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE, 0x11af, 0x0000131c, 0x131c, 0x11b0, 0x00001341, 0x1341, 0x11b2, 0x00001388, 0x1388)
USER32(NtUserGetDisplayAutoRotationPreferences           ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE, 0x13c1, 0x0000131d, 0x131d, 0x13e9, 0x00001342, 0x1342, 0x1434, 0x00001389, 0x1389)
USER32(NtUserGetDisplayAutoRotationPreferencesByProcessId,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE, 0x13c0, 0x0000131e, 0x131e, 0x13e8, 0x00001343, 0x1343, 0x1433, 0x0000138a, 0x138a)
USER32(NtUserGetDisplayConfigBufferSizes                 ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE, 0x1152, 0x12ac, 0x1159, 0x0000131f, 0x131f, 0x115b, 0x00001344, 0x1344, 0x115c, 0x0000138b, 0x138b)
USER32(NtUserGetDoubleClickTime                          , 0x1188, 0x1193, 0x1192, 0x10ba, 0x11a1, 0x10bb, 0x11a6, 0x10b6, 0x11ae, 0x000110b6, 0x10b6, 0x11af, 0x000110b7, 0x10b7, 0x11b1, 0x000110b8, 0x10b8)
USER32(NtUserGetDpiForMonitor                            ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE,   NONE, 0x1407, 0x00001345, 0x1345, 0x1454, 0x0000138c, 0x138c)
USER32(NtUserGetDpiMetrics                               ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE, 0x11b0, 0x0000138d, 0x138d)
USER32(NtUserGetForegroundWindow                         , 0x1189, 0x1194, 0x1193, 0x103b, 0x11a2, 0x103c, 0x11a7, 0x103c, 0x11ad, 0x0001103d, 0x103d, 0x11ae, 0x0001103e, 0x103e, 0x11af, 0x0001103f, 0x103f)
USER32(NtUserGetGUIThreadInfo                            , 0x118b, 0x1196, 0x1195, 0x1104, 0x11a4, 0x1105, 0x11a9, 0x10ff, 0x11aa, 0x000010ff, 0x10ff, 0x11ac, 0x00001100, 0x1100, 0x11ad, 0x00001101, 0x1101)
USER32(NtUserGetGestureConfig                            ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE, 0x1297, 0x12ad, 0x12d4, 0x00001320, 0x1320, 0x12dd, 0x00001346, 0x1346, 0x12e9, 0x0000138e, 0x138e)
USER32(NtUserGetGestureExtArgs                           ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE, 0x1294, 0x12ae, 0x12d6, 0x00001321, 0x1321, 0x12df, 0x00001347, 0x1347, 0x12eb, 0x0000138f, 0x138f)
USER32(NtUserGetGestureInfo                              ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE, 0x1293, 0x12af, 0x12d7, 0x00001322, 0x1322, 0x12e0, 0x00001348, 0x1348, 0x12ec, 0x00001390, 0x1390)
USER32(NtUserGetGlobalIMEStatus                          ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE, 0x11ac, 0x00001323, 0x1323,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE)
USER32(NtUserGetGuiResources                             , 0x118a, 0x1195, 0x1194, 0x1244, 0x11a3, 0x129c, 0x11a8, 0x12b0, 0x11ab, 0x00071324, 0x1324, 0x11ad, 0x00071349, 0x1349, 0x11ae, 0x00071391, 0x1391)
USER32(NtUserGetHimetricScaleFactorFromPixelLocation     ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE,   NONE, 0x1405, 0x0000134a, 0x134a, 0x1452, 0x00001392, 0x1392)
USER32(NtUserGetIconInfo                                 ,   NONE, 0x1197, 0x1196, 0x104e, 0x11a5, 0x104f, 0x11aa, 0x104f, 0x11a9, 0x00001050, 0x1050, 0x11ab, 0x00001051, 0x1051, 0x11ac, 0x00001052, 0x1052)
USER32(NtUserGetIconSize                                 ,   NONE, 0x1198, 0x1197, 0x1089, 0x11a6, 0x108a, 0x11ab, 0x1088, 0x11a8, 0x00111088, 0x1088, 0x11aa, 0x00111089, 0x1089, 0x11ab, 0x0011108a, 0x108a)
USER32(NtUserGetImeHotKey                                , 0x118e, 0x1199, 0x1198, 0x1245, 0x11a7, 0x129d, 0x11ac, 0x12b1, 0x11a7, 0x00001325, 0x1325, 0x11a9, 0x0000134b, 0x134b, 0x11aa, 0x00001393, 0x1393)
USER32(NtUserGetInputLocaleInfo                          ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE, 0x11ae, 0x12b3, 0x11a5, 0x00001327, 0x1327, 0x11a7, 0x0000134d, 0x134d, 0x11a8, 0x00001395, 0x1395)
USER32(NtUserGetInternalWindowPos                        , 0x1190, 0x119b, 0x119a, 0x1247, 0x11a9, 0x129f, 0x11af, 0x12b4, 0x11a4, 0x000c1328, 0x1328, 0x11a6, 0x000c134e, 0x134e, 0x11a7, 0x000c1396, 0x1396)
USER32(NtUserGetKeyNameText                              ,   NONE, 0x119f, 0x119e, 0x1248, 0x11ad, 0x12a0, 0x11b3, 0x12b5, 0x11a0, 0x000a1329, 0x1329, 0x11a2, 0x000a134f, 0x134f, 0x11a3, 0x000a1397, 0x1397)
USER32(NtUserGetKeyState                                 ,   NONE, 0x11a0, 0x119f, 0x1003, 0x11ae, 0x1003, 0x11b4, 0x1003, 0x119f, 0x00031004, 0x1004, 0x11a1, 0x00031005, 0x1005, 0x11a2, 0x00031006, 0x1006)
USER32(NtUserGetKeyboardLayoutList                       , 0x1191, 0x119c, 0x119b, 0x1058, 0x11aa, 0x1059, 0x11b0, 0x1059, 0x11a3, 0x00001059, 0x1059, 0x11a5, 0x0000105a, 0x105a, 0x11a6, 0x0000105b, 0x105b)
USER32(NtUserGetKeyboardLayoutName                       ,   NONE, 0x119d, 0x119c, 0x1249, 0x11ab, 0x12a1, 0x11b1, 0x12b6, 0x11a2, 0x0000132a, 0x132a, 0x11a4, 0x00001350, 0x1350, 0x11a5, 0x00001398, 0x1398)
USER32(NtUserGetKeyboardState                            , 0x1193, 0x119e, 0x119d, 0x1079, 0x11ac, 0x107a, 0x11b2, 0x1078, 0x11a1, 0x00041078, 0x1078, 0x11a3, 0x00041079, 0x1079, 0x11a4, 0x0004107a, 0x107a)
USER32(NtUserGetLayeredWindowAttributes                  ,   NONE, 0x1244, 0x1240, 0x124a, 0x125c, 0x12a2, 0x1269, 0x12b7, 0x1295, 0x0014132b, 0x132b, 0x1298, 0x00141351, 0x1351, 0x12a2, 0x00141399, 0x1399)
USER32(NtUserGetListBoxInfo                              , 0x1196, 0x11a1, 0x11a0, 0x124b, 0x11af, 0x12a3, 0x11b5, 0x12b8, 0x119e, 0x0003132c, 0x132c, 0x11a0, 0x00031352, 0x1352, 0x11a1, 0x0003139a, 0x139a)
USER32(NtUserGetMenuBarInfo                              , 0x1197, 0x11a2, 0x11a1, 0x10c5, 0x11b0, 0x10c6, 0x11b6, 0x10c1, 0x119d, 0x000010c1, 0x10c1, 0x119f, 0x000010c2, 0x10c2, 0x11a0, 0x000010c3, 0x10c3)
USER32(NtUserGetMenuIndex                                ,   NONE, 0x11a3, 0x11a2, 0x124c, 0x11b1, 0x12a4, 0x11b7, 0x12b9, 0x119c, 0x0005132d, 0x132d, 0x119e, 0x00051353, 0x1353, 0x119f, 0x0005139b, 0x139b)
USER32(NtUserGetMenuItemRect                             , 0x1199, 0x11a4, 0x11a3, 0x124d, 0x11b2, 0x12a5, 0x11b8, 0x12ba, 0x119b, 0x0014132e, 0x132e, 0x119d, 0x00141354, 0x1354, 0x119e, 0x0014139c, 0x139c)
USER32(NtUserGetMessage                                  ,   NONE, 0x11a5, 0x11a4, 0x1006, 0x11b3, 0x1006, 0x11b9, 0x1006, 0x119a, 0x00001007, 0x1007, 0x119c, 0x00001008, 0x1008, 0x119d, 0x00001009, 0x1009)
USER32(NtUserGetMouseMovePointsEx                        , 0x119b, 0x11a6, 0x11a5, 0x124e, 0x11b4, 0x12a6, 0x11ba, 0x12bb, 0x1199, 0x0000132f, 0x132f, 0x119b, 0x00001355, 0x1355, 0x119c, 0x0000139d, 0x139d)
USER32(NtUserGetObjectInformation                        , 0x119c, 0x11a7, 0x11a6, 0x106b, 0x11b5, 0x106c, 0x11bb, 0x106c, 0x1198, 0x0000106c, 0x106c, 0x119a, 0x0000106d, 0x106d, 0x119b, 0x0000106e, 0x106e)
USER32(NtUserGetOpenClipboardWindow                      , 0x119d, 0x11a8, 0x11a7, 0x10dc, 0x11b6, 0x10dd, 0x11bc, 0x10d8, 0x1197, 0x000110d8, 0x10d8, 0x1199, 0x000110d9, 0x10d9, 0x119a, 0x000110da, 0x10da)
USER32(NtUserGetOwnerTransformedMonitorRect              ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE,   NONE, 0x140b, 0x00001356, 0x1356, 0x1000, 0x00001000, 0x1000)
USER32(NtUserGetPhysicalDeviceRect                       ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE,   NONE, 0x12ca, 0x00001357, 0x1357, 0x12d5, 0x0000139e, 0x139e)
USER32(NtUserGetPointerCursorId                          ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE, 0x12d2, 0x00001330, 0x1330, 0x12db, 0x00001358, 0x1358, 0x12e7, 0x0000139f, 0x139f)
USER32(NtUserGetPointerDevice                            ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE, 0x12cc, 0x00001331, 0x1331, 0x12d1, 0x00001359, 0x1359, 0x12dc, 0x000013a0, 0x13a0)
USER32(NtUserGetPointerDeviceCursors                     ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE, 0x12c8, 0x00001332, 0x1332, 0x12cd, 0x0000135a, 0x135a, 0x12d8, 0x000013a1, 0x13a1)
USER32(NtUserGetPointerDeviceProperties                  ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE, 0x12cb, 0x00001333, 0x1333, 0x12d0, 0x0000135b, 0x135b, 0x12db, 0x000013a2, 0x13a2)
USER32(NtUserGetPointerDeviceRects                       ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE, 0x12c9, 0x00001334, 0x1334, 0x12ce, 0x0000135c, 0x135c, 0x12d9, 0x000013a3, 0x13a3)
USER32(NtUserGetPointerDevices                           ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE, 0x12cd, 0x00001335, 0x1335, 0x12d2, 0x0000135d, 0x135d, 0x12dd, 0x000013a4, 0x13a4)
USER32(NtUserGetPointerFrameArrivalTimes                 ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE, 0x12e4, 0x000013a5, 0x13a5)
USER32(NtUserGetPointerInfoList                          ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE, 0x12d1, 0x00001336, 0x1336, 0x12da, 0x0000135e, 0x135e, 0x12e6, 0x000013a6, 0x13a6)
USER32(NtUserGetPointerInputTransform                    ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE,   NONE, 0x12d9, 0x0000135f, 0x135f, 0x12e5, 0x000013a7, 0x13a7)
USER32(NtUserGetPointerType                              ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE, 0x12d3, 0x00001337, 0x1337, 0x12dc, 0x00001360, 0x1360, 0x12e8, 0x000013a8, 0x13a8)
USER32(NtUserGetPrecisionTouchPadConfiguration           ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE,   NONE, 0x12d4, 0x00001361, 0x1361, 0x12df, 0x000013a9, 0x13a9)
USER32(NtUserGetPriorityClipboardFormat                  , 0x119e, 0x11a9, 0x11a8, 0x124f, 0x11b7, 0x12a7, 0x11bd, 0x12bc, 0x1196, 0x00091338, 0x1338, 0x1198, 0x00091362, 0x1362, 0x1199, 0x000913aa, 0x13aa)
USER32(NtUserGetProcessDpiAwareness                      ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE,   NONE, 0x1406, 0x00001363, 0x1363, 0x1453, 0x000013ab, 0x13ab)
USER32(NtUserGetProcessUIContextInformation              ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE, 0x13cc, 0x00001339, 0x1339, 0x13f5, 0x00001364, 0x1364, 0x1440, 0x000013ac, 0x13ac)
USER32(NtUserGetProcessWindowStation                     , 0x119f, 0x11aa, 0x11a9, 0x1021, 0x11b8, 0x1022, 0x11be, 0x1022, 0x1195, 0x00011023, 0x1023, 0x1197, 0x00011024, 0x1024, 0x1198, 0x00011025, 0x1025)
USER32(NtUserGetProp                                     ,   NONE,   NONE,   NONE,   NONE, 0x1228, 0x100e, 0x1232, 0x100e, 0x124c, 0x0000100f, 0x100f, 0x124f, 0x00001010, 0x1010, 0x1255, 0x00001011, 0x1011)
USER32(NtUserGetQueueEventStatus                         ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE, 0x13d7, 0x0000133a, 0x133a, 0x1400, 0x00001365, 0x1365, 0x144d, 0x000013ad, 0x13ad)
USER32(NtUserGetRawInputBuffer                           ,   NONE, 0x11ab, 0x11aa, 0x1250, 0x11b9, 0x12a8, 0x11bf, 0x12bd, 0x1194, 0x0000133b, 0x133b, 0x1196, 0x00001366, 0x1366, 0x1197, 0x000013ae, 0x13ae)
USER32(NtUserGetRawInputData                             ,   NONE, 0x11ac, 0x11ab, 0x1251, 0x11ba, 0x12a9, 0x11c0, 0x12be, 0x1193, 0x0000133c, 0x133c, 0x1195, 0x00001367, 0x1367, 0x1196, 0x000013af, 0x13af)
USER32(NtUserGetRawInputDeviceInfo                       ,   NONE, 0x11ad, 0x11ac, 0x1252, 0x11bb, 0x12aa, 0x11c1, 0x12bf, 0x1192, 0x0000133d, 0x133d, 0x1194, 0x00001368, 0x1368, 0x1195, 0x000013b0, 0x13b0)
USER32(NtUserGetRawInputDeviceList                       ,   NONE, 0x11ae, 0x11ad, 0x1253, 0x11bc, 0x12ab, 0x11c2, 0x12c0, 0x1191, 0x0000133e, 0x133e, 0x1193, 0x00001369, 0x1369, 0x1194, 0x000013b1, 0x13b1)
USER32(NtUserGetRawPointerDeviceData                     ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE, 0x12c7, 0x0000133f, 0x133f, 0x12cc, 0x0000136a, 0x136a, 0x12d7, 0x000013b2, 0x13b2)
USER32(NtUserGetRegisteredRawInputDevices                ,   NONE, 0x11af, 0x11ae, 0x1254, 0x11bd, 0x12ac, 0x11c3, 0x12c1, 0x1190, 0x00001340, 0x1340, 0x1192, 0x0000136b, 0x136b, 0x1193, 0x000013b3, 0x13b3)
USER32(NtUserGetScrollBarInfo                            , 0x11a0, 0x11b0, 0x11af, 0x1094, 0x11be, 0x1095, 0x11c4, 0x1093, 0x118f, 0x000c1093, 0x1093, 0x1191, 0x000c1094, 0x1094, 0x1192, 0x000c1095, 0x1095)
USER32(NtUserGetSystemMenu                               , 0x11a1, 0x11b1, 0x11b0, 0x1060, 0x11bf, 0x1061, 0x11c5, 0x1061, 0x118e, 0x00071061, 0x1061, 0x1190, 0x00071062, 0x1062, 0x1191, 0x00071063, 0x1063)
/* i#487: this has a different sysnum on some platforms.  It must be before NtUserGetThreadDesktop (i#1418). */
USER32(GetThreadDesktop                                  ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE)
USER32(NtUserGetThreadDesktop                            ,   NONE, 0x11b2, 0x11b1, 0x1084, 0x11c0, 0x1085, 0x11c6, 0x1083, 0x118d, 0x00051083, 0x1083, 0x118f, 0x00051084, 0x1084, 0x1190, 0x00051085, 0x1085)
USER32(NtUserGetThreadState                              , 0x11a3, 0x11b3, 0x11b2, 0x1000, 0x11c1, 0x1000, 0x11c7, 0x1000, 0x118c, 0x00041001, 0x1001, 0x118e, 0x00041002, 0x1002, 0x118f, 0x00041003, 0x1003)
USER32(NtUserGetTitleBarInfo                             , 0x11a4, 0x11b4, 0x11b3, 0x1090, 0x11c2, 0x1091, 0x11c8, 0x108f, 0x118b, 0x0007108f, 0x108f, 0x118d, 0x00071090, 0x1090, 0x118e, 0x00071091, 0x1091)
USER32(NtUserGetTopLevelWindow                           ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE, 0x11c9, 0x12c2, 0x118a, 0x00001341, 0x1341, 0x118c, 0x0000136c, 0x136c, 0x118d, 0x000013b4, 0x13b4)
USER32(NtUserGetTouchInputInfo                           ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE, 0x1290, 0x12c3, 0x12a0, 0x00001342, 0x1342, 0x12a3, 0x0000136d, 0x136d, 0x12ad, 0x000013b5, 0x13b5)
USER32(NtUserGetTouchValidationStatus                    ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE, 0x12c1, 0x00001343, 0x1343, 0x12c4, 0x0000136e, 0x136e, 0x12cf, 0x000013b6, 0x13b6)
USER32(NtUserGetUpdateRect                               ,   NONE, 0x11b5, 0x11b4, 0x1053, 0x11c4, 0x1054, 0x11cb, 0x1054, 0x1188, 0x000c1054, 0x1054, 0x118a, 0x000c1055, 0x1055, 0x118b, 0x000c1056, 0x1056)
USER32(NtUserGetUpdateRgn                                ,   NONE, 0x11b6, 0x11b5, 0x1087, 0x11c5, 0x1088, 0x11cc, 0x1086, 0x1187, 0x000c1086, 0x1086, 0x1189, 0x000c1087, 0x1087, 0x118a, 0x000c1088, 0x1088)
USER32(NtUserGetUpdatedClipboardFormats                  ,   NONE,   NONE,   NONE,   NONE, 0x11c3, 0x12ad, 0x11ca, 0x12c4, 0x1189, 0x00001344, 0x1344, 0x118b, 0x0000136f, 0x136f, 0x118c, 0x000013b7, 0x13b7)
USER32(NtUserGetWOWClass                                 ,   NONE, 0x11b9, 0x11b8, 0x1255, 0x11c8, 0x12ae, 0x11d2, 0x12c5, 0x1180, 0x00001345, 0x1345, 0x1182, 0x00001370, 0x1370, 0x1183, 0x000013b8, 0x13b8)
USER32(NtUserGetWindowBand                               ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE, 0x1186, 0x00001346, 0x1346, 0x1188, 0x00001371, 0x1371, 0x1189, 0x000013b9, 0x13b9)
USER32(NtUserGetWindowCompositionAttribute               ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE, 0x11ce, 0x12c6, 0x1184, 0x00001347, 0x1347, 0x1186, 0x00001372, 0x1372, 0x1187, 0x000013ba, 0x13ba)
USER32(NtUserGetWindowCompositionInfo                    ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE, 0x11cd, 0x12c7, 0x1185, 0x00001348, 0x1348, 0x1187, 0x00001373, 0x1373, 0x1188, 0x000013bb, 0x13bb)
USER32(NtUserGetWindowDC                                 , 0x11a7, 0x11b7, 0x11b6, 0x1063, 0x11c6, 0x1064, 0x11cf, 0x1064, 0x1183, 0x00031064, 0x1064, 0x1185, 0x00031065, 0x1065, 0x1186, 0x00031066, 0x1066)
USER32(NtUserGetWindowDisplayAffinity                    ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE, 0x11d0, 0x12c8, 0x1182, 0x00001349, 0x1349, 0x1184, 0x00001374, 0x1374, 0x1185, 0x000013bc, 0x13bc)
USER32(NtUserGetWindowFeedbackSetting                    ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE, 0x12bf, 0x0000134a, 0x134a, 0x12c2, 0x00001375, 0x1375, 0x12cd, 0x000013bd, 0x13bd)
USER32(NtUserGetWindowMinimizeRect                       ,   NONE,   NONE,   NONE,   NONE, 0x1277, 0x12af, 0x1281, 0x12c9, 0x127a, 0x0000134b, 0x134b, 0x127d, 0x00001376, 0x1376, 0x1287, 0x000013be, 0x13be)
USER32(NtUserGetWindowPlacement                          , 0x11a8, 0x11b8, 0x11b7, 0x10d9, 0x11c7, 0x10da, 0x11d1, 0x10d5, 0x1181, 0x000710d5, 0x10d5, 0x1183, 0x000710d6, 0x10d6, 0x1184, 0x000710d7, 0x10d7)
USER32(NtUserGetWindowRgnEx                              ,   NONE,   NONE,   NONE,   NONE, 0x123a, 0x12b0, 0x1246, 0x12ca, 0x1232, 0x0000134c, 0x134c, 0x1235, 0x00001377, 0x1377, 0x1238, 0x000013bf, 0x13bf)
USER32(NtUserGhostWindowFromHungWindow                   ,   NONE,   NONE,   NONE,   NONE, 0x11c9, 0x12b1, 0x11d3, 0x12cb, 0x117f, 0x0000134d, 0x134d, 0x1181, 0x00001378, 0x1378, 0x1182, 0x000013c0, 0x13c0)
USER32(NtUserHandleDelegatedInput                        ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE, 0x11e0, 0x0000134e, 0x134e, 0x11e1, 0x00001379, 0x1379, 0x11e3, 0x000013c1, 0x13c1)
USER32(NtUserHardErrorControl                            ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE, 0x134f,   NONE,       NONE, 0x137a, 0x1218, 0x000013c2, 0x13c2)
USER32(NtUserHideCaret                                   , 0x11ab, 0x11bb, 0x11ba, 0x101e, 0x11cb, 0x101f, 0x11d5, 0x101f, 0x1212, 0x00031020, 0x1020, 0x1215, 0x00031021, 0x1021, 0x1217, 0x00031022, 0x1022)
USER32(NtUserHidePointerContactVisualization             ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE, 0x12c0, 0x00001350, 0x1350, 0x12c3, 0x0000137b, 0x137b, 0x12ce, 0x000013c3, 0x13c3)
USER32(NtUserHiliteMenuItem                              , 0x11ac, 0x11bc, 0x11bb, 0x1257, 0x11cc, 0x12b3, 0x11d6, 0x12cd, 0x1211, 0x00141351, 0x1351, 0x1214, 0x0014137c, 0x137c, 0x1216, 0x001413c4, 0x13c4)
USER32(NtUserHungWindowFromGhostWindow                   ,   NONE,   NONE,   NONE,   NONE, 0x11cd, 0x12b4, 0x11d7, 0x12ce, 0x1210, 0x00001352, 0x1352, 0x1213, 0x0000137d, 0x137d, 0x1215, 0x000013c5, 0x13c5)
USER32(NtUserHwndQueryRedirectionInfo                    ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE, 0x1337, 0x12cf, 0x137f, 0x00001353, 0x1353, 0x1391, 0x0000137e, 0x137e, 0x13ca, 0x000013c6, 0x13c6)
USER32(NtUserHwndSetRedirectionInfo                      ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE, 0x1338, 0x12d0, 0x137e, 0x00001354, 0x1354, 0x1390, 0x0000137f, 0x137f, 0x13c9, 0x000013c7, 0x13c7)
USER32(NtUserImpersonateDdeClientWindow                  , 0x11ad, 0x11bd, 0x11bc, 0x1258, 0x11ce, 0x12b5, 0x11d8, 0x12d1, 0x120f, 0x00001355, 0x1355, 0x1212, 0x00001380, 0x1380, 0x1214, 0x000013c8, 0x13c8)
USER32(NtUserInitTask                                    ,   NONE, 0x11c0, 0x11bf, 0x1259, 0x11d1, 0x12b6, 0x11db, 0x12d2, 0x120c, 0x00001356, 0x1356, 0x120f, 0x00001381, 0x1381, 0x1211, 0x000013c9, 0x13c9)
USER32(NtUserInitialize                                  ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE, 0x1357,   NONE,       NONE, 0x1382, 0x1213, 0x000013ca, 0x13ca)
USER32(NtUserInitializeClientPfnArrays                   ,   NONE, 0x11bf, 0x11be, 0x125b, 0x11d0, 0x12b8, 0x11da, 0x12d4, 0x120d, 0x00001358, 0x1358, 0x1210, 0x00001383, 0x1383, 0x1212, 0x000013cb, 0x13cb)
USER32(NtUserInitializeInputDeviceInjection              ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE, 0x1458, 0x000013cc, 0x13cc)
USER32(NtUserInitializePointerDeviceInjection            ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE, 0x1459, 0x000013cd, 0x13cd)
USER32(NtUserInitializeTouchInjection                    ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE, 0x12c2, 0x00001359, 0x1359, 0x12c5, 0x00001384, 0x1384, 0x12d0, 0x000013ce, 0x13ce)
USER32(NtUserInjectDeviceInput                           ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE, 0x145b, 0x000013cf, 0x13cf)
USER32(NtUserInjectGesture                               ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE, 0x1292, 0x12d5, 0x12d8, 0x0000135a, 0x135a, 0x12e1, 0x00001385, 0x1385, 0x12ed, 0x000013d0, 0x13d0)
USER32(NtUserInjectKeyboardInput                         ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE, 0x145d, 0x000013d1, 0x13d1)
USER32(NtUserInjectMouseInput                            ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE, 0x145c, 0x000013d2, 0x13d2)
USER32(NtUserInjectPointerInput                          ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE, 0x145e, 0x000013d3, 0x13d3)
USER32(NtUserInjectTouchInput                            ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE, 0x12c3, 0x0000135b, 0x135b, 0x12c6, 0x00001386, 0x1386, 0x12d1, 0x000013d4, 0x13d4)
USER32(NtUserInternalClipCursor                          ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE, 0x13d4, 0x0000135c, 0x135c, 0x13fd, 0x00001387, 0x1387, 0x144a, 0x000013d5, 0x13d5)
USER32(NtUserInternalGetWindowIcon                       ,   NONE,   NONE,   NONE,   NONE, 0x11d3, 0x12b9, 0x11dd, 0x12d6, 0x120a, 0x0000135d, 0x135d, 0x120d, 0x00001388, 0x1388, 0x120f, 0x000013d6, 0x13d6)
USER32(NtUserInternalGetWindowText                       ,   NONE, 0x11c1, 0x11c0, 0x1062, 0x11d2, 0x1063, 0x11dc, 0x1063, 0x120b, 0x000c1063, 0x1063, 0x120e, 0x000c1064, 0x1064, 0x1210, 0x000c1065, 0x1065)
USER32(NtUserInvalidateRect                              , 0x11b2, 0x11c2, 0x11c1, 0x1004, 0x11d4, 0x1004, 0x11de, 0x1004, 0x1209, 0x000c1005, 0x1005, 0x120c, 0x000c1006, 0x1006, 0x120e, 0x000c1007, 0x1007)
USER32(NtUserInvalidateRgn                               , 0x11b3, 0x11c3, 0x11c2, 0x10cc, 0x11d5, 0x10cd, 0x11df, 0x10c8, 0x1208, 0x000c10c8, 0x10c8, 0x120b, 0x000c10c9, 0x10c9, 0x120d, 0x000c10ca, 0x10ca)
USER32(NtUserIsChildWindowDpiMessageEnabled              ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE, 0x1246, 0x000013d7, 0x13d7)
USER32(NtUserIsClipboardFormatAvailable                  , 0x11b4, 0x11c4, 0x11c3, 0x102e, 0x11d6, 0x102f, 0x11e0, 0x102f, 0x1207, 0x00041030, 0x1030, 0x120a, 0x00041031, 0x1031, 0x120c, 0x00041032, 0x1032)
USER32(NtUserIsMouseInPointerEnabled                     ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE, 0x13cf, 0x0000135e, 0x135e, 0x13f8, 0x00001389, 0x1389, 0x1445, 0x000013d8, 0x13d8)
USER32(NtUserIsMouseInputEnabled                         ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE, 0x13d3, 0x0000135f, 0x135f, 0x13fc, 0x0000138a, 0x138a, 0x1449, 0x000013d9, 0x13d9)
USER32(NtUserIsTopLevelWindow                            ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE, 0x11e1, 0x12d7, 0x1206, 0x00001360, 0x1360, 0x1209, 0x0000138b, 0x138b, 0x120b, 0x000013da, 0x13da)
USER32(NtUserIsTouchWindow                               ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE, 0x128d, 0x12d8, 0x12a1, 0x00001361, 0x1361, 0x12a4, 0x0000138c, 0x138c, 0x12ae, 0x000013db, 0x13db)
USER32(NtUserIsWindowBroadcastingDpiToChildren           ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE, 0x1247, 0x000013dc, 0x13dc)
USER32(NtUserKillTimer                                   , 0x11b5, 0x11c5, 0x11c4, 0x101a, 0x11d7, 0x101b, 0x11e2, 0x101b, 0x1205, 0x0007101c, 0x101c, 0x1208, 0x0007101d, 0x101d, 0x120a, 0x0007101e, 0x101e)
USER32(NtUserLayoutCompleted                             ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE, 0x11ff, 0x00001362, 0x1362, 0x1201, 0x0000138d, 0x138d, 0x1203, 0x000013dd, 0x13dd)
USER32(NtUserLinkDpiCursor                               ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE,   NONE, 0x1409, 0x0000138e, 0x138e, 0x1456, 0x000013de, 0x13de)
USER32(NtUserLoadKeyboardLayoutEx                        ,   NONE, 0x11c6, 0x11c5, 0x125c, 0x11d8, 0x12ba, 0x11e3, 0x12d9, 0x1204, 0x00001363, 0x1363, 0x1207, 0x0000138f, 0x138f, 0x1209, 0x000013df, 0x13df)
USER32(NtUserLockWindowStation                           , 0x11b7, 0x11c7, 0x11c6, 0x125d, 0x11d9, 0x12bb, 0x11e4, 0x12da, 0x1203, 0x00041364, 0x1364, 0x1206, 0x00041390, 0x1390, 0x1208, 0x000413e0, 0x13e0)
USER32(NtUserLockWindowUpdate                            , 0x11b8, 0x11c8, 0x11c7, 0x110c, 0x11da, 0x110d, 0x11e5, 0x1106, 0x1202, 0x00031106, 0x1106, 0x1205, 0x00031107, 0x1107, 0x1207, 0x00031108, 0x1108)
USER32(NtUserLockWorkStation                             , 0x11b9, 0x11c9, 0x11c8, 0x125e, 0x11db, 0x12bc, 0x11e6, 0x12db, 0x1201, 0x00011365, 0x1365, 0x1204, 0x00011391, 0x1391, 0x1206, 0x000113e1, 0x13e1)
USER32(NtUserLogicalToPerMonitorDPIPhysicalPoint         ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE,   NONE, 0x1202, 0x00001392, 0x1392, 0x1204, 0x000013e2, 0x13e2)
USER32(NtUserLogicalToPhysicalPoint                      ,   NONE,   NONE,   NONE,   NONE, 0x11dc, 0x12bd, 0x11e7, 0x12dc, 0x1200, 0x00001366, 0x1366, 0x1203, 0x00001393, 0x1393, 0x1205, 0x000013e3, 0x13e3)
USER32(NtUserLogon                                       ,   NONE,   NONE,   NONE, 0x2059,   NONE, 0x205f,   NONE, 0x205d,   NONE, 0x00002002,   NONE,   NONE, 0x00002003,   NONE,   NONE, 0x00002003,   NONE)
USER32(NtUserMNDragLeave                                 ,   NONE, 0x11ce, 0x11cd, 0x125f, 0x11e1, 0x12be, 0x11ec, 0x12dd, 0x11fa, 0x00011367, 0x1367, 0x11fc, 0x00011394, 0x1394, 0x11fe, 0x000113e4, 0x13e4)
USER32(NtUserMNDragOver                                  ,   NONE, 0x11cf, 0x11ce, 0x1260, 0x11e2, 0x12bf, 0x11ed, 0x12de, 0x11f9, 0x00001368, 0x1368, 0x11fb, 0x00001395, 0x1395, 0x11fd, 0x000013e5, 0x13e5)
USER32(NtUserMagControl                                  ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE, 0x1334, 0x12df, 0x137c, 0x00001369, 0x1369, 0x138e, 0x00001396, 0x1396, 0x13c7, 0x000013e6, 0x13e6)
USER32(NtUserMagGetContextInformation                    ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE, 0x1336, 0x12e0, 0x137a, 0x0000136a, 0x136a, 0x138c, 0x00001397, 0x1397, 0x13c5, 0x000013e7, 0x13e7)
USER32(NtUserMagSetContextInformation                    ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE, 0x1335, 0x12e1, 0x137b, 0x0000136b, 0x136b, 0x138d, 0x00001398, 0x1398, 0x13c6, 0x000013e8, 0x13e8)
USER32(NtUserManageGestureHandlerWindow                  ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE, 0x1295, 0x12e2,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE)
USER32(NtUserMapVirtualKeyEx                             ,   NONE, 0x11ca, 0x11c9, 0x105a, 0x11dd, 0x105b, 0x11e8, 0x105b, 0x11fe, 0x0000105b, 0x105b, 0x1200, 0x0000105c, 0x105c, 0x1202, 0x0000105d, 0x105d)
USER32(NtUserMenuItemFromPoint                           , 0x11bb, 0x11cb, 0x11ca, 0x1261, 0x11de, 0x12c0, 0x11e9, 0x12e3, 0x11fd, 0x0000136c, 0x136c, 0x11ff, 0x00001399, 0x1399, 0x1201, 0x000013e9, 0x13e9)
USER32(NtUserMessageCall                                 ,   NONE, 0x11cc, 0x11cb, 0x1007, 0x11df, 0x1007, 0x11ea, 0x1007, 0x11fc, 0x00001008, 0x1008, 0x11fe, 0x00001009, 0x1009, 0x1200, 0x0000100a, 0x100a)
USER32(NtUserMinMaximize                                 ,   NONE, 0x11cd, 0x11cc, 0x1262, 0x11e0, 0x12c1, 0x11eb, 0x12e4, 0x11fb, 0x000c136d, 0x136d, 0x11fd, 0x000c139a, 0x139a, 0x11ff, 0x000c13ea, 0x13ea)
USER32(NtUserModifyUserStartupInfoFlags                  ,   NONE, 0x11d0, 0x11cf, 0x1114, 0x11e3, 0x1115, 0x11ee, 0x110e, 0x11f8, 0x0005110e, 0x110e, 0x11fa, 0x0005110f, 0x110f, 0x11fc, 0x00051110, 0x1110)
USER32(NtUserModifyWindowTouchCapability                 ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE, 0x128c, 0x12e5, 0x12a2, 0x0000136e, 0x136e, 0x12a5, 0x0000139b, 0x139b, 0x12af, 0x000013eb, 0x13eb)
USER32(NtUserMoveWindow                                  , 0x11c1, 0x11d1, 0x11d0, 0x105d, 0x11e4, 0x105e, 0x11ef, 0x105e, 0x11f7, 0x0000105e, 0x105e, 0x11f9, 0x0000105f, 0x105f, 0x11fb, 0x00001060, 0x1060)
USER32(NtUserNavigateFocus                               ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE, 0x1466, 0x000013ec, 0x13ec)
USER32(NtUserNotifyIMEStatus                             , 0x11c2, 0x11d2, 0x11d1, 0x1263, 0x11e5, 0x12c2, 0x11f0, 0x12e6, 0x11f6, 0x000c136f, 0x136f, 0x11f8, 0x000c139c, 0x139c, 0x11fa, 0x000c13ed, 0x13ed)
USER32(NtUserNotifyProcessCreate                         ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE, 0x108d,   NONE,       NONE, 0x108e, 0x11f9, 0x0000108f, 0x108f)
USER32(NtUserNotifyWinEvent                              ,   NONE, 0x11d4, 0x11d3, 0x102c, 0x11e7, 0x102d, 0x11f2, 0x102d, 0x11f4, 0x0016102e, 0x102e, 0x11f6, 0x0016102f, 0x102f, 0x11f8, 0x00161030, 0x1030)
USER32(NtUserOpenClipboard                               ,   NONE, 0x11d5, 0x11d4, 0x10d3, 0x11e8, 0x10d4, 0x11f3, 0x10cf, 0x11f3, 0x000710cf, 0x10cf, 0x11f5, 0x000710d0, 0x10d0, 0x11f7, 0x000710d1, 0x10d1)
USER32(NtUserOpenDesktop                                 ,   NONE, 0x11d6, 0x11d5, 0x10ab, 0x11e9, 0x10ac, 0x11f4, 0x10a7, 0x11f2, 0x000010a7, 0x10a7, 0x11f4, 0x000010a8, 0x10a8, 0x11f6, 0x000010a9, 0x10a9)
USER32(NtUserOpenInputDesktop                            , 0x11c7, 0x11d7, 0x11d6, 0x1264, 0x11ea, 0x12c3, 0x11f5, 0x12e7, 0x11f1, 0x000a1370, 0x1370, 0x11f3, 0x000a139d, 0x139d, 0x11f5, 0x000a13ee, 0x13ee)
USER32(NtUserOpenThreadDesktop                           ,   NONE,   NONE,   NONE,   NONE, 0x11eb, 0x2063, 0x11f6, 0x12e8, 0x11f0, 0x00001371, 0x1371, 0x11f2, 0x0000139e, 0x139e, 0x11f4, 0x000013ef, 0x13ef)
USER32(NtUserOpenWindowStation                           ,   NONE, 0x11d8, 0x11d7, 0x10a1, 0x11ec, 0x10a2, 0x11f7, 0x10a0, 0x11ef, 0x000010a0, 0x10a0, 0x11f1, 0x000010a1, 0x10a1, 0x11f3, 0x000010a2, 0x10a2)
USER32(NtUserPaintDesktop                                , 0x11c9, 0x11d9, 0x11d8, 0x111b, 0x11ed, 0x111c, 0x11f8, 0x1115, 0x11ee, 0x00001115, 0x1115, 0x11f0, 0x00001116, 0x1116, 0x11f2, 0x00001117, 0x1117)
USER32(NtUserPaintMenuBar                                ,   NONE, 0x1237, 0x1233, 0x10f2, 0x124f, 0x10f3, 0x125c, 0x10ee, 0x1218, 0x000010ee, 0x10ee, 0x121b, 0x000010ef, 0x10ef, 0x121e, 0x000010f0, 0x10f0)
USER32(NtUserPaintMonitor                                ,   NONE,   NONE,   NONE,   NONE, 0x11ee, 0x12c5, 0x11f9, 0x12e9, 0x11ed, 0x00001372, 0x1372, 0x11ef, 0x0000139f, 0x139f, 0x11f1, 0x000013f0, 0x13f0)
USER32(NtUserPeekMessage                                 ,   NONE, 0x11da, 0x11d9, 0x1001, 0x11ef, 0x1001, 0x11fa, 0x1001, 0x11ec, 0x00001002, 0x1002, 0x11ee, 0x00001003, 0x1003, 0x11f0, 0x00001004, 0x1004)
USER32(NtUserPerMonitorDPIPhysicalToLogicalPoint         ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE,   NONE, 0x11e8, 0x000013a0, 0x13a0, 0x11ea, 0x000013f1, 0x13f1)
USER32(NtUserPhysicalToLogicalPoint                      ,   NONE,   NONE,   NONE,   NONE, 0x11f0, 0x12c6, 0x11fb, 0x12ea, 0x11eb, 0x00001373, 0x1373, 0x11ed, 0x000013a1, 0x13a1, 0x11ef, 0x000013f2, 0x13f2)
USER32(NtUserPostMessage                                 ,   NONE, 0x11db, 0x11da, 0x100e, 0x11f1, 0x100f, 0x11fc, 0x100f, 0x11ea, 0x00001010, 0x1010, 0x11ec, 0x00001011, 0x1011, 0x11ee, 0x00001012, 0x1012)
USER32(NtUserPostThreadMessage                           ,   NONE, 0x11dc, 0x11db, 0x105e, 0x11f2, 0x105f, 0x11fd, 0x105f, 0x11e9, 0x0000105f, 0x105f, 0x11eb, 0x00001060, 0x1060, 0x11ed, 0x00001061, 0x1061)
USER32(NtUserPrintWindow                                 ,   NONE, 0x11dd, 0x11dc, 0x1265, 0x11f3, 0x12c7, 0x11fe, 0x12eb, 0x11e8, 0x000e1374, 0x1374, 0x11ea, 0x000e13a2, 0x13a2, 0x11ec, 0x000e13f3, 0x13f3)
USER32(NtUserProcessConnect                              ,   NONE, 0x11de, 0x11dd, 0x10fa, 0x11f4, 0x10fb, 0x11ff, 0x10f6, 0x11e7, 0x000010f6, 0x10f6, 0x11e9, 0x000010f7, 0x10f7, 0x11eb, 0x000010f8, 0x10f8)
USER32(NtUserPromoteMouseInPointer                       ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE, 0x1375,   NONE,       NONE, 0x13a3, 0x1446, 0x000013f4, 0x13f4)
USER32(NtUserPromotePointer                              ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE, 0x12ce, 0x00001376, 0x1376, 0x12d6, 0x000013a4, 0x13a4, 0x12e1, 0x000013f5, 0x13f5)
USER32(NtUserQueryBSDRWindow                             ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE, 0x11e6, 0x00001377, 0x1377, 0x11e7, 0x000013a5, 0x13a5, 0x11e9, 0x000013f6, 0x13f6)
USER32(NtUserQueryDisplayConfig                          ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE, 0x1154, 0x12ec, 0x1157, 0x00001378, 0x1378, 0x1159, 0x000013a6, 0x13a6, 0x115a, 0x000013f7, 0x13f7)
USER32(NtUserQueryInformationThread                      ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE, 0x1379,   NONE,       NONE, 0x13a7, 0x11e8, 0x000013f8, 0x13f8)
USER32(NtUserQuerySendMessage                            , 0x11d0, 0x11e1, 0x11e0, 0x1268, 0x11f7, 0x12ca, 0x1202, 0x12ef, 0x11e3, 0x0000137b, 0x137b, 0x11e4, 0x000013a9, 0x13a9, 0x11e6, 0x000013fa, 0x13fa)
USER32(NtUserQueryUserCounters                           , 0x11d1, 0x11e2,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE)
USER32(NtUserQueryWindow                                 , 0x11d2, 0x11e3, 0x11e1, 0x100f, 0x11f8, 0x1010, 0x1203, 0x1010, 0x11e2, 0x00071011, 0x1011, 0x11e3, 0x00071012, 0x1012, 0x11e5, 0x00071013, 0x1013)
USER32(NtUserRealChildWindowFromPoint                    , 0x11d3, 0x11e4, 0x11e2, 0x1269, 0x11f9, 0x12cb, 0x1204, 0x12f0, 0x11e1, 0x0000137c, 0x137c, 0x11e2, 0x000013aa, 0x13aa, 0x11e4, 0x000013fb, 0x13fb)
USER32(NtUserRealInternalGetMessage                      ,   NONE, 0x11e5, 0x11e3, 0x10ee, 0x11fa, 0x10ef, 0x1205, 0x10ea, 0x1214, 0x000010ea, 0x10ea, 0x1217, 0x000010eb, 0x10eb, 0x1219, 0x000010ec, 0x10ec)
USER32(NtUserRealWaitMessageEx                           ,   NONE, 0x11e6, 0x11e4, 0x126a, 0x11fb, 0x12cc, 0x1206, 0x12f1, 0x1215, 0x0000137d, 0x137d, 0x1218, 0x000013ab, 0x13ab, 0x121a, 0x000013fc, 0x13fc)
USER32(NtUserRedrawWindow                                , 0x11d4, 0x11e7, 0x11e5, 0x1012, 0x11fc, 0x1013, 0x1207, 0x1013, 0x1278, 0x00141014, 0x1014, 0x127b, 0x00141015, 0x1015, 0x1285, 0x00141016, 0x1016)
USER32(NtUserRegisterBSDRWindow                          ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE, 0x1277, 0x0000137e, 0x137e, 0x127a, 0x000013ac, 0x13ac, 0x1284, 0x000013fd, 0x13fd)
USER32(NtUserRegisterClassExWOW                          ,   NONE, 0x11e8, 0x11e6, 0x10b4, 0x11fd, 0x10b5, 0x1208, 0x10b0, 0x1276, 0x000010b0, 0x10b0, 0x1279, 0x000010b1, 0x10b1, 0x1283, 0x000010b2, 0x10b2)
USER32(NtUserRegisterDManipHook                          ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE, 0x127d, 0x000013fe, 0x13fe)
USER32(NtUserRegisterEdgy                                ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE, 0x12be, 0x0000137f, 0x137f, 0x12c1, 0x000013ad, 0x13ad, 0x12cb, 0x000013ff, 0x13ff)
USER32(NtUserRegisterErrorReportingDialog                ,   NONE,   NONE,   NONE,   NONE, 0x11fe, 0x12cd, 0x1209, 0x12f2, 0x1275, 0x00001380, 0x1380, 0x1278, 0x000013ae, 0x13ae, 0x1282, 0x00001400, 0x1400)
USER32(NtUserRegisterHotKey                              , 0x11d6, 0x11ea, 0x11e8, 0x126b, 0x1200, 0x12ce, 0x120b, 0x12f3, 0x1273, 0x00141381, 0x1381, 0x1276, 0x001413af, 0x13af, 0x127c, 0x00141401, 0x1401)
USER32(NtUserRegisterManipulationThread                  ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE, 0x127f, 0x00001402, 0x1402)
USER32(NtUserRegisterPointerDeviceNotifications          ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE, 0x12ca, 0x00001382, 0x1382, 0x12cf, 0x000013b0, 0x13b0, 0x12da, 0x00001403, 0x1403)
USER32(NtUserRegisterPointerInputTarget                  ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE, 0x12d0, 0x00001383, 0x1383, 0x12d8, 0x000013b1, 0x13b1, 0x12e3, 0x00001404, 0x1404)
USER32(NtUserRegisterRawInputDevices                     ,   NONE, 0x11eb, 0x11e9, 0x126c, 0x1201, 0x12cf, 0x120c, 0x12f4, 0x1272, 0x00001384, 0x1384, 0x1275, 0x000013b2, 0x13b2, 0x127b, 0x00001405, 0x1405)
USER32(NtUserRegisterServicesProcess                     ,   NONE,   NONE,   NONE, 0x205b,   NONE, 0x2061, 0x120d, 0x12f5, 0x1271, 0x00001385, 0x1385, 0x1274, 0x000013b3, 0x13b3, 0x127a, 0x00001406, 0x1406)
USER32(NtUserRegisterSessionPort                         ,   NONE,   NONE,   NONE,   NONE, 0x1270, 0x12d0, 0x127c, 0x12f6, 0x1282, 0x00001386, 0x1386, 0x1285, 0x000013b4, 0x13b4, 0x128f, 0x00001407, 0x1407)
USER32(NtUserRegisterShellPTPListener                    ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE, 0x12cc, 0x00001408, 0x1408)
USER32(NtUserRegisterTasklist                            , 0x11d7, 0x11ec, 0x11ea, 0x126d, 0x1202, 0x12d1, 0x120e, 0x12f7, 0x1270, 0x00031387, 0x1387, 0x1273, 0x000313b5, 0x13b5, 0x1279, 0x00031409, 0x1409)
USER32(NtUserRegisterTouchHitTestingWindow               ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE, 0x12c4, 0x00001388, 0x1388, 0x12c7, 0x000013b6, 0x13b6, 0x12d2, 0x0000140a, 0x140a)
USER32(NtUserRegisterTouchPadCapable                     ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE,   NONE, 0x12cb, 0x000013b7, 0x13b7, 0x12d6, 0x0000140b, 0x140b)
USER32(NtUserRegisterUserApiHook                         ,   NONE, 0x11e9, 0x11e7, 0x126e, 0x11ff, 0x12d2, 0x120a, 0x12f8, 0x1274, 0x00001389, 0x1389, 0x1277, 0x000013b8, 0x13b8, 0x1281, 0x0000140c, 0x140c)
USER32(NtUserRegisterWindowMessage                       ,   NONE, 0x11ed, 0x11eb, 0x1036, 0x1203, 0x1037, 0x120f, 0x1037, 0x126f, 0x00001038, 0x1038, 0x1272, 0x00001039, 0x1039, 0x1278, 0x0000103a, 0x103a)
USER32(NtUserReleaseDwmHitTestWaiters                    ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE, 0x121b, 0x0000140d, 0x140d)
USER32(NtUserRemoteConnect                               ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE, 0x138a,   NONE,       NONE, 0x13b9, 0x1294, 0x0000140e, 0x140e)
USER32(NtUserRemoteRedrawRectangle                       ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE, 0x138b,   NONE,       NONE, 0x13ba, 0x1293, 0x0000140f, 0x140f)
USER32(NtUserRemoteRedrawScreen                          ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE, 0x138c,   NONE,       NONE, 0x13bb, 0x1292, 0x00001410, 0x1410)
USER32(NtUserRemoteStopScreenUpdates                     ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE, 0x138d,   NONE,       NONE, 0x13bc, 0x1291, 0x00001411, 0x1411)
USER32(NtUserRemoveClipboardFormatListener               ,   NONE,   NONE,   NONE,   NONE, 0x1204, 0x12d7, 0x1210, 0x12fd, 0x126e, 0x0000138e, 0x138e, 0x1271, 0x000013bd, 0x13bd, 0x1277, 0x00001412, 0x1412)
USER32(NtUserRemoveMenu                                  , 0x11d9, 0x11ee, 0x11ec, 0x10fe, 0x1205, 0x10ff, 0x1211, 0x10fa, 0x126d, 0x000a10fa, 0x10fa, 0x1270, 0x000a10fb, 0x10fb, 0x1276, 0x000a10fc, 0x10fc)
USER32(NtUserRemoveProp                                  ,   NONE, 0x11ef, 0x11ed, 0x1045, 0x1206, 0x1046, 0x1212, 0x1046, 0x126c, 0x00071047, 0x1047, 0x126f, 0x00071048, 0x1048, 0x1275, 0x00071049, 0x1049)
USER32(NtUserReportInertia                               ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE,   NONE, 0x1408, 0x000013be, 0x13be, 0x1455, 0x00001413, 0x1413)
USER32(NtUserResolveDesktopForWOW                        , 0x11dc, 0x11f1, 0x11ef, 0x1273, 0x1208, 0x12d8, 0x1213, 0x12fe, 0x126b, 0x0000138f, 0x138f, 0x126e, 0x000013bf, 0x13bf, 0x1274, 0x00001414, 0x1414)
USER32(NtUserSBGetParms                                  ,   NONE, 0x11f2, 0x11f0, 0x104d, 0x1209, 0x104e, 0x1214, 0x104e, 0x126a, 0x0014104f, 0x104f, 0x126d, 0x00141050, 0x1050, 0x1273, 0x00141051, 0x1051)
USER32(NtUserScrollDC                                    ,   NONE, 0x11f3, 0x11f1, 0x106a, 0x120a, 0x106b, 0x1215, 0x106b, 0x1269, 0x0000106b, 0x106b, 0x126c, 0x0000106c, 0x106c, 0x1272, 0x0000106d, 0x106d)
USER32(NtUserScrollWindowEx                              ,   NONE, 0x11f4, 0x11f2, 0x10c2, 0x120b, 0x10c3, 0x1216, 0x10be, 0x1268, 0x000010be, 0x10be, 0x126b, 0x000010bf, 0x10bf, 0x1271, 0x000010c0, 0x10c0)
USER32(NtUserSelectPalette                               ,   NONE, 0x11f5, 0x11f3, 0x101c, 0x120c, 0x101d, 0x1217, 0x101d,   NONE,       NONE, 0x101e,   NONE,       NONE, 0x101f, 0x1270, 0x000c1020, 0x1020)
USER32(NtUserSendEventMessage                            ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE, 0x1221, 0x00001390, 0x1390, 0x1224, 0x000013c0, 0x13c0, 0x1227, 0x00001415, 0x1415)
USER32(NtUserSendInput                                   , 0x11e1, 0x11f6, 0x11f4, 0x1083, 0x120d, 0x1084, 0x1218, 0x1082, 0x1266, 0x00001082, 0x1082, 0x1269, 0x00001083, 0x1083, 0x126f, 0x00001084, 0x1084)
USER32(NtUserSendTouchInput                              ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE, 0x128e, 0x12ff,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE)
USER32(NtUserSetActivationFilter                         ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE,   NONE, 0x13ef, 0x000013c1, 0x13c1, 0x143a, 0x00001416, 0x1416)
USER32(NtUserSetActiveProcess                            ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE, 0x13bf, 0x00001391, 0x1391, 0x13e7, 0x000013c2, 0x13c2,   NONE,       NONE,   NONE)
USER32(NtUserSetActiveProcessForMonitor                  ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE, 0x1432, 0x00001417, 0x1417)
USER32(NtUserSetActiveWindow                             , 0x11e4, 0x11f7, 0x11f5, 0x10e4, 0x120e, 0x10e5, 0x1219, 0x10e0, 0x1265, 0x000310e0, 0x10e0, 0x1268, 0x000310e1, 0x10e1, 0x126e, 0x000310e2, 0x10e2)
USER32(NtUserSetAutoRotation                             ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE, 0x13c3, 0x00001393, 0x1393, 0x13eb, 0x000013c4, 0x13c4, 0x1436, 0x00001419, 0x1419)
USER32(NtUserSetBrokeredForeground                       ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE, 0x13c9, 0x00001394, 0x1394, 0x13f1, 0x000013c5, 0x13c5, 0x143c, 0x0000141a, 0x141a)
USER32(NtUserSetCalibrationData                          ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE, 0x12c6, 0x00001395, 0x1395, 0x12c9, 0x000013c6, 0x13c6, 0x12d4, 0x0000141b, 0x141b)
USER32(NtUserSetCapture                                  , 0x11e6, 0x11f9, 0x11f7, 0x1048, 0x1210, 0x1049, 0x121b, 0x1049, 0x1263, 0x0003104a, 0x104a, 0x1266, 0x0003104b, 0x104b, 0x126c, 0x0003104c, 0x104c)
USER32(NtUserSetChildWindowNoActivate                    ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE, 0x121c, 0x1301, 0x1262, 0x00001396, 0x1396, 0x1265, 0x000013c7, 0x13c7, 0x126b, 0x0000141c, 0x141c)
USER32(NtUserSetClassLong                                ,   NONE, 0x11fa, 0x11f8, 0x10c4, 0x1211, 0x10c5, 0x121d, 0x10c0, 0x1261, 0x000010c0, 0x10c0, 0x1264, 0x000010c1, 0x10c1, 0x126a, 0x000010c2, 0x10c2)
USER32(NtUserSetClassLongPtr                             ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE, 0x13d8,   NONE,       NONE, 0x140c,   NONE,       NONE, 0x1468)
USER32(NtUserSetClassWord                                , 0x11e8, 0x11fb, 0x11f9, 0x1275, 0x1212, 0x12da, 0x121e, 0x1302, 0x1260, 0x000c1397, 0x1397, 0x1263, 0x000c13c8, 0x13c8, 0x1269, 0x000c141d, 0x141d)
USER32(NtUserSetClipboardData                            ,   NONE, 0x11fc, 0x11fa, 0x10d5, 0x1213, 0x10d6, 0x121f, 0x10d1, 0x125f, 0x000f10d1, 0x10d1, 0x1262, 0x000f10d2, 0x10d2, 0x1268, 0x000f10d3, 0x10d3)
USER32(NtUserSetClipboardViewer                          , 0x11ea, 0x11fd, 0x11fb, 0x1121, 0x1214, 0x1122, 0x1220, 0x111a, 0x125e, 0x0003111a, 0x111a, 0x1261, 0x0003111b, 0x111b, 0x1267, 0x0003111c, 0x111c)
USER32(NtUserSetConsoleReserveKeys                       , 0x11eb, 0x11fe, 0x11fc, 0x1123, 0x1215, 0x1124,   NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE)
USER32(NtUserSetCoreWindow                               ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE, 0x1464, 0x0000141e, 0x141e)
USER32(NtUserSetCoreWindowPartner                        ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE, 0x1465, 0x0000141f, 0x141f)
USER32(NtUserSetCursor                                   , 0x11ec, 0x11ff, 0x11fd, 0x1019, 0x1216, 0x101a, 0x1221, 0x101a, 0x125d, 0x0004101b, 0x101b, 0x1260, 0x0004101c, 0x101c, 0x1266, 0x0004101d, 0x101d)
USER32(NtUserSetCursorContents                           , 0x11ed, 0x1200, 0x11fe, 0x1276, 0x1217, 0x12db, 0x1222, 0x1303, 0x125c, 0x00051398, 0x1398, 0x125f, 0x000513c9, 0x13c9, 0x1265, 0x00051420, 0x1420)
USER32(NtUserSetCursorIconData                           ,   NONE, 0x1201, 0x11ff, 0x10a8, 0x1218, 0x10a9, 0x1223, 0x10a4, 0x125b, 0x000010a4, 0x10a4, 0x125e, 0x000010a5, 0x10a5, 0x1264, 0x000010a6, 0x10a6)
USER32(NtUserSetDbgTag                                   , 0x11ef, 0x1202,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE)
USER32(NtUserSetDisplayAutoRotationPreferences           ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE, 0x13c2, 0x00001399, 0x1399, 0x13ea, 0x000013ca, 0x13ca, 0x1435, 0x00001421, 0x1421)
USER32(NtUserSetDisplayConfig                            ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE, 0x1153, 0x1304, 0x1158, 0x0000139a, 0x139a, 0x115a, 0x000013cb, 0x13cb, 0x115b, 0x00001422, 0x1422)
USER32(NtUserSetDisplayMapping                           ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE, 0x12c5, 0x0000139b, 0x139b, 0x12c8, 0x000013cc, 0x13cc, 0x12d3, 0x00001423, 0x1423)
USER32(NtUserSetFallbackForeground                       ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE, 0x13c8, 0x0000139c, 0x139c, 0x13f0, 0x000013cd, 0x13cd, 0x143b, 0x00001424, 0x1424)
USER32(NtUserSetFeatureReportResponse                    ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE, 0x145a, 0x00001425, 0x1425)
USER32(NtUserSetFocus                                    , 0x11f0, 0x1203, 0x1200, 0x1050, 0x1219, 0x1051, 0x1224, 0x1051, 0x125a, 0x00031052, 0x1052, 0x125d, 0x00031053, 0x1053, 0x1263, 0x00031054, 0x1054)
USER32(NtUserSetGestureConfig                            ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE, 0x1296, 0x1305, 0x12d5, 0x0000139d, 0x139d, 0x12de, 0x000013ce, 0x13ce, 0x12ea, 0x00001426, 0x1426)
USER32(NtUserSetImeHotKey                                ,   NONE, 0x1204, 0x1201, 0x1277, 0x121a, 0x12dc, 0x1225, 0x1306, 0x1259, 0x0000139e, 0x139e, 0x125c, 0x000013cf, 0x13cf, 0x1262, 0x00001427, 0x1427)
USER32(NtUserSetImeOwnerWindow                           ,   NONE, 0x1206, 0x1203, 0x1279, 0x121c, 0x12de, 0x1227, 0x1308, 0x1257, 0x000813a0, 0x13a0, 0x125a, 0x000813d1, 0x13d1, 0x1260, 0x00081429, 0x1429)
USER32(NtUserSetImmersiveBackgroundWindow                ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE, 0x1249, 0x000013a1, 0x13a1, 0x124c, 0x000013d2, 0x13d2, 0x1252, 0x0000142a, 0x142a)
USER32(NtUserSetInformationThread                        ,   NONE, 0x1208, 0x1205, 0x10e5, 0x121e, 0x10e6, 0x1228, 0x10e1, 0x1256, 0x000010e1, 0x10e1, 0x1259, 0x000010e2, 0x10e2, 0x125f, 0x000010e3, 0x10e3)
USER32(NtUserSetInternalWindowPos                        , 0x11f6, 0x1209, 0x1206, 0x127a, 0x121f, 0x12df, 0x1229, 0x1309, 0x1255, 0x001413a2, 0x13a2, 0x1258, 0x001413d3, 0x13d3, 0x125e, 0x0014142b, 0x142b)
USER32(NtUserSetKeyboardState                            , 0x11f7, 0x120a, 0x1207, 0x10f3, 0x1220, 0x10f4, 0x122a, 0x10ef, 0x1254, 0x000410ef, 0x10ef, 0x1257, 0x000410f0, 0x10f0, 0x125d, 0x000410f1, 0x10f1)
USER32(NtUserSetLayeredWindowAttributes                  , 0x122e, 0x1245, 0x1241, 0x127b, 0x125d, 0x12e0, 0x126a, 0x130a, 0x1294, 0x001413a3, 0x13a3, 0x1297, 0x001413d4, 0x13d4, 0x12a1, 0x0014142c, 0x142c)
USER32(NtUserSetLogonNotifyWindow                        , 0x11f8, 0x120b, 0x1208, 0x127c,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE)
USER32(NtUserSetManipulationInputTarget                  ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE, 0x1280, 0x0000142d, 0x142d)
USER32(NtUserSetMenu                                     ,   NONE, 0x120c, 0x1209, 0x127d, 0x1221, 0x12e1, 0x122b, 0x130b, 0x1253, 0x000c13a4, 0x13a4, 0x1256, 0x000c13d5, 0x13d5, 0x125c, 0x000c142e, 0x142e)
USER32(NtUserSetMenuContextHelpId                        , 0x11fa, 0x120d, 0x120a, 0x127e, 0x1222, 0x12e2, 0x122c, 0x130c, 0x1252, 0x000513a5, 0x13a5, 0x1255, 0x000513d6, 0x13d6, 0x125b, 0x0005142f, 0x142f)
USER32(NtUserSetMenuDefaultItem                          , 0x11fb, 0x120e, 0x120b, 0x1107, 0x1223, 0x1108, 0x122d, 0x1102, 0x1251, 0x000a1102, 0x1102, 0x1254, 0x000a1103, 0x1103, 0x125a, 0x000a1104, 0x1104)
USER32(NtUserSetMenuFlagRtoL                             ,   NONE, 0x120f, 0x120c, 0x127f, 0x1224, 0x12e3, 0x122e, 0x130d, 0x1250, 0x000413a6, 0x13a6, 0x1253, 0x000413d7, 0x13d7, 0x1259, 0x00041430, 0x1430)
USER32(NtUserSetMirrorRendering                          ,   NONE,   NONE,   NONE,   NONE, 0x1302, 0x12e4, 0x1332, 0x130e, 0x1379, 0x000013a7, 0x13a7, 0x138b, 0x000013d8, 0x13d8, 0x13c4, 0x00001431, 0x1431)
USER32(NtUserSetObjectInformation                        , 0x11fd, 0x1210, 0x120d, 0x1280, 0x1225, 0x12e5, 0x122f, 0x130f, 0x124f, 0x000013a8, 0x13a8, 0x1252, 0x000013d9, 0x13d9, 0x1258, 0x00001432, 0x1432)
USER32(NtUserSetParent                                   , 0x11fe, 0x1211, 0x120e, 0x1078, 0x1226, 0x1079, 0x1230, 0x1077, 0x124e, 0x00081077, 0x1077, 0x1251, 0x00081078, 0x1078, 0x1257, 0x00081079, 0x1079)
USER32(NtUserSetPrecisionTouchPadConfiguration           ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE,   NONE, 0x12d5, 0x000013da, 0x13da, 0x12e0, 0x00001433, 0x1433)
USER32(NtUserSetProcessDPIAware                          ,   NONE,   NONE,   NONE,   NONE, 0x1234, 0x12e6, 0x123e, 0x1310, 0x123e, 0x000013a9, 0x13a9,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE)
USER32(NtUserSetProcessDpiAwareness                      ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE,   NONE, 0x1241, 0x000013db, 0x13db, 0x1244, 0x00001434, 0x1434)
USER32(NtUserSetProcessRestrictionExemption              ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE, 0x13cd, 0x000013aa, 0x13aa, 0x13f6, 0x000013dc, 0x13dc, 0x1441, 0x00001435, 0x1435)
USER32(NtUserSetProcessUIAccessZorder                    ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE, 0x123d, 0x000013ab, 0x13ab, 0x1240, 0x000013dd, 0x13dd, 0x1243, 0x00001436, 0x1436)
USER32(NtUserSetProcessWindowStation                     , 0x11ff, 0x1212, 0x120f, 0x10ac, 0x1227, 0x10ad, 0x1231, 0x10a8, 0x124d, 0x000410a8, 0x10a8, 0x1250, 0x000410a9, 0x10a9, 0x1256, 0x000410aa, 0x10aa)
USER32(NtUserSetProp                                     ,   NONE, 0x1213, 0x1210, 0x104b, 0x1229, 0x104c, 0x1233, 0x104c, 0x124b, 0x0010104d, 0x104d, 0x124e, 0x0010104e, 0x104e, 0x1254, 0x0010104f, 0x104f)
USER32(NtUserSetRipFlags                                 , 0x1201, 0x1214,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE)
USER32(NtUserSetScrollInfo                               , 0x1202, 0x1215, 0x1211, 0x102f, 0x122a, 0x1030, 0x1234, 0x1030, 0x124a, 0x00141031, 0x1031, 0x124d, 0x00141032, 0x1032, 0x1253, 0x00141033, 0x1033)
USER32(NtUserSetSensorPresence                           ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE, 0x13ac,   NONE,       NONE, 0x1001, 0x1002, 0x00001002, 0x1002)
USER32(NtUserSetShellWindowEx                            , 0x1203, 0x1216, 0x1212, 0x1281, 0x122b, 0x12e7, 0x1235, 0x1311, 0x1248, 0x000813ad, 0x13ad, 0x124b, 0x000813de, 0x13de, 0x1251, 0x00081437, 0x1437)
USER32(NtUserSetSysColors                                ,   NONE, 0x1217, 0x1213, 0x1282, 0x122c, 0x12e8, 0x1236, 0x1312, 0x1247, 0x001113ae, 0x13ae, 0x124a, 0x001113df, 0x13df, 0x1250, 0x00111438, 0x1438)
USER32(NtUserSetSystemCursor                             ,   NONE, 0x1218, 0x1214, 0x1283, 0x122d, 0x12e9, 0x1237, 0x1313, 0x1246, 0x000513af, 0x13af, 0x1249, 0x000513e0, 0x13e0, 0x124f, 0x00051439, 0x1439)
USER32(NtUserSetSystemMenu                               , 0x1206, 0x1219, 0x1215, 0x110d, 0x122e, 0x110e, 0x1238, 0x1107, 0x1245, 0x00081107, 0x1107, 0x1248, 0x00081108, 0x1108, 0x124e, 0x00081109, 0x1109)
USER32(NtUserSetSystemTimer                              , 0x1207, 0x121a, 0x1216, 0x1284, 0x122f, 0x12ea, 0x1239, 0x1314, 0x1244, 0x000013b0, 0x13b0, 0x1247, 0x000013e1, 0x13e1, 0x124d, 0x0000143a, 0x143a)
USER32(NtUserSetThreadDesktop                            , 0x1208, 0x121b, 0x1217, 0x1092, 0x1230, 0x1093, 0x123a, 0x1091, 0x1242, 0x00041091, 0x1091, 0x1245, 0x00041092, 0x1092, 0x124b, 0x00041093, 0x1093)
USER32(NtUserSetThreadInputBlocked                       ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE, 0x1243, 0x000013b1, 0x13b1, 0x1246, 0x000013e2, 0x13e2, 0x124c, 0x0000143b, 0x143b)
USER32(NtUserSetThreadState                              ,   NONE, 0x121d, 0x1219, 0x10dd, 0x1232, 0x10de, 0x123c, 0x10d9, 0x1240, 0x000510d9, 0x10d9, 0x1243, 0x000510da, 0x10da, 0x1249, 0x000510db, 0x10db)
USER32(NtUserSetTimer                                    , 0x120b, 0x121e, 0x121a, 0x1017, 0x1233, 0x1018, 0x123d, 0x1018, 0x123f, 0x00001019, 0x1019, 0x1242, 0x0000101a, 0x101a, 0x1248, 0x0000101b, 0x101b)
USER32(NtUserSetWinEventHook                             ,   NONE, 0x1228, 0x1224, 0x1109, 0x1240, 0x110a, 0x124c, 0x1104, 0x122c, 0x00001104, 0x1104, 0x122f, 0x00001105, 0x1105, 0x1232, 0x00001106, 0x1106)
USER32(NtUserSetWindowArrangement                        ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE, 0x1442, 0x0000143d, 0x143d)
USER32(NtUserSetWindowBand                               ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE, 0x123c, 0x000013b3, 0x13b3, 0x123f, 0x000013e4, 0x13e4, 0x1242, 0x0000143e, 0x143e)
USER32(NtUserSetWindowCompositionAttribute               ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE, 0x123f, 0x1316, 0x123b, 0x000013b4, 0x13b4, 0x123e, 0x000013e5, 0x13e5, 0x1241, 0x0000143f, 0x143f)
USER32(NtUserSetWindowCompositionTransition              ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE, 0x1239, 0x000013b5, 0x13b5, 0x123c, 0x000013e6, 0x13e6, 0x123f, 0x00001440, 0x1440)
USER32(NtUserSetWindowDisplayAffinity                    ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE, 0x1240, 0x1317, 0x1238, 0x000013b6, 0x13b6, 0x123b, 0x000013e7, 0x13e7, 0x123e, 0x00001441, 0x1441)
USER32(NtUserSetWindowFNID                               ,   NONE, 0x121f, 0x121b, 0x1096, 0x1235, 0x1097, 0x1241, 0x1095, 0x1237, 0x00071095, 0x1095, 0x123a, 0x00071096, 0x1096, 0x123d, 0x00071097, 0x1097)
USER32(NtUserSetWindowFeedbackSetting                    ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE, 0x12bd, 0x000013b7, 0x13b7, 0x12c0, 0x000013e8, 0x13e8, 0x12ca, 0x00001442, 0x1442)
USER32(NtUserSetWindowLong                               ,   NONE, 0x1220, 0x121c, 0x105b, 0x1236, 0x105c, 0x1242, 0x105c, 0x1236, 0x0000105c, 0x105c, 0x1239, 0x0000105d, 0x105d, 0x123c, 0x0000105e, 0x105e)
USER32(NtUserSetWindowLongPtr                            ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE, 0x13d9,   NONE,       NONE, 0x140d,   NONE,       NONE, 0x1469)
USER32(NtUserSetWindowPlacement                          , 0x120e, 0x1221, 0x121d, 0x10e6, 0x1237, 0x10e7, 0x1243, 0x10e2, 0x1235, 0x000710e2, 0x10e2, 0x1238, 0x000710e3, 0x10e3, 0x123b, 0x000710e4, 0x10e4)
USER32(NtUserSetWindowPos                                , 0x120f, 0x1222, 0x121e, 0x1023, 0x1238, 0x1024, 0x1244, 0x1024, 0x1234, 0x00001025, 0x1025, 0x1237, 0x00001026, 0x1026, 0x123a, 0x00001027, 0x1027)
USER32(NtUserSetWindowRgn                                ,   NONE, 0x1223, 0x121f, 0x10ce, 0x1239, 0x10cf, 0x1245, 0x10ca, 0x1233, 0x000c10ca, 0x10ca, 0x1236, 0x000c10cb, 0x10cb, 0x1239, 0x000c10cc, 0x10cc)
USER32(NtUserSetWindowRgnEx                              ,   NONE,   NONE,   NONE,   NONE, 0x123b, 0x12ec, 0x1247, 0x1318, 0x1231, 0x000013b8, 0x13b8, 0x1234, 0x000013e9, 0x13e9, 0x1237, 0x00001443, 0x1443)
USER32(NtUserSetWindowShowState                          ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE, 0x1443, 0x00001444, 0x1444)
USER32(NtUserSetWindowStationUser                        ,   NONE, 0x1226, 0x1222, 0x1286, 0x123e, 0x12ed, 0x124a, 0x1319, 0x122e, 0x000013b9, 0x13b9, 0x1231, 0x000013ea, 0x13ea, 0x1234, 0x00001445, 0x1445)
USER32(NtUserSetWindowWord                               , 0x1214, 0x1227, 0x1223, 0x10ec, 0x123f, 0x10ed, 0x124b, 0x10e8, 0x122d, 0x000010e8, 0x10e8, 0x1230, 0x000010e9, 0x10e9, 0x1233, 0x000010ea, 0x10ea)
USER32(NtUserSetWindowsHookAW                            ,   NONE, 0x1224, 0x1220, 0x1106, 0x123c, 0x1107, 0x1248, 0x1101, 0x1230, 0x00001101, 0x1101, 0x1233, 0x00001102, 0x1102, 0x1236, 0x00001103, 0x1103)
USER32(NtUserSetWindowsHookEx                            ,   NONE, 0x1225, 0x1221, 0x108d, 0x123d, 0x108e, 0x1249, 0x108c, 0x122f, 0x0000108c, 0x108c, 0x1232, 0x0000108d, 0x108d, 0x1235, 0x0000108e, 0x108e)
USER32(NtUserSfmDestroyLogicalSurfaceBinding             ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE, 0x128b, 0x131a,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE)
USER32(NtUserSfmDxBindSwapChain                          ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE, 0x1282, 0x131b,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE)
USER32(NtUserSfmDxGetSwapChainStats                      ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE, 0x1288, 0x131c,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE)
USER32(NtUserSfmDxOpenSwapChain                          ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE, 0x1283, 0x131d,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE)
USER32(NtUserSfmDxQuerySwapChainBindingStatus            ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE, 0x1286, 0x131e,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE)
USER32(NtUserSfmDxReleaseSwapChain                       ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE, 0x1284, 0x131f,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE)
USER32(NtUserSfmDxReportPendingBindingsToDwm             ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE, 0x1287, 0x1320,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE)
USER32(NtUserSfmDxSetSwapChainBindingStatus              ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE, 0x1285, 0x1321,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE)
USER32(NtUserSfmDxSetSwapChainStats                      ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE, 0x1289, 0x1322,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE)
USER32(NtUserSfmGetLogicalSurfaceBinding                 ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE, 0x128a, 0x1323,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE)
USER32(NtUserShowCaret                                   , 0x1216, 0x1229, 0x1225, 0x1024, 0x1241, 0x1025, 0x124d, 0x1025, 0x122b, 0x00031026, 0x1026, 0x122e, 0x00031027, 0x1027, 0x1231, 0x00031028, 0x1028)
USER32(NtUserShowScrollBar                               , 0x1217, 0x122a, 0x1226, 0x103c, 0x1242, 0x103d, 0x124e, 0x103d, 0x122a, 0x000c103e, 0x103e, 0x122d, 0x000c103f, 0x103f, 0x1230, 0x000c1040, 0x1040)
USER32(NtUserShowSystemCursor                            ,   NONE,   NONE,   NONE,   NONE, 0x1303, 0x12ee, 0x1333, 0x1324, 0x1378, 0x000013ba, 0x13ba, 0x138a, 0x000013eb, 0x13eb, 0x13c3, 0x00001446, 0x1446)
USER32(NtUserShowWindow                                  , 0x1218, 0x122b, 0x1227, 0x1057, 0x1243, 0x1058, 0x124f, 0x1058, 0x1229, 0x00081058, 0x1058, 0x122c, 0x00081059, 0x1059, 0x122f, 0x0008105a, 0x105a)
USER32(NtUserShowWindowAsync                             , 0x1219, 0x122c, 0x1228, 0x1122, 0x1244, 0x1123, 0x1250, 0x111b, 0x1228, 0x0008111b, 0x111b, 0x122b, 0x0008111c, 0x111c, 0x122e, 0x0008111d, 0x111d)
USER32(NtUserShutdownBlockReasonCreate                   ,   NONE,   NONE,   NONE,   NONE,   NONE, 0x2067,   NONE, 0x2060, 0x1227, 0x000013bb, 0x13bb, 0x122a, 0x000013ec, 0x13ec, 0x122d, 0x00001447, 0x1447)
USER32(NtUserShutdownBlockReasonDestroy                  ,   NONE,   NONE,   NONE,   NONE,   NONE, 0x2069,   NONE, 0x2062, 0x1225, 0x000013bd, 0x13bd, 0x1228, 0x000013ee, 0x13ee, 0x122b, 0x00001449, 0x1449)
USER32(NtUserShutdownBlockReasonQuery                    ,   NONE,   NONE,   NONE,   NONE,   NONE, 0x2068,   NONE, 0x2061, 0x1226, 0x000013bc, 0x13bc, 0x1229, 0x000013ed, 0x13ed, 0x122c, 0x00001448, 0x1448)
USER32(NtUserSignalRedirectionStartComplete              ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE, 0x13be, 0x000013be, 0x13be, 0x13e6, 0x000013ef, 0x13ef, 0x1431, 0x0000144a, 0x144a)
USER32(NtUserSlicerControl                               ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE, 0x137d, 0x000013bf, 0x13bf, 0x138f, 0x000013f0, 0x13f0, 0x13c8, 0x0000144b, 0x144b)
USER32(NtUserSoundSentry                                 ,   NONE,   NONE,   NONE,   NONE, 0x1245, 0x12ef, 0x1251, 0x1325, 0x1224, 0x000013c0, 0x13c0, 0x1227, 0x000013f1, 0x13f1, 0x122a, 0x0000144c, 0x144c)
USER32(NtUserSwitchDesktop                               , 0x121b, 0x122e, 0x122a, 0x1288, 0x1246, 0x12f0, 0x1252, 0x1326, 0x1223, 0x000513c1, 0x13c1, 0x1226, 0x000513f2, 0x13f2, 0x1229, 0x0005144d, 0x144d)
USER32(NtUserSystemParametersInfo                        ,   NONE, 0x122f, 0x122b, 0x1041, 0x1247, 0x1042, 0x1253, 0x1042, 0x1222, 0x00001043, 0x1043, 0x1225, 0x00001044, 0x1044, 0x1228, 0x00001045, 0x1045)
USER32(NtUserTestForInteractiveUser                      ,   NONE,   NONE,   NONE, 0x1289, 0x1248, 0x12f1, 0x1254, 0x1327, 0x1220, 0x000413c2, 0x13c2, 0x1223, 0x000413f3, 0x13f3, 0x1226, 0x0004144e, 0x144e)
USER32(NtUserThunkedMenuInfo                             ,   NONE, 0x1231, 0x122d, 0x110e, 0x1249, 0x110f, 0x1255, 0x1108, 0x121f, 0x00001108, 0x1108, 0x1222, 0x00001109, 0x1109, 0x1225, 0x0000110a, 0x110a)
USER32(NtUserThunkedMenuItemInfo                         ,   NONE, 0x1232, 0x122e, 0x1099, 0x124a, 0x109a, 0x1256, 0x1098, 0x121e, 0x00001098, 0x1098, 0x1221, 0x00001099, 0x1099, 0x1224, 0x0000109a, 0x109a)
USER32(NtUserToUnicodeEx                                 ,   NONE, 0x1233, 0x122f, 0x107a, 0x124b, 0x107b, 0x1257, 0x1079, 0x121d, 0x00001079, 0x1079, 0x1220, 0x0000107a, 0x107a, 0x1223, 0x0000107b, 0x107b)
USER32(NtUserTrackMouseEvent                             , 0x1221, 0x1234, 0x1230, 0x10df, 0x124c, 0x10e0, 0x1258, 0x10db, 0x121c, 0x000010db, 0x10db, 0x121f, 0x000010dc, 0x10dc, 0x1222, 0x000010dd, 0x10dd)
USER32(NtUserTrackPopupMenuEx                            , 0x1222, 0x1235, 0x1231, 0x128a, 0x124d, 0x12f2, 0x1259, 0x1328, 0x121b, 0x000013c3, 0x13c3,   NONE, 0x000013f4, 0x13f4, 0x1221, 0x0000144f, 0x144f)
USER32(NtUserTransformPoint                              ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE,   NONE, 0x1403, 0x000013f5, 0x13f5, 0x1450, 0x00001450, 0x1450)
USER32(NtUserTransformRect                               ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE,   NONE, 0x1404, 0x000013f6, 0x13f6, 0x1451, 0x00001451, 0x1451)
USER32(NtUserTranslateAccelerator                        ,   NONE, 0x1238, 0x1234, 0x1010, 0x1250, 0x1011, 0x125d, 0x1011, 0x1217, 0x00001012, 0x1012, 0x121a, 0x00001013, 0x1013, 0x121d, 0x00001014, 0x1014)
USER32(NtUserTranslateMessage                            ,   NONE, 0x1239, 0x1235, 0x100d, 0x1251, 0x100d, 0x125e, 0x100d, 0x1216, 0x0000100e, 0x100e, 0x1219, 0x0000100f, 0x100f, 0x121c, 0x00001010, 0x1010)
USER32(NtUserUndelegateInput                             ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE, 0x1279, 0x000013c4, 0x13c4, 0x127c, 0x000013f7, 0x13f7, 0x1286, 0x00001452, 0x1452)
USER32(NtUserUnhookWinEvent                              , 0x1226, 0x123b, 0x1237, 0x110a, 0x1253, 0x110b, 0x1260, 0x1105, 0x129e, 0x00001105, 0x1105, 0x12a1, 0x00001106, 0x1106, 0x12ab, 0x00001107, 0x1107)
USER32(NtUserUnhookWindowsHookEx                         , 0x1225, 0x123a, 0x1236, 0x1070, 0x1252, 0x1071, 0x125f, 0x1070, 0x129f, 0x00001070, 0x1070, 0x12a2, 0x00001071, 0x1071, 0x12ac, 0x00001072, 0x1072)
USER32(NtUserUnloadKeyboardLayout                        ,   NONE, 0x123c, 0x1238, 0x128b, 0x1254, 0x12f3, 0x1261, 0x1329, 0x129d, 0x000313c5, 0x13c5, 0x12a0, 0x000313f8, 0x13f8, 0x12aa, 0x00031453, 0x1453)
USER32(NtUserUnlockWindowStation                         , 0x1228, 0x123d, 0x1239, 0x128c, 0x1255, 0x12f4, 0x1262, 0x132a, 0x129c, 0x000413c6, 0x13c6, 0x129f, 0x000413f9, 0x13f9, 0x12a9, 0x00041454, 0x1454)
USER32(NtUserUnregisterClass                             ,   NONE, 0x123e, 0x123a, 0x10bf, 0x1256, 0x10c0, 0x1263, 0x10bb, 0x129b, 0x000010bb, 0x10bb, 0x129e, 0x000010bc, 0x10bc, 0x12a8, 0x000010bd, 0x10bd)
USER32(NtUserUnregisterHotKey                            , 0x122a, 0x1240, 0x123c, 0x128d, 0x1258, 0x12f5, 0x1265, 0x132b, 0x1299, 0x000813c7, 0x13c7, 0x129c, 0x000813fa, 0x13fa, 0x12a6, 0x00081455, 0x1455)
USER32(NtUserUnregisterSessionPort                       ,   NONE,   NONE,   NONE,   NONE, 0x1271, 0x12f6, 0x127d, 0x132c, 0x1281, 0x000013c8, 0x13c8, 0x1284, 0x000013fb, 0x13fb, 0x128e, 0x00001456, 0x1456)
USER32(NtUserUnregisterUserApiHook                       ,   NONE, 0x123f, 0x123b, 0x128e, 0x1257, 0x12f7, 0x1264, 0x132d, 0x129a, 0x000013c9, 0x13c9, 0x129d, 0x000013fc, 0x13fc, 0x12a7, 0x00001457, 0x1457)
USER32(NtUserUpdateDefaultDesktopThumbnail               ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE, 0x123a, 0x000013ca, 0x13ca, 0x123d, 0x000013fd, 0x13fd, 0x1240, 0x00001458, 0x1458)
USER32(NtUserUpdateInputContext                          , 0x122b, 0x1241, 0x123d, 0x128f, 0x1259, 0x12f8, 0x1266, 0x132e, 0x1298, 0x000013cb, 0x13cb, 0x129b, 0x000013fe, 0x13fe, 0x12a5, 0x00001459, 0x1459)
USER32(NtUserUpdateInstance                              ,   NONE, 0x1242, 0x123e, 0x1290, 0x125a, 0x12f9, 0x1267, 0x132f, 0x1297, 0x000c13cc, 0x13cc, 0x129a, 0x000c13ff, 0x13ff, 0x12a4, 0x000c145a, 0x145a)
USER32(NtUserUpdateLayeredWindow                         , 0x122d, 0x1243, 0x123f, 0x1291, 0x125b, 0x12fa, 0x1268, 0x1330, 0x1296, 0x000013cd, 0x13cd, 0x1299, 0x00001400, 0x1400, 0x12a3, 0x0000145b, 0x145b)
USER32(NtUserUpdatePerUserSystemParameters               ,   NONE, 0x1246, 0x1242, 0x1292, 0x125e, 0x12fb, 0x126b, 0x1331, 0x1293, 0x000713ce, 0x13ce, 0x1296, 0x00071401, 0x1401, 0x12a0, 0x0007145c, 0x145c)
USER32(NtUserUpdateWindowInputSinkHints                  ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE,   NONE, 0x1402, 0x00001402, 0x1402, 0x144f, 0x0000145d, 0x145d)
USER32(NtUserUpdateWindowTransform                       ,   NONE,   NONE,   NONE,   NONE, 0x1272, 0x12fc, 0x127e, 0x1332, 0x127f, 0x000013cf, 0x13cf, 0x1282, 0x00001403, 0x1403, 0x128c, 0x0000145e, 0x145e)
USER32(NtUserUserConnectToServer                         ,   NONE,   NONE,   NONE, 0x205e,   NONE, 0x2066,   NONE, 0x205f,   NONE, 0x00002004,   NONE,   NONE, 0x00002000,   NONE,   NONE, 0x00002000,   NONE)
USER32(NtUserUserHandleGrantAccess                       , 0x1230, 0x1247, 0x1243, 0x1293, 0x125f, 0x12fd, 0x126c, 0x1333, 0x1292, 0x000e13d0, 0x13d0, 0x1295, 0x000e1404, 0x1404, 0x129f, 0x000e145f, 0x145f)
USER32(NtUserValidateHandleSecure                        , 0x1231, 0x1248, 0x1244, 0x1294, 0x1260, 0x12fe, 0x126d, 0x1334, 0x1291, 0x000313d1, 0x13d1, 0x1294, 0x00031405, 0x1405, 0x129e, 0x00031460, 0x1460)
USER32(NtUserValidateRect                                , 0x1232, 0x1249, 0x1245, 0x10d1, 0x1261, 0x10d2, 0x126e, 0x10cd, 0x1290, 0x000710cd, 0x10cd, 0x1293, 0x000710ce, 0x10ce, 0x129d, 0x000710cf, 0x10cf)
USER32(NtUserValidateTimerCallback                       ,   NONE, 0x124a, 0x1246, 0x1015, 0x1262, 0x1016, 0x126f, 0x1016, 0x128f, 0x00041017, 0x1017, 0x1292, 0x00041018, 0x1018, 0x129c, 0x00041019, 0x1019)
USER32(NtUserVkKeyScanEx                                 ,   NONE, 0x124b, 0x1247, 0x1027, 0x1263, 0x1028, 0x1270, 0x1028, 0x128e, 0x000f1029, 0x1029, 0x1291, 0x000f102a, 0x102a, 0x129b, 0x000f102b, 0x102b)
USER32(NtUserWaitAvailableMessageEx                      ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE, 0x1288, 0x000013d2, 0x13d2, 0x128b, 0x00001406, 0x1406, 0x1295, 0x00001461, 0x1461)
USER32(NtUserWaitForInputIdle                            ,   NONE, 0x124c, 0x1248, 0x1295, 0x1264, 0x12ff, 0x1271, 0x1335, 0x128d, 0x000c13d3, 0x13d3, 0x1290, 0x000c1407, 0x1407, 0x129a, 0x000c1462, 0x1462)
USER32(NtUserWaitForMsgAndEvent                          ,   NONE, 0x124d, 0x1249, 0x1296, 0x1265, 0x1300, 0x1272, 0x1336, 0x128c, 0x000313d4, 0x13d4, 0x128f, 0x00031408, 0x1408, 0x1299, 0x00031463, 0x1463)
USER32(NtUserWaitForRedirectionStartComplete             ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE, 0x13bd, 0x000013d5, 0x13d5, 0x13e5, 0x00001409, 0x1409, 0x1430, 0x00001464, 0x1464)
USER32(NtUserWaitMessage                                 , 0x1236, 0x124e, 0x124a, 0x100c, 0x1266, 0x100c, 0x1273, 0x100c, 0x128b, 0x0001100d, 0x100d, 0x128e, 0x0001100e, 0x100e, 0x1298, 0x0001100f, 0x100f)
USER32(NtUserWin32PoolAllocationStats                    , 0x1237, 0x124f, 0x124b, 0x1297, 0x1267, 0x1301,   NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE)
USER32(NtUserWindowFromPhysicalPoint                     ,   NONE,   NONE,   NONE,   NONE, 0x1268, 0x1302, 0x1274, 0x1337, 0x128a, 0x000013d6, 0x13d6, 0x128d, 0x0000140a, 0x140a, 0x1297, 0x00001465, 0x1465)
USER32(NtUserWindowFromPoint                             , 0x1238, 0x1250, 0x124c, 0x1013, 0x1269, 0x1014, 0x1275, 0x1014, 0x1289, 0x00001015, 0x1015, 0x128c, 0x00001016, 0x1016, 0x1296, 0x00001017, 0x1017)
USER32(NtUserYieldTask                                   ,   NONE, 0x1251, 0x124d, 0x1298, 0x126a, 0x1303, 0x1276, 0x1338, 0x1000, 0x00011000, 0x1000, 0x1000, 0x00011000, 0x1000, 0x1001, 0x00011001, 0x1001)
USER32(NtValidateCompositionSurfaceHandle                ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE, 0x13d7,   NONE,       NONE, 0x140b, 0x13cc, 0x00001466, 0x1466)
USER32(NtVisualCaptureBits                               ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE, 0x13dc, 0x00001467, 0x1467)

/* a number of these duplicate user32 wrappers (see comments at top of file) so not
 * listed here, even though on win2k the imm32 wrapper is not present in user32
 */
IMM32(NtUserAssociateInputContext                       , 0x1128, 0x1132, 0x1131, 0x1225, 0x113c, 0x1275, 0x113d, 0x1282, 0x116e, 0x000012dd, 0x12dd, 0x1170, 0x000012fe, 0x12fe, 0x1171, 0x00001343, 0x1343)
IMM32(NtUserBuildHimcList                               , 0x112d, 0x1137, 0x1136, 0x1227, 0x1141, 0x1277, 0x1142, 0x1284, 0x1169, 0x000012e1, 0x12e1, 0x116b, 0x00001302, 0x1302, 0x116c, 0x00001347, 0x1347)
IMM32(NtUserCreateInputContext                          , 0x114b, 0x1155, 0x1154, 0x122e, 0x1162, 0x1281, 0x1167, 0x1290, 0x1144, 0x000012f0, 0x12f0, 0x1146, 0x00001313, 0x1313, 0x1147, 0x00001358, 0x1358)
IMM32(NtUserDestroyInputContext                         , 0x1157, 0x1161, 0x1160, 0x1233, 0x116c, 0x1284, 0x1171, 0x1293, 0x1179, 0x000012f7, 0x12f7, 0x117b, 0x0000131a, 0x131a, 0x117c, 0x0000135f, 0x135f)
IMM32(NtUserGetAppImeLevel                              , 0x1174, 0x117e, 0x117d, 0x123e, 0x118c, 0x1296, 0x1191, 0x12a6, 0x11c6, 0x00031312, 0x1312, 0x11c7, 0x00031336, 0x1336, 0x11c9, 0x0003137c, 0x137c)
IMM32(NtUserGetImeInfoEx                                , 0x118f, 0x119a, 0x1199, 0x1246, 0x11a8, 0x129e, 0x11ad, 0x12b2, 0x11a6, 0x00001326, 0x1326, 0x11a8, 0x0000134c, 0x134c, 0x11a9, 0x00001394, 0x1394)
IMM32(NtUserQueryInputContext                           , 0x11cf, 0x11e0, 0x11df, 0x1267, 0x11f6, 0x12c9, 0x1201, 0x12ee, 0x11e4, 0x0000137a, 0x137a, 0x11e5, 0x000013a8, 0x13a8, 0x11e7, 0x000013f9, 0x13f9)
IMM32(NtUserSetAppImeLevel                              , 0x11e5, 0x11f8, 0x11f6, 0x1274, 0x120f, 0x12d9, 0x121a, 0x1300, 0x1264, 0x00071392, 0x1392, 0x1267, 0x000713c3, 0x13c3, 0x126d, 0x00071418, 0x1418)
IMM32(NtUserSetImeInfoEx                                , 0x11f2, 0x1205, 0x1202, 0x1278, 0x121b, 0x12dd, 0x1226, 0x1307, 0x1258, 0x0000139f, 0x139f, 0x125b, 0x000013d0, 0x13d0, 0x1261, 0x00001428, 0x1428)
IMM32(NtUserSetThreadLayoutHandles                      , 0x1209, 0x121c, 0x1218, 0x1285, 0x1231, 0x12eb, 0x123b, 0x1315, 0x1241, 0x000013b2, 0x13b2, 0x1244, 0x000013e3, 0x13e3, 0x124a, 0x0000143c, 0x143c)

GDI32(NtGdiAbortDoc                                     ,   NONE, 0x1000, 0x1000, 0x1127, 0x1000, 0x1128, 0x1000, 0x111f, 0x113c, 0x0003114c, 0x114c, 0x113e, 0x0003115b, 0x115b, 0x113f, 0x00031167, 0x1167)
GDI32(NtGdiAbortPath                                    ,   NONE, 0x1001, 0x1001, 0x1128, 0x1001, 0x1129, 0x1001, 0x1120, 0x113b, 0x0003114d, 0x114d, 0x113d, 0x0003115c, 0x115c, 0x113e, 0x00031168, 0x1168)
GDI32(NtGdiAddEmbFontToDC                               ,   NONE, 0x10d6, 0x10d5, 0x1129, 0x10de, 0x112a, 0x10e0, 0x1121, 0x105a, 0x0000114e, 0x114e, 0x105b, 0x0000115d, 0x115d, 0x105c, 0x00001169, 0x1169)
GDI32(NtGdiAddFontMemResourceEx                         ,   NONE, 0x1004, 0x1004, 0x1116, 0x1004, 0x1117, 0x1004, 0x1110, 0x1138, 0x00001110, 0x1110, 0x113a, 0x00001111, 0x1111, 0x113b, 0x00001112, 0x1112)
GDI32(NtGdiAddFontResourceW                             ,   NONE, 0x1002, 0x1002, 0x112a, 0x1002, 0x112b, 0x1002, 0x1122, 0x113a, 0x0000114f, 0x114f, 0x113c, 0x0000115e, 0x115e, 0x113d, 0x0000116a, 0x116a)
GDI32(NtGdiAddRemoteFontToDC                            ,   NONE, 0x1003, 0x1003, 0x112b, 0x1003, 0x112c, 0x1003, 0x1123, 0x1139, 0x00001150, 0x1150, 0x113b, 0x0000115f, 0x115f, 0x113c, 0x0000116b, 0x116b)
GDI32(NtGdiAddRemoteMMInstanceToDC                      ,   NONE, 0x1006, 0x1006, 0x112c, 0x1006, 0x112d, 0x1006, 0x1124, 0x1136, 0x000c1151, 0x1151, 0x1138, 0x000c1160, 0x1160, 0x1139, 0x000c116c, 0x116c)
GDI32(NtGdiAlphaBlend                                   ,   NONE, 0x1007, 0x1007, 0x107d, 0x1007, 0x107e, 0x1007, 0x107c, 0x1135, 0x0000107c, 0x107c, 0x1137, 0x0000107d, 0x107d, 0x1138, 0x0000107e, 0x107e)
GDI32(NtGdiAngleArc                                     ,   NONE, 0x1008, 0x1008, 0x112d, 0x1008, 0x112e, 0x1008, 0x1125, 0x1134, 0x00001152, 0x1152, 0x1136, 0x00001161, 0x1161, 0x1137, 0x0000116d, 0x116d)
GDI32(NtGdiAnyLinkedFonts                               , 0x1009, 0x1009, 0x1009, 0x112e, 0x1009, 0x112f, 0x1009, 0x1126, 0x1133, 0x00011153, 0x1153, 0x1135, 0x00011162, 0x1162, 0x1136, 0x0001116e, 0x116e)
GDI32(NtGdiArcInternal                                  ,   NONE, 0x100b, 0x100b, 0x112f, 0x100b, 0x1130, 0x100b, 0x1127, 0x1131, 0x00001154, 0x1154, 0x1133, 0x00001163, 0x1163, 0x1134, 0x0000116f, 0x116f)
GDI32(NtGdiBRUSHOBJ_DeleteRbrush                        ,   NONE, 0x1298, 0x1294, 0x1130, 0x12b9, 0x1131, 0x12d9, 0x1128, 0x1301, 0x00001155, 0x1155, 0x130a, 0x00001164, 0x1164, 0x1316, 0x00001170, 0x1170)
GDI32(NtGdiBRUSHOBJ_hGetColorTransform                  , 0x1265, 0x127d, 0x1279, 0x1131, 0x129e, 0x1132, 0x12be, 0x1129, 0x12e3, 0x00001156, 0x1156, 0x12ec, 0x00001165, 0x1165, 0x12f8, 0x00001171, 0x1171)
GDI32(NtGdiBRUSHOBJ_pvAllocRbrush                       , 0x1263, 0x127b, 0x1277, 0x1132, 0x129c, 0x1133, 0x12bc, 0x112a, 0x12e5, 0x00001157, 0x1157, 0x12ee, 0x00001166, 0x1166, 0x12fa, 0x00001172, 0x1172)
GDI32(NtGdiBRUSHOBJ_pvGetRbrush                         , 0x1264, 0x127c, 0x1278, 0x1133, 0x129d, 0x1134, 0x12bd, 0x112b, 0x12e4, 0x00001158, 0x1158, 0x12ed, 0x00001167, 0x1167, 0x12f9, 0x00001173, 0x1173)
GDI32(NtGdiBRUSHOBJ_ulGetBrushColor                     , 0x1262, 0x127a, 0x1276, 0x1134, 0x129b, 0x1135, 0x12bb, 0x112c, 0x12e6, 0x00001159, 0x1159, 0x12ef, 0x00001168, 0x1168, 0x12fb, 0x00001174, 0x1174)
GDI32(NtGdiBeginGdiRendering                            ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE, 0x100c, 0x112d, 0x1130, 0x0000115a, 0x115a, 0x1132, 0x00001169, 0x1169, 0x1133, 0x00001175, 0x1175)
GDI32(NtGdiBeginPath                                    ,   NONE, 0x100c, 0x100c, 0x110f, 0x100c, 0x1110, 0x100d, 0x1109, 0x112f, 0x00031109, 0x1109, 0x1131, 0x0003110a, 0x110a, 0x1132, 0x0003110b, 0x110b)
GDI32(NtGdiBitBlt                                       ,   NONE, 0x100d, 0x100d, 0x1008, 0x100d, 0x1008, 0x100e, 0x1008, 0x112e, 0x00001009, 0x1009, 0x1130, 0x0000100a, 0x100a, 0x1131, 0x0000100b, 0x100b)
GDI32(NtGdiCLIPOBJ_bEnum                                , 0x125c, 0x1274, 0x1270, 0x1135, 0x1295, 0x1136, 0x12b5, 0x112e, 0x12e2, 0x0000115b, 0x115b, 0x12eb, 0x0000116a, 0x116a, 0x12f7, 0x00001176, 0x1176)
GDI32(NtGdiCLIPOBJ_cEnumStart                           , 0x125d, 0x1275, 0x1271, 0x1136, 0x1296, 0x1137, 0x12b6, 0x112f, 0x12e1, 0x0000115c, 0x115c, 0x12ea, 0x0000116b, 0x116b, 0x12f6, 0x00001177, 0x1177)
GDI32(NtGdiCLIPOBJ_ppoGetPath                           , 0x125e, 0x1276, 0x1272, 0x1137, 0x1297, 0x1138, 0x12b7, 0x1130, 0x12e0, 0x0000115d, 0x115d, 0x12e9, 0x0000116c, 0x116c, 0x12f5, 0x00001178, 0x1178)
GDI32(NtGdiCancelDC                                     ,   NONE, 0x100e, 0x100e, 0x1138, 0x100e, 0x1139, 0x100f, 0x1131, 0x112d, 0x0003115e, 0x115e, 0x112f, 0x0003116d, 0x116d, 0x1130, 0x00031179, 0x1179)
GDI32(NtGdiChangeGhostFont                              ,   NONE, 0x10d5, 0x10d4, 0x1139, 0x10dd, 0x113a, 0x10df, 0x1132, 0x105b, 0x0000115f, 0x115f, 0x105c, 0x0000116e, 0x116e, 0x105d, 0x0000117a, 0x117a)
GDI32(NtGdiCheckBitmapBits                              ,   NONE, 0x100f, 0x100f, 0x113a, 0x100f, 0x113b, 0x1010, 0x1133, 0x112c, 0x00001160, 0x1160, 0x112e, 0x0000116f, 0x116f, 0x112f, 0x0000117b, 0x117b)
GDI32(NtGdiClearBitmapAttributes                        ,   NONE, 0x1011, 0x1011, 0x113b, 0x1011, 0x113c, 0x1012, 0x1134, 0x112a, 0x00051161, 0x1161, 0x112c, 0x00051170, 0x1170, 0x112d, 0x0005117c, 0x117c)
GDI32(NtGdiClearBrushAttributes                         ,   NONE, 0x1012, 0x1012, 0x113c, 0x1012, 0x113d, 0x1013, 0x1135, 0x1129, 0x00051162, 0x1162, 0x112b, 0x00051171, 0x1171, 0x112c, 0x0005117d, 0x117d)
GDI32(NtGdiCloseFigure                                  ,   NONE, 0x1010, 0x1010, 0x1105, 0x1010, 0x1106, 0x1011, 0x1100, 0x112b, 0x00031100, 0x1100, 0x112d, 0x00031101, 0x1101, 0x112e, 0x00031102, 0x1102)
GDI32(NtGdiColorCorrectPalette                          ,   NONE, 0x1013, 0x1013, 0x113d, 0x1013, 0x113e, 0x1014, 0x1136, 0x1128, 0x00001163, 0x1163, 0x112a, 0x00001172, 0x1172, 0x112b, 0x0000117e, 0x117e)
GDI32(NtGdiCombineRgn                                   ,   NONE, 0x1014, 0x1014, 0x1033, 0x1014, 0x1034, 0x1015, 0x1034, 0x1127, 0x00111035, 0x1035, 0x1129, 0x00111036, 0x1036, 0x112a, 0x00111037, 0x1037)
GDI32(NtGdiCombineTransform                             ,   NONE, 0x1015, 0x1015, 0x10f4, 0x1015, 0x10f5, 0x1016, 0x10f0, 0x1126, 0x000a10f0, 0x10f0, 0x1128, 0x000a10f1, 0x10f1, 0x1129, 0x000a10f2, 0x10f2)
GDI32(NtGdiComputeXformCoefficients                     ,   NONE, 0x1016, 0x1016, 0x108c, 0x1016, 0x108d, 0x1017, 0x108b, 0x1125, 0x0003108b, 0x108b, 0x1127, 0x0003108c, 0x108c, 0x1128, 0x0003108d, 0x108d)
GDI32(NtGdiConfigureOPMProtectedOutput                  ,   NONE,   NONE,   NONE,   NONE, 0x1017, 0x113f, 0x1018, 0x1137, 0x1124, 0x00001164, 0x1164, 0x1126, 0x00001173, 0x1173, 0x1127, 0x0000117f, 0x117f)
GDI32(NtGdiConsoleTextOut                               , 0x1015, 0x1017, 0x1017, 0x106d, 0x1018, 0x106e,   NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE)
GDI32(NtGdiConvertMetafileRect                          ,   NONE, 0x1018, 0x1018, 0x113e, 0x1019, 0x1140, 0x1019, 0x1138, 0x1123, 0x00071165, 0x1165, 0x1125, 0x00071174, 0x1174, 0x1126, 0x00071180, 0x1180)
GDI32(NtGdiCreateBitmap                                 ,   NONE, 0x1019, 0x1019, 0x106c, 0x101a, 0x106d, 0x101a, 0x106d, 0x1122, 0x0000106d, 0x106d, 0x1124, 0x0000106e, 0x106e, 0x1125, 0x0000106f, 0x106f)
GDI32(NtGdiCreateBitmapFromDxSurface                    ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE, 0x101b, 0x1139, 0x1121, 0x00001166, 0x1166, 0x1123, 0x00001175, 0x1175, 0x1124, 0x00001181, 0x1181)
GDI32(NtGdiCreateBitmapFromDxSurface2                   ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE, 0x1120, 0x00001167, 0x1167, 0x1122, 0x00001176, 0x1176, 0x1123, 0x00001182, 0x1182)
GDI32(NtGdiCreateClientObj                              ,   NONE, 0x101a, 0x101a, 0x10be, 0x101b, 0x10bf, 0x101c, 0x10ba, 0x111f, 0x000410ba, 0x10ba, 0x1121, 0x000410bb, 0x10bb, 0x1122, 0x000410bc, 0x10bc)
GDI32(NtGdiCreateColorSpace                             ,   NONE, 0x101b, 0x101b, 0x1124, 0x101c, 0x1125, 0x101d, 0x111c, 0x111e, 0x0004111c, 0x111c, 0x1120, 0x0004111d, 0x111d, 0x1121, 0x0004111e, 0x111e)
GDI32(NtGdiCreateColorTransform                         ,   NONE, 0x101c, 0x101c, 0x113f, 0x101d, 0x1141, 0x101e, 0x113a, 0x111d, 0x00001168, 0x1168, 0x111f, 0x00001177, 0x1177, 0x1120, 0x00001183, 0x1183)
GDI32(NtGdiCreateCompatibleBitmap                       ,   NONE, 0x101d, 0x101d, 0x104a, 0x101e, 0x104b, 0x101f, 0x104b, 0x111c, 0x000b104c, 0x104c, 0x111e, 0x000b104d, 0x104d, 0x111f, 0x000b104e, 0x104e)
GDI32(NtGdiCreateCompatibleDC                           ,   NONE, 0x101e, 0x101e, 0x1054, 0x101f, 0x1055, 0x1020, 0x1055, 0x111b, 0x00031055, 0x1055, 0x111d, 0x00031056, 0x1056, 0x111e, 0x00031057, 0x1057)
GDI32(NtGdiCreateDIBBrush                               ,   NONE, 0x101f, 0x101f, 0x1075, 0x1020, 0x1076, 0x1021, 0x1074, 0x111a, 0x00001074, 0x1074, 0x111c, 0x00001075, 0x1075, 0x111d, 0x00001076, 0x1076)
GDI32(NtGdiCreateDIBSection                             ,   NONE, 0x1021, 0x1021, 0x109b, 0x1022, 0x109c, 0x1023, 0x109a, 0x1118, 0x0000109a, 0x109a, 0x111a, 0x0000109b, 0x109b, 0x111b, 0x0000109c, 0x109c)
GDI32(NtGdiCreateDIBitmapInternal                       ,   NONE, 0x1020, 0x1020, 0x10a0, 0x1021, 0x10a1, 0x1022, 0x109f, 0x1119, 0x0000109f, 0x109f, 0x111b, 0x000010a0, 0x10a0, 0x111c, 0x000010a1, 0x10a1)
GDI32(NtGdiCreateEllipticRgn                            ,   NONE, 0x1022, 0x1022, 0x1140, 0x1023, 0x1142, 0x1024, 0x113b, 0x1116, 0x00111169, 0x1169, 0x1118, 0x00111178, 0x1178, 0x1119, 0x00111184, 0x1184)
GDI32(NtGdiCreateHalftonePalette                        , 0x1021, 0x1023, 0x1023, 0x10f1, 0x1024, 0x10f2, 0x1025, 0x10ed, 0x1115, 0x000310ed, 0x10ed, 0x1117, 0x000310ee, 0x10ee, 0x1118, 0x000310ef, 0x10ef)
GDI32(NtGdiCreateHatchBrushInternal                     ,   NONE, 0x1024, 0x1024, 0x1141, 0x1025, 0x1143, 0x1026, 0x113c, 0x1114, 0x000a116a, 0x116a, 0x1116, 0x000a1179, 0x1179, 0x1117, 0x000a1185, 0x1185)
GDI32(NtGdiCreateMetafileDC                             ,   NONE, 0x1025, 0x1025, 0x1142, 0x1026, 0x1144, 0x1027, 0x113d, 0x1113, 0x0003116b, 0x116b, 0x1115, 0x0003117a, 0x117a, 0x1116, 0x00031186, 0x1186)
GDI32(NtGdiCreateOPMProtectedOutputs                    ,   NONE,   NONE,   NONE,   NONE, 0x1027, 0x1145, 0x1028, 0x113e, 0x1112, 0x0000116c, 0x116c, 0x1114, 0x0000117b, 0x117b, 0x1115, 0x00001187, 0x1187)
GDI32(NtGdiCreatePaletteInternal                        ,   NONE, 0x1026, 0x1026, 0x10b0, 0x1028, 0x10b1, 0x1029, 0x10ac, 0x1111, 0x000510ac, 0x10ac, 0x1113, 0x000510ad, 0x10ad, 0x1114, 0x000510ae, 0x10ae)
GDI32(NtGdiCreatePatternBrushInternal                   ,   NONE, 0x1027, 0x1027, 0x10b5, 0x1029, 0x10b6, 0x102a, 0x10b1, 0x1110, 0x000a10b1, 0x10b1, 0x1112, 0x000a10b2, 0x10b2, 0x1113, 0x000a10b3, 0x10b3)
GDI32(NtGdiCreatePen                                    ,   NONE, 0x1028, 0x1028, 0x1056, 0x102a, 0x1057, 0x102b, 0x1057, 0x110f, 0x00111057, 0x1057, 0x1111, 0x00111058, 0x1058, 0x1112, 0x00111059, 0x1059)
GDI32(NtGdiCreateRectRgn                                ,   NONE, 0x1029, 0x1029, 0x1085, 0x102b, 0x1086, 0x102c, 0x1084, 0x110e, 0x00111084, 0x1084, 0x1110, 0x00111085, 0x1085, 0x1111, 0x00111086, 0x1086)
GDI32(NtGdiCreateRoundRectRgn                           ,   NONE, 0x102a, 0x102a, 0x1143, 0x102c, 0x1146, 0x102d, 0x113f, 0x110d, 0x0000116d, 0x116d, 0x110f, 0x0000117c, 0x117c, 0x1110, 0x00001188, 0x1188)
GDI32(NtGdiCreateServerMetaFile                         ,   NONE, 0x102b, 0x102b, 0x1144, 0x102d, 0x1147, 0x102e, 0x1140, 0x110c, 0x0000116e, 0x116e, 0x110e, 0x0000117d, 0x117d, 0x110f, 0x00001189, 0x1189)
GDI32(NtGdiCreateSessionMappedDIBSection                ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE, 0x1117, 0x0000116f, 0x116f, 0x1119, 0x0000117e, 0x117e, 0x111a, 0x0000118a, 0x118a)
GDI32(NtGdiCreateSolidBrush                             ,   NONE, 0x102c, 0x102c, 0x10bc, 0x102e, 0x10bd, 0x102f, 0x10b8, 0x110b, 0x000510b8, 0x10b8, 0x110d, 0x000510b9, 0x10b9, 0x110e, 0x000510ba, 0x10ba)
GDI32(NtGdiD3dContextCreate                             ,   NONE, 0x102d, 0x102d, 0x1145, 0x102f, 0x1148, 0x1030, 0x1141, 0x110a, 0x00001170, 0x1170, 0x110c, 0x0000117f, 0x117f, 0x110d, 0x0000118b, 0x118b)
GDI32(NtGdiD3dContextDestroy                            ,   NONE, 0x102e, 0x102e, 0x1146, 0x1030, 0x1149, 0x1031, 0x1142, 0x1109, 0x00001171, 0x1171, 0x110b, 0x00001180, 0x1180, 0x110c, 0x0000118c, 0x118c)
GDI32(NtGdiD3dContextDestroyAll                         ,   NONE, 0x102f, 0x102f, 0x1147, 0x1031, 0x114a, 0x1032, 0x1143, 0x1108, 0x00041172, 0x1172, 0x110a, 0x00041181, 0x1181, 0x110b, 0x0004118d, 0x118d)
GDI32(NtGdiD3dDrawPrimitives2                           ,   NONE, 0x1031, 0x1031, 0x1064, 0x1033, 0x1065, 0x1034, 0x1065, 0x1106, 0x00001065, 0x1065, 0x1108, 0x00001066, 0x1066, 0x1109, 0x00001067, 0x1067)
GDI32(NtGdiD3dValidateTextureStageState                 ,   NONE, 0x1030, 0x1030, 0x1148, 0x1032, 0x114b, 0x1033, 0x1144, 0x1107, 0x00001173, 0x1173, 0x1109, 0x00001182, 0x1182, 0x110a, 0x0000118e, 0x118e)
GDI32(NtGdiDDCCIGetCapabilitiesString                   ,   NONE,   NONE,   NONE,   NONE, 0x1300, 0x114c, 0x132c, 0x1145, 0x136f, 0x00001174, 0x1174, 0x1381, 0x00001183, 0x1183, 0x13ba, 0x0000118f, 0x118f)
GDI32(NtGdiDDCCIGetCapabilitiesStringLength             ,   NONE,   NONE,   NONE,   NONE, 0x12ff, 0x114d, 0x132b, 0x1146, 0x1370, 0x00001175, 0x1175, 0x1382, 0x00001184, 0x1184, 0x13bb, 0x00001190, 0x1190)
GDI32(NtGdiDDCCIGetTimingReport                         ,   NONE,   NONE,   NONE,   NONE, 0x1301, 0x114e, 0x132d, 0x1147, 0x136e, 0x00001176, 0x1176, 0x1380, 0x00001185, 0x1185, 0x13b9, 0x00001191, 0x1191)
GDI32(NtGdiDDCCIGetVCPFeature                           ,   NONE,   NONE,   NONE,   NONE, 0x12fc, 0x114f, 0x1328, 0x1148, 0x1373, 0x00001177, 0x1177, 0x1385, 0x00001186, 0x1186, 0x13be, 0x00001192, 0x1192)
GDI32(NtGdiDDCCISaveCurrentSettings                     ,   NONE,   NONE,   NONE,   NONE, 0x12fe, 0x1150, 0x132a, 0x1149, 0x1371, 0x00001178, 0x1178, 0x1383, 0x00001187, 0x1187, 0x13bc, 0x00001193, 0x1193)
GDI32(NtGdiDDCCISetVCPFeature                           ,   NONE,   NONE,   NONE,   NONE, 0x12fd, 0x1151, 0x1329, 0x114a, 0x1372, 0x00001179, 0x1179, 0x1384, 0x00001188, 0x1188, 0x13bd, 0x00001194, 0x1194)
GDI32(NtGdiDdAddAttachedSurface                         ,   NONE, 0x1033, 0x1033, 0x1149, 0x1035, 0x1152, 0x1036, 0x114b, 0x1104, 0x0000117a, 0x117a, 0x1106, 0x00001189, 0x1189, 0x1107, 0x00001195, 0x1195)
GDI32(NtGdiDdAlphaBlt                                   ,   NONE, 0x1034, 0x1034, 0x114a, 0x1036, 0x1153, 0x1037, 0x114c, 0x1103, 0x0000117b, 0x117b, 0x1105, 0x0000118a, 0x118a, 0x1106, 0x00001196, 0x1196)
GDI32(NtGdiDdAttachSurface                              ,   NONE, 0x1035, 0x1035, 0x114b, 0x1037, 0x1154, 0x1038, 0x114d, 0x1102, 0x0005117c, 0x117c, 0x1104, 0x0005118b, 0x118b, 0x1105, 0x00051197, 0x1197)
GDI32(NtGdiDdBeginMoCompFrame                           ,   NONE, 0x1036, 0x1036, 0x114c, 0x1038, 0x1155, 0x1039, 0x114e, 0x1101, 0x0000117d, 0x117d, 0x1103, 0x0000118c, 0x118c, 0x1104, 0x00001198, 0x1198)
GDI32(NtGdiDdBlt                                        ,   NONE, 0x1037, 0x1037, 0x107e, 0x1039, 0x107f, 0x103a, 0x107d, 0x1100, 0x0000107d, 0x107d, 0x1102, 0x0000107e, 0x107e, 0x1103, 0x0000107f, 0x107f)
GDI32(NtGdiDdCanCreateD3DBuffer                         ,   NONE, 0x1039, 0x1039, 0x114d, 0x103b, 0x1156, 0x103c, 0x114f, 0x10fe, 0x0000117e, 0x117e, 0x1100, 0x0000118d, 0x118d, 0x1101, 0x00001199, 0x1199)
GDI32(NtGdiDdCanCreateSurface                           ,   NONE, 0x1038, 0x1038, 0x10a6, 0x103a, 0x10a7, 0x103b, 0x10a2, 0x10ff, 0x000010a2, 0x10a2, 0x1101, 0x000010a3, 0x10a3, 0x1102, 0x000010a4, 0x10a4)
GDI32(NtGdiDdColorControl                               ,   NONE, 0x103a, 0x103a, 0x114e, 0x103c, 0x1157, 0x103d, 0x1150, 0x10fd, 0x0000117f, 0x117f, 0x10ff, 0x0000118e, 0x118e, 0x1100, 0x0000119a, 0x119a)
GDI32(NtGdiDdCreateD3DBuffer                            ,   NONE, 0x103d, 0x103d, 0x114f, 0x103f, 0x1158, 0x1040, 0x1151, 0x10fa, 0x00001180, 0x1180, 0x10fc, 0x0000118f, 0x118f, 0x10fd, 0x0000119b, 0x119b)
GDI32(NtGdiDdCreateDirectDrawObject                     ,   NONE, 0x103b, 0x103b, 0x1150, 0x103d, 0x1159, 0x103e, 0x1152, 0x10fc, 0x00031181, 0x1181, 0x10fe, 0x00031190, 0x1190, 0x10ff, 0x0003119c, 0x119c)
GDI32(NtGdiDdCreateFullscreenSprite                     ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE, 0x132e, 0x1153, 0x1377, 0x00001182, 0x1182, 0x1389, 0x00001191, 0x1191, 0x13c2, 0x0000119d, 0x119d)
GDI32(NtGdiDdCreateMoComp                               ,   NONE, 0x103e, 0x103e, 0x1151, 0x1040, 0x115a, 0x1041, 0x1154, 0x10f9, 0x00001183, 0x1183, 0x10fb, 0x00001192, 0x1192, 0x10fc, 0x0000119e, 0x119e)
GDI32(NtGdiDdCreateSurface                              ,   NONE, 0x103c, 0x103c, 0x10a7, 0x103e, 0x10a8, 0x103f, 0x10a3, 0x10fb, 0x000010a3, 0x10a3, 0x10fd, 0x000010a4, 0x10a4, 0x10fe, 0x000010a5, 0x10a5)
GDI32(NtGdiDdCreateSurfaceEx                            ,   NONE, 0x105e, 0x105e, 0x10c6, 0x1060, 0x10c7, 0x1061, 0x10c2, 0x10d9, 0x000a10c2, 0x10c2, 0x10db, 0x000a10c3, 0x10c3, 0x10dc, 0x000a10c4, 0x10c4)
GDI32(NtGdiDdCreateSurfaceObject                        ,   NONE, 0x103f, 0x103f, 0x10c7, 0x1041, 0x10c8, 0x1042, 0x10c3, 0x10f8, 0x000010c3, 0x10c3, 0x10fa, 0x000010c4, 0x10c4, 0x10fb, 0x000010c5, 0x10c5)
GDI32(NtGdiDdDDIAbandonSwapChain                        ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE, 0x13a8, 0x0000119f, 0x119f)
GDI32(NtGdiDdDDIAcquireKeyedMutex                       ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE, 0x131c, 0x1155, 0x1353, 0x00001184, 0x1184, 0x135c, 0x00001193, 0x1193, 0x136c, 0x000011a0, 0x11a0)
GDI32(NtGdiDdDDIAcquireKeyedMutex2                      ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE, 0x1344, 0x00001185, 0x1185, 0x134d, 0x00001194, 0x1194, 0x135d, 0x000011a1, 0x11a1)
GDI32(NtGdiDdDDIAcquireSwapChain                        ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE, 0x13a6, 0x000011a2, 0x11a2)
GDI32(NtGdiDdDDIAdjustFullscreenGamma                   ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE, 0x13b0, 0x000011a3, 0x11a3)
GDI32(NtGdiDdDDICacheHybridQueryValue                   ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE,   NONE, 0x1374, 0x00001195, 0x1195, 0x1399, 0x000011a4, 0x11a4)
GDI32(NtGdiDdDDIChangeVideoMemoryReservation            ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE, 0x13a2, 0x000011a5, 0x11a5)
GDI32(NtGdiDdDDICheckExclusiveOwnership                 ,   NONE,   NONE,   NONE,   NONE, 0x12f2, 0x115b, 0x1315, 0x1156, 0x135a, 0x00001186, 0x1186, 0x1363, 0x00001196, 0x1196, 0x1373, 0x000011a6, 0x11a6)
GDI32(NtGdiDdDDICheckMonitorPowerState                  ,   NONE,   NONE,   NONE,   NONE, 0x12f1, 0x115c, 0x1314, 0x1157, 0x135b, 0x00001187, 0x1187, 0x1364, 0x00001197, 0x1197, 0x1374, 0x000011a7, 0x11a7)
GDI32(NtGdiDdDDICheckMultiPlaneOverlaySupport           ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE,   NONE, 0x1372, 0x00001198, 0x1198, 0x1384, 0x000011a8, 0x11a8)
GDI32(NtGdiDdDDICheckMultiPlaneOverlaySupport2          ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE, 0x13aa, 0x000011a9, 0x11a9)
GDI32(NtGdiDdDDICheckOcclusion                          ,   NONE,   NONE,   NONE,   NONE, 0x12ef, 0x115d, 0x1312, 0x1158, 0x135d, 0x00001188, 0x1188, 0x1366, 0x00001199, 0x1199, 0x1376, 0x000011aa, 0x11aa)
GDI32(NtGdiDdDDICheckSharedResourceAccess               ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE, 0x1321, 0x1159, 0x134e, 0x00001189, 0x1189, 0x1357, 0x0000119a, 0x119a, 0x1367, 0x000011ab, 0x11ab)
GDI32(NtGdiDdDDICheckVidPnExclusiveOwnership            ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE, 0x1320, 0x115a, 0x134f, 0x0000118a, 0x118a, 0x1358, 0x0000119b, 0x119b, 0x1368, 0x000011ac, 0x11ac)
GDI32(NtGdiDdDDICloseAdapter                            ,   NONE,   NONE,   NONE,   NONE, 0x12d7, 0x115e, 0x12f9, 0x115b, 0x131e, 0x0000118b, 0x118b, 0x1327, 0x0000119c, 0x119c, 0x1333, 0x000011ad, 0x11ad)
GDI32(NtGdiDdDDIConfigureSharedResource                 ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE, 0x131e, 0x115c, 0x1351, 0x0000118c, 0x118c, 0x135a, 0x0000119d, 0x119d, 0x136a, 0x000011ae, 0x11ae)
GDI32(NtGdiDdDDICreateAllocation                        ,   NONE,   NONE,   NONE,   NONE, 0x12be, 0x115f, 0x12df, 0x115d, 0x133f, 0x0000118d, 0x118d, 0x1348, 0x0000119e, 0x119e, 0x1358, 0x000011af, 0x11af)
GDI32(NtGdiDdDDICreateContext                           ,   NONE,   NONE,   NONE,   NONE, 0x12c6, 0x1160, 0x12e7, 0x115e, 0x1332, 0x0000118e, 0x118e, 0x133b, 0x0000119f, 0x119f, 0x1348, 0x000011b0, 0x11b0)
GDI32(NtGdiDdDDICreateContextVirtual                    ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE, 0x1396, 0x000011b1, 0x11b1)
GDI32(NtGdiDdDDICreateDCFromMemory                      ,   NONE,   NONE,   NONE,   NONE, 0x12e4, 0x1161, 0x1307, 0x115f, 0x130f, 0x0000118f, 0x118f, 0x1318, 0x000011a0, 0x11a0, 0x1324, 0x000011b2, 0x11b2)
GDI32(NtGdiDdDDICreateDevice                            ,   NONE,   NONE,   NONE,   NONE, 0x12c4, 0x1162, 0x12e5, 0x1160, 0x1334, 0x00001190, 0x1190, 0x133d, 0x000011a1, 0x11a1, 0x134a, 0x000011b3, 0x11b3)
GDI32(NtGdiDdDDICreateKeyedMutex                        ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE, 0x1319, 0x1161, 0x1356, 0x00001191, 0x1191, 0x135f, 0x000011a2, 0x11a2, 0x136f, 0x000011b4, 0x11b4)
GDI32(NtGdiDdDDICreateKeyedMutex2                       ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE, 0x1346, 0x00001192, 0x1192, 0x134f, 0x000011a3, 0x11a3, 0x135f, 0x000011b5, 0x11b5)
GDI32(NtGdiDdDDICreateOutputDupl                        ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE, 0x134b, 0x00001193, 0x1193, 0x1354, 0x000011a4, 0x11a4, 0x1364, 0x000011b6, 0x11b6)
GDI32(NtGdiDdDDICreateOverlay                           ,   NONE,   NONE,   NONE,   NONE, 0x12dd, 0x1163, 0x1300, 0x1162, 0x1316, 0x00001194, 0x1194, 0x131f, 0x000011a5, 0x11a5, 0x132b, 0x000011b7, 0x11b7)
GDI32(NtGdiDdDDICreatePagingQueue                       ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE, 0x138c, 0x000011b8, 0x11b8)
GDI32(NtGdiDdDDICreateSwapChain                         ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE, 0x13a3, 0x000011b9, 0x11b9)
GDI32(NtGdiDdDDICreateSynchronizationObject             ,   NONE,   NONE,   NONE,   NONE, 0x12c8, 0x1164, 0x12e9, 0x1163, 0x1330, 0x00001195, 0x1195, 0x1339, 0x000011a6, 0x11a6, 0x1346, 0x000011ba, 0x11ba)
GDI32(NtGdiDdDDIDestroyAllocation                       ,   NONE,   NONE,   NONE,   NONE, 0x12c1, 0x1165, 0x12e2, 0x1164, 0x1337, 0x00001196, 0x1196, 0x1340, 0x000011a7, 0x11a7, 0x134d, 0x000011bb, 0x11bb)
GDI32(NtGdiDdDDIDestroyAllocation2                      ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE, 0x134e, 0x000011bc, 0x11bc)
GDI32(NtGdiDdDDIDestroyContext                          ,   NONE,   NONE,   NONE,   NONE, 0x12c7, 0x1166, 0x12e8, 0x1165, 0x1331, 0x00001197, 0x1197, 0x133a, 0x000011a8, 0x11a8, 0x1347, 0x000011bd, 0x11bd)
GDI32(NtGdiDdDDIDestroyDCFromMemory                     ,   NONE,   NONE,   NONE,   NONE, 0x12e5, 0x1167, 0x1308, 0x1166, 0x130e, 0x00001198, 0x1198, 0x1317, 0x000011a9, 0x11a9, 0x1323, 0x000011be, 0x11be)
GDI32(NtGdiDdDDIDestroyDevice                           ,   NONE,   NONE,   NONE,   NONE, 0x12c5, 0x1168, 0x12e6, 0x1167, 0x1333, 0x00001199, 0x1199, 0x133c, 0x000011aa, 0x11aa, 0x1349, 0x000011bf, 0x11bf)
GDI32(NtGdiDdDDIDestroyKeyedMutex                       ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE, 0x131b, 0x1168, 0x1354, 0x0000119a, 0x119a, 0x135d, 0x000011ab, 0x11ab, 0x136d, 0x000011c0, 0x11c0)
GDI32(NtGdiDdDDIDestroyOutputDupl                       ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE, 0x134a, 0x0000119b, 0x119b, 0x1353, 0x000011ac, 0x11ac, 0x1363, 0x000011c1, 0x11c1)
GDI32(NtGdiDdDDIDestroyOverlay                          ,   NONE,   NONE,   NONE,   NONE, 0x12e0, 0x1169, 0x1303, 0x1169, 0x1313, 0x0000119c, 0x119c, 0x131c, 0x000011ad, 0x11ad, 0x1328, 0x000011c2, 0x11c2)
GDI32(NtGdiDdDDIDestroyPagingQueue                      ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE, 0x138d, 0x000011c3, 0x11c3)
GDI32(NtGdiDdDDIDestroySynchronizationObject            ,   NONE,   NONE,   NONE,   NONE, 0x12c9, 0x116a, 0x12eb, 0x116a, 0x132e, 0x0000119d, 0x119d, 0x1337, 0x000011ae, 0x11ae, 0x1344, 0x000011c4, 0x11c4)
GDI32(NtGdiDdDDIEnumAdapters                            ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE, 0x1320, 0x0000119e, 0x119e, 0x1329, 0x000011af, 0x11af, 0x1335, 0x000011c5, 0x11c5)
GDI32(NtGdiDdDDIEnumAdapters2                           ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE, 0x1336, 0x000011c6, 0x11c6)
GDI32(NtGdiDdDDIEscape                                  ,   NONE,   NONE,   NONE,   NONE, 0x12d9, 0x116b, 0x12fb, 0x116b, 0x131c, 0x0000119f, 0x119f, 0x1325, 0x000011b0, 0x11b0, 0x1331, 0x000011c7, 0x11c7)
GDI32(NtGdiDdDDIEvict                                   ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE, 0x1386, 0x000011c8, 0x11c8)
GDI32(NtGdiDdDDIFlipOverlay                             ,   NONE,   NONE,   NONE,   NONE, 0x12df, 0x116c, 0x1302, 0x116c, 0x1314, 0x000011a0, 0x11a0, 0x131d, 0x000011b1, 0x11b1, 0x1329, 0x000011c9, 0x11c9)
GDI32(NtGdiDdDDIFreeGpuVirtualAddress                   ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE, 0x1394, 0x000011ca, 0x11ca)
GDI32(NtGdiDdDDIGetCachedHybridQueryValue               ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE,   NONE, 0x1373, 0x000011b2, 0x11b2, 0x1398, 0x000011cb, 0x11cb)
GDI32(NtGdiDdDDIGetContextInProcessSchedulingPriority   ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE, 0x1365, 0x000011a1, 0x11a1, 0x136e, 0x000011b3, 0x11b3, 0x1380, 0x000011cc, 0x11cc)
GDI32(NtGdiDdDDIGetContextSchedulingPriority            ,   NONE,   NONE,   NONE,   NONE, 0x12e7, 0x116d, 0x130a, 0x116d, 0x130c, 0x000011a2, 0x11a2, 0x1315, 0x000011b4, 0x11b4, 0x1321, 0x000011cd, 0x11cd)
GDI32(NtGdiDdDDIGetDWMVerticalBlankEvent                ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE, 0x137d, 0x000011ce, 0x11ce)
GDI32(NtGdiDdDDIGetDeviceState                          ,   NONE,   NONE,   NONE,   NONE, 0x12e3, 0x116e, 0x1306, 0x116e, 0x1310, 0x000011a3, 0x11a3, 0x1319, 0x000011b5, 0x11b5, 0x1325, 0x000011cf, 0x11cf)
GDI32(NtGdiDdDDIGetDisplayModeList                      ,   NONE,   NONE,   NONE,   NONE, 0x12d0, 0x116f, 0x12f2, 0x116f, 0x1327, 0x000011a4, 0x11a4, 0x1330, 0x000011b6, 0x11b6, 0x133d, 0x000011d0, 0x11d0)
GDI32(NtGdiDdDDIGetMultisampleMethodList                ,   NONE,   NONE,   NONE,   NONE, 0x12d2, 0x1170, 0x12f4, 0x1170, 0x1325, 0x000011a5, 0x11a5, 0x132e, 0x000011b7, 0x11b7, 0x133b, 0x000011d1, 0x11d1)
GDI32(NtGdiDdDDIGetOverlayState                         ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE, 0x131f, 0x1171, 0x1350, 0x000011a6, 0x11a6, 0x1359, 0x000011b8, 0x11b8, 0x1369, 0x000011d2, 0x11d2)
GDI32(NtGdiDdDDIGetPresentHistory                       ,   NONE,   NONE,   NONE,   NONE, 0x12dc, 0x1171, 0x12fe, 0x1172, 0x1318, 0x000011a7, 0x11a7, 0x1321, 0x000011b9, 0x11b9, 0x132d, 0x000011d3, 0x11d3)
GDI32(NtGdiDdDDIGetPresentQueueEvent                    ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE, 0x12ff, 0x1173, 0x1317, 0x000011a8, 0x11a8, 0x1320, 0x000011ba, 0x11ba, 0x132c, 0x000011d4, 0x11d4)
GDI32(NtGdiDdDDIGetProcessSchedulingPriorityClass       ,   NONE,   NONE,   NONE,   NONE, 0x12e9, 0x1172, 0x130c, 0x1174, 0x130a, 0x000011a9, 0x11a9, 0x1313, 0x000011bb, 0x11bb, 0x131f, 0x000011d5, 0x11d5)
GDI32(NtGdiDdDDIGetResourcePresentPrivateDriverData     ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE, 0x1391, 0x000011d6, 0x11d6)
GDI32(NtGdiDdDDIGetRuntimeData                          ,   NONE,   NONE,   NONE,   NONE, 0x12cc, 0x1173, 0x12ee, 0x1175, 0x132b, 0x000011aa, 0x11aa, 0x1334, 0x000011bc, 0x11bc, 0x1341, 0x000011d7, 0x11d7)
GDI32(NtGdiDdDDIGetScanLine                             ,   NONE,   NONE,   NONE,   NONE, 0x12eb, 0x1174, 0x130e, 0x1176, 0x1308, 0x000011ab, 0x11ab, 0x1311, 0x000011bd, 0x11bd, 0x131d, 0x000011d8, 0x11d8)
GDI32(NtGdiDdDDIGetSetSwapChainMetadata                 ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE, 0x13a5, 0x000011d9, 0x11d9)
GDI32(NtGdiDdDDIGetSharedPrimaryHandle                  ,   NONE,   NONE,   NONE,   NONE, 0x12d8, 0x1175, 0x12fa, 0x1177, 0x131d, 0x000011ac, 0x11ac, 0x1326, 0x000011be, 0x11be, 0x1332, 0x000011da, 0x11da)
GDI32(NtGdiDdDDIGetSharedResourceAdapterLuid            ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE, 0x1366, 0x000011ad, 0x11ad, 0x136f, 0x000011bf, 0x11bf, 0x1381, 0x000011db, 0x11db)
GDI32(NtGdiDdDDIInvalidateActiveVidPn                   ,   NONE,   NONE,   NONE,   NONE, 0x12ee, 0x1176, 0x1311, 0x1178, 0x135e, 0x000011ae, 0x11ae, 0x1367, 0x000011c0, 0x11c0, 0x1377, 0x000011dc, 0x11dc)
GDI32(NtGdiDdDDIInvalidateCache                         ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE, 0x1390, 0x000011dd, 0x11dd)
GDI32(NtGdiDdDDILock                                    ,   NONE,   NONE,   NONE,   NONE, 0x12ce, 0x1177, 0x12f0, 0x1179, 0x1329, 0x000011af, 0x11af, 0x1332, 0x000011c1, 0x11c1, 0x133f, 0x000011de, 0x11de)
GDI32(NtGdiDdDDILock2                                   ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE, 0x138e, 0x000011df, 0x11df)
GDI32(NtGdiDdDDIMakeResident                            ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE, 0x1385, 0x000011e0, 0x11e0)
GDI32(NtGdiDdDDIMapGpuVirtualAddress                    ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE, 0x1392, 0x000011e1, 0x11e1)
GDI32(NtGdiDdDDIMarkDeviceAsError                       ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE, 0x13b2, 0x000011e2, 0x11e2)
GDI32(NtGdiDdDDINetDispGetNextChunkInfo                 ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE,   NONE, 0x1375, 0x000011c2, 0x11c2, 0x139a, 0x000011e3, 0x11e3)
GDI32(NtGdiDdDDINetDispQueryMiracastDisplayDeviceStatus ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE,   NONE, 0x1379, 0x000011c3, 0x11c3, 0x139f, 0x000011e4, 0x11e4)
GDI32(NtGdiDdDDINetDispQueryMiracastDisplayDeviceSupport,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE,   NONE, 0x1376, 0x000011c4, 0x11c4, 0x139b, 0x000011e5, 0x11e5)
GDI32(NtGdiDdDDINetDispStartMiracastDisplayDevice       ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE,   NONE, 0x1377, 0x000011c5, 0x11c5, 0x139c, 0x000011e6, 0x11e6)
GDI32(NtGdiDdDDINetDispStartMiracastDisplayDeviceEx     ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE, 0x139d, 0x000011e7, 0x11e7)
GDI32(NtGdiDdDDINetDispStopMiracastDisplayDevice        ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE,   NONE, 0x1378, 0x000011c6, 0x11c6, 0x139e, 0x000011e8, 0x11e8)
GDI32(NtGdiDdDDINetDispStopSessions                     ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE, 0x13a0, 0x000011e9, 0x11e9)
GDI32(NtGdiDdDDIOfferAllocations                        ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE, 0x134d, 0x000011b0, 0x11b0, 0x1356, 0x000011c7, 0x11c7, 0x1366, 0x000011ea, 0x11ea)
GDI32(NtGdiDdDDIOpenAdapterFromDeviceName               ,   NONE,   NONE,   NONE,   NONE, 0x12d5, 0x1178, 0x12f7, 0x117a, 0x1322, 0x000011b1, 0x11b1, 0x132b, 0x000011c8, 0x11c8, 0x1338, 0x000011eb, 0x11eb)
GDI32(NtGdiDdDDIOpenAdapterFromHdc                      ,   NONE,   NONE,   NONE,   NONE, 0x12d6, 0x1179, 0x12f8, 0x117b, 0x1321, 0x000011b2, 0x11b2, 0x132a, 0x000011c9, 0x11c9, 0x1337, 0x000011ec, 0x11ec)
GDI32(NtGdiDdDDIOpenAdapterFromLuid                     ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE, 0x131f, 0x000011b3, 0x11b3, 0x1328, 0x000011ca, 0x11ca, 0x1334, 0x000011ed, 0x11ed)
GDI32(NtGdiDdDDIOpenKeyedMutex                          ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE, 0x131a, 0x117c, 0x1355, 0x000011b4, 0x11b4, 0x135e, 0x000011cb, 0x11cb, 0x136e, 0x000011ee, 0x11ee)
GDI32(NtGdiDdDDIOpenKeyedMutex2                         ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE, 0x1345, 0x000011b5, 0x11b5, 0x134e, 0x000011cc, 0x11cc, 0x135e, 0x000011ef, 0x11ef)
GDI32(NtGdiDdDDIOpenNtHandleFromName                    ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE, 0x133b, 0x000011b6, 0x11b6, 0x1344, 0x000011cd, 0x11cd, 0x1353, 0x000011f0, 0x11f0)
GDI32(NtGdiDdDDIOpenResource                            ,   NONE,   NONE,   NONE,   NONE, 0x12c0, 0x117a, 0x12e1, 0x117d, 0x133a, 0x000011b7, 0x11b7, 0x1343, 0x000011ce, 0x11ce, 0x1352, 0x000011f1, 0x11f1)
GDI32(NtGdiDdDDIOpenResourceFromNtHandle                ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE, 0x1338, 0x000011b8, 0x11b8, 0x1341, 0x000011cf, 0x11cf, 0x134f, 0x000011f2, 0x11f2)
GDI32(NtGdiDdDDIOpenSwapChain                           ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE, 0x13a4, 0x000011f3, 0x11f3)
GDI32(NtGdiDdDDIOpenSyncObjectFromNtHandle              ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE, 0x1339, 0x000011b9, 0x11b9, 0x1342, 0x000011d0, 0x11d0, 0x1350, 0x000011f4, 0x11f4)
GDI32(NtGdiDdDDIOpenSyncObjectFromNtHandle2             ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE, 0x1351, 0x000011f5, 0x11f5)
GDI32(NtGdiDdDDIOpenSyncObjectNtHandleFromName          ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE, 0x1354, 0x000011f6, 0x11f6)
GDI32(NtGdiDdDDIOpenSynchronizationObject               ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE, 0x12ea, 0x117e, 0x132f, 0x000011ba, 0x11ba, 0x1338, 0x000011d1, 0x11d1, 0x1345, 0x000011f7, 0x11f7)
GDI32(NtGdiDdDDIOutputDuplGetFrameInfo                  ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE, 0x1349, 0x000011bb, 0x11bb, 0x1352, 0x000011d2, 0x11d2, 0x1362, 0x000011f8, 0x11f8)
GDI32(NtGdiDdDDIOutputDuplGetMetaData                   ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE, 0x1348, 0x000011bc, 0x11bc, 0x1351, 0x000011d3, 0x11d3, 0x1361, 0x000011f9, 0x11f9)
GDI32(NtGdiDdDDIOutputDuplGetPointerShapeData           ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE, 0x1347, 0x000011bd, 0x11bd, 0x1350, 0x000011d4, 0x11d4, 0x1360, 0x000011fa, 0x11fa)
GDI32(NtGdiDdDDIOutputDuplPresent                       ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE, 0x1342, 0x000011be, 0x11be, 0x134b, 0x000011d5, 0x11d5, 0x135b, 0x000011fb, 0x11fb)
GDI32(NtGdiDdDDIOutputDuplReleaseFrame                  ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE, 0x1340, 0x000011bf, 0x11bf, 0x1349, 0x000011d6, 0x11d6, 0x1359, 0x000011fc, 0x11fc)
GDI32(NtGdiDdDDIPinDirectFlipResources                  ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE, 0x1361, 0x000011c0, 0x11c0, 0x136a, 0x000011d7, 0x11d7, 0x137a, 0x000011fd, 0x11fd)
GDI32(NtGdiDdDDIPollDisplayChildren                     ,   NONE,   NONE,   NONE,   NONE, 0x12ed, 0x117b, 0x1310, 0x117f, 0x135f, 0x000011c1, 0x11c1, 0x1368, 0x000011d8, 0x11d8, 0x1378, 0x000011fe, 0x11fe)
GDI32(NtGdiDdDDIPresent                                 ,   NONE,   NONE,   NONE,   NONE, 0x12d3, 0x117c, 0x12f5, 0x1180, 0x1324, 0x000011c2, 0x11c2, 0x132d, 0x000011d9, 0x11d9, 0x133a, 0x000011ff, 0x11ff)
GDI32(NtGdiDdDDIPresentMultiPlaneOverlay                ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE,   NONE, 0x1371, 0x000011da, 0x11da, 0x1383, 0x00001200, 0x1200)
GDI32(NtGdiDdDDIPresentMultiPlaneOverlay2               ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE, 0x13ab, 0x00001201, 0x1201)
GDI32(NtGdiDdDDIQueryAdapterInfo                        ,   NONE,   NONE,   NONE,   NONE, 0x12cd, 0x117d, 0x12ef, 0x1181, 0x132a, 0x000011c3, 0x11c3, 0x1333, 0x000011db, 0x11db, 0x1340, 0x00001202, 0x1202)
GDI32(NtGdiDdDDIQueryAllocationResidency                ,   NONE,   NONE,   NONE,   NONE, 0x12c3, 0x117e, 0x12e4, 0x1182, 0x1335, 0x000011c4, 0x11c4, 0x133e, 0x000011dc, 0x11dc, 0x134b, 0x00001203, 0x1203)
GDI32(NtGdiDdDDIQueryClockCalibration                   ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE, 0x13ae, 0x00001204, 0x1204)
GDI32(NtGdiDdDDIQueryRemoteVidPnSourceFromGdiDisplayName,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE, 0x1341, 0x000011c5, 0x11c5, 0x134a, 0x000011dd, 0x11dd, 0x135a, 0x00001205, 0x1205)
GDI32(NtGdiDdDDIQueryResourceInfo                       ,   NONE,   NONE,   NONE,   NONE, 0x12bf, 0x117f, 0x12e0, 0x1183, 0x133e, 0x000011c6, 0x11c6, 0x1347, 0x000011de, 0x11de, 0x1357, 0x00001206, 0x1206)
GDI32(NtGdiDdDDIQueryResourceInfoFromNtHandle           ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE, 0x133d, 0x000011c7, 0x11c7, 0x1346, 0x000011df, 0x11df, 0x1356, 0x00001207, 0x1207)
GDI32(NtGdiDdDDIQueryStatistics                         ,   NONE,   NONE,   NONE,   NONE, 0x12da, 0x1180, 0x12fc, 0x1184, 0x131b, 0x000011c8, 0x11c8, 0x1324, 0x000011e0, 0x11e0, 0x1330, 0x00001208, 0x1208)
GDI32(NtGdiDdDDIQueryVidPnExclusiveOwnership            ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE, 0x13af, 0x00001209, 0x1209)
GDI32(NtGdiDdDDIQueryVideoMemoryInfo                    ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE, 0x13a1, 0x0000120a, 0x120a)
GDI32(NtGdiDdDDIReclaimAllocations                      ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE, 0x134c, 0x000011c9, 0x11c9, 0x1355, 0x000011e1, 0x11e1, 0x1365, 0x0000120b, 0x120b)
GDI32(NtGdiDdDDIReclaimAllocations2                     ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE, 0x13ac, 0x0000120c, 0x120c)
GDI32(NtGdiDdDDIReleaseKeyedMutex                       ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE, 0x131d, 0x1185, 0x1352, 0x000011ca, 0x11ca, 0x135b, 0x000011e2, 0x11e2, 0x136b, 0x0000120d, 0x120d)
GDI32(NtGdiDdDDIReleaseKeyedMutex2                      ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE, 0x1343, 0x000011cb, 0x11cb, 0x134c, 0x000011e3, 0x11e3, 0x135c, 0x0000120e, 0x120e)
GDI32(NtGdiDdDDIReleaseProcessVidPnSourceOwners         ,   NONE,   NONE,   NONE,   NONE, 0x12ea, 0x1181, 0x130d, 0x1186, 0x1309, 0x000011cc, 0x11cc, 0x1312, 0x000011e4, 0x11e4, 0x131e, 0x0000120f, 0x120f)
GDI32(NtGdiDdDDIReleaseSwapChain                        ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE, 0x13a7, 0x00001210, 0x1210)
GDI32(NtGdiDdDDIRender                                  ,   NONE,   NONE,   NONE,   NONE, 0x12d4, 0x1182, 0x12f6, 0x1187, 0x1323, 0x000011cd, 0x11cd, 0x132c, 0x000011e5, 0x11e5, 0x1339, 0x00001211, 0x1211)
GDI32(NtGdiDdDDIReserveGpuVirtualAddress                ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE, 0x1393, 0x00001212, 0x1212)
GDI32(NtGdiDdDDISetAllocationPriority                   ,   NONE,   NONE,   NONE,   NONE, 0x12c2, 0x1183, 0x12e3, 0x1188, 0x1336, 0x000011ce, 0x11ce, 0x133f, 0x000011e6, 0x11e6, 0x134c, 0x00001213, 0x1213)
GDI32(NtGdiDdDDISetContextInProcessSchedulingPriority   ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE, 0x1364, 0x000011cf, 0x11cf, 0x136d, 0x000011e7, 0x11e7, 0x137f, 0x00001214, 0x1214)
GDI32(NtGdiDdDDISetContextSchedulingPriority            ,   NONE,   NONE,   NONE,   NONE, 0x12e6, 0x1184, 0x1309, 0x1189, 0x130d, 0x000011d0, 0x11d0, 0x1316, 0x000011e8, 0x11e8, 0x1322, 0x00001215, 0x1215)
GDI32(NtGdiDdDDISetDisplayMode                          ,   NONE,   NONE,   NONE,   NONE, 0x12d1, 0x1185, 0x12f3, 0x118a, 0x1326, 0x000011d1, 0x11d1, 0x132f, 0x000011e9, 0x11e9, 0x133c, 0x00001216, 0x1216)
GDI32(NtGdiDdDDISetDisplayPrivateDriverFormat           ,   NONE,   NONE,   NONE,   NONE, 0x12f3, 0x1186, 0x1316, 0x118b, 0x1359, 0x000011d2, 0x11d2, 0x1362, 0x000011ea, 0x11ea, 0x1372, 0x00001217, 0x1217)
GDI32(NtGdiDdDDISetDodIndirectSwapchain                 ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE, 0x13a9, 0x00001218, 0x1218)
GDI32(NtGdiDdDDISetGammaRamp                            ,   NONE,   NONE,   NONE,   NONE, 0x12e2, 0x1187, 0x1305, 0x118c, 0x1311, 0x000011d3, 0x11d3, 0x131a, 0x000011eb, 0x11eb, 0x1326, 0x00001219, 0x1219)
GDI32(NtGdiDdDDISetProcessSchedulingPriorityClass       ,   NONE,   NONE,   NONE,   NONE, 0x12e8, 0x1188, 0x130b, 0x118d, 0x130b, 0x000011d4, 0x11d4, 0x1314, 0x000011ec, 0x11ec, 0x1320, 0x0000121a, 0x121a)
GDI32(NtGdiDdDDISetQueuedLimit                          ,   NONE,   NONE,   NONE,   NONE, 0x12ec, 0x1189, 0x130f, 0x118e, 0x1360, 0x000011d5, 0x11d5, 0x1369, 0x000011ed, 0x11ed, 0x1379, 0x0000121b, 0x121b)
GDI32(NtGdiDdDDISetStablePowerState                     ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE, 0x13ad, 0x0000121c, 0x121c)
GDI32(NtGdiDdDDISetStereoEnabled                        ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE, 0x1367, 0x000011d6, 0x11d6, 0x1370, 0x000011ee, 0x11ee, 0x1382, 0x0000121d, 0x121d)
GDI32(NtGdiDdDDISetSyncRefreshCountWaitTarget           ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE, 0x137e, 0x0000121e, 0x121e)
GDI32(NtGdiDdDDISetVidPnSourceHwProtection              ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE, 0x13b1, 0x0000121f, 0x121f)
GDI32(NtGdiDdDDISetVidPnSourceOwner                     ,   NONE,   NONE,   NONE,   NONE, 0x12db, 0x118a, 0x12fd, 0x118f, 0x131a, 0x000011d7, 0x11d7, 0x1323, 0x000011ef, 0x11ef, 0x132f, 0x00001220, 0x1220)
GDI32(NtGdiDdDDISetVidPnSourceOwner1                    ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE, 0x1319, 0x000011d8, 0x11d8, 0x1322, 0x000011f0, 0x11f0, 0x132e, 0x00001221, 0x1221)
GDI32(NtGdiDdDDIShareObjects                            ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE, 0x133c, 0x000011d9, 0x11d9, 0x1345, 0x000011f1, 0x11f1, 0x1355, 0x00001222, 0x1222)
GDI32(NtGdiDdDDISharedPrimaryLockNotification           ,   NONE,   NONE,   NONE,   NONE, 0x12f4, 0x118b, 0x1317, 0x1190, 0x1358, 0x000011da, 0x11da, 0x1361, 0x000011f2, 0x11f2, 0x1371, 0x00001223, 0x1223)
GDI32(NtGdiDdDDISharedPrimaryUnLockNotification         ,   NONE,   NONE,   NONE,   NONE, 0x12f5, 0x118c, 0x1318, 0x1191, 0x1357, 0x000011db, 0x11db, 0x1360, 0x000011f3, 0x11f3, 0x1370, 0x00001224, 0x1224)
GDI32(NtGdiDdDDISignalSynchronizationObject             ,   NONE,   NONE,   NONE,   NONE, 0x12cb, 0x118d, 0x12ed, 0x1192, 0x132c, 0x000011dc, 0x11dc, 0x1335, 0x000011f4, 0x11f4, 0x1342, 0x00001225, 0x1225)
GDI32(NtGdiDdDDISignalSynchronizationObjectFromCpu      ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE, 0x1388, 0x00001226, 0x1226)
GDI32(NtGdiDdDDISignalSynchronizationObjectFromGpu      ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE, 0x138a, 0x00001227, 0x1227)
GDI32(NtGdiDdDDISignalSynchronizationObjectFromGpu2     ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE, 0x138b, 0x00001228, 0x1228)
GDI32(NtGdiDdDDISubmitCommand                           ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE, 0x1397, 0x00001229, 0x1229)
GDI32(NtGdiDdDDIUnlock                                  ,   NONE,   NONE,   NONE,   NONE, 0x12cf, 0x118e, 0x12f1, 0x1193, 0x1328, 0x000011dd, 0x11dd, 0x1331, 0x000011f5, 0x11f5, 0x133e, 0x0000122a, 0x122a)
GDI32(NtGdiDdDDIUnlock2                                 ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE, 0x138f, 0x0000122b, 0x122b)
GDI32(NtGdiDdDDIUnpinDirectFlipResources                ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE, 0x1362, 0x000011de, 0x11de, 0x136b, 0x000011f6, 0x11f6, 0x137b, 0x0000122c, 0x122c)
GDI32(NtGdiDdDDIUpdateGpuVirtualAddress                 ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE, 0x1395, 0x0000122d, 0x122d)
GDI32(NtGdiDdDDIUpdateOverlay                           ,   NONE,   NONE,   NONE,   NONE, 0x12de, 0x118f, 0x1301, 0x1194, 0x1315, 0x000011df, 0x11df, 0x131e, 0x000011f7, 0x11f7, 0x132a, 0x0000122e, 0x122e)
GDI32(NtGdiDdDDIWaitForIdle                             ,   NONE,   NONE,   NONE,   NONE, 0x12f0, 0x1190, 0x1313, 0x1195, 0x135c, 0x000011e0, 0x11e0, 0x1365, 0x000011f8, 0x11f8, 0x1375, 0x0000122f, 0x122f)
GDI32(NtGdiDdDDIWaitForSynchronizationObject            ,   NONE,   NONE,   NONE,   NONE, 0x12ca, 0x1191, 0x12ec, 0x1196, 0x132d, 0x000011e1, 0x11e1, 0x1336, 0x000011f9, 0x11f9, 0x1343, 0x00001230, 0x1230)
GDI32(NtGdiDdDDIWaitForSynchronizationObjectFromCpu     ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE, 0x1387, 0x00001231, 0x1231)
GDI32(NtGdiDdDDIWaitForSynchronizationObjectFromGpu     ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE, 0x1389, 0x00001232, 0x1232)
GDI32(NtGdiDdDDIWaitForVerticalBlankEvent               ,   NONE,   NONE,   NONE,   NONE, 0x12e1, 0x1192, 0x1304, 0x1197, 0x1312, 0x000011e2, 0x11e2, 0x131b, 0x000011fa, 0x11fa, 0x1327, 0x00001233, 0x1233)
GDI32(NtGdiDdDDIWaitForVerticalBlankEvent2              ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE, 0x1363, 0x000011e3, 0x11e3, 0x136c, 0x000011fb, 0x11fb, 0x137c, 0x00001234, 0x1234)
GDI32(NtGdiDdDeleteDirectDrawObject                     ,   NONE, 0x1040, 0x1040, 0x1152, 0x1042, 0x1193, 0x1043, 0x1198, 0x10f7, 0x000411e4, 0x11e4, 0x10f9, 0x000411fc, 0x11fc, 0x10fa, 0x00041235, 0x1235)
GDI32(NtGdiDdDeleteSurfaceObject                        ,   NONE, 0x1041, 0x1041, 0x10a2, 0x1043, 0x10a3, 0x1044, 0x10a1, 0x10f6, 0x000410a1, 0x10a1, 0x10f8, 0x000410a2, 0x10a2, 0x10f9, 0x000410a3, 0x10a3)
GDI32(NtGdiDdDestroyD3DBuffer                           ,   NONE, 0x1044, 0x1044, 0x1153, 0x1046, 0x1194, 0x1047, 0x1199, 0x10f3, 0x000411e5, 0x11e5, 0x10f5, 0x000411fd, 0x11fd, 0x10f6, 0x00041236, 0x1236)
GDI32(NtGdiDdDestroyFullscreenSprite                    ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE, 0x1330, 0x119a, 0x1375, 0x000011e6, 0x11e6, 0x1387, 0x000011fe, 0x11fe, 0x13c0, 0x00001237, 0x1237)
GDI32(NtGdiDdDestroyMoComp                              ,   NONE, 0x1042, 0x1042, 0x1154, 0x1044, 0x1195, 0x1045, 0x119b, 0x10f5, 0x000011e7, 0x11e7, 0x10f7, 0x000011ff, 0x11ff, 0x10f8, 0x00001238, 0x1238)
GDI32(NtGdiDdDestroySurface                             ,   NONE, 0x1043, 0x1043, 0x10a9, 0x1045, 0x10aa, 0x1046, 0x10a5, 0x10f4, 0x000510a5, 0x10a5, 0x10f6, 0x000510a6, 0x10a6, 0x10f7, 0x000510a7, 0x10a7)
GDI32(NtGdiDdEndMoCompFrame                             ,   NONE, 0x1045, 0x1045, 0x1155, 0x1047, 0x1196, 0x1048, 0x119c, 0x10f2, 0x000011e8, 0x11e8, 0x10f4, 0x00001200, 0x1200, 0x10f5, 0x00001239, 0x1239)
GDI32(NtGdiDdFlip                                       ,   NONE, 0x1046, 0x1046, 0x1156, 0x1048, 0x1197, 0x1049, 0x119d, 0x10f1, 0x000011e9, 0x11e9, 0x10f3, 0x00001201, 0x1201, 0x10f4, 0x0000123a, 0x123a)
GDI32(NtGdiDdFlipToGDISurface                           ,   NONE, 0x1047, 0x1047, 0x1157, 0x1049, 0x1198, 0x104a, 0x119e, 0x10f0, 0x000011ea, 0x11ea, 0x10f2, 0x00001202, 0x1202, 0x10f3, 0x0000123b, 0x123b)
GDI32(NtGdiDdGetAvailDriverMemory                       ,   NONE, 0x1048, 0x1048, 0x1158, 0x104a, 0x1199, 0x104b, 0x119f, 0x10ef, 0x000011eb, 0x11eb, 0x10f1, 0x00001203, 0x1203, 0x10f2, 0x0000123c, 0x123c)
GDI32(NtGdiDdGetBltStatus                               ,   NONE, 0x1049, 0x1049, 0x1159, 0x104b, 0x119a, 0x104c, 0x11a0, 0x10ee, 0x000011ec, 0x11ec, 0x10f0, 0x00001204, 0x1204, 0x10f1, 0x0000123d, 0x123d)
GDI32(NtGdiDdGetDC                                      ,   NONE, 0x104a, 0x104a, 0x115a, 0x104c, 0x119b, 0x104d, 0x11a1, 0x10ed, 0x000511ed, 0x11ed, 0x10ef, 0x00051205, 0x1205, 0x10f0, 0x0005123e, 0x123e)
GDI32(NtGdiDdGetDriverInfo                              ,   NONE, 0x104b, 0x104b, 0x115b, 0x104d, 0x119c, 0x104e, 0x11a2, 0x10ec, 0x000011ee, 0x11ee, 0x10ee, 0x00001206, 0x1206, 0x10ef, 0x0000123f, 0x123f)
GDI32(NtGdiDdGetDriverState                             ,   NONE, 0x1032, 0x1032, 0x115c, 0x1034, 0x119d, 0x1035, 0x11a3, 0x1105, 0x000011ef, 0x11ef, 0x1107, 0x00001207, 0x1207, 0x1108, 0x00001240, 0x1240)
GDI32(NtGdiDdGetDxHandle                                ,   NONE, 0x104c, 0x104c, 0x115d, 0x104e, 0x119e, 0x104f, 0x11a4, 0x10eb, 0x000a11f0, 0x11f0, 0x10ed, 0x000a1208, 0x1208, 0x10ee, 0x000a1241, 0x1241)
GDI32(NtGdiDdGetFlipStatus                              ,   NONE, 0x104d, 0x104d, 0x115e, 0x104f, 0x119f, 0x1050, 0x11a5, 0x10ea, 0x000011f1, 0x11f1, 0x10ec, 0x00001209, 0x1209, 0x10ed, 0x00001242, 0x1242)
GDI32(NtGdiDdGetInternalMoCompInfo                      ,   NONE, 0x104e, 0x104e, 0x115f, 0x1050, 0x11a0, 0x1051, 0x11a6, 0x10e9, 0x000011f2, 0x11f2, 0x10eb, 0x0000120a, 0x120a, 0x10ec, 0x00001243, 0x1243)
GDI32(NtGdiDdGetMoCompBuffInfo                          ,   NONE, 0x104f, 0x104f, 0x1160, 0x1051, 0x11a1, 0x1052, 0x11a7, 0x10e8, 0x000011f3, 0x11f3, 0x10ea, 0x0000120b, 0x120b, 0x10eb, 0x00001244, 0x1244)
GDI32(NtGdiDdGetMoCompFormats                           ,   NONE, 0x1051, 0x1051, 0x1161, 0x1053, 0x11a2, 0x1054, 0x11a8, 0x10e6, 0x000011f4, 0x11f4, 0x10e8, 0x0000120c, 0x120c, 0x10e9, 0x00001245, 0x1245)
GDI32(NtGdiDdGetMoCompGuids                             ,   NONE, 0x1050, 0x1050, 0x1162, 0x1052, 0x11a3, 0x1053, 0x11a9, 0x10e7, 0x000011f5, 0x11f5, 0x10e9, 0x0000120d, 0x120d, 0x10ea, 0x00001246, 0x1246)
GDI32(NtGdiDdGetScanLine                                ,   NONE, 0x1052, 0x1052, 0x1163, 0x1054, 0x11a4, 0x1055, 0x11aa, 0x10e5, 0x000011f6, 0x11f6, 0x10e7, 0x0000120e, 0x120e, 0x10e8, 0x00001247, 0x1247)
GDI32(NtGdiDdLock                                       ,   NONE, 0x1053, 0x1053, 0x1164, 0x1055, 0x11a5, 0x1056, 0x11ab, 0x10e4, 0x000011f7, 0x11f7, 0x10e6, 0x0000120f, 0x120f, 0x10e7, 0x00001248, 0x1248)
GDI32(NtGdiDdLockD3D                                    ,   NONE, 0x1054, 0x1054, 0x10c9, 0x1056, 0x10ca, 0x1057, 0x10c5, 0x10e3, 0x000010c5, 0x10c5, 0x10e5, 0x000010c6, 0x10c6, 0x10e6, 0x000010c7, 0x10c7)
GDI32(NtGdiDdNotifyFullscreenSpriteUpdate               ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE, 0x132f, 0x11ac, 0x1376, 0x000011f8, 0x11f8, 0x1388, 0x00001210, 0x1210, 0x13c1, 0x00001249, 0x1249)
GDI32(NtGdiDdQueryDirectDrawObject                      ,   NONE, 0x1055, 0x1055, 0x1165, 0x1057, 0x11a6, 0x1058, 0x11ad, 0x10e2, 0x000011f9, 0x11f9, 0x10e4, 0x00001211, 0x1211, 0x10e5, 0x0000124a, 0x124a)
GDI32(NtGdiDdQueryMoCompStatus                          ,   NONE, 0x1056, 0x1056, 0x1166, 0x1058, 0x11a7, 0x1059, 0x11ae, 0x10e1, 0x000011fa, 0x11fa, 0x10e3, 0x00001212, 0x1212, 0x10e4, 0x0000124b, 0x124b)
GDI32(NtGdiDdQueryVisRgnUniqueness                      ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE, 0x1331, 0x11af, 0x1374, 0x000011fb, 0x11fb, 0x1386, 0x00001213, 0x1213, 0x13bf, 0x0000124c, 0x124c)
GDI32(NtGdiDdReenableDirectDrawObject                   ,   NONE, 0x1057, 0x1057, 0x1167, 0x1059, 0x11a8, 0x105a, 0x11b0, 0x10e0, 0x000511fc, 0x11fc, 0x10e2, 0x00051214, 0x1214, 0x10e3, 0x0005124d, 0x124d)
GDI32(NtGdiDdReleaseDC                                  ,   NONE, 0x1058, 0x1058, 0x1168, 0x105a, 0x11a9, 0x105b, 0x11b1, 0x10df, 0x000411fd, 0x11fd, 0x10e1, 0x00041215, 0x1215, 0x10e2, 0x0004124e, 0x124e)
GDI32(NtGdiDdRenderMoComp                               ,   NONE, 0x1059, 0x1059, 0x1169, 0x105b, 0x11aa, 0x105c, 0x11b2, 0x10de, 0x000011fe, 0x11fe, 0x10e0, 0x00001216, 0x1216, 0x10e1, 0x0000124f, 0x124f)
GDI32(NtGdiDdResetVisrgn                                ,   NONE, 0x105a, 0x105a, 0x10ae, 0x105c, 0x10af, 0x105d, 0x10aa, 0x10dd, 0x000910aa, 0x10aa, 0x10df, 0x000910ab, 0x10ab, 0x10e0, 0x000910ac, 0x10ac)
GDI32(NtGdiDdSetColorKey                                ,   NONE, 0x105b, 0x105b, 0x116a, 0x105d, 0x11ab, 0x105e, 0x11b3, 0x10dc, 0x000011ff, 0x11ff, 0x10de, 0x00001217, 0x1217, 0x10df, 0x00001250, 0x1250)
GDI32(NtGdiDdSetExclusiveMode                           ,   NONE, 0x105c, 0x105c, 0x116b, 0x105e, 0x11ac, 0x105f, 0x11b4, 0x10db, 0x00001200, 0x1200, 0x10dd, 0x00001218, 0x1218, 0x10de, 0x00001251, 0x1251)
GDI32(NtGdiDdSetGammaRamp                               ,   NONE, 0x105d, 0x105d, 0x116c, 0x105f, 0x11ad, 0x1060, 0x11b5, 0x10da, 0x000f1201, 0x1201, 0x10dc, 0x000f1219, 0x1219, 0x10dd, 0x000f1252, 0x1252)
GDI32(NtGdiDdSetOverlayPosition                         ,   NONE, 0x105f, 0x105f, 0x116d, 0x1061, 0x11ae, 0x1062, 0x11b6, 0x10d8, 0x00001202, 0x1202, 0x10da, 0x0000121a, 0x121a, 0x10db, 0x00001253, 0x1253)
GDI32(NtGdiDdUnattachSurface                            ,   NONE, 0x1060, 0x1060, 0x116e, 0x1062, 0x11af, 0x1063, 0x11b7, 0x10d7, 0x00051203, 0x1203, 0x10d9, 0x0005121b, 0x121b, 0x10da, 0x00051254, 0x1254)
GDI32(NtGdiDdUnlock                                     ,   NONE, 0x1061, 0x1061, 0x116f, 0x1063, 0x11b0, 0x1064, 0x11b8, 0x10d6, 0x00001204, 0x1204, 0x10d8, 0x0000121c, 0x121c, 0x10d9, 0x00001255, 0x1255)
GDI32(NtGdiDdUnlockD3D                                  ,   NONE, 0x1062, 0x1062, 0x10ca, 0x1064, 0x10cb, 0x1065, 0x10c6, 0x10d5, 0x000010c6, 0x10c6, 0x10d7, 0x000010c7, 0x10c7, 0x10d8, 0x000010c8, 0x10c8)
GDI32(NtGdiDdUpdateOverlay                              ,   NONE, 0x1063, 0x1063, 0x1170, 0x1065, 0x11b1, 0x1066, 0x11b9, 0x10d4, 0x00001205, 0x1205, 0x10d6, 0x0000121d, 0x121d, 0x10d7, 0x00001256, 0x1256)
GDI32(NtGdiDdWaitForVerticalBlank                       ,   NONE, 0x1064, 0x1064, 0x1171, 0x1066, 0x11b2, 0x1067, 0x11ba, 0x10d3, 0x00001206, 0x1206, 0x10d5, 0x0000121e, 0x121e, 0x10d6, 0x00001257, 0x1257)
GDI32(NtGdiDeleteClientObj                              ,   NONE, 0x1077, 0x1077, 0x1088, 0x1079, 0x1089, 0x107a, 0x1087, 0x10c0, 0x00031087, 0x1087, 0x10c2, 0x00031088, 0x1088, 0x10c3, 0x00031089, 0x1089)
GDI32(NtGdiDeleteColorSpace                             ,   NONE, 0x1078, 0x1078, 0x1125, 0x107a, 0x1126, 0x107b, 0x111d, 0x10bf, 0x0003111d, 0x111d, 0x10c1, 0x0003111e, 0x111e, 0x10c2, 0x0003111f, 0x111f)
GDI32(NtGdiDeleteColorTransform                         ,   NONE, 0x1079, 0x1079, 0x1172, 0x107b, 0x11b3, 0x107c, 0x11bb, 0x10be, 0x00081207, 0x1207, 0x10c0, 0x0008121f, 0x121f, 0x10c1, 0x00081258, 0x1258)
GDI32(NtGdiDeleteObjectApp                              ,   NONE, 0x107a, 0x107a, 0x1022, 0x107c, 0x1023, 0x107d, 0x1023, 0x10bd, 0x00031024, 0x1024, 0x10bf, 0x00031025, 0x1025, 0x10c0, 0x00031026, 0x1026)
GDI32(NtGdiDescribePixelFormat                          ,   NONE, 0x107b, 0x107b, 0x1173, 0x107d, 0x11b4, 0x107e, 0x11bc, 0x10bc, 0x00141208, 0x1208, 0x10be, 0x00141220, 0x1220, 0x10bf, 0x00141259, 0x1259)
GDI32(NtGdiDestroyOPMProtectedOutput                    ,   NONE,   NONE,   NONE,   NONE, 0x107e, 0x11b5, 0x107f, 0x11bd, 0x10bb, 0x00001209, 0x1209, 0x10bd, 0x00001221, 0x1221, 0x10be, 0x0000125a, 0x125a)
GDI32(NtGdiDestroyPhysicalMonitor                       ,   NONE,   NONE,   NONE,   NONE, 0x12fb, 0x11b6, 0x1327, 0x11be, 0x136a, 0x0000120a, 0x120a, 0x137c, 0x00001222, 0x1222, 0x13b5, 0x0000125b, 0x125b)
GDI32(NtGdiDoBanding                                    ,   NONE, 0x107d, 0x107d, 0x1174, 0x1080, 0x11b7, 0x1081, 0x11bf,   NONE, 0x0014120b, 0x120b, 0x10bb, 0x00141223, 0x1223, 0x10bc, 0x0014125c, 0x125c)
GDI32(NtGdiDoPalette                                    ,   NONE, 0x107e, 0x107e, 0x1046, 0x1081, 0x1047, 0x1082, 0x1047, 0x10b8, 0x00001048, 0x1048, 0x10ba, 0x00001049, 0x1049, 0x10bb, 0x0000104a, 0x104a)
GDI32(NtGdiDrawEscape                                   ,   NONE, 0x107f, 0x107f, 0x1175, 0x1082, 0x11b8, 0x1083, 0x11c0, 0x10b7, 0x0014120c, 0x120c, 0x10b9, 0x00141224, 0x1224, 0x10ba, 0x0014125d, 0x125d)
GDI32(NtGdiDrawStream                                   ,   NONE, 0x129a, 0x1296, 0x1061, 0x12bb, 0x1062, 0x12db, 0x1062, 0x1304, 0x00001062, 0x1062, 0x130d, 0x00001063, 0x1063, 0x1319, 0x00001064, 0x1064)
GDI32(NtGdiDvpAcquireNotification                       ,   NONE, 0x1074, 0x1074, 0x1176, 0x1076, 0x11b9, 0x1077, 0x11c1, 0x10c3, 0x0000120d, 0x120d, 0x10c5, 0x00001225, 0x1225, 0x10c6, 0x0000125e, 0x125e)
GDI32(NtGdiDvpCanCreateVideoPort                        ,   NONE, 0x1065, 0x1065, 0x1177, 0x1067, 0x11ba, 0x1068, 0x11c2, 0x10d2, 0x0000120e, 0x120e, 0x10d4, 0x00001226, 0x1226, 0x10d5, 0x0000125f, 0x125f)
GDI32(NtGdiDvpColorControl                              ,   NONE, 0x1066, 0x1066, 0x1178, 0x1068, 0x11bb, 0x1069, 0x11c3, 0x10d1, 0x0000120f, 0x120f, 0x10d3, 0x00001227, 0x1227, 0x10d4, 0x00001260, 0x1260)
GDI32(NtGdiDvpCreateVideoPort                           ,   NONE, 0x1067, 0x1067, 0x1179, 0x1069, 0x11bc, 0x106a, 0x11c4, 0x10d0, 0x00001210, 0x1210, 0x10d2, 0x00001228, 0x1228, 0x10d3, 0x00001261, 0x1261)
GDI32(NtGdiDvpDestroyVideoPort                          ,   NONE, 0x1068, 0x1068, 0x117a, 0x106a, 0x11bd, 0x106b, 0x11c5, 0x10cf, 0x00001211, 0x1211, 0x10d1, 0x00001229, 0x1229, 0x10d2, 0x00001262, 0x1262)
GDI32(NtGdiDvpFlipVideoPort                             ,   NONE, 0x1069, 0x1069, 0x117b, 0x106b, 0x11be, 0x106c, 0x11c6, 0x10ce, 0x00001212, 0x1212, 0x10d0, 0x0000122a, 0x122a, 0x10d1, 0x00001263, 0x1263)
GDI32(NtGdiDvpGetVideoPortBandwidth                     ,   NONE, 0x106a, 0x106a, 0x117c, 0x106c, 0x11bf, 0x106d, 0x11c7, 0x10cd, 0x00001213, 0x1213, 0x10cf, 0x0000122b, 0x122b, 0x10d0, 0x00001264, 0x1264)
GDI32(NtGdiDvpGetVideoPortConnectInfo                   ,   NONE, 0x1070, 0x1070, 0x117d, 0x1072, 0x11c0, 0x1073, 0x11c8, 0x10c7, 0x00001214, 0x1214, 0x10c9, 0x0000122c, 0x122c, 0x10ca, 0x00001265, 0x1265)
GDI32(NtGdiDvpGetVideoPortField                         ,   NONE, 0x106b, 0x106b, 0x117e, 0x106d, 0x11c1, 0x106e, 0x11c9, 0x10cc, 0x00001215, 0x1215, 0x10ce, 0x0000122d, 0x122d, 0x10cf, 0x00001266, 0x1266)
GDI32(NtGdiDvpGetVideoPortFlipStatus                    ,   NONE, 0x106c, 0x106c, 0x117f, 0x106e, 0x11c2, 0x106f, 0x11ca, 0x10cb, 0x00001216, 0x1216, 0x10cd, 0x0000122e, 0x122e, 0x10ce, 0x00001267, 0x1267)
GDI32(NtGdiDvpGetVideoPortInputFormats                  ,   NONE, 0x106d, 0x106d, 0x1180, 0x106f, 0x11c3, 0x1070, 0x11cb, 0x10ca, 0x00001217, 0x1217, 0x10cc, 0x0000122f, 0x122f, 0x10cd, 0x00001268, 0x1268)
GDI32(NtGdiDvpGetVideoPortLine                          ,   NONE, 0x106e, 0x106e, 0x1181, 0x1070, 0x11c4, 0x1071, 0x11cc, 0x10c9, 0x00001218, 0x1218, 0x10cb, 0x00001230, 0x1230, 0x10cc, 0x00001269, 0x1269)
GDI32(NtGdiDvpGetVideoPortOutputFormats                 ,   NONE, 0x106f, 0x106f, 0x1182, 0x1071, 0x11c5, 0x1072, 0x11cd, 0x10c8, 0x00001219, 0x1219, 0x10ca, 0x00001231, 0x1231, 0x10cb, 0x0000126a, 0x126a)
GDI32(NtGdiDvpGetVideoSignalStatus                      ,   NONE, 0x1071, 0x1071, 0x1183, 0x1073, 0x11c6, 0x1074, 0x11ce, 0x10c6, 0x0000121a, 0x121a, 0x10c8, 0x00001232, 0x1232, 0x10c9, 0x0000126b, 0x126b)
GDI32(NtGdiDvpReleaseNotification                       ,   NONE, 0x1075, 0x1075, 0x1184, 0x1077, 0x11c7, 0x1078, 0x11cf, 0x10c2, 0x0000121b, 0x121b, 0x10c4, 0x00001233, 0x1233, 0x10c5, 0x0000126c, 0x126c)
GDI32(NtGdiDvpUpdateVideoPort                           ,   NONE, 0x1072, 0x1072, 0x1185, 0x1074, 0x11c8, 0x1075, 0x11d0, 0x10c5, 0x0000121c, 0x121c, 0x10c7, 0x00001234, 0x1234, 0x10c8, 0x0000126d, 0x126d)
GDI32(NtGdiDvpWaitForVideoPortSync                      ,   NONE, 0x1073, 0x1073, 0x1186, 0x1075, 0x11c9, 0x1076, 0x11d1, 0x10c4, 0x0000121d, 0x121d, 0x10c6, 0x00001235, 0x1235, 0x10c7, 0x0000126e, 0x126e)
GDI32(NtGdiDwmCreatedBitmapRemotingOutput               ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE, 0x1307, 0x0000121e, 0x121e, 0x1310, 0x00001236, 0x1236, 0x131c, 0x0000126f, 0x126f)
GDI32(NtGdiDwmGetDirtyRgn                               ,   NONE,   NONE,   NONE,   NONE, 0x12bc, 0x11ca,   NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE)
GDI32(NtGdiDwmGetSurfaceData                            ,   NONE,   NONE,   NONE,   NONE, 0x12bd, 0x11cb,   NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE)
GDI32(NtGdiDxgGenericThunk                              ,   NONE, 0x1076, 0x1076, 0x1187, 0x1078, 0x11cc, 0x1079, 0x11d2, 0x10c1, 0x0000121f, 0x121f, 0x10c3, 0x00001237, 0x1237, 0x10c4, 0x00001270, 0x1270)
GDI32(NtGdiEllipse                                      , 0x11e0, 0x1080, 0x1080, 0x1188, 0x1083, 0x11cd, 0x1084, 0x11d3, 0x10b6, 0x00001220, 0x1220, 0x10b8, 0x00001238, 0x1238, 0x10b9, 0x00001271, 0x1271)
GDI32(NtGdiEnableEudc                                   , 0x107c, 0x1081, 0x1081, 0x1189, 0x1084, 0x11ce, 0x1085, 0x11d4, 0x10b5, 0x00041221, 0x1221, 0x10b7, 0x00041239, 0x1239, 0x10b8, 0x00041272, 0x1272)
GDI32(NtGdiEndDoc                                       ,   NONE, 0x1082, 0x1082, 0x118a, 0x1085, 0x11cf, 0x1086, 0x11d5, 0x10b4, 0x00031222, 0x1222, 0x10b6, 0x0003123a, 0x123a, 0x10b7, 0x00031273, 0x1273)
GDI32(NtGdiEndGdiRendering                              ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE, 0x1087, 0x11d6, 0x10b3, 0x00001223, 0x1223, 0x10b5, 0x0000123b, 0x123b, 0x10b6, 0x00001274, 0x1274)
GDI32(NtGdiEndPage                                      ,   NONE, 0x1083, 0x1083, 0x118b, 0x1086, 0x11d0, 0x1088, 0x11d7, 0x10b2, 0x00031224, 0x1224, 0x10b4, 0x0003123c, 0x123c, 0x10b5, 0x00031275, 0x1275)
GDI32(NtGdiEndPath                                      ,   NONE, 0x1084, 0x1084, 0x1110, 0x1087, 0x1111, 0x1089, 0x110a, 0x10b1, 0x0003110a, 0x110a, 0x10b3, 0x0003110b, 0x110b, 0x10b4, 0x0003110c, 0x110c)
GDI32(NtGdiEngAlphaBlend                                , 0x1254, 0x126c, 0x1268, 0x118c, 0x128d, 0x11d1, 0x12ad, 0x11d8, 0x12a7, 0x00001225, 0x1225, 0x12aa, 0x0000123d, 0x123d, 0x12b4, 0x00001276, 0x1276)
GDI32(NtGdiEngAssociateSurface                          , 0x123f, 0x1257, 0x1253, 0x118d, 0x1278, 0x11d2, 0x1298, 0x11d9, 0x12bc, 0x00001226, 0x1226, 0x12bf, 0x0000123e, 0x123e, 0x12c9, 0x00001277, 0x1277)
GDI32(NtGdiEngBitBlt                                    , 0x124b, 0x1263, 0x125f, 0x118e, 0x1284, 0x11d3, 0x12a4, 0x11da, 0x12b0, 0x00001227, 0x1227, 0x12b3, 0x0000123f, 0x123f, 0x12bd, 0x00001278, 0x1278)
GDI32(NtGdiEngCheckAbort                                , 0x127b, 0x1293, 0x128f, 0x118f, 0x12b4, 0x11d4, 0x12d4, 0x11db, 0x12fb, 0x00001228, 0x1228, 0x1304, 0x00001240, 0x1240, 0x1310, 0x00001279, 0x1279)
GDI32(NtGdiEngComputeGlyphSet                           ,   NONE, 0x125c, 0x1258, 0x1190, 0x127d, 0x11d5, 0x129d, 0x11dc, 0x12b7, 0x00001229, 0x1229, 0x12ba, 0x00001241, 0x1241, 0x12c4, 0x0000127a, 0x127a)
GDI32(NtGdiEngCopyBits                                  , 0x1245, 0x125d, 0x1259, 0x1191, 0x127e, 0x11d6, 0x129e, 0x11dd, 0x12b6, 0x0000122a, 0x122a, 0x12b9, 0x00001242, 0x1242, 0x12c3, 0x0000127b, 0x127b)
GDI32(NtGdiEngCreateBitmap                              , 0x1240, 0x1258, 0x1254, 0x1192, 0x1279, 0x11d7, 0x1299, 0x11de, 0x12bb, 0x0000122b, 0x122b, 0x12be, 0x00001243, 0x1243, 0x12c8, 0x0000127c, 0x127c)
GDI32(NtGdiEngCreateClip                                , 0x1260, 0x1278, 0x1274, 0x1193, 0x1299, 0x11d8, 0x12b9, 0x11df, 0x12de, 0x0000122c, 0x122c, 0x12e7, 0x00001244, 0x1244, 0x12f3, 0x0000127d, 0x127d)
GDI32(NtGdiEngCreateDeviceBitmap                        , 0x1242, 0x125a, 0x1256, 0x1194, 0x127b, 0x11d9, 0x129b, 0x11e0, 0x12b9, 0x0000122d, 0x122d, 0x12bc, 0x00001245, 0x1245, 0x12c6, 0x0000127e, 0x127e)
GDI32(NtGdiEngCreateDeviceSurface                       , 0x1241, 0x1259, 0x1255, 0x1195, 0x127a, 0x11da, 0x129a, 0x11e1, 0x12ba, 0x0000122e, 0x122e, 0x12bd, 0x00001246, 0x1246, 0x12c7, 0x0000127f, 0x127f)
GDI32(NtGdiEngCreatePalette                             , 0x1243, 0x125b, 0x1257, 0x1196, 0x127c, 0x11db, 0x129c, 0x11e2, 0x12b8, 0x0000122f, 0x122f, 0x12bb, 0x00001247, 0x1247, 0x12c5, 0x00001280, 0x1280)
GDI32(NtGdiEngDeleteClip                                , 0x1261, 0x1279, 0x1275, 0x1197, 0x129a, 0x11dc, 0x12ba, 0x11e3, 0x12dd, 0x00001230, 0x1230, 0x12e6, 0x00001248, 0x1248, 0x12f2, 0x00001281, 0x1281)
GDI32(NtGdiEngDeletePalette                             , 0x1246, 0x125e, 0x125a, 0x1198, 0x127f, 0x11dd, 0x129f, 0x11e4, 0x12b5, 0x00001231, 0x1231, 0x12b8, 0x00001249, 0x1249, 0x12c2, 0x00001282, 0x1282)
GDI32(NtGdiEngDeletePath                                , 0x125f, 0x1277, 0x1273, 0x1199, 0x1298, 0x11de, 0x12b8, 0x11e5, 0x12df, 0x00001232, 0x1232, 0x12e8, 0x0000124a, 0x124a, 0x12f4, 0x00001283, 0x1283)
GDI32(NtGdiEngDeleteSurface                             , 0x1247, 0x125f, 0x125b, 0x119a, 0x1280, 0x11df, 0x12a0, 0x11e6, 0x12b4, 0x00001233, 0x1233, 0x12b7, 0x0000124b, 0x124b, 0x12c1, 0x00001284, 0x1284)
GDI32(NtGdiEngEraseSurface                              , 0x1248, 0x1260, 0x125c, 0x119b, 0x1281, 0x11e0, 0x12a1, 0x11e7, 0x12b3, 0x00001234, 0x1234, 0x12b6, 0x0000124c, 0x124c, 0x12c0, 0x00001285, 0x1285)
GDI32(NtGdiEngFillPath                                  , 0x1250, 0x1268, 0x1264, 0x119c, 0x1289, 0x11e1, 0x12a9, 0x11e8, 0x12ab, 0x00001235, 0x1235, 0x12ae, 0x0000124d, 0x124d, 0x12b8, 0x00001286, 0x1286)
GDI32(NtGdiEngGradientFill                              , 0x1255, 0x126d, 0x1269, 0x119d, 0x128e, 0x11e2, 0x12ae, 0x11e9, 0x12a6, 0x00001236, 0x1236, 0x12a9, 0x0000124e, 0x124e, 0x12b3, 0x00001287, 0x1287)
GDI32(NtGdiEngLineTo                                    , 0x1253, 0x126b, 0x1267, 0x119e, 0x128c, 0x11e3, 0x12ac, 0x11ea, 0x12a8, 0x00001237, 0x1237, 0x12ab, 0x0000124f, 0x124f, 0x12b5, 0x00001288, 0x1288)
GDI32(NtGdiEngLockSurface                               , 0x124a, 0x1262, 0x125e, 0x119f, 0x1283, 0x11e4, 0x12a3, 0x11eb, 0x12b1, 0x00001238, 0x1238, 0x12b4, 0x00001250, 0x1250, 0x12be, 0x00001289, 0x1289)
GDI32(NtGdiEngMarkBandingSurface                        , 0x124e, 0x1266, 0x1262, 0x11a0, 0x1287, 0x11e5, 0x12a7, 0x11ec, 0x12ad, 0x00001239, 0x1239, 0x12b0, 0x00001251, 0x1251, 0x12ba, 0x0000128a, 0x128a)
GDI32(NtGdiEngPaint                                     , 0x1252, 0x126a, 0x1266, 0x11a1, 0x128b, 0x11e6, 0x12ab, 0x11ed, 0x12a9, 0x0000123a, 0x123a, 0x12ac, 0x00001252, 0x1252, 0x12b6, 0x0000128b, 0x128b)
GDI32(NtGdiEngPlgBlt                                    , 0x124d, 0x1265, 0x1261, 0x11a2, 0x1286, 0x11e7, 0x12a6, 0x11ee, 0x12ae, 0x0000123b, 0x123b, 0x12b1, 0x00001253, 0x1253, 0x12bb, 0x0000128c, 0x128c)
GDI32(NtGdiEngStretchBlt                                , 0x124c, 0x1264, 0x1260, 0x11a3, 0x1285, 0x11e8, 0x12a5, 0x11ef, 0x12af, 0x0000123c, 0x123c, 0x12b2, 0x00001254, 0x1254, 0x12bc, 0x0000128d, 0x128d)
GDI32(NtGdiEngStretchBltROP                             , 0x1258, 0x1270, 0x126c, 0x11a4, 0x1291, 0x11e9, 0x12b1, 0x11f0, 0x12a3, 0x0000123d, 0x123d, 0x12a6, 0x00001255, 0x1255, 0x12b0, 0x0000128e, 0x128e)
GDI32(NtGdiEngStrokeAndFillPath                         , 0x1251, 0x1269, 0x1265, 0x11a5, 0x128a, 0x11ea, 0x12aa, 0x11f1, 0x12aa, 0x0000123e, 0x123e, 0x12ad, 0x00001256, 0x1256, 0x12b7, 0x0000128f, 0x128f)
GDI32(NtGdiEngStrokePath                                , 0x124f, 0x1267, 0x1263, 0x11a6, 0x1288, 0x11eb, 0x12a8, 0x11f2, 0x12ac, 0x0000123f, 0x123f, 0x12af, 0x00001257, 0x1257, 0x12b9, 0x00001290, 0x1290)
GDI32(NtGdiEngTextOut                                   , 0x1257, 0x126f, 0x126b, 0x11a7, 0x1290, 0x11ec, 0x12b0, 0x11f3, 0x12a4, 0x00001240, 0x1240, 0x12a7, 0x00001258, 0x1258, 0x12b1, 0x00001291, 0x1291)
GDI32(NtGdiEngTransparentBlt                            , 0x1256, 0x126e, 0x126a, 0x11a8, 0x128f, 0x11ed, 0x12af, 0x11f4, 0x12a5, 0x00001241, 0x1241, 0x12a8, 0x00001259, 0x1259, 0x12b2, 0x00001292, 0x1292)
GDI32(NtGdiEngUnlockSurface                             , 0x1249, 0x1261, 0x125d, 0x11a9, 0x1282, 0x11ee, 0x12a2, 0x11f5, 0x12b2, 0x00001242, 0x1242, 0x12b5, 0x0000125a, 0x125a, 0x12bf, 0x00001293, 0x1293)
GDI32(NtGdiEnumFontChunk                                ,   NONE, 0x1085, 0x1085, 0x10a5, 0x1088, 0x10a6,   NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE)
GDI32(NtGdiEnumFontClose                                ,   NONE, 0x1086, 0x1086, 0x10a3, 0x1089, 0x10a4,   NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE)
GDI32(NtGdiEnumFontOpen                                 ,   NONE, 0x1087, 0x1087, 0x10a4, 0x108a, 0x10a5,   NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE)
GDI32(NtGdiEnumFonts                                    ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE, 0x108a, 0x11f6, 0x10b0, 0x00001243, 0x1243, 0x10b2, 0x0000125b, 0x125b, 0x10b3, 0x00001294, 0x1294)
GDI32(NtGdiEnumObjects                                  ,   NONE, 0x1088, 0x1088, 0x11aa, 0x108b, 0x11ef, 0x108b, 0x11f7, 0x10af, 0x00001244, 0x1244, 0x10b1, 0x0000125c, 0x125c, 0x10b2, 0x00001295, 0x1295)
GDI32(NtGdiEqualRgn                                     ,   NONE, 0x1089, 0x1089, 0x1117, 0x108c, 0x1118, 0x108c, 0x1111, 0x10ae, 0x00051111, 0x1111, 0x10b0, 0x00051112, 0x1112, 0x10b1, 0x00051113, 0x1113)
GDI32(NtGdiEudcLoadUnloadLink                           ,   NONE, 0x108a, 0x108a, 0x11ab, 0x108d, 0x11f0, 0x108d, 0x11f8, 0x10ad, 0x00001245, 0x1245, 0x10af, 0x0000125d, 0x125d, 0x10b0, 0x00001296, 0x1296)
GDI32(NtGdiExcludeClipRect                              ,   NONE, 0x108b, 0x108b, 0x109a, 0x108e, 0x109b, 0x108e, 0x1099, 0x10ac, 0x00001099, 0x1099, 0x10ae, 0x0000109a, 0x109a, 0x10af, 0x0000109b, 0x109b)
GDI32(NtGdiExtCreatePen                                 ,   NONE, 0x108c, 0x108c, 0x10af, 0x108f, 0x10b0, 0x108f, 0x10ab, 0x10ab, 0x000010ab, 0x10ab, 0x10ad, 0x000010ac, 0x10ac, 0x10ae, 0x000010ad, 0x10ad)
GDI32(NtGdiExtCreateRegion                              ,   NONE, 0x108d, 0x108d, 0x108b, 0x1090, 0x108c, 0x1090, 0x108a, 0x10aa, 0x000a108a, 0x108a, 0x10ac, 0x000a108b, 0x108b, 0x10ad, 0x000a108c, 0x108c)
GDI32(NtGdiExtEscape                                    ,   NONE, 0x108e, 0x108e, 0x111c, 0x1091, 0x111d, 0x1091, 0x1116, 0x10a9, 0x00001116, 0x1116, 0x10ab, 0x00001117, 0x1117, 0x10ac, 0x00001118, 0x1118)
GDI32(NtGdiExtFloodFill                                 ,   NONE, 0x108f, 0x108f, 0x11ac, 0x1092, 0x11f1, 0x1092, 0x11f9, 0x10a8, 0x00001246, 0x1246, 0x10aa, 0x0000125e, 0x125e, 0x10ab, 0x00001297, 0x1297)
GDI32(NtGdiExtGetObjectW                                ,   NONE, 0x1090, 0x1090, 0x1051, 0x1093, 0x1052, 0x1093, 0x1052, 0x10a7, 0x00001053, 0x1053, 0x10a9, 0x00001054, 0x1054, 0x10aa, 0x00001055, 0x1055)
GDI32(NtGdiExtSelectClipRgn                             ,   NONE, 0x1091, 0x1091, 0x102d, 0x1094, 0x102e, 0x1094, 0x102e, 0x10a6, 0x0010102f, 0x102f, 0x10a8, 0x00101030, 0x1030, 0x10a9, 0x00101031, 0x1031)
GDI32(NtGdiExtTextOutW                                  ,   NONE, 0x1092, 0x1092, 0x1037, 0x1095, 0x1038, 0x1095, 0x1038, 0x10a5, 0x00001039, 0x1039, 0x10a7, 0x0000103a, 0x103a, 0x10a8, 0x0000103b, 0x103b)
GDI32(NtGdiFONTOBJ_cGetAllGlyphHandles                  , 0x126f, 0x1287, 0x1283, 0x11ad, 0x12a8, 0x11f2, 0x12c8, 0x11fa, 0x12ef, 0x00001247, 0x1247, 0x12f8, 0x0000125f, 0x125f, 0x1304, 0x00001298, 0x1298)
GDI32(NtGdiFONTOBJ_cGetGlyphs                           , 0x126a, 0x1282, 0x127e, 0x11ae, 0x12a3, 0x11f3, 0x12c3, 0x11fb, 0x12ec, 0x00001248, 0x1248, 0x12f5, 0x00001260, 0x1260, 0x1301, 0x00001299, 0x1299)
GDI32(NtGdiFONTOBJ_pQueryGlyphAttrs                     , 0x126d, 0x1285, 0x1281, 0x11af, 0x12a6, 0x11f4, 0x12c6, 0x11fc, 0x12e9, 0x00001249, 0x1249, 0x12f2, 0x00001261, 0x1261, 0x12fe, 0x0000129a, 0x129a)
GDI32(NtGdiFONTOBJ_pfdg                                 , 0x126c, 0x1284, 0x1280, 0x11b0, 0x12a5, 0x11f5, 0x12c5, 0x11fd, 0x12ea, 0x0000124a, 0x124a, 0x12f3, 0x00001262, 0x1262, 0x12ff, 0x0000129b, 0x129b)
GDI32(NtGdiFONTOBJ_pifi                                 , 0x126b, 0x1283, 0x127f, 0x11b1, 0x12a4, 0x11f6, 0x12c4, 0x11fe, 0x12eb, 0x0000124b, 0x124b, 0x12f4, 0x00001263, 0x1263, 0x1300, 0x0000129c, 0x129c)
GDI32(NtGdiFONTOBJ_pvTrueTypeFontFile                   , 0x126e, 0x1286, 0x1282, 0x11b2, 0x12a7, 0x11f7, 0x12c7, 0x11ff, 0x12f0, 0x0000124c, 0x124c, 0x12f9, 0x00001264, 0x1264, 0x1305, 0x0000129d, 0x129d)
GDI32(NtGdiFONTOBJ_pxoGetXform                          , 0x1269, 0x1281, 0x127d, 0x11b3, 0x12a2, 0x11f8, 0x12c2, 0x1200, 0x12ed, 0x0000124d, 0x124d, 0x12f6, 0x00001265, 0x1265, 0x1302, 0x0000129e, 0x129e)
GDI32(NtGdiFONTOBJ_vGetInfo                             , 0x1268, 0x1280, 0x127c, 0x11b4, 0x12a1, 0x11f9, 0x12c1, 0x1201, 0x12ee, 0x0000124e, 0x124e, 0x12f7, 0x00001266, 0x1266, 0x1303, 0x0000129f, 0x129f)
GDI32(NtGdiFillPath                                     ,   NONE, 0x1093, 0x1093, 0x1111, 0x1096, 0x1112, 0x1096, 0x110b, 0x10a4, 0x0003110b, 0x110b, 0x10a6, 0x0003110c, 0x110c, 0x10a7, 0x0003110d, 0x110d)
GDI32(NtGdiFillRgn                                      ,   NONE, 0x1094, 0x1094, 0x10d8, 0x1097, 0x10d9, 0x1097, 0x10d4, 0x10a3, 0x000010d4, 0x10d4, 0x10a5, 0x000010d5, 0x10d5, 0x10a6, 0x000010d6, 0x10d6)
GDI32(NtGdiFlattenPath                                  ,   NONE, 0x1095, 0x1095, 0x11b5, 0x1098, 0x11fa, 0x1098, 0x1202, 0x10a2, 0x0003124f, 0x124f, 0x10a4, 0x00031267, 0x1267, 0x10a5, 0x000312a0, 0x12a0)
GDI32(NtGdiFlush                                        ,   NONE, 0x1097, 0x1096, 0x1011, 0x1099, 0x1012, 0x1099, 0x1012, 0x10a1, 0x00011013, 0x1013, 0x10a3, 0x00011014, 0x1014, 0x10a4, 0x00011015, 0x1015)
GDI32(NtGdiFontIsLinked                                 , 0x100a, 0x100a, 0x100a, 0x11b6, 0x100a, 0x11fb, 0x100a, 0x1203, 0x1132, 0x00031250, 0x1250, 0x1134, 0x00031268, 0x1268, 0x1135, 0x000312a1, 0x12a1)
GDI32(NtGdiForceUFIMapping                              ,   NONE, 0x1098, 0x1097, 0x11b7, 0x109a, 0x11fc, 0x109a, 0x1204, 0x10a0, 0x00071251, 0x1251, 0x10a2, 0x00071269, 0x1269, 0x10a3, 0x000712a2, 0x12a2)
GDI32(NtGdiFrameRgn                                     ,   NONE, 0x1099, 0x1098, 0x11b8, 0x109b, 0x11fd, 0x109b, 0x1205, 0x109f, 0x00001252, 0x1252, 0x10a1, 0x0000126a, 0x126a, 0x10a2, 0x000012a3, 0x12a3)
GDI32(NtGdiFullscreenControl                            , 0x1096, 0x109a, 0x1099, 0x11b9, 0x109c, 0x11fe, 0x109c, 0x1206, 0x109e, 0x00001253, 0x1253, 0x10a0, 0x0000126b, 0x126b, 0x10a1, 0x000012a4, 0x12a4)
GDI32(NtGdiGetAndSetDCDword                             ,   NONE, 0x109b, 0x109a, 0x1067, 0x109d, 0x1068, 0x109d, 0x1068, 0x109d, 0x00141068, 0x1068, 0x109f, 0x00141069, 0x1069, 0x10a0, 0x0014106a, 0x106a)
GDI32(NtGdiGetAppClipBox                                ,   NONE, 0x109c, 0x109b, 0x1042, 0x109e, 0x1043, 0x109e, 0x1043, 0x109c, 0x00071044, 0x1044, 0x109e, 0x00071045, 0x1045, 0x109f, 0x00071046, 0x1046)
GDI32(NtGdiGetBitmapBits                                ,   NONE, 0x109d, 0x109c, 0x10e2, 0x109f, 0x10e3, 0x109f, 0x10de, 0x109b, 0x000a10de, 0x10de, 0x109d, 0x000a10df, 0x10df, 0x109e, 0x000a10e0, 0x10e0)
GDI32(NtGdiGetBitmapDimension                           ,   NONE, 0x109e, 0x109d, 0x1101, 0x10a0, 0x1102, 0x10a0, 0x10fc, 0x109a, 0x000510fc, 0x10fc, 0x109c, 0x000510fd, 0x10fd, 0x109d, 0x000510fe, 0x10fe)
GDI32(NtGdiGetBoundsRect                                ,   NONE, 0x109f, 0x109e, 0x11ba, 0x10a1, 0x11ff, 0x10a1, 0x1207, 0x1099, 0x000c1254, 0x1254, 0x109b, 0x000c126c, 0x126c, 0x109c, 0x000c12a5, 0x12a5)
GDI32(NtGdiGetCOPPCompatibleOPMInformation              ,   NONE,   NONE,   NONE,   NONE, 0x10ab, 0x1200, 0x10ab, 0x1208, 0x108f, 0x00001255, 0x1255, 0x1091, 0x0000126d, 0x126d, 0x1092, 0x000012a6, 0x12a6)
GDI32(NtGdiGetCertificate                               ,   NONE,   NONE,   NONE,   NONE, 0x10a2, 0x1201, 0x10a2, 0x1209, 0x1098, 0x00001256, 0x1256, 0x109a, 0x0000126e, 0x126e, 0x109b, 0x000012a7, 0x12a7)
GDI32(NtGdiGetCertificateSize                           ,   NONE,   NONE,   NONE,   NONE, 0x10a3, 0x1202, 0x10a3, 0x120a, 0x1097, 0x00001257, 0x1257, 0x1099, 0x0000126f, 0x126f, 0x109a, 0x000012a8, 0x12a8)
GDI32(NtGdiGetCharABCWidthsW                            ,   NONE, 0x10a0, 0x109f, 0x11bb, 0x10a4, 0x1203, 0x10a4, 0x120b, 0x1096, 0x00001258, 0x1258, 0x1098, 0x00001270, 0x1270, 0x1099, 0x000012a9, 0x12a9)
GDI32(NtGdiGetCharSet                                   ,   NONE, 0x10a2, 0x10a1, 0x1009, 0x10a6, 0x1009, 0x10a6, 0x1009, 0x1094, 0x0003100a, 0x100a, 0x1096, 0x0003100b, 0x100b, 0x1097, 0x0003100c, 0x100c)
GDI32(NtGdiGetCharWidthInfo                             , 0x10a0, 0x10a4, 0x10a3, 0x10d0, 0x10a8, 0x10d1, 0x10a8, 0x10cc, 0x1092, 0x000710cc, 0x10cc, 0x1094, 0x000710cd, 0x10cd, 0x1095, 0x000710ce, 0x10ce)
GDI32(NtGdiGetCharWidthW                                ,   NONE, 0x10a3, 0x10a2, 0x10cb, 0x10a7, 0x10cc, 0x10a7, 0x10c7, 0x1093, 0x000010c7, 0x10c7, 0x1095, 0x000010c8, 0x10c8, 0x1096, 0x000010c9, 0x10c9)
GDI32(NtGdiGetCharacterPlacementW                       ,   NONE, 0x10a1, 0x10a0, 0x11bc, 0x10a5, 0x1204, 0x10a5, 0x120c, 0x1095, 0x00001259, 0x1259, 0x1097, 0x00001271, 0x1271, 0x1098, 0x000012aa, 0x12aa)
GDI32(NtGdiGetCodePage                                  , 0x10b6,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE)
GDI32(NtGdiGetColorAdjustment                           ,   NONE, 0x10a5, 0x10a4, 0x11bd, 0x10a9, 0x1205, 0x10a9, 0x120d, 0x1091, 0x0007125a, 0x125a, 0x1093, 0x00071272, 0x1272, 0x1094, 0x000712ab, 0x12ab)
GDI32(NtGdiGetColorSpaceforBitmap                       ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE, 0x125b,   NONE,       NONE, 0x1273, 0x1093, 0x000412ac, 0x12ac)
GDI32(NtGdiGetCurrentDpiInfo                            ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE,   NONE, 0x1090, 0x00001274, 0x1274, 0x1091, 0x000012ad, 0x12ad)
GDI32(NtGdiGetDCDword                                   ,   NONE, 0x10a7, 0x10a6, 0x103e, 0x10ac, 0x103f, 0x10ac, 0x103f, 0x108e, 0x000c1040, 0x1040, 0x108f, 0x000c1041, 0x1041, 0x1090, 0x000c1042, 0x1042)
GDI32(NtGdiGetDCObject                                  ,   NONE, 0x10a9, 0x10a8, 0x1034, 0x10ae, 0x1035, 0x10ae, 0x1035, 0x108c, 0x00081036, 0x1036, 0x108d, 0x00081037, 0x1037, 0x108e, 0x00081038, 0x1038)
GDI32(NtGdiGetDCPoint                                   ,   NONE, 0x10aa, 0x10a9, 0x1073, 0x10af, 0x1074, 0x10af, 0x1073, 0x108b, 0x000c1073, 0x1073, 0x108c, 0x000c1074, 0x1074, 0x108d, 0x000c1075, 0x1075)
GDI32(NtGdiGetDCforBitmap                               ,   NONE, 0x10a8, 0x10a7, 0x109c, 0x10ad, 0x109d, 0x10ad, 0x109b, 0x108d, 0x0004109b, 0x109b, 0x108e, 0x0004109c, 0x109c, 0x108f, 0x0004109d, 0x109d)
GDI32(NtGdiGetDIBitsInternal                            ,   NONE, 0x10ae, 0x10ad, 0x1086, 0x10b3, 0x1087, 0x10b3, 0x1085, 0x1087, 0x00001085, 0x1085, 0x1088, 0x00001086, 0x1086, 0x1089, 0x00001087, 0x1087)
GDI32(NtGdiGetDeviceCaps                                ,   NONE, 0x10ab, 0x10aa, 0x11bf, 0x10b0, 0x1207, 0x10b0, 0x120f, 0x108a, 0x0008125c, 0x125c, 0x108b, 0x00081275, 0x1275, 0x108c, 0x000812ae, 0x12ae)
GDI32(NtGdiGetDeviceCapsAll                             ,   NONE, 0x10ad, 0x10ac, 0x11c0, 0x10b2, 0x1208, 0x10b2, 0x1210, 0x1088, 0x0007125d, 0x125d, 0x1089, 0x00071276, 0x1276, 0x108a, 0x000712af, 0x12af)
GDI32(NtGdiGetDeviceGammaRamp                           ,   NONE, 0x10ac, 0x10ab, 0x11c1, 0x10b1, 0x1209, 0x10b1, 0x1211, 0x1089, 0x0000125e, 0x125e, 0x108a, 0x00001277, 0x1277, 0x108b, 0x000012b0, 0x12b0)
GDI32(NtGdiGetDeviceWidth                               ,   NONE, 0x1117, 0x1116, 0x11c2, 0x111f, 0x120a, 0x1121, 0x1212, 0x1019, 0x0003125f, 0x125f, 0x101a, 0x00031278, 0x1278, 0x101b, 0x000312b1, 0x12b1)
GDI32(NtGdiGetDhpdev                                    ,   NONE, 0x1292, 0x128e, 0x11c3, 0x12b3, 0x120b, 0x12d3, 0x1213, 0x12fc, 0x00001260, 0x1260, 0x1305, 0x00001279, 0x1279, 0x1311, 0x000012b2, 0x12b2)
GDI32(NtGdiGetETM                                       ,   NONE, 0x10af, 0x10ae, 0x11c4, 0x10b4, 0x120c, 0x10b4, 0x1214, 0x1086, 0x00071261, 0x1261, 0x1087, 0x0007127a, 0x127a, 0x1088, 0x000712b3, 0x12b3)
GDI32(NtGdiGetEmbUFI                                    ,   NONE, 0x10d2, 0x10d1, 0x11c5, 0x10da, 0x120d, 0x10dc, 0x1215, 0x105e, 0x00001262, 0x1262, 0x105f, 0x0000127b, 0x127b, 0x1060, 0x000012b4, 0x12b4)
GDI32(NtGdiGetEmbedFonts                                ,   NONE, 0x10d4, 0x10d3, 0x11c6, 0x10dc, 0x120e, 0x10de, 0x1216, 0x105c, 0x00011263, 0x1263, 0x105d, 0x0001127c, 0x127c, 0x105e, 0x000112b5, 0x12b5)
GDI32(NtGdiGetEudcTimeStampEx                           ,   NONE, 0x10b0, 0x10af, 0x11c7, 0x10b5, 0x120f, 0x10b5, 0x1217, 0x1085, 0x000a1264, 0x1264, 0x1086, 0x000a127d, 0x127d, 0x1087, 0x000a12b6, 0x12b6)
GDI32(NtGdiGetFontData                                  ,   NONE, 0x10b1, 0x10b0, 0x10db, 0x10b6, 0x10dc, 0x10b6, 0x10d7, 0x1084, 0x000010d7, 0x10d7, 0x1085, 0x000010d8, 0x10d8, 0x1086, 0x000010d9, 0x10d9)
GDI32(NtGdiGetFontFileData                              ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE, 0x10b7, 0x1218, 0x1083, 0x00001265, 0x1265, 0x1084, 0x0000127e, 0x127e, 0x1085, 0x000012b7, 0x12b7)
GDI32(NtGdiGetFontFileInfo                              ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE, 0x10b8, 0x1219, 0x1082, 0x00001266, 0x1266, 0x1083, 0x0000127f, 0x127f, 0x1084, 0x000012b8, 0x12b8)
GDI32(NtGdiGetFontResourceInfoInternalW                 ,   NONE, 0x10b2, 0x10b1, 0x11c8, 0x10b7, 0x1210, 0x10b9, 0x121a, 0x1081, 0x00001267, 0x1267, 0x1082, 0x00001280, 0x1280, 0x1083, 0x000012b9, 0x12b9)
GDI32(NtGdiGetFontUnicodeRanges                         , 0x10cf, 0x10d7, 0x10d6, 0x11c9, 0x10df, 0x1211, 0x10e1, 0x121b, 0x1059, 0x00071268, 0x1268, 0x105a, 0x00071281, 0x1281, 0x105b, 0x000712ba, 0x12ba)
GDI32(NtGdiGetGlyphIndicesW                             , 0x10af, 0x10b3, 0x10b2, 0x11ca, 0x10b8, 0x1212, 0x10ba, 0x121c, 0x1080, 0x00001269, 0x1269, 0x1081, 0x00001282, 0x1282, 0x1082, 0x000012bb, 0x12bb)
GDI32(NtGdiGetGlyphIndicesWInternal                     ,   NONE, 0x10b4, 0x10b3, 0x11cb, 0x10b9, 0x1213, 0x10bb, 0x121d, 0x107f, 0x0000126a, 0x126a, 0x1080, 0x00001283, 0x1283, 0x1081, 0x000012bc, 0x12bc)
GDI32(NtGdiGetGlyphOutline                              ,   NONE, 0x10b5, 0x10b4, 0x11cc, 0x10ba, 0x1214, 0x10bc, 0x121e, 0x107e, 0x0000126b, 0x126b, 0x107f, 0x00001284, 0x1284, 0x1080, 0x000012bd, 0x12bd)
GDI32(NtGdiGetKerningPairs                              ,   NONE, 0x10b6, 0x10b5, 0x11cd, 0x10bc, 0x1215, 0x10be, 0x121f, 0x107c, 0x000c126c, 0x126c, 0x107d, 0x000c1285, 0x1285, 0x107e, 0x000c12be, 0x12be)
GDI32(NtGdiGetLinkedUFIs                                ,   NONE, 0x10b7, 0x10b6, 0x11ce, 0x10bd, 0x1216, 0x10bf, 0x1220, 0x107b, 0x000c126d, 0x126d, 0x107c, 0x000c1286, 0x1286, 0x107d, 0x000c12bf, 0x12bf)
GDI32(NtGdiGetMiterLimit                                ,   NONE, 0x10b8, 0x10b7, 0x11cf, 0x10be, 0x1217, 0x10c0, 0x1221, 0x107a, 0x0007126e, 0x126e, 0x107b, 0x00071287, 0x1287, 0x107c, 0x000712c0, 0x12c0)
GDI32(NtGdiGetMonitorID                                 ,   NONE, 0x10b9, 0x10b8, 0x11d0, 0x10bf, 0x1218, 0x10c1, 0x1222, 0x1079, 0x000c126f, 0x126f, 0x107a, 0x000c1288, 0x1288, 0x107b, 0x000c12c1, 0x12c1)
GDI32(NtGdiGetNearestColor                              ,   NONE, 0x10ba, 0x10b9, 0x1071, 0x10c0, 0x1072, 0x10c2, 0x1071, 0x1078, 0x00071071, 0x1071, 0x1079, 0x00071072, 0x1072, 0x107a, 0x00071073, 0x1073)
GDI32(NtGdiGetNearestPaletteIndex                       ,   NONE, 0x10bb, 0x10ba, 0x10c8, 0x10c1, 0x10c9, 0x10c3, 0x10c4, 0x1077, 0x000510c4, 0x10c4, 0x1078, 0x000510c5, 0x10c5, 0x1079, 0x000510c6, 0x10c6)
GDI32(NtGdiGetNumberOfPhysicalMonitors                  ,   NONE,   NONE,   NONE,   NONE, 0x12f8, 0x1219, 0x1324, 0x1223, 0x136d, 0x00001270, 0x1270, 0x137f, 0x00001289, 0x1289, 0x13b8, 0x000012c2, 0x12c2)
GDI32(NtGdiGetOPMInformation                            ,   NONE,   NONE,   NONE,   NONE, 0x10bb, 0x121a, 0x10bd, 0x1224, 0x107d, 0x00001271, 0x1271, 0x107e, 0x0000128a, 0x128a, 0x107f, 0x000012c3, 0x12c3)
GDI32(NtGdiGetOPMRandomNumber                           ,   NONE,   NONE,   NONE,   NONE, 0x10c3, 0x121b, 0x10c5, 0x1225, 0x1075, 0x00001272, 0x1272, 0x1076, 0x0000128b, 0x128b, 0x1077, 0x000012c4, 0x12c4)
GDI32(NtGdiGetObjectBitmapHandle                        ,   NONE, 0x10bc, 0x10bb, 0x11d1, 0x10c2, 0x121c, 0x10c4, 0x1226, 0x1076, 0x00051273, 0x1273, 0x1077, 0x0005128c, 0x128c, 0x1078, 0x000512c5, 0x12c5)
GDI32(NtGdiGetOutlineTextMetricsInternalW               ,   NONE, 0x10bd, 0x10bc, 0x10b7, 0x10c4, 0x10b8, 0x10c6, 0x10b3, 0x1074, 0x000010b3, 0x10b3, 0x1075, 0x000010b4, 0x10b4, 0x1076, 0x000010b5, 0x10b5)
GDI32(NtGdiGetPath                                      ,   NONE, 0x10be, 0x10bd, 0x11d2, 0x10c5, 0x121d, 0x10c7, 0x1227, 0x1073, 0x00141274, 0x1274, 0x1074, 0x0014128d, 0x128d, 0x1075, 0x001412c6, 0x12c6)
GDI32(NtGdiGetPerBandInfo                               ,   NONE, 0x107c, 0x107c, 0x11d3, 0x107f, 0x121e, 0x1080, 0x1228, 0x10ba, 0x00071275, 0x1275, 0x10bc, 0x0007128e, 0x128e, 0x10bd, 0x000712c7, 0x12c7)
GDI32(NtGdiGetPhysicalMonitorDescription                ,   NONE,   NONE,   NONE,   NONE, 0x12fa, 0x121f, 0x1326, 0x1229, 0x136b, 0x00001276, 0x1276, 0x137d, 0x0000128f, 0x128f, 0x13b6, 0x000012c8, 0x12c8)
GDI32(NtGdiGetPhysicalMonitors                          ,   NONE,   NONE,   NONE,   NONE, 0x12f9, 0x1220, 0x1325, 0x122a, 0x136c, 0x00001277, 0x1277, 0x137e, 0x00001290, 0x1290, 0x13b7, 0x000012c9, 0x12c9)
GDI32(NtGdiGetPixel                                     ,   NONE, 0x10bf, 0x10be, 0x10c3, 0x10c6, 0x10c4, 0x10c8, 0x10bf, 0x1072, 0x000b10bf, 0x10bf, 0x1073, 0x000b10c0, 0x10c0, 0x1074, 0x000b10c1, 0x10c1)
GDI32(NtGdiGetRandomRgn                                 ,   NONE, 0x10c0, 0x10bf, 0x102a, 0x10c7, 0x102b, 0x10c9, 0x102b, 0x1071, 0x0010102c, 0x102c, 0x1072, 0x0010102d, 0x102d, 0x1073, 0x0010102e, 0x102e)
GDI32(NtGdiGetRasterizerCaps                            ,   NONE, 0x10c1, 0x10c0, 0x10eb, 0x10c8, 0x10ec, 0x10ca, 0x10e7, 0x1070, 0x000510e7, 0x10e7, 0x1071, 0x000510e8, 0x10e8, 0x1072, 0x000510e9, 0x10e9)
GDI32(NtGdiGetRealizationInfo                           ,   NONE, 0x10c2, 0x10c1, 0x11d4, 0x10c9, 0x1221, 0x10cb, 0x122b, 0x106f, 0x00071278, 0x1278, 0x1070, 0x00071291, 0x1291, 0x1071, 0x000712ca, 0x12ca)
GDI32(NtGdiGetRegionData                                ,   NONE, 0x10c3, 0x10c2, 0x103f, 0x10ca, 0x1040, 0x10cc, 0x1040, 0x106e, 0x000a1041, 0x1041, 0x106f, 0x000a1042, 0x1042, 0x1070, 0x000a1043, 0x1043)
GDI32(NtGdiGetRgnBox                                    ,   NONE, 0x10c4, 0x10c3, 0x1066, 0x10cb, 0x1067, 0x10cd, 0x1067, 0x106d, 0x00051067, 0x1067, 0x106e, 0x00051068, 0x1068, 0x106f, 0x00051069, 0x1069)
GDI32(NtGdiGetServerMetaFileBits                        ,   NONE, 0x10c5, 0x10c4, 0x11d5, 0x10cc, 0x1222, 0x10ce, 0x122c, 0x106c, 0x00001279, 0x1279, 0x106d, 0x00001292, 0x1292, 0x106e, 0x000012cb, 0x12cb)
GDI32(NtGdiGetSpoolMessage                              , 0x10c2, 0x10c6, 0x10c5, 0x11d6, 0x10cd, 0x1223, 0x10cf, 0x122d, 0x106b, 0x0000127a, 0x127a, 0x106c, 0x00001293, 0x1293, 0x106d, 0x000012cc, 0x12cc)
GDI32(NtGdiGetStats                                     ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE, 0x127b,   NONE,       NONE, 0x1294, 0x106c, 0x000012cd, 0x12cd)
GDI32(NtGdiGetStockObject                               ,   NONE, 0x10c8, 0x10c7, 0x10d4, 0x10cf, 0x10d5, 0x10d1, 0x10d0, 0x1069, 0x000310d0, 0x10d0, 0x106a, 0x000310d1, 0x10d1, 0x106b, 0x000310d2, 0x10d2)
GDI32(NtGdiGetStringBitmapW                             ,   NONE, 0x10c9, 0x10c8, 0x11d8, 0x10d0, 0x1225, 0x10d2, 0x122f, 0x1068, 0x0000127c, 0x127c, 0x1069, 0x00001295, 0x1295, 0x106a, 0x000012ce, 0x12ce)
GDI32(NtGdiGetSuggestedOPMProtectedOutputArraySize      ,   NONE,   NONE,   NONE,   NONE, 0x10d1, 0x1226, 0x10d3, 0x1230, 0x1067, 0x0000127d, 0x127d, 0x1068, 0x00001296, 0x1296, 0x1069, 0x000012cf, 0x12cf)
GDI32(NtGdiGetSystemPaletteUse                          ,   NONE, 0x10ca, 0x10c9, 0x1118, 0x10d2, 0x1119, 0x10d4, 0x1112, 0x1066, 0x00031112, 0x1112, 0x1067, 0x00031113, 0x1113, 0x1068, 0x00031114, 0x1114)
GDI32(NtGdiGetTextCharsetInfo                           , 0x10c7, 0x10cb, 0x10ca, 0x104c, 0x10d3, 0x104d, 0x10d5, 0x104d, 0x1065, 0x000c104e, 0x104e, 0x1066, 0x000c104f, 0x104f, 0x1067, 0x000c1050, 0x1050)
GDI32(NtGdiGetTextExtent                                ,   NONE, 0x10cc, 0x10cb, 0x1095, 0x10d4, 0x1096, 0x10d6, 0x1094, 0x1064, 0x00001094, 0x1094, 0x1065, 0x00001095, 0x1095, 0x1066, 0x00001096, 0x1096)
GDI32(NtGdiGetTextExtentExW                             ,   NONE, 0x10cd, 0x10cc, 0x11d9, 0x10d5, 0x1227, 0x10d7, 0x1231, 0x1063, 0x0000127e, 0x127e, 0x1064, 0x00001297, 0x1297, 0x1065, 0x000012d0, 0x12d0)
GDI32(NtGdiGetTextFaceW                                 ,   NONE, 0x10ce, 0x10cd, 0x1081, 0x10d6, 0x1082, 0x10d8, 0x1080, 0x1062, 0x00141080, 0x1080, 0x1063, 0x00141081, 0x1081, 0x1064, 0x00141082, 0x1082)
GDI32(NtGdiGetTextMetricsW                              ,   NONE, 0x10cf, 0x10ce, 0x1076, 0x10d7, 0x1077, 0x10d9, 0x1075, 0x1061, 0x000c1075, 0x1075, 0x1062, 0x000c1076, 0x1076, 0x1063, 0x000c1077, 0x1077)
GDI32(NtGdiGetTransform                                 ,   NONE, 0x10d0, 0x10cf, 0x10e0, 0x10d8, 0x10e1, 0x10da, 0x10dc, 0x1060, 0x000c10dc, 0x10dc, 0x1061, 0x000c10dd, 0x10dd, 0x1062, 0x000c10de, 0x10de)
GDI32(NtGdiGetUFI                                       ,   NONE, 0x10d1, 0x10d0, 0x11da, 0x10d9, 0x1228, 0x10db, 0x1232, 0x105f, 0x0000127f, 0x127f, 0x1060, 0x00001298, 0x1298, 0x1061, 0x000012d1, 0x12d1)
GDI32(NtGdiGetUFIPathname                               ,   NONE, 0x10d3, 0x10d2, 0x11db, 0x10db, 0x1229, 0x10dd, 0x1233, 0x105d, 0x00001280, 0x1280, 0x105e, 0x00001299, 0x1299, 0x105f, 0x000012d2, 0x12d2)
GDI32(NtGdiGetWidthTable                                ,   NONE, 0x10d8, 0x10d7, 0x1069, 0x10e0, 0x106a, 0x10e2, 0x106a, 0x1058, 0x0000106a, 0x106a, 0x1059, 0x0000106b, 0x106b, 0x105a, 0x0000106c, 0x106c)
GDI32(NtGdiGradientFill                                 ,   NONE, 0x10d9, 0x10d8, 0x11dc, 0x10e1, 0x122a, 0x10e3, 0x1234, 0x1057, 0x00001281, 0x1281, 0x1058, 0x0000129a, 0x129a, 0x1059, 0x000012d3, 0x12d3)
GDI32(NtGdiHLSurfGetInformation                         ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE, 0x12dd, 0x1235, 0x1306, 0x00001282, 0x1282, 0x130f, 0x0000129b, 0x129b, 0x131b, 0x000012d4, 0x12d4)
GDI32(NtGdiHLSurfSetInformation                         ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE, 0x12de, 0x1236, 0x1305, 0x00001283, 0x1283, 0x130e, 0x0000129c, 0x129c, 0x131a, 0x000012d5, 0x12d5)
GDI32(NtGdiHT_Get8BPPFormatPalette                      , 0x127c, 0x1294, 0x1290, 0x11dd, 0x12b5, 0x122b, 0x12d5, 0x1237, 0x12fe, 0x00001284, 0x1284, 0x1307, 0x0000129d, 0x129d, 0x1313, 0x000012d6, 0x12d6)
GDI32(NtGdiHT_Get8BPPMaskPalette                        , 0x127d, 0x1295, 0x1291, 0x11de, 0x12b6, 0x122c, 0x12d6, 0x1238, 0x12fd, 0x00001285, 0x1285, 0x1306, 0x0000129e, 0x129e, 0x1312, 0x000012d7, 0x12d7)
GDI32(NtGdiHfontCreate                                  ,   NONE, 0x10da, 0x10d9, 0x105c, 0x10e2, 0x105d, 0x10e4, 0x105d, 0x1056, 0x0000105d, 0x105d, 0x1057, 0x0000105e, 0x105e, 0x1058, 0x0000105f, 0x105f)
GDI32(NtGdiIcmBrushInfo                                 ,   NONE, 0x10db, 0x10da, 0x11df, 0x10e3, 0x122d, 0x10e5, 0x1239, 0x1055, 0x00001286, 0x1286, 0x1056, 0x0000129f, 0x129f, 0x1057, 0x000012d8, 0x12d8)
GDI32(NtGdiInit                                         ,   NONE, 0x10dc, 0x10db, 0x11e0, 0x10e4, 0x122e, 0x10e6, 0x123a, 0x1054, 0x00011287, 0x1287, 0x1055, 0x000112a0, 0x12a0, 0x1056, 0x000112d9, 0x12d9)
GDI32(NtGdiInitSpool                                    , 0x10d5, 0x10dd, 0x10dc, 0x11e1, 0x10e5, 0x122f, 0x10e7, 0x123b, 0x1053, 0x00011288, 0x1288, 0x1054, 0x000112a1, 0x12a1, 0x1055, 0x000112da, 0x12da)
GDI32(NtGdiIntersectClipRect                            ,   NONE, 0x10de, 0x10dd, 0x101f, 0x10e6, 0x1020, 0x10e8, 0x1020, 0x1052, 0x00001021, 0x1021, 0x1053, 0x00001022, 0x1022, 0x1054, 0x00001023, 0x1023)
GDI32(NtGdiInvertRgn                                    ,   NONE, 0x10df, 0x10de, 0x1065, 0x10e7, 0x1066, 0x10e9, 0x1066, 0x1051, 0x00071066, 0x1066, 0x1052, 0x00071067, 0x1067, 0x1053, 0x00071068, 0x1068)
GDI32(NtGdiLineTo                                       ,   NONE, 0x10e0, 0x10df, 0x1040, 0x10e8, 0x1041, 0x10ea, 0x1041, 0x1050, 0x000c1042, 0x1042, 0x1051, 0x000c1043, 0x1043, 0x1052, 0x000c1044, 0x1044)
GDI32(NtGdiMakeFontDir                                  ,   NONE, 0x10e1, 0x10e0, 0x11e2, 0x10e9, 0x1230, 0x10eb, 0x123c, 0x104f, 0x00001289, 0x1289, 0x1050, 0x000012a2, 0x12a2, 0x1051, 0x000012db, 0x12db)
GDI32(NtGdiMakeInfoDC                                   ,   NONE, 0x10e2, 0x10e1, 0x11e3, 0x10ea, 0x1231, 0x10ec, 0x123d, 0x104e, 0x0007128a, 0x128a, 0x104f, 0x000712a3, 0x12a3, 0x1050, 0x000712dc, 0x12dc)
GDI32(NtGdiMakeObjectUnXferable                         ,   NONE,   NONE,   NONE, 0x11e4,   NONE, 0x1232,   NONE, 0x123e, 0x1368, 0x0000128b, 0x128b, 0x137a, 0x000012a4, 0x12a4, 0x13b3, 0x000012dd, 0x12dd)
GDI32(NtGdiMakeObjectXferable                           ,   NONE,   NONE,   NONE, 0x11e5,   NONE, 0x1233,   NONE, 0x123f, 0x1369, 0x0000128c, 0x128c, 0x137b, 0x000012a5, 0x12a5, 0x13b4, 0x000012de, 0x12de)
GDI32(NtGdiMaskBlt                                      ,   NONE, 0x10e3, 0x10e2, 0x1068, 0x10eb, 0x1069, 0x10ed, 0x1069, 0x104d, 0x00001069, 0x1069, 0x104e, 0x0000106a, 0x106a, 0x104f, 0x0000106b, 0x106b)
GDI32(NtGdiMirrorWindowOrg                              ,   NONE, 0x1118, 0x1117, 0x11e6, 0x1120, 0x1234, 0x1122, 0x1240, 0x1018, 0x0003128d, 0x128d, 0x1019, 0x000312a6, 0x12a6, 0x101a, 0x000312df, 0x12df)
GDI32(NtGdiModifyWorldTransform                         ,   NONE, 0x10e4, 0x10e3, 0x10da, 0x10ec, 0x10db, 0x10ee, 0x10d6, 0x104c, 0x000c10d6, 0x10d6, 0x104d, 0x000c10d7, 0x10d7, 0x104e, 0x000c10d8, 0x10d8)
GDI32(NtGdiMonoBitmap                                   ,   NONE, 0x10e5, 0x10e4, 0x11e7, 0x10ed, 0x1235, 0x10ef, 0x1241, 0x104b, 0x0004128e, 0x128e, 0x104c, 0x000412a7, 0x12a7, 0x104d, 0x000412e0, 0x12e0)
GDI32(NtGdiMoveTo                                       ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE, 0x128f,   NONE,       NONE, 0x12a8, 0x104c, 0x001412e1, 0x12e1)
GDI32(NtGdiOffsetClipRgn                                ,   NONE, 0x10e7, 0x10e6, 0x11e9, 0x10ef, 0x1237, 0x10f1, 0x1243, 0x1049, 0x000c1290, 0x1290, 0x104a, 0x000c12a9, 0x12a9, 0x104b, 0x000c12e2, 0x12e2)
GDI32(NtGdiOffsetRgn                                    ,   NONE, 0x10e8, 0x10e7, 0x107f, 0x10f0, 0x1080, 0x10f2, 0x107e, 0x1048, 0x000a107e, 0x107e, 0x1049, 0x000a107f, 0x107f, 0x104a, 0x000a1080, 0x1080)
GDI32(NtGdiOpenDCW                                      ,   NONE, 0x10e9, 0x10e8, 0x10de, 0x10f1, 0x10df, 0x10f3, 0x10da, 0x1047, 0x000010da, 0x10da, 0x1048, 0x000010db, 0x10db, 0x1049, 0x000010dc, 0x10dc)
GDI32(NtGdiPATHOBJ_bEnum                                , 0x1276, 0x128e, 0x128a, 0x11ea, 0x12af, 0x1238, 0x12cf, 0x1244, 0x12f9, 0x00001291, 0x1291, 0x1302, 0x000012aa, 0x12aa, 0x130e, 0x000012e3, 0x12e3)
GDI32(NtGdiPATHOBJ_bEnumClipLines                       , 0x1279, 0x1291, 0x128d, 0x11eb, 0x12b2, 0x1239, 0x12d2, 0x1245, 0x12f6, 0x00001292, 0x1292, 0x12ff, 0x000012ab, 0x12ab, 0x130b, 0x000012e4, 0x12e4)
GDI32(NtGdiPATHOBJ_vEnumStart                           , 0x1277, 0x128f, 0x128b, 0x11ec, 0x12b0, 0x123a, 0x12d0, 0x1246, 0x12f8, 0x00001293, 0x1293, 0x1301, 0x000012ac, 0x12ac, 0x130d, 0x000012e5, 0x12e5)
GDI32(NtGdiPATHOBJ_vEnumStartClipLines                  , 0x1278, 0x1290, 0x128c, 0x11ed, 0x12b1, 0x123b, 0x12d1, 0x1247, 0x12f7, 0x00001294, 0x1294, 0x1300, 0x000012ad, 0x12ad, 0x130c, 0x000012e6, 0x12e6)
GDI32(NtGdiPATHOBJ_vGetBounds                           , 0x1275, 0x128d, 0x1289, 0x11ee, 0x12ae, 0x123c, 0x12ce, 0x1248, 0x12fa, 0x00001295, 0x1295, 0x1303, 0x000012ae, 0x12ae, 0x130f, 0x000012e7, 0x12e7)
GDI32(NtGdiPatBlt                                       ,   NONE, 0x10ea, 0x10e9, 0x1059, 0x10f2, 0x105a, 0x10f4, 0x105a, 0x1046, 0x0000105a, 0x105a, 0x1047, 0x0000105b, 0x105b, 0x1048, 0x0000105c, 0x105c)
GDI32(NtGdiPathToRegion                                 ,   NONE, 0x10ec, 0x10eb, 0x11ef, 0x10f4, 0x123d, 0x10f6, 0x1249, 0x1044, 0x00031296, 0x1296, 0x1045, 0x000312af, 0x12af, 0x1046, 0x000312e8, 0x12e8)
GDI32(NtGdiPlgBlt                                       ,   NONE, 0x10ed, 0x10ec, 0x11f0, 0x10f5, 0x123e, 0x10f7, 0x124a, 0x1043, 0x00001297, 0x1297, 0x1044, 0x000012b0, 0x12b0, 0x1045, 0x000012e9, 0x12e9)
GDI32(NtGdiPolyDraw                                     ,   NONE, 0x10ee, 0x10ed, 0x11f1, 0x10f6, 0x123f, 0x10f8, 0x124b, 0x1042, 0x00141298, 0x1298, 0x1043, 0x001412b1, 0x12b1, 0x1044, 0x001412ea, 0x12ea)
GDI32(NtGdiPolyPatBlt                                   ,   NONE, 0x10eb, 0x10ea, 0x106f, 0x10f3, 0x1070, 0x10f5, 0x106f, 0x1045, 0x0000106f, 0x106f, 0x1046, 0x00001070, 0x1070, 0x1047, 0x00001071, 0x1071)
GDI32(NtGdiPolyPolyDraw                                 ,   NONE, 0x10ef, 0x10ee, 0x1047, 0x10f7, 0x1048, 0x10f9, 0x1048, 0x1041, 0x00001049, 0x1049, 0x1042, 0x0000104a, 0x104a, 0x1043, 0x0000104b, 0x104b)
GDI32(NtGdiPolyTextOutW                                 ,   NONE, 0x10f0, 0x10ef, 0x11f2, 0x10f8, 0x1240, 0x10fa, 0x124c, 0x1040, 0x00001299, 0x1299, 0x1041, 0x000012b2, 0x12b2, 0x1042, 0x000012eb, 0x12eb)
GDI32(NtGdiPtInRegion                                   ,   NONE, 0x10f1, 0x10f0, 0x11f3, 0x10f9, 0x1241, 0x10fb, 0x124d, 0x103f, 0x000a129a, 0x129a, 0x1040, 0x000a12b3, 0x12b3, 0x1041, 0x000a12ec, 0x12ec)
GDI32(NtGdiPtVisible                                    ,   NONE, 0x10f2, 0x10f1, 0x11f4, 0x10fa, 0x1242, 0x10fc, 0x124e, 0x103e, 0x000c129b, 0x129b, 0x103f, 0x000c12b4, 0x12b4, 0x1040, 0x000c12ed, 0x12ed)
GDI32(NtGdiQueryFontAssocInfo                           ,   NONE, 0x10f4, 0x10f3, 0x10f9, 0x10fc, 0x10fa, 0x10fe, 0x10f5, 0x103c, 0x000310f5, 0x10f5, 0x103d, 0x000310f6, 0x10f6, 0x103e, 0x000310f7, 0x10f7)
GDI32(NtGdiQueryFonts                                   , 0x10eb, 0x10f3, 0x10f2, 0x11f5, 0x10fb, 0x1243, 0x10fd, 0x124f, 0x103d, 0x000a129c, 0x129c, 0x103e, 0x000a12b5, 0x12b5, 0x103f, 0x000a12ee, 0x12ee)
GDI32(NtGdiRectInRegion                                 ,   NONE, 0x10f6, 0x10f5, 0x10c1, 0x10fe, 0x10c2, 0x1100, 0x10bd, 0x103a, 0x000510bd, 0x10bd, 0x103b, 0x000510be, 0x10be, 0x103c, 0x000510bf, 0x10bf)
GDI32(NtGdiRectVisible                                  ,   NONE, 0x10f7, 0x10f6, 0x1032, 0x10ff, 0x1033, 0x1101, 0x1033, 0x1039, 0x00071034, 0x1034, 0x103a, 0x00071035, 0x1035, 0x103b, 0x00071036, 0x1036)
GDI32(NtGdiRectangle                                    ,   NONE, 0x10f5, 0x10f4, 0x1091, 0x10fd, 0x1092, 0x10ff, 0x1090, 0x103b, 0x00001090, 0x1090, 0x103c, 0x00001091, 0x1091, 0x103d, 0x00001092, 0x1092)
GDI32(NtGdiRemoveFontMemResourceEx                      ,   NONE, 0x10f9, 0x10f8, 0x1119, 0x1101, 0x111a, 0x1103, 0x1113, 0x1037, 0x00031113, 0x1113, 0x1038, 0x00031114, 0x1114, 0x1039, 0x00031115, 0x1115)
GDI32(NtGdiRemoveFontResourceW                          ,   NONE, 0x10f8, 0x10f7, 0x11f6, 0x1100, 0x1244, 0x1102, 0x1250, 0x1038, 0x0000129d, 0x129d, 0x1039, 0x000012b6, 0x12b6, 0x103a, 0x000012ef, 0x12ef)
GDI32(NtGdiRemoveMergeFont                              ,   NONE, 0x1005, 0x1005, 0x11f7, 0x1005, 0x1245, 0x1005, 0x1251, 0x1137, 0x0007129e, 0x129e, 0x1139, 0x000712b7, 0x12b7, 0x113a, 0x000712f0, 0x12f0)
GDI32(NtGdiResetDC                                      ,   NONE, 0x10fa, 0x10f9, 0x11f8, 0x1102, 0x1246, 0x1104, 0x1252, 0x1036, 0x0000129f, 0x129f, 0x1037, 0x000012b8, 0x12b8, 0x1038, 0x000012f1, 0x12f1)
GDI32(NtGdiResizePalette                                ,   NONE, 0x10fb, 0x10fa, 0x11f9, 0x1103, 0x1247, 0x1105, 0x1253, 0x1035, 0x000512a0, 0x12a0, 0x1036, 0x000512b9, 0x12b9, 0x1037, 0x000512f2, 0x12f2)
GDI32(NtGdiRestoreDC                                    ,   NONE, 0x10fc, 0x10fb, 0x1039, 0x1104, 0x103a, 0x1106, 0x103a, 0x1034, 0x0008103b, 0x103b, 0x1035, 0x0008103c, 0x103c, 0x1036, 0x0008103d, 0x103d)
GDI32(NtGdiRoundRect                                    ,   NONE, 0x10fd, 0x10fc, 0x11fa, 0x1105, 0x1248, 0x1107, 0x1254, 0x1033, 0x000012a1, 0x12a1, 0x1034, 0x000012ba, 0x12ba, 0x1035, 0x000012f3, 0x12f3)
GDI32(NtGdiSTROBJ_bEnum                                 , 0x1270, 0x1288, 0x1284, 0x11fb, 0x12a9, 0x1249, 0x12c9, 0x1255, 0x12f5, 0x000012a2, 0x12a2, 0x12fe, 0x000012bb, 0x12bb, 0x130a, 0x000012f4, 0x12f4)
GDI32(NtGdiSTROBJ_bEnumPositionsOnly                    , 0x1271, 0x1289, 0x1285, 0x11fc, 0x12aa, 0x124a, 0x12ca, 0x1256, 0x12f4, 0x000012a3, 0x12a3, 0x12fd, 0x000012bc, 0x12bc, 0x1309, 0x000012f5, 0x12f5)
GDI32(NtGdiSTROBJ_bGetAdvanceWidths                     , 0x1272, 0x128a, 0x1286, 0x11fd, 0x12ab, 0x124b, 0x12cb, 0x1257, 0x12f3, 0x000012a4, 0x12a4, 0x12fc, 0x000012bd, 0x12bd, 0x1308, 0x000012f6, 0x12f6)
GDI32(NtGdiSTROBJ_dwGetCodePage                         , 0x1274, 0x128c, 0x1288, 0x11fe, 0x12ad, 0x124c, 0x12cd, 0x1258, 0x12f1, 0x000012a5, 0x12a5, 0x12fa, 0x000012be, 0x12be, 0x1306, 0x000012f7, 0x12f7)
GDI32(NtGdiSTROBJ_vEnumStart                            , 0x1273, 0x128b, 0x1287, 0x11ff, 0x12ac, 0x124d, 0x12cc, 0x1259, 0x12f2, 0x000012a6, 0x12a6, 0x12fb, 0x000012bf, 0x12bf, 0x1307, 0x000012f8, 0x12f8)
GDI32(NtGdiSaveDC                                       ,   NONE, 0x10fe, 0x10fd, 0x103a, 0x1106, 0x103b, 0x1108, 0x103b, 0x1032, 0x0003103c, 0x103c, 0x1033, 0x0003103d, 0x103d, 0x1034, 0x0003103e, 0x103e)
GDI32(NtGdiScaleViewportExtEx                           ,   NONE, 0x10ff, 0x10fe, 0x1200, 0x1107, 0x124e, 0x1109, 0x125a, 0x1031, 0x000012a7, 0x12a7, 0x1032, 0x000012c0, 0x12c0, 0x1033, 0x000012f9, 0x12f9)
GDI32(NtGdiScaleWindowExtEx                             ,   NONE, 0x1100, 0x10ff, 0x1201, 0x1108, 0x124f, 0x110a, 0x125b, 0x1030, 0x000012a8, 0x12a8, 0x1031, 0x000012c1, 0x12c1, 0x1032, 0x000012fa, 0x12fa)
GDI32(NtGdiSelectBitmap                                 ,   NONE, 0x1101, 0x1100, 0x100b, 0x1109, 0x100b, 0x110b, 0x100b, 0x102f, 0x0007100c, 0x100c, 0x1030, 0x0007100d, 0x100d, 0x1031, 0x0007100e, 0x100e)
GDI32(NtGdiSelectBrush                                  ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE, 0x12a9,   NONE,       NONE, 0x12c2, 0x1030, 0x000712fb, 0x12fb)
GDI32(NtGdiSelectClipPath                               ,   NONE, 0x1103, 0x1102, 0x1203, 0x110b, 0x1251, 0x110d, 0x125d, 0x102d, 0x000812aa, 0x12aa, 0x102e, 0x000812c3, 0x12c3, 0x102f, 0x000812fc, 0x12fc)
GDI32(NtGdiSelectFont                                   ,   NONE, 0x1104, 0x1103, 0x1038, 0x110c, 0x1039, 0x110e, 0x1039, 0x102c, 0x0007103a, 0x103a, 0x102d, 0x0007103b, 0x103b, 0x102e, 0x0007103c, 0x103c)
GDI32(NtGdiSelectPen                                    ,   NONE, 0x1105, 0x1104, 0x1204, 0x110d, 0x1252, 0x110f, 0x125e, 0x102b, 0x000712ab, 0x12ab, 0x102c, 0x000712c4, 0x12c4, 0x102d, 0x000712fd, 0x12fd)
GDI32(NtGdiSetBitmapAttributes                          ,   NONE, 0x1106, 0x1105, 0x1205, 0x110e, 0x1253, 0x1110, 0x125f, 0x102a, 0x000512ac, 0x12ac, 0x102b, 0x000512c5, 0x12c5, 0x102c, 0x000512fe, 0x12fe)
GDI32(NtGdiSetBitmapBits                                ,   NONE, 0x1107, 0x1106, 0x10b8, 0x110f, 0x10b9, 0x1111, 0x10b4, 0x1029, 0x000a10b4, 0x10b4, 0x102a, 0x000a10b5, 0x10b5, 0x102b, 0x000a10b6, 0x10b6)
GDI32(NtGdiSetBitmapDimension                           ,   NONE, 0x1108, 0x1107, 0x111d, 0x1110, 0x111e, 0x1112, 0x1117, 0x1028, 0x00111117, 0x1117, 0x1029, 0x00111118, 0x1118, 0x102a, 0x00111119, 0x1119)
GDI32(NtGdiSetBoundsRect                                ,   NONE, 0x1109, 0x1108, 0x10ff, 0x1111, 0x1100, 0x1113, 0x10fb, 0x1027, 0x000c10fb, 0x10fb, 0x1028, 0x000c10fc, 0x10fc, 0x1029, 0x000c10fd, 0x10fd)
GDI32(NtGdiSetBrushAttributes                           ,   NONE, 0x110a, 0x1109, 0x1206, 0x1112, 0x1254, 0x1114, 0x1260, 0x1026, 0x000512ad, 0x12ad, 0x1027, 0x000512c6, 0x12c6, 0x1028, 0x000512ff, 0x12ff)
GDI32(NtGdiSetBrushOrg                                  ,   NONE, 0x110b, 0x110a, 0x10b1, 0x1113, 0x10b2, 0x1115, 0x10ad, 0x1025, 0x001410ad, 0x10ad, 0x1026, 0x001410ae, 0x10ae, 0x1027, 0x001410af, 0x10af)
GDI32(NtGdiSetColorAdjustment                           ,   NONE, 0x110c, 0x110b, 0x1207, 0x1114, 0x1255, 0x1116, 0x1261, 0x1024, 0x000712ae, 0x12ae, 0x1025, 0x000712c7, 0x12c7, 0x1026, 0x00071300, 0x1300)
GDI32(NtGdiSetColorSpace                                ,   NONE, 0x110d, 0x110c, 0x1208, 0x1115, 0x1256, 0x1117, 0x1262, 0x1023, 0x000712af, 0x12af, 0x1024, 0x000712c8, 0x12c8, 0x1025, 0x00071301, 0x1301)
GDI32(NtGdiSetDIBitsToDeviceInternal                    ,   NONE, 0x110f, 0x110e, 0x1028, 0x1117, 0x1029, 0x1119, 0x1029, 0x1021, 0x0000102a, 0x102a, 0x1022, 0x0000102b, 0x102b, 0x1023, 0x0000102c, 0x102c)
GDI32(NtGdiSetDeviceGammaRamp                           ,   NONE, 0x110e, 0x110d, 0x1209, 0x1116, 0x1257, 0x1118, 0x1263, 0x1022, 0x000012b0, 0x12b0, 0x1023, 0x000012c9, 0x12c9, 0x1024, 0x00001302, 0x1302)
GDI32(NtGdiSetFontEnumeration                           ,   NONE, 0x1110, 0x110f, 0x111e, 0x1118, 0x111f, 0x111a, 0x1118, 0x1020, 0x00041118, 0x1118, 0x1021, 0x00041119, 0x1119, 0x1022, 0x0004111a, 0x111a)
GDI32(NtGdiSetFontXform                                 ,   NONE, 0x1111, 0x1110, 0x120a, 0x1119, 0x1258, 0x111b, 0x1264, 0x101f, 0x000c12b1, 0x12b1, 0x1020, 0x000c12ca, 0x12ca, 0x1021, 0x000c1303, 0x1303)
GDI32(NtGdiSetIcmMode                                   ,   NONE, 0x1112, 0x1111, 0x120b, 0x111a, 0x1259, 0x111c, 0x1265, 0x101e, 0x000c12b2, 0x12b2, 0x101f, 0x000c12cb, 0x12cb, 0x1020, 0x000c1304, 0x1304)
GDI32(NtGdiSetLayout                                    ,   NONE, 0x1119, 0x1118, 0x1097, 0x1121, 0x1098, 0x1123, 0x1096, 0x1017, 0x000c1096, 0x1096, 0x1018, 0x000c1097, 0x1097, 0x1019, 0x000c1098, 0x1098)
GDI32(NtGdiSetLinkedUFIs                                ,   NONE, 0x1113, 0x1112, 0x120c, 0x111b, 0x125a, 0x111d, 0x1266, 0x101d, 0x000c12b3, 0x12b3, 0x101e, 0x000c12cc, 0x12cc, 0x101f, 0x000c1305, 0x1305)
GDI32(NtGdiSetMagicColors                               , 0x110a, 0x1114, 0x1113, 0x120d, 0x111c, 0x125b, 0x111e, 0x1267, 0x101c, 0x000012b4, 0x12b4, 0x101d, 0x000012cd, 0x12cd, 0x101e, 0x00001306, 0x1306)
GDI32(NtGdiSetMetaRgn                                   ,   NONE, 0x1115, 0x1114, 0x10e8, 0x111d, 0x10e9, 0x111f, 0x10e4, 0x101b, 0x000310e4, 0x10e4, 0x101c, 0x000310e5, 0x10e5, 0x101d, 0x000310e6, 0x10e6)
GDI32(NtGdiSetMiterLimit                                ,   NONE, 0x1116, 0x1115, 0x10e9, 0x111e, 0x10ea, 0x1120, 0x10e5, 0x101a, 0x000c10e5, 0x10e5, 0x101b, 0x000c10e6, 0x10e6, 0x101c, 0x000c10e7, 0x10e7)
GDI32(NtGdiSetOPMSigningKeyAndSequenceNumbers           ,   NONE,   NONE,   NONE,   NONE, 0x1122, 0x125c, 0x1124, 0x1268, 0x1016, 0x000012b5, 0x12b5, 0x1017, 0x000012ce, 0x12ce, 0x1018, 0x00001307, 0x1307)
GDI32(NtGdiSetPUMPDOBJ                                  ,   NONE, 0x1297, 0x1293, 0x120e, 0x12b8, 0x125d, 0x12d8, 0x1269, 0x1302, 0x000012b6, 0x12b6, 0x130b, 0x000012cf, 0x12cf, 0x1317, 0x00001308, 0x1308)
GDI32(NtGdiSetPixel                                     ,   NONE, 0x111a, 0x1119, 0x10b3, 0x1123, 0x10b4, 0x1125, 0x10af, 0x1015, 0x001410af, 0x10af, 0x1016, 0x001410b0, 0x10b0, 0x1017, 0x001410b1, 0x10b1)
GDI32(NtGdiSetPixelFormat                               ,   NONE, 0x111b, 0x111a, 0x120f, 0x1124, 0x125e, 0x1126, 0x126a, 0x1014, 0x000812b7, 0x12b7, 0x1015, 0x000812d0, 0x12d0, 0x1016, 0x00081309, 0x1309)
GDI32(NtGdiSetRectRgn                                   ,   NONE, 0x111c, 0x111b, 0x1210, 0x1125, 0x125f, 0x1127, 0x126b, 0x1013, 0x000012b8, 0x12b8, 0x1014, 0x000012d1, 0x12d1, 0x1015, 0x0000130a, 0x130a)
GDI32(NtGdiSetSizeDevice                                ,   NONE, 0x1121, 0x1120, 0x1211, 0x112a, 0x1260, 0x112b, 0x126c, 0x100f, 0x000c12b9, 0x12b9, 0x1010, 0x000c12d2, 0x12d2, 0x1011, 0x000c130b, 0x130b)
GDI32(NtGdiSetSystemPaletteUse                          ,   NONE, 0x111d, 0x111c, 0x1212, 0x1126, 0x1261, 0x1128, 0x126d, 0x1012, 0x000712ba, 0x12ba, 0x1013, 0x000712d3, 0x12d3, 0x1014, 0x0007130c, 0x130c)
GDI32(NtGdiSetTextJustification                         ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE, 0x12bb,   NONE,       NONE, 0x12d4, 0x1013, 0x000c130d, 0x130d)
GDI32(NtGdiSetUMPDSandboxState                          ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE, 0x1303, 0x000012bc, 0x12bc, 0x130c, 0x000012d5, 0x12d5, 0x1318, 0x0000130e, 0x130e)
GDI32(NtGdiSetVirtualResolution                         ,   NONE, 0x1120, 0x111f, 0x10ea, 0x1129, 0x10eb, 0x112a, 0x10e6, 0x1010, 0x000010e6, 0x10e6, 0x1011, 0x000010e7, 0x10e7, 0x1012, 0x000010e8, 0x10e8)
GDI32(NtGdiSetupPublicCFONT                             ,   NONE, 0x111f, 0x111e, 0x110b, 0x1128, 0x110c,   NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE)
GDI32(NtGdiSfmGetNotificationTokens                     ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE, 0x12dc, 0x126f,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE)
GDI32(NtGdiStartDoc                                     ,   NONE, 0x1122, 0x1121, 0x1214, 0x112b, 0x1263, 0x112c, 0x1270, 0x100e, 0x000012bd, 0x12bd, 0x100f, 0x000012d6, 0x12d6, 0x1010, 0x0000130f, 0x130f)
GDI32(NtGdiStartPage                                    ,   NONE, 0x1123, 0x1122, 0x1215, 0x112c, 0x1264, 0x112d, 0x1271, 0x100d, 0x000312be, 0x12be, 0x100e, 0x000312d7, 0x12d7, 0x100f, 0x00031310, 0x1310)
GDI32(NtGdiStretchBlt                                   ,   NONE, 0x1124, 0x1123, 0x1030, 0x112d, 0x1031, 0x112e, 0x1031, 0x100c, 0x00001032, 0x1032, 0x100d, 0x00001033, 0x1033, 0x100e, 0x00001034, 0x1034)
GDI32(NtGdiStretchDIBitsInternal                        ,   NONE, 0x1125, 0x1124, 0x1082, 0x112e, 0x1083, 0x112f, 0x1081, 0x100b, 0x00001081, 0x1081, 0x100c, 0x00001082, 0x1082, 0x100d, 0x00001083, 0x1083)
GDI32(NtGdiStrokeAndFillPath                            ,   NONE, 0x1126, 0x1125, 0x1216, 0x112f, 0x1265, 0x1130, 0x1272, 0x100a, 0x000312bf, 0x12bf, 0x100b, 0x000312d8, 0x12d8, 0x100c, 0x00031311, 0x1311)
GDI32(NtGdiStrokePath                                   ,   NONE, 0x1127, 0x1126, 0x1217, 0x1130, 0x1266, 0x1131, 0x1273, 0x1009, 0x000312c0, 0x12c0, 0x100a, 0x000312d9, 0x12d9, 0x100b, 0x00031312, 0x1312)
GDI32(NtGdiSwapBuffers                                  ,   NONE, 0x1128, 0x1127, 0x1218, 0x1131, 0x1267, 0x1132, 0x1274, 0x1008, 0x000312c1, 0x12c1, 0x1009, 0x000312da, 0x12da, 0x100a, 0x00031313, 0x1313)
GDI32(NtGdiTransformPoints                              ,   NONE, 0x1129, 0x1128, 0x1072, 0x1132, 0x1073, 0x1133, 0x1072, 0x1007, 0x00001072, 0x1072, 0x1008, 0x00001073, 0x1073, 0x1009, 0x00001074, 0x1074)
GDI32(NtGdiTransparentBlt                               ,   NONE, 0x112a, 0x1129, 0x1219, 0x1133, 0x1268, 0x1134, 0x1275, 0x1006, 0x000012c2, 0x12c2, 0x1007, 0x000012db, 0x12db, 0x1008, 0x00001314, 0x1314)
GDI32(NtGdiUMPDEngFreeUserMem                           ,   NONE, 0x1299, 0x1295, 0x121a, 0x12ba, 0x1269, 0x12da, 0x1276, 0x1300, 0x000012c3, 0x12c3, 0x1309, 0x000012dc, 0x12dc, 0x1315, 0x00001315, 0x1315)
GDI32(NtGdiUnloadPrinterDriver                          ,   NONE, 0x112b, 0x112a, 0x121b, 0x1134, 0x126a, 0x1135, 0x1277, 0x1005, 0x000512c4, 0x12c4, 0x1006, 0x000512dd, 0x12dd, 0x1007, 0x00051316, 0x1316)
GDI32(NtGdiUnmapMemFont                                 ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE, 0x12c5,   NONE,       NONE, 0x12de, 0x1006, 0x00001317, 0x1317)
GDI32(NtGdiUnrealizeObject                              ,   NONE, 0x112d, 0x112c, 0x108f, 0x1136, 0x1090, 0x1137, 0x108e, 0x1003, 0x0003108e, 0x108e, 0x1004, 0x0003108f, 0x108f, 0x1005, 0x00031090, 0x1090)
GDI32(NtGdiUpdateColors                                 ,   NONE, 0x112e, 0x112d, 0x121d, 0x1137, 0x126c, 0x1138, 0x1279, 0x1002, 0x000312c6, 0x12c6, 0x1003, 0x000312df, 0x12df, 0x1004, 0x00031318, 0x1318)
GDI32(NtGdiUpdateTransform                              ,   NONE, 0x1296, 0x1292, 0x121e, 0x12b7, 0x126d, 0x12d7, 0x127a, 0x12ff, 0x000012c7, 0x12c7, 0x1308, 0x000012e0, 0x12e0, 0x1314, 0x00001319, 0x1319)
GDI32(NtGdiWidenPath                                    ,   NONE, 0x112f, 0x112e, 0x121f, 0x1138, 0x126e, 0x1139, 0x127b, 0x1001, 0x000312c8, 0x12c8, 0x1002, 0x000312e1, 0x12e1, 0x1003, 0x0003131a, 0x131a)
GDI32(NtGdiXFORMOBJ_bApplyXform                         , 0x1266, 0x127e, 0x127a, 0x1220, 0x129f, 0x126f, 0x12bf, 0x127c, 0x12e8, 0x000012c9, 0x12c9, 0x12f1, 0x000012e2, 0x12e2, 0x12fd, 0x0000131b, 0x131b)
GDI32(NtGdiXFORMOBJ_iGetXform                           , 0x1267, 0x127f, 0x127b, 0x1221, 0x12a0, 0x1270, 0x12c0, 0x127d, 0x12e7, 0x000012ca, 0x12ca, 0x12f0, 0x000012e3, 0x12e3, 0x12fc, 0x0000131c, 0x131c)
GDI32(NtGdiXLATEOBJ_cGetPalette                         , 0x1259, 0x1271, 0x126d, 0x1222, 0x1292, 0x1271, 0x12b2, 0x127e, 0x12dc, 0x000012cb, 0x12cb, 0x12e5, 0x000012e4, 0x12e4, 0x12f1, 0x0000131d, 0x131d)
GDI32(NtGdiXLATEOBJ_hGetColorTransform                  , 0x125b, 0x1273, 0x126f, 0x1223, 0x1294, 0x1272, 0x12b4, 0x127f, 0x12da, 0x000012cc, 0x12cc, 0x12e3, 0x000012e5, 0x12e5, 0x12ef, 0x0000131e, 0x131e)
GDI32(NtGdiXLATEOBJ_iXlate                              , 0x125a, 0x1272, 0x126e, 0x1224, 0x1293, 0x1273, 0x12b3, 0x1280, 0x12db, 0x000012cd, 0x12cd, 0x12e4, 0x000012e6, 0x12e6, 0x12f0, 0x0000131f, 0x131f)

KERNEL32(AddConsoleAliasInternal                           ,   NONE,   NONE,   NONE, 0x2040,   NONE, 0x2046,   NONE, 0x2047,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE)
KERNEL32(AllocConsoleInternal                              ,   NONE,   NONE,   NONE, 0x203c,   NONE, 0x2042,   NONE, 0x2042,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE)
KERNEL32(AttachConsoleInternal                             ,   NONE,   NONE,   NONE, 0x203e,   NONE, 0x2044,   NONE, 0x2044,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE)
KERNEL32(CloseConsoleHandle                                ,   NONE,   NONE,   NONE, 0x2003,   NONE, 0x2003,   NONE, 0x2003,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE)
KERNEL32(ConnectConsoleInternal                            ,   NONE,   NONE,   NONE, 0x203b,   NONE, 0x2041,   NONE, 0x2041,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE)
KERNEL32(ConsoleMenuControl                                ,   NONE,   NONE,   NONE, 0x2032,   NONE, 0x2038,   NONE, 0x2038,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE)
KERNEL32(CreateConsoleScreenBuffer                         ,   NONE,   NONE,   NONE, 0x2010,   NONE, 0x2010,   NONE, 0x2010,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE)
KERNEL32(DuplicateConsoleHandle                            ,   NONE,   NONE,   NONE, 0x2004,   NONE, 0x2004,   NONE, 0x2004,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE)
KERNEL32(ExpungeConsoleCommandHistoryInternal              ,   NONE,   NONE,   NONE, 0x2046,   NONE, 0x204c,   NONE, 0x204d,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE)
KERNEL32(FillConsoleOutput                                 ,   NONE,   NONE,   NONE, 0x200f,   NONE, 0x200f,   NONE, 0x200f,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE)
KERNEL32(FlushConsoleInputBuffer                           ,   NONE,   NONE,   NONE, 0x2021,   NONE, 0x2024,   NONE, 0x2024,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE)
KERNEL32(FreeConsoleInternal                               ,   NONE,   NONE,   NONE, 0x203d,   NONE, 0x2043,   NONE, 0x2043,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE)
KERNEL32(GenerateConsoleCtrlEvent                          ,   NONE,   NONE,   NONE, 0x201f,   NONE, 0x2022,   NONE, 0x2022,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE)
KERNEL32(GetConsoleAliasExesInternal                       ,   NONE,   NONE,   NONE, 0x2045,   NONE, 0x204b,   NONE, 0x204c,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE)
KERNEL32(GetConsoleAliasExesLengthInternal                 ,   NONE,   NONE,   NONE, 0x2043,   NONE, 0x2049,   NONE, 0x204a,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE)
KERNEL32(GetConsoleAliasInternal                           ,   NONE,   NONE,   NONE, 0x2041,   NONE, 0x2047,   NONE, 0x2048,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE)
KERNEL32(GetConsoleAliasesInternal                         ,   NONE,   NONE,   NONE, 0x2044,   NONE, 0x204a,   NONE, 0x204b,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE)
KERNEL32(GetConsoleAliasesLengthInternal                   ,   NONE,   NONE,   NONE, 0x2042,   NONE, 0x2048,   NONE, 0x2049,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE)
KERNEL32(GetConsoleCP                                      ,   NONE,   NONE,   NONE, 0x202a,   NONE, 0x2030,   NONE, 0x2030,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE)
KERNEL32(GetConsoleCharType                                ,   NONE,   NONE,   NONE, 0x204d,   NONE, 0x2052,   NONE, 0x2053,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE)
KERNEL32(GetConsoleCommandHistoryInternal                  ,   NONE,   NONE,   NONE, 0x2049,   NONE, 0x204f,   NONE, 0x2050,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE)
KERNEL32(GetConsoleCommandHistoryLengthInternal            ,   NONE,   NONE,   NONE, 0x2048,   NONE, 0x204e,   NONE, 0x204f,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE)
KERNEL32(GetConsoleCursorInfo                              ,   NONE,   NONE,   NONE, 0x2018,   NONE, 0x201a,   NONE, 0x201a,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE)
KERNEL32(GetConsoleCursorMode                              ,   NONE,   NONE,   NONE, 0x2050,   NONE, 0x2055,   NONE, 0x2056,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE)
KERNEL32(GetConsoleDisplayMode                             ,   NONE,   NONE,   NONE, 0x2038,   NONE, 0x203e,   NONE, 0x203e,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE)
KERNEL32(GetConsoleFontInfo                                ,   NONE,   NONE,   NONE, 0x201b,   NONE, 0x201d,   NONE, 0x201d,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE)
KERNEL32(GetConsoleFontSize                                ,   NONE,   NONE,   NONE, 0x201c,   NONE, 0x201e,   NONE, 0x201e,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE)
KERNEL32(GetConsoleHandleInformation                       ,   NONE,   NONE,   NONE, 0x2005,   NONE, 0x2005,   NONE, 0x2005,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE)
KERNEL32(GetConsoleHardwareState                           ,   NONE,   NONE,   NONE, 0x2036,   NONE, 0x203c,   NONE, 0x203c,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE)
KERNEL32(GetConsoleHistoryInfo                             ,   NONE,   NONE,   NONE,   NONE,   NONE, 0x2012,   NONE, 0x2012,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE)
KERNEL32(GetConsoleInput                                   ,   NONE,   NONE,   NONE, 0x2009,   NONE, 0x2009,   NONE, 0x2009,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE)
KERNEL32(GetConsoleKeyboardLayoutNameWorker                ,   NONE,   NONE,   NONE, 0x202e,   NONE, 0x2034,   NONE, 0x2034,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE)
KERNEL32(GetConsoleLangId                                  ,   NONE,   NONE,   NONE, 0x203f,   NONE, 0x2045,   NONE, 0x2045,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE)
KERNEL32(GetConsoleMode                                    ,   NONE,   NONE,   NONE, 0x2012,   NONE, 0x2014,   NONE, 0x2014,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE)
KERNEL32(GetConsoleNlsMode                                 ,   NONE,   NONE,   NONE, 0x2053,   NONE, 0x2058,   NONE, 0x2059,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE)
KERNEL32(GetConsoleOutputCP                                ,   NONE,   NONE,   NONE, 0x202c,   NONE, 0x2032,   NONE, 0x2032,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE)
KERNEL32(GetConsoleProcessList                             ,   NONE,   NONE,   NONE, 0x2013,   NONE, 0x2015,   NONE, 0x2015,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE)
KERNEL32(GetConsoleScreenBufferInfo                        ,   NONE,   NONE,   NONE, 0x2017,   NONE, 0x2019,   NONE, 0x2019,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE)
KERNEL32(GetConsoleScreenBufferInfoEx                      ,   NONE,   NONE,   NONE,   NONE,   NONE, 0x2013,   NONE, 0x2013,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE)
KERNEL32(GetConsoleSelectionInfo                           ,   NONE,   NONE,   NONE, 0x2019,   NONE, 0x201b,   NONE, 0x201b,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE)
KERNEL32(GetConsoleTitleInternal                           ,   NONE,   NONE,   NONE, 0x204b,   NONE, 0x2050,   NONE, 0x2051,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE)
KERNEL32(GetConsoleWindow                                  ,   NONE,   NONE,   NONE, 0x202f,   NONE, 0x2035,   NONE, 0x2035,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE)
KERNEL32(GetCurrentConsoleFont                             ,   NONE,   NONE,   NONE, 0x201d,   NONE, 0x201f,   NONE, 0x201f,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE)
KERNEL32(GetCurrentConsoleFontEx                           ,   NONE,   NONE,   NONE,   NONE,   NONE, 0x2020,   NONE, 0x2020,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE)
KERNEL32(GetLargestConsoleWindowSize                       ,   NONE,   NONE,   NONE, 0x2016,   NONE, 0x2018,   NONE, 0x2018,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE)
KERNEL32(GetNumberOfConsoleFonts                           ,   NONE,   NONE,   NONE, 0x2014,   NONE, 0x2016,   NONE, 0x2016,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE)
KERNEL32(GetNumberOfConsoleInputEvents                     ,   NONE,   NONE,   NONE, 0x2015,   NONE, 0x2017,   NONE, 0x2017,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE)
KERNEL32(GetNumberOfConsoleMouseButtons                    ,   NONE,   NONE,   NONE, 0x201a,   NONE, 0x201c,   NONE, 0x201c,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE)
KERNEL32(InvalidateConsoleDIBits                           ,   NONE,   NONE,   NONE, 0x2011,   NONE, 0x2011,   NONE, 0x2011,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE)
KERNEL32(NtWow64ConsoleLaunchServerProcess                 ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE, 0x0000300d,   NONE,   NONE, 0x0000300c,   NONE,   NONE, 0x0000300c,   NONE)
KERNEL32(NtWow64CsrBaseCheckRunApp                         ,   NONE,   NONE,   NONE, 0x3011,   NONE, 0x300b,   NONE, 0x300b,   NONE, 0x00003009,   NONE,   NONE, 0x00003008,   NONE,   NONE, 0x00003008,   NONE)
KERNEL32(NtWow64CsrBaseClientConnectToServer               ,   NONE,   NONE,   NONE, 0x300b,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE)
KERNEL32(NtWow64CsrBaseQueryModuleData                     ,   NONE,   NONE,   NONE, 0x3013,   NONE, 0x300c,   NONE, 0x300c,   NONE, 0x0000300a,   NONE,   NONE, 0x00003009,   NONE,   NONE, 0x00003009,   NONE)
KERNEL32(NtWow64CsrBasepCreateActCtx                       ,   NONE,   NONE,   NONE, 0x300f,   NONE, 0x300a,   NONE, 0x300a,   NONE, 0x00003008,   NONE,   NONE, 0x00003007,   NONE,   NONE, 0x00003007,   NONE)
KERNEL32(NtWow64CsrBasepCreateProcess                      ,   NONE,   NONE,   NONE, 0x3004,   NONE, 0x3003,   NONE, 0x3003,   NONE, 0x00003003,   NONE,   NONE, 0x00003002,   NONE,   NONE, 0x00003002,   NONE)
KERNEL32(NtWow64CsrBasepCreateThread                       ,   NONE,   NONE,   NONE, 0x300a,   NONE, 0x3009,   NONE, 0x3009,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE)
KERNEL32(NtWow64CsrBasepDefineDosDevice                    ,   NONE,   NONE,   NONE, 0x3002,   NONE, 0x3002,   NONE, 0x3002,   NONE, 0x00003002,   NONE,   NONE, 0x00003001,   NONE,   NONE, 0x00003001,   NONE)
KERNEL32(NtWow64CsrBasepExitProcess                        ,   NONE,   NONE,   NONE, 0x3005,   NONE, 0x3004,   NONE, 0x3004,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE)
KERNEL32(NtWow64CsrBasepGetProcessShutdownParam            ,   NONE,   NONE,   NONE, 0x3007,   NONE, 0x3006,   NONE, 0x3006,   NONE, 0x00003005,   NONE,   NONE, 0x00003004,   NONE,   NONE, 0x00003004,   NONE)
KERNEL32(NtWow64CsrBasepGetTempFile                        ,   NONE,   NONE,   NONE, 0x3003,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE)
KERNEL32(NtWow64CsrBasepNlsCreateSection                   ,   NONE,   NONE,   NONE, 0x300e,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE)
KERNEL32(NtWow64CsrBasepNlsGetUserInfo                     ,   NONE,   NONE,   NONE, 0x3012,   NONE, 0x300f,   NONE, 0x300e,   NONE, 0x0000300c,   NONE,   NONE, 0x0000300b,   NONE,   NONE, 0x0000300b,   NONE)
KERNEL32(NtWow64CsrBasepNlsSetMultipleUserInfo             ,   NONE,   NONE,   NONE, 0x300d,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE)
KERNEL32(NtWow64CsrBasepNlsSetUserInfo                     ,   NONE,   NONE,   NONE, 0x300c,   NONE, 0x300d,   NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE)
KERNEL32(NtWow64CsrBasepNlsUpdateCacheCount                ,   NONE,   NONE,   NONE, 0x3010,   NONE, 0x300e,   NONE, 0x300d,   NONE, 0x0000300b,   NONE,   NONE, 0x0000300a,   NONE,   NONE, 0x0000300a,   NONE)
KERNEL32(NtWow64CsrBasepRefreshIniFileMapping              ,   NONE,   NONE,   NONE, 0x3001,   NONE, 0x3001,   NONE, 0x3001,   NONE, 0x00003001,   NONE,   NONE, 0x00003000,   NONE,   NONE, 0x00003000,   NONE)
KERNEL32(NtWow64CsrBasepSetClientTimeZoneInformation       ,   NONE,   NONE,   NONE, 0x3009,   NONE, 0x3008,   NONE, 0x3008,   NONE, 0x00003007,   NONE,   NONE, 0x00003006,   NONE,   NONE, 0x00003006,   NONE)
KERNEL32(NtWow64CsrBasepSetProcessShutdownParam            ,   NONE,   NONE,   NONE, 0x3006,   NONE, 0x3005,   NONE, 0x3005,   NONE, 0x00003004,   NONE,   NONE, 0x00003003,   NONE,   NONE, 0x00003003,   NONE)
KERNEL32(NtWow64CsrBasepSetTermsrvAppInstallMode           ,   NONE,   NONE,   NONE, 0x3008,   NONE, 0x3007,   NONE, 0x3007,   NONE, 0x00003006,   NONE,   NONE, 0x00003005,   NONE,   NONE, 0x00003005,   NONE)
KERNEL32(NtWow64CsrBasepSoundSentryNotification            ,   NONE,   NONE,   NONE, 0x3000,   NONE, 0x3000,   NONE, 0x3000,   NONE, 0x00003000,   NONE,   NONE, 0x0000300d,   NONE,   NONE, 0x0000300d,   NONE)
KERNEL32(OpenConsoleWInternal                              ,   NONE,   NONE,   NONE, 0x2000,   NONE, 0x2000,   NONE, 0x2000,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE)
KERNEL32(QueryConsoleIMEInternal                           ,   NONE,   NONE,   NONE,   NONE,   NONE, 0x205c,   NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE)
KERNEL32(ReadConsoleInternal                               ,   NONE,   NONE,   NONE, 0x2001,   NONE, 0x2001,   NONE, 0x2001,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE)
KERNEL32(ReadConsoleOutputInternal                         ,   NONE,   NONE,   NONE, 0x200b,   NONE, 0x200b,   NONE, 0x200b,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE)
KERNEL32(ReadConsoleOutputString                           ,   NONE,   NONE,   NONE, 0x200d,   NONE, 0x200d,   NONE, 0x200d,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE)
KERNEL32(RegisterConsoleIMEInternal                        ,   NONE,   NONE,   NONE, 0x2055,   NONE, 0x205a,   NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE)
KERNEL32(RegisterConsoleOS2                                ,   NONE,   NONE,   NONE, 0x2051,   NONE, 0x2056,   NONE, 0x2057,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE)
KERNEL32(RegisterConsoleVDM                                ,   NONE,   NONE,   NONE, 0x2034,   NONE, 0x203a,   NONE, 0x203a,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE)
KERNEL32(ScrollConsoleScreenBufferInternal                 ,   NONE,   NONE,   NONE, 0x2026,   NONE, 0x202c,   NONE, 0x202c,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE)
KERNEL32(SetConsoleActiveScreenBuffer                      ,   NONE,   NONE,   NONE, 0x2020,   NONE, 0x2023,   NONE, 0x2023,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE)
KERNEL32(SetConsoleCP                                      ,   NONE,   NONE,   NONE, 0x202b,   NONE, 0x2031,   NONE, 0x2031,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE)
KERNEL32(SetConsoleCommandHistoryMode                      ,   NONE,   NONE,   NONE, 0x204a,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE)
KERNEL32(SetConsoleCursor                                  ,   NONE,   NONE,   NONE, 0x2030,   NONE, 0x2036,   NONE, 0x2036,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE)
KERNEL32(SetConsoleCursorInfo                              ,   NONE,   NONE,   NONE, 0x2024,   NONE, 0x2027,   NONE, 0x2027,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE)
KERNEL32(SetConsoleCursorMode                              ,   NONE,   NONE,   NONE, 0x204f,   NONE, 0x2054,   NONE, 0x2055,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE)
KERNEL32(SetConsoleCursorPosition                          ,   NONE,   NONE,   NONE, 0x2023,   NONE, 0x2026,   NONE, 0x2026,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE)
KERNEL32(SetConsoleDisplayMode                             ,   NONE,   NONE,   NONE, 0x2035,   NONE, 0x203b,   NONE, 0x203b,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE)
KERNEL32(SetConsoleFont                                    ,   NONE,   NONE,   NONE, 0x2028,   NONE, 0x202e,   NONE, 0x202e,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE)
KERNEL32(SetConsoleHandleInformation                       ,   NONE,   NONE,   NONE, 0x2006,   NONE, 0x2006,   NONE, 0x2006,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE)
KERNEL32(SetConsoleHardwareState                           ,   NONE,   NONE,   NONE, 0x2037,   NONE, 0x203d,   NONE, 0x203d,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE)
KERNEL32(SetConsoleHistoryInfo                             ,   NONE,   NONE,   NONE,   NONE,   NONE, 0x2028,   NONE, 0x2028,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE)
KERNEL32(SetConsoleIcon                                    ,   NONE,   NONE,   NONE, 0x2029,   NONE, 0x202f,   NONE, 0x202f,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE)
KERNEL32(SetConsoleKeyShortcuts                            ,   NONE,   NONE,   NONE, 0x2039,   NONE, 0x203f,   NONE, 0x203f,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE)
KERNEL32(SetConsoleLocalEUDC                               ,   NONE,   NONE,   NONE, 0x204e,   NONE, 0x2053,   NONE, 0x2054,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE)
KERNEL32(SetConsoleMenuClose                               ,   NONE,   NONE,   NONE, 0x203a,   NONE, 0x2040,   NONE, 0x2040,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE)
KERNEL32(SetConsoleMode                                    ,   NONE,   NONE,   NONE, 0x201e,   NONE, 0x2021,   NONE, 0x2021,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE)
KERNEL32(SetConsoleNlsMode                                 ,   NONE,   NONE,   NONE, 0x2054,   NONE, 0x2059,   NONE, 0x205a,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE)
KERNEL32(SetConsoleNumberOfCommandsInternal                ,   NONE,   NONE,   NONE, 0x2047,   NONE, 0x204d,   NONE, 0x204e,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE)
KERNEL32(SetConsoleOS2OemFormat                            ,   NONE,   NONE,   NONE, 0x2052,   NONE, 0x2057,   NONE, 0x2058,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE)
KERNEL32(SetConsoleOutputCPInternal                        ,   NONE,   NONE,   NONE, 0x202d,   NONE, 0x2033,   NONE, 0x2033,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE)
KERNEL32(SetConsolePaletteInternal                         ,   NONE,   NONE,   NONE, 0x2033,   NONE, 0x2039,   NONE, 0x2039,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE)
KERNEL32(SetConsoleScreenBufferInfoEx                      ,   NONE,   NONE,   NONE,   NONE,   NONE, 0x2029,   NONE, 0x2029,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE)
KERNEL32(SetConsoleScreenBufferSize                        ,   NONE,   NONE,   NONE, 0x2022,   NONE, 0x2025,   NONE, 0x2025,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE)
KERNEL32(SetConsoleTextAttribute                           ,   NONE,   NONE,   NONE, 0x2027,   NONE, 0x202d,   NONE, 0x202d,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE)
KERNEL32(SetConsoleTitleInternal                           ,   NONE,   NONE,   NONE, 0x204c,   NONE, 0x2051,   NONE, 0x2052,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE)
KERNEL32(SetConsoleWindowInfo                              ,   NONE,   NONE,   NONE, 0x2025,   NONE, 0x202a,   NONE, 0x202a,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE)
KERNEL32(SetCurrentConsoleFontEx                           ,   NONE,   NONE,   NONE,   NONE,   NONE, 0x202b,   NONE, 0x202b,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE)
KERNEL32(SetLastConsoleEventActiveInternal                 ,   NONE,   NONE,   NONE, 0x2008,   NONE, 0x2008,   NONE, 0x2008,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE)
KERNEL32(ShowConsoleCursor                                 ,   NONE,   NONE,   NONE, 0x2031,   NONE, 0x2037,   NONE, 0x2037,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE)
KERNEL32(UnregisterConsoleIMEInternal                      ,   NONE,   NONE,   NONE, 0x2056,   NONE, 0x205b,   NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE)
KERNEL32(VerifyConsoleIoHandle                             ,   NONE,   NONE,   NONE, 0x2007,   NONE, 0x2007,   NONE, 0x2007,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE)
KERNEL32(WriteConsoleInputInternal                         ,   NONE,   NONE,   NONE, 0x200a,   NONE, 0x200a,   NONE, 0x200a,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE)
KERNEL32(WriteConsoleInternal                              ,   NONE,   NONE,   NONE, 0x2002,   NONE, 0x2002,   NONE, 0x2002,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE)
KERNEL32(WriteConsoleOutputInternal                        ,   NONE,   NONE,   NONE, 0x200c,   NONE, 0x200c,   NONE, 0x200c,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE)
KERNEL32(WriteConsoleOutputString                          ,   NONE,   NONE,   NONE, 0x200e,   NONE, 0x200e,   NONE, 0x200e,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE)

NTDLL(NtAcceptConnectPort                               , 0x0000, 0x0000, 0x0000, 0x0060, 0x0000, 0x0060, 0x0000, 0x00000060, 0x01ac, 0x00000061, 0x0061, 0x0001, 0x00000001, 0x0001, 0x0002, 0x00000002, 0x0002)
NTDLL(NtAccessCheck                                     , 0x0001, 0x0001, 0x0001, 0x0061, 0x0001, 0x0061, 0x0001, 0x00000061, 0x01ab, 0x00000062, 0x0062, 0x01b0, 0x00000062, 0x0062, 0x0000, 0x00000000, 0x0000)
NTDLL(NtAccessCheckAndAuditAlarm                        , 0x0002, 0x0002, 0x0002, 0x0026, 0x0002, 0x0026, 0x0002, 0x00000026, 0x01aa, 0x00000027, 0x0027, 0x01af, 0x00000028, 0x0028, 0x01b7, 0x00000029, 0x0029)
NTDLL(NtAccessCheckByType                               , 0x0003, 0x0003, 0x0003, 0x0062, 0x0003, 0x0062, 0x0003, 0x00000062, 0x01a9, 0x00000063, 0x0063, 0x01ae, 0x00000063, 0x0063, 0x01b6, 0x00000063, 0x0063)
NTDLL(NtAccessCheckByTypeAndAuditAlarm                  , 0x0004, 0x0004, 0x0004, 0x0056, 0x0004, 0x0056, 0x0004, 0x00000056, 0x01a8, 0x00000057, 0x0057, 0x01ad, 0x00000058, 0x0058, 0x01b5, 0x00000059, 0x0059)
NTDLL(NtAccessCheckByTypeResultList                     , 0x0005, 0x0005, 0x0005, 0x0063, 0x0005, 0x0063, 0x0005, 0x00000063, 0x01a7, 0x00000064, 0x0064, 0x01ac, 0x00000064, 0x0064, 0x01b4, 0x00000064, 0x0064)
NTDLL(NtAccessCheckByTypeResultListAndAuditAlarm        , 0x0006, 0x0006, 0x0006, 0x0064, 0x0006, 0x0064, 0x0006, 0x00000064, 0x01a6, 0x00000065, 0x0065, 0x01ab, 0x00000065, 0x0065, 0x01b3, 0x00000065, 0x0065)
NTDLL(NtAccessCheckByTypeResultListAndAuditAlarmByHandle, 0x0007, 0x0007, 0x0007, 0x0065, 0x0007, 0x0065, 0x0007, 0x00000065, 0x01a5, 0x00000066, 0x0066, 0x01aa, 0x00000066, 0x0066, 0x01b2, 0x00000066, 0x0066)
NTDLL(NtAcquireCMFViewOwnership                         ,   NONE,   NONE,   NONE,   NONE, 0x0185, 0x0066,   NONE,       NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE)
NTDLL(NtAddAtom                                         , 0x0008, 0x0008, 0x0008, 0x0044, 0x0008, 0x0044, 0x0008, 0x00000044, 0x01a3, 0x000a0045, 0x0045, 0x01a8, 0x000a0046, 0x0046, 0x01b0, 0x000a0047, 0x0047)
NTDLL(NtAddAtomEx                                       ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE, 0x01a4, 0x00000067, 0x0067, 0x01a9, 0x00000067, 0x0067, 0x01b1, 0x00110067, 0x0067)
NTDLL(NtAddBootEntry                                    ,   NONE, 0x0009, 0x0009, 0x0066, 0x0009, 0x0067, 0x0009, 0x00000066, 0x01a2, 0x00000068, 0x0068, 0x01a7, 0x00000068, 0x0068, 0x01af, 0x00000068, 0x0068)
NTDLL(NtAddDriverEntry                                  ,   NONE,   NONE, 0x000a, 0x0067, 0x000a, 0x0068, 0x000a, 0x00000067, 0x01a1, 0x00000069, 0x0069, 0x01a6, 0x00000069, 0x0069, 0x01ae, 0x00000069, 0x0069)
NTDLL(NtAdjustGroupsToken                               , 0x0009, 0x000a, 0x000b, 0x0068, 0x000b, 0x0069, 0x000b, 0x00000068, 0x019f, 0x0000006a, 0x006a, 0x01a4, 0x0000006a, 0x006a, 0x01ac, 0x0000006a, 0x006a)
NTDLL(NtAdjustPrivilegesToken                           , 0x000a, 0x000b, 0x000c, 0x003e, 0x000c, 0x003e, 0x000c, 0x0000003e, 0x019e, 0x0000003f, 0x003f, 0x01a3, 0x00000040, 0x0040, 0x01ab, 0x00000041, 0x0041)
NTDLL(NtAdjustTokenClaimsAndDeviceGroups                ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE, 0x01a0, 0x0000006b, 0x006b, 0x01a5, 0x0000006b, 0x006b, 0x01ad, 0x0000006b, 0x006b)
NTDLL(NtAlertResumeThread                               , 0x000b, 0x000c, 0x000d, 0x0069, 0x000d, 0x006a, 0x000d, 0x00000069, 0x019d, 0x0007006c, 0x006c, 0x01a2, 0x0007006c, 0x006c, 0x01aa, 0x0007006c, 0x006c)
NTDLL(NtAlertThread                                     , 0x000c, 0x000d, 0x000e, 0x006a, 0x000e, 0x006b, 0x000e, 0x0000006a, 0x019c, 0x0003006d, 0x006d, 0x01a1, 0x0003006d, 0x006d, 0x01a9, 0x0003006d, 0x006d)
NTDLL(NtAlertThreadByThreadId                           ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE, 0x019b, 0x0004006e, 0x006e, 0x01a0, 0x0004006e, 0x006e, 0x01a8, 0x0004006e, 0x006e)
NTDLL(NtAllocateLocallyUniqueId                         , 0x000d, 0x000e, 0x000f, 0x006b, 0x000f, 0x006c, 0x000f, 0x0000006b, 0x019a, 0x0004006f, 0x006f, 0x019f, 0x0004006f, 0x006f, 0x01a7, 0x0004006f, 0x006f)
NTDLL(NtAllocateReserveObject                           ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE, 0x0010, 0x0000006c, 0x0199, 0x00000070, 0x0070, 0x019e, 0x00000070, 0x0070, 0x01a6, 0x00000070, 0x0070)
NTDLL(NtAllocateUserPhysicalPages                       , 0x000e, 0x000f, 0x0010, 0x006c, 0x0010, 0x006d, 0x0011, 0x0000006d, 0x0198, 0x00000071, 0x0071, 0x019d, 0x00000071, 0x0071, 0x01a5, 0x00000071, 0x0071)
NTDLL(NtAllocateUuids                                   , 0x000f, 0x0010, 0x0011, 0x006d, 0x0011, 0x006e, 0x0012, 0x0000006e, 0x0197, 0x00110072, 0x0072, 0x019c, 0x00110072, 0x0072, 0x01a4, 0x00110072, 0x0072)
NTDLL(NtAllocateVirtualMemory                           , 0x0010, 0x0011, 0x0012, 0x0015, 0x0012, 0x0015, 0x0013, 0x00000015, 0x0196, 0x00000016, 0x0016, 0x019b, 0x00000017, 0x0017, 0x01a3, 0x00000018, 0x0018)
NTDLL(NtAlpcAcceptConnectPort                           ,   NONE,   NONE,   NONE,   NONE, 0x0013, 0x006f, 0x0014, 0x0000006f, 0x0195, 0x00000073, 0x0073, 0x019a, 0x00000073, 0x0073, 0x01a2, 0x00000073, 0x0073)
NTDLL(NtAlpcCancelMessage                               ,   NONE,   NONE,   NONE,   NONE, 0x0014, 0x0070, 0x0015, 0x00000070, 0x0194, 0x00000074, 0x0074, 0x0199, 0x00000074, 0x0074, 0x01a1, 0x00000074, 0x0074)
NTDLL(NtAlpcConnectPort                                 ,   NONE,   NONE,   NONE,   NONE, 0x0015, 0x0071, 0x0016, 0x00000071, 0x0192, 0x00000075, 0x0075, 0x0197, 0x00000075, 0x0075, 0x019f, 0x00000075, 0x0075)
NTDLL(NtAlpcConnectPortEx                               ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE, 0x0193, 0x00000076, 0x0076, 0x0198, 0x00000076, 0x0076, 0x01a0, 0x00000076, 0x0076)
NTDLL(NtAlpcCreatePort                                  ,   NONE,   NONE,   NONE,   NONE, 0x0016, 0x0072, 0x0017, 0x00000072, 0x0191, 0x00000077, 0x0077, 0x0196, 0x00000077, 0x0077, 0x019e, 0x00000077, 0x0077)
NTDLL(NtAlpcCreatePortSection                           ,   NONE,   NONE,   NONE,   NONE, 0x0017, 0x0073, 0x0018, 0x00000073, 0x0190, 0x00000078, 0x0078, 0x0195, 0x00000078, 0x0078, 0x019d, 0x00000078, 0x0078)
NTDLL(NtAlpcCreateResourceReserve                       ,   NONE,   NONE,   NONE,   NONE, 0x0018, 0x0074, 0x0019, 0x00000074, 0x018f, 0x00000079, 0x0079, 0x0194, 0x00000079, 0x0079, 0x019c, 0x00000079, 0x0079)
NTDLL(NtAlpcCreateSectionView                           ,   NONE,   NONE,   NONE,   NONE, 0x0019, 0x0075, 0x001a, 0x00000075, 0x018e, 0x0000007a, 0x007a, 0x0193, 0x0000007a, 0x007a, 0x019b, 0x0000007a, 0x007a)
NTDLL(NtAlpcCreateSecurityContext                       ,   NONE,   NONE,   NONE,   NONE, 0x001a, 0x0076, 0x001b, 0x00000076, 0x018d, 0x0000007b, 0x007b, 0x0192, 0x0000007b, 0x007b, 0x019a, 0x0000007b, 0x007b)
NTDLL(NtAlpcDeletePortSection                           ,   NONE,   NONE,   NONE,   NONE, 0x001b, 0x0077, 0x001c, 0x00000077, 0x018c, 0x0000007c, 0x007c, 0x0191, 0x0000007c, 0x007c, 0x0199, 0x0000007c, 0x007c)
NTDLL(NtAlpcDeleteResourceReserve                       ,   NONE,   NONE,   NONE,   NONE, 0x001c, 0x0078, 0x001d, 0x00000078, 0x018b, 0x0000007d, 0x007d, 0x0190, 0x0000007d, 0x007d, 0x0198, 0x0000007d, 0x007d)
NTDLL(NtAlpcDeleteSectionView                           ,   NONE,   NONE,   NONE,   NONE, 0x001d, 0x0079, 0x001e, 0x00000079, 0x018a, 0x0000007e, 0x007e, 0x018f, 0x0000007e, 0x007e, 0x0197, 0x0000007e, 0x007e)
NTDLL(NtAlpcDeleteSecurityContext                       ,   NONE,   NONE,   NONE,   NONE, 0x001e, 0x007a, 0x001f, 0x0000007a, 0x0189, 0x0000007f, 0x007f, 0x018e, 0x0000007f, 0x007f, 0x0196, 0x0000007f, 0x007f)
NTDLL(NtAlpcDisconnectPort                              ,   NONE,   NONE,   NONE,   NONE, 0x001f, 0x007b, 0x0020, 0x0000007b, 0x0188, 0x00000080, 0x0080, 0x018d, 0x00000080, 0x0080, 0x0195, 0x00000080, 0x0080)
NTDLL(NtAlpcImpersonateClientContainerOfPort            ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE, 0x0194, 0x00000081, 0x0081)
NTDLL(NtAlpcImpersonateClientOfPort                     ,   NONE,   NONE,   NONE,   NONE, 0x0020, 0x007c, 0x0021, 0x0000007c, 0x0187, 0x00000081, 0x0081, 0x018c, 0x00000081, 0x0081, 0x0193, 0x00000082, 0x0082)
NTDLL(NtAlpcOpenSenderProcess                           ,   NONE,   NONE,   NONE,   NONE, 0x0021, 0x007d, 0x0022, 0x0000007d, 0x0186, 0x00000082, 0x0082, 0x018b, 0x00000082, 0x0082, 0x0192, 0x00000083, 0x0083)
NTDLL(NtAlpcOpenSenderThread                            ,   NONE,   NONE,   NONE,   NONE, 0x0022, 0x007e, 0x0023, 0x0000007e, 0x0185, 0x00000083, 0x0083, 0x018a, 0x00000083, 0x0083, 0x0191, 0x00000084, 0x0084)
NTDLL(NtAlpcQueryInformation                            ,   NONE,   NONE,   NONE,   NONE, 0x0023, 0x007f, 0x0024, 0x0000007f, 0x0184, 0x00000084, 0x0084, 0x0189, 0x00000084, 0x0084, 0x0190, 0x00000085, 0x0085)
NTDLL(NtAlpcQueryInformationMessage                     ,   NONE,   NONE,   NONE,   NONE, 0x0024, 0x0080, 0x0025, 0x00000080, 0x0183, 0x00000085, 0x0085, 0x0188, 0x00000085, 0x0085, 0x018f, 0x00000086, 0x0086)
NTDLL(NtAlpcRevokeSecurityContext                       ,   NONE,   NONE,   NONE,   NONE, 0x0025, 0x0081, 0x0026, 0x00000081, 0x0182, 0x00000086, 0x0086, 0x0187, 0x00000086, 0x0086, 0x018e, 0x00000087, 0x0087)
NTDLL(NtAlpcSendWaitReceivePort                         ,   NONE,   NONE,   NONE,   NONE, 0x0026, 0x0082, 0x0027, 0x00000082, 0x0181, 0x00000087, 0x0087, 0x0186, 0x00000087, 0x0087, 0x018d, 0x00000088, 0x0088)
NTDLL(NtAlpcSetInformation                              ,   NONE,   NONE,   NONE,   NONE, 0x0027, 0x0083, 0x0028, 0x00000083, 0x0180, 0x00000088, 0x0088, 0x0185, 0x00000088, 0x0088, 0x018c, 0x00000089, 0x0089)
NTDLL(NtApphelpCacheControl                             ,   NONE,   NONE, 0x0013, 0x0049, 0x0028, 0x0049, 0x0029, 0x00000049, 0x017f, 0x0000004a, 0x004a, 0x0184, 0x0000004b, 0x004b, 0x018b, 0x0000004c, 0x004c)
NTDLL(NtAreMappedFilesTheSame                           , 0x0011, 0x0012, 0x0014, 0x006e, 0x0029, 0x0084, 0x002a, 0x00000084, 0x017e, 0x00050089, 0x0089, 0x0183, 0x00050089, 0x0089, 0x018a, 0x0005008a, 0x008a)
NTDLL(NtAssignProcessToJobObject                        , 0x0012, 0x0013, 0x0015, 0x006f, 0x002a, 0x0085, 0x002b, 0x00000085, 0x017d, 0x0008008a, 0x008a, 0x0182, 0x0008008a, 0x008a, 0x0189, 0x0008008b, 0x008b)
NTDLL(NtAssociateWaitCompletionPacket                   ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE, 0x017c, 0x0000008b, 0x008b, 0x0181, 0x0000008b, 0x008b, 0x0188, 0x0000008c, 0x008c)
NTDLL(NtCallbackReturn                                  , 0x0013, 0x0014, 0x0016, 0x0002, 0x002b, 0x0002, 0x002c, 0x00000002, 0x017b, 0x00000003, 0x0003, 0x0180, 0x00000004, 0x0004, 0x0187, 0x00000005, 0x0005)
NTDLL(NtCancelDeviceWakeupRequest                       , 0x0016, 0x0015, 0x0017, 0x0070, 0x002c, 0x0086,   NONE,       NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE)
NTDLL(NtCancelIoFile                                    , 0x0014, 0x0016, 0x0018, 0x005a, 0x002d, 0x005a, 0x002d, 0x0000005a, 0x017a, 0x0000005b, 0x005b, 0x017f, 0x0000005c, 0x005c, 0x0186, 0x0000005d, 0x005d)
NTDLL(NtCancelIoFileEx                                  ,   NONE,   NONE,   NONE,   NONE, 0x0171, 0x0087, 0x002e, 0x00000086, 0x0179, 0x0000008c, 0x008c, 0x017e, 0x0000008c, 0x008c, 0x0185, 0x0000008d, 0x008d)
NTDLL(NtCancelSynchronousIoFile                         ,   NONE,   NONE,   NONE,   NONE, 0x0172, 0x0088, 0x002f, 0x00000087, 0x0178, 0x0000008d, 0x008d, 0x017d, 0x0000008d, 0x008d, 0x0184, 0x0000008e, 0x008e)
NTDLL(NtCancelTimer                                     , 0x0015, 0x0017, 0x0019, 0x005e, 0x002e, 0x005e, 0x0030, 0x0000005e, 0x0177, 0x0007005f, 0x005f, 0x017c, 0x00070060, 0x0060, 0x0183, 0x00070061, 0x0061)
NTDLL(NtCancelTimer2                                    ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE,   NONE,       NONE,   NONE, 0x0043, 0x0000008e, 0x008e, 0x0044, 0x0000008f, 0x008f)
NTDLL(NtCancelWaitCompletionPacket                      ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE, 0x0176, 0x0000008e, 0x008e, 0x017b, 0x0000008f, 0x008f, 0x0182, 0x00000090, 0x0090)
NTDLL(NtClearEvent                                      , 0x0017, 0x0018, 0x001a, 0x003b, 0x002f, 0x003b, 0x0031, 0x0000003b, 0x0175, 0x0003003c, 0x003c, 0x017a, 0x0003003d, 0x003d, 0x0181, 0x0003003e, 0x003e)
NTDLL(NtClose                                           , 0x0018, 0x0019, 0x001b, 0x000c, 0x0030, 0x000c, 0x0032, 0x0000000c, 0x0174, 0x0000000d, 0x000d, 0x0179, 0x0003000e, 0x000e, 0x0180, 0x0003000f, 0x000f)
NTDLL(NtCloseObjectAuditAlarm                           , 0x0019, 0x001a, 0x001c, 0x0038, 0x0031, 0x0038, 0x0033, 0x00000038, 0x0173, 0x00000039, 0x0039, 0x0178, 0x0000003a, 0x003a, 0x017f, 0x0000003b, 0x003b)
NTDLL(NtCommitComplete                                  ,   NONE,   NONE,   NONE,   NONE, 0x005f, 0x0089, 0x0034, 0x00000088, 0x0172, 0x0000008f, 0x008f, 0x0177, 0x00000090, 0x0090, 0x017e, 0x00000091, 0x0091)
NTDLL(NtCommitEnlistment                                ,   NONE,   NONE,   NONE,   NONE, 0x0057, 0x008a, 0x0035, 0x00000089, 0x0171, 0x00000090, 0x0090, 0x0176, 0x00000091, 0x0091, 0x017d, 0x00000092, 0x0092)
NTDLL(NtCommitTransaction                               ,   NONE,   NONE,   NONE,   NONE, 0x005b, 0x008b, 0x0036, 0x0000008a, 0x0170, 0x00000091, 0x0091, 0x0175, 0x00000092, 0x0092, 0x017c, 0x00000093, 0x0093)
NTDLL(NtCompactKeys                                     ,   NONE, 0x001b, 0x001d, 0x0071, 0x0032, 0x008c, 0x0037, 0x0000008b, 0x016f, 0x00000092, 0x0092, 0x0174, 0x00000093, 0x0093, 0x017b, 0x00000094, 0x0094)
NTDLL(NtCompareObjects                                  ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE, 0x017a, 0x00000095, 0x0095)
NTDLL(NtCompareTokens                                   ,   NONE, 0x001c, 0x001e, 0x0072, 0x0033, 0x008d, 0x0038, 0x0000008c, 0x016e, 0x00000093, 0x0093, 0x0173, 0x00000094, 0x0094, 0x0179, 0x00000096, 0x0096)
NTDLL(NtCompleteConnectPort                             , 0x001a, 0x001d, 0x001f, 0x0073, 0x0034, 0x008e, 0x0039, 0x0000008d, 0x016d, 0x00000094, 0x0094, 0x0172, 0x00000095, 0x0095, 0x0178, 0x00000097, 0x0097)
NTDLL(NtCompressKey                                     ,   NONE, 0x001e, 0x0020, 0x0074, 0x0035, 0x008f, 0x003a, 0x0000008e, 0x016c, 0x00030095, 0x0095, 0x0171, 0x00030096, 0x0096, 0x0177, 0x00030098, 0x0098)
NTDLL(NtConnectPort                                     , 0x001b, 0x001f, 0x0021, 0x0075, 0x0036, 0x0090, 0x003b, 0x0000008f, 0x016b, 0x00000096, 0x0096, 0x0170, 0x00000097, 0x0097, 0x0176, 0x00000099, 0x0099)
NTDLL(NtContinue                                        , 0x001c, 0x0020, 0x0022, 0x0040, 0x0037, 0x0040, 0x003c, 0x00000040, 0x016a, 0x00000041, 0x0041, 0x016f, 0x00000042, 0x0042, 0x0175, 0x00000043, 0x0043)
NTDLL(NtCreateChannel                                   , 0x00f1,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE)
NTDLL(NtCreateDebugObject                               ,   NONE, 0x0021, 0x0023, 0x0076, 0x0038, 0x0091, 0x003d, 0x00000090, 0x0169, 0x00000097, 0x0097, 0x016e, 0x00000098, 0x0098, 0x0174, 0x0000009a, 0x009a)
NTDLL(NtCreateDirectoryObject                           , 0x001d, 0x0022, 0x0024, 0x0077, 0x0039, 0x0092, 0x003e, 0x00000091, 0x0168, 0x00000098, 0x0098, 0x016d, 0x00000099, 0x0099, 0x0173, 0x0000009b, 0x009b)
NTDLL(NtCreateDirectoryObjectEx                         ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE, 0x0167, 0x00000099, 0x0099, 0x016c, 0x0000009a, 0x009a, 0x0172, 0x0000009c, 0x009c)
NTDLL(NtCreateEnlistment                                ,   NONE,   NONE,   NONE,   NONE, 0x006f, 0x0093, 0x003f, 0x00000092, 0x0166, 0x0000009a, 0x009a, 0x016b, 0x0000009b, 0x009b, 0x0171, 0x0000009d, 0x009d)
NTDLL(NtCreateEvent                                     , 0x001e, 0x0023, 0x0025, 0x0045, 0x003a, 0x0045, 0x0040, 0x00000045, 0x0165, 0x00000046, 0x0046, 0x016a, 0x00000047, 0x0047, 0x0170, 0x00000048, 0x0048)
NTDLL(NtCreateEventPair                                 , 0x001f, 0x0024, 0x0026, 0x0078, 0x003b, 0x0094, 0x0041, 0x00000093, 0x0164, 0x0000009b, 0x009b, 0x0169, 0x0000009c, 0x009c, 0x016f, 0x0000009e, 0x009e)
NTDLL(NtCreateFile                                      , 0x0020, 0x0025, 0x0027, 0x0052, 0x003c, 0x0052, 0x0042, 0x00000052, 0x0163, 0x00000053, 0x0053, 0x0168, 0x00000054, 0x0054, 0x016e, 0x00000055, 0x0055)
NTDLL(NtCreateIRTimer                                   ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE, 0x0161, 0x0000009c, 0x009c, 0x0165, 0x0000009d, 0x009d, 0x016b, 0x0000009f, 0x009f)
NTDLL(NtCreateIoCompletion                              , 0x0021, 0x0026, 0x0028, 0x0079, 0x003d, 0x0095, 0x0043, 0x00000094, 0x0162, 0x0000009d, 0x009d, 0x0167, 0x0000009e, 0x009e, 0x016d, 0x000000a0, 0x00a0)
NTDLL(NtCreateJobObject                                 , 0x0022, 0x0027, 0x0029, 0x007a, 0x003e, 0x0096, 0x0044, 0x00000095, 0x0160, 0x0000009e, 0x009e, 0x0164, 0x0000009f, 0x009f, 0x016a, 0x000000a1, 0x00a1)
NTDLL(NtCreateJobSet                                    ,   NONE, 0x0028, 0x002a, 0x007b, 0x003f, 0x0097, 0x0045, 0x00000096, 0x015f, 0x0000009f, 0x009f, 0x0163, 0x000000a0, 0x00a0, 0x0169, 0x000000a2, 0x00a2)
NTDLL(NtCreateKey                                       , 0x0023, 0x0029, 0x002b, 0x001a, 0x0040, 0x001a, 0x0046, 0x0000001a, 0x015e, 0x0000001b, 0x001b, 0x0162, 0x0000001c, 0x001c, 0x0168, 0x0000001d, 0x001d)
NTDLL(NtCreateKeyTransacted                             ,   NONE,   NONE,   NONE,   NONE, 0x0041, 0x0098, 0x0048, 0x00000097, 0x015c, 0x000000a0, 0x00a0, 0x0160, 0x000000a1, 0x00a1, 0x0166, 0x000000a3, 0x00a3)
NTDLL(NtCreateKeyedEvent                                ,   NONE, 0x0117, 0x0121, 0x007c, 0x0168, 0x0099, 0x0047, 0x00000098, 0x015d, 0x000000a1, 0x00a1, 0x0161, 0x000000a2, 0x00a2, 0x0167, 0x000000a4, 0x00a4)
NTDLL(NtCreateLowBoxToken                               ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE, 0x0149, 0x000000a2, 0x00a2, 0x014d, 0x000000a3, 0x00a3, 0x0153, 0x000000a5, 0x00a5)
NTDLL(NtCreateMailslotFile                              , 0x0024, 0x002a, 0x002c, 0x007d, 0x0042, 0x009a, 0x0049, 0x00000099, 0x015b, 0x000000a3, 0x00a3, 0x015f, 0x000000a4, 0x00a4, 0x0165, 0x000000a6, 0x00a6)
NTDLL(NtCreateMutant                                    , 0x0025, 0x002b, 0x002d, 0x007e, 0x0043, 0x009b, 0x004a, 0x0000009a, 0x015a, 0x000000a4, 0x00a4, 0x015e, 0x000000a5, 0x00a5, 0x0164, 0x000000a7, 0x00a7)
NTDLL(NtCreateNamedPipeFile                             , 0x0026, 0x002c, 0x002e, 0x007f, 0x0044, 0x009c, 0x004b, 0x0000009b, 0x0159, 0x000000a5, 0x00a5, 0x015d, 0x000000a6, 0x00a6, 0x0163, 0x000000a8, 0x00a8)
NTDLL(NtCreatePagingFile                                , 0x0027, 0x002d, 0x002f, 0x0080, 0x0046, 0x009d, 0x004c, 0x0000009c, 0x0158, 0x000000a6, 0x00a6, 0x015c, 0x000000a7, 0x00a7, 0x0162, 0x000000a9, 0x00a9)
NTDLL(NtCreatePartition                                 ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE, 0x014a, 0x000000aa, 0x00aa)
NTDLL(NtCreatePort                                      , 0x0028, 0x002e, 0x0030, 0x0081, 0x0047, 0x009e, 0x004d, 0x0000009d, 0x0157, 0x000000a7, 0x00a7, 0x015b, 0x000000a8, 0x00a8, 0x0161, 0x000000ab, 0x00ab)
NTDLL(NtCreatePrivateNamespace                          ,   NONE,   NONE,   NONE,   NONE, 0x0045, 0x009f, 0x004e, 0x0000009e, 0x0156, 0x000000a8, 0x00a8, 0x015a, 0x000000a9, 0x00a9, 0x0160, 0x000000ac, 0x00ac)
NTDLL(NtCreateProcess                                   , 0x0029, 0x002f, 0x0031, 0x0082, 0x0048, 0x00a0, 0x004f, 0x0000009f, 0x0155, 0x000000a9, 0x00a9, 0x0159, 0x000000aa, 0x00aa, 0x015f, 0x000000ad, 0x00ad)
NTDLL(NtCreateProcessEx                                 ,   NONE, 0x0030, 0x0032, 0x004a, 0x0049, 0x004a, 0x0050, 0x0000004a, 0x0154, 0x0000004b, 0x004b, 0x0158, 0x0000004c, 0x004c, 0x015e, 0x0000004d, 0x004d)
NTDLL(NtCreateProfile                                   , 0x002a, 0x0031, 0x0033, 0x0083, 0x004a, 0x00a1, 0x0051, 0x000000a0, 0x0153, 0x000000aa, 0x00aa, 0x0157, 0x000000ab, 0x00ab, 0x015d, 0x000000ae, 0x00ae)
NTDLL(NtCreateProfileEx                                 ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE, 0x0052, 0x000000a1, 0x0152, 0x000000ab, 0x00ab, 0x0156, 0x000000ac, 0x00ac, 0x015c, 0x000000af, 0x00af)
NTDLL(NtCreateResourceManager                           ,   NONE,   NONE,   NONE,   NONE, 0x006b, 0x00a2, 0x0053, 0x000000a2, 0x0151, 0x000000ac, 0x00ac, 0x0155, 0x000000ad, 0x00ad, 0x015b, 0x000000b0, 0x00b0)
NTDLL(NtCreateSection                                   , 0x002b, 0x0032, 0x0034, 0x0047, 0x004b, 0x0047, 0x0054, 0x00000047, 0x0150, 0x00000048, 0x0048, 0x0154, 0x00000049, 0x0049, 0x015a, 0x0000004a, 0x004a)
NTDLL(NtCreateSemaphore                                 , 0x002c, 0x0033, 0x0035, 0x0084, 0x004c, 0x00a3, 0x0055, 0x000000a3, 0x014f, 0x000000ad, 0x00ad, 0x0153, 0x000000ae, 0x00ae, 0x0159, 0x000000b1, 0x00b1)
NTDLL(NtCreateSymbolicLinkObject                        , 0x002d, 0x0034, 0x0036, 0x0085, 0x004d, 0x00a4, 0x0056, 0x000000a4, 0x014e, 0x000000ae, 0x00ae, 0x0152, 0x000000af, 0x00af, 0x0158, 0x000000b2, 0x00b2)
NTDLL(NtCreateThread                                    , 0x002e, 0x0035, 0x0037, 0x004b, 0x004e, 0x004b, 0x0057, 0x0000004b, 0x014d, 0x0000004c, 0x004c, 0x0151, 0x0000004d, 0x004d, 0x0157, 0x0000004e, 0x004e)
NTDLL(NtCreateThreadEx                                  ,   NONE,   NONE,   NONE,   NONE, 0x017e, 0x00a5, 0x0058, 0x000000a5, 0x014c, 0x000000af, 0x00af, 0x0150, 0x000000b0, 0x00b0, 0x0156, 0x000000b3, 0x00b3)
NTDLL(NtCreateTimer                                     , 0x002f, 0x0036, 0x0038, 0x0086, 0x004f, 0x00a6, 0x0059, 0x000000a6, 0x014b, 0x000000b0, 0x00b0, 0x014f, 0x000000b1, 0x00b1, 0x0155, 0x000000b4, 0x00b4)
NTDLL(NtCreateTimer2                                    ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE,   NONE,       NONE,   NONE, 0x0166, 0x000000b2, 0x00b2, 0x016c, 0x000000b5, 0x00b5)
NTDLL(NtCreateToken                                     , 0x0030, 0x0037, 0x0039, 0x0087, 0x0050, 0x00a7, 0x005a, 0x000000a7, 0x0148, 0x000000b1, 0x00b1, 0x014c, 0x000000b3, 0x00b3, 0x0152, 0x000000b6, 0x00b6)
NTDLL(NtCreateTokenEx                                   ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE, 0x014a, 0x000000b2, 0x00b2, 0x014e, 0x000000b4, 0x00b4, 0x0154, 0x000000b7, 0x00b7)
NTDLL(NtCreateTransaction                               ,   NONE,   NONE,   NONE,   NONE, 0x0051, 0x00a8, 0x005b, 0x000000a8, 0x0147, 0x000000b3, 0x00b3, 0x014b, 0x000000b5, 0x00b5, 0x0151, 0x000000b8, 0x00b8)
NTDLL(NtCreateTransactionManager                        ,   NONE,   NONE,   NONE,   NONE, 0x0064, 0x00a9, 0x005c, 0x000000a9, 0x0146, 0x000000b4, 0x00b4, 0x014a, 0x000000b6, 0x00b6, 0x0150, 0x000000b9, 0x00b9)
NTDLL(NtCreateUserProcess                               ,   NONE,   NONE,   NONE,   NONE, 0x017f, 0x00aa, 0x005d, 0x000000aa, 0x0145, 0x000000b5, 0x00b5, 0x0149, 0x000000b7, 0x00b7, 0x014f, 0x000000ba, 0x00ba)
NTDLL(NtCreateWaitCompletionPacket                      ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE, 0x0143, 0x000000b6, 0x00b6, 0x0147, 0x000000b8, 0x00b8, 0x014d, 0x000000bb, 0x00bb)
NTDLL(NtCreateWaitablePort                              , 0x0031, 0x0038, 0x003a, 0x0088, 0x0073, 0x00ab, 0x005e, 0x000000ab, 0x0144, 0x000000b7, 0x00b7, 0x0148, 0x000000b9, 0x00b9, 0x014e, 0x000000bc, 0x00bc)
NTDLL(NtCreateWnfStateName                              ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE, 0x0142, 0x000000b8, 0x00b8, 0x0146, 0x000000ba, 0x00ba, 0x014c, 0x000000bd, 0x00bd)
NTDLL(NtCreateWorkerFactory                             ,   NONE,   NONE,   NONE,   NONE, 0x0177, 0x00ac, 0x005f, 0x000000ac, 0x0141, 0x000000b9, 0x00b9, 0x0145, 0x000000bb, 0x00bb, 0x014b, 0x000000be, 0x00be)
NTDLL(NtDebugActiveProcess                              ,   NONE, 0x0039, 0x003b, 0x0089, 0x0074, 0x00ad, 0x0060, 0x000000ad, 0x0140, 0x000800ba, 0x00ba, 0x0144, 0x000800bc, 0x00bc, 0x0149, 0x000800bf, 0x00bf)
NTDLL(NtDebugContinue                                   ,   NONE, 0x003a, 0x003c, 0x008a, 0x0075, 0x00ae, 0x0061, 0x000000ae, 0x013f, 0x000000bb, 0x00bb, 0x0143, 0x000000bd, 0x00bd, 0x0148, 0x000000c0, 0x00c0)
NTDLL(NtDelayExecution                                  , 0x0032, 0x003b, 0x003d, 0x0031, 0x0076, 0x0031, 0x0062, 0x00000031, 0x013e, 0x00060032, 0x0032, 0x0142, 0x00060033, 0x0033, 0x0147, 0x00060034, 0x0034)
NTDLL(NtDeleteAtom                                      , 0x0033, 0x003c, 0x003e, 0x008b, 0x0077, 0x00af, 0x0063, 0x000000af, 0x013d, 0x000400bc, 0x00bc, 0x0141, 0x000400be, 0x00be, 0x0146, 0x000400c1, 0x00c1)
NTDLL(NtDeleteBootEntry                                 ,   NONE, 0x003d, 0x003f, 0x008c, 0x0078, 0x00b0, 0x0064, 0x000000b0, 0x013c, 0x000000bd, 0x00bd, 0x0140, 0x000000bf, 0x00bf, 0x0145, 0x000000c2, 0x00c2)
NTDLL(NtDeleteDriverEntry                               ,   NONE,   NONE, 0x0040, 0x008d, 0x0079, 0x00b1, 0x0065, 0x000000b1, 0x013b, 0x000000be, 0x00be, 0x013f, 0x000000c0, 0x00c0, 0x0144, 0x000000c3, 0x00c3)
NTDLL(NtDeleteFile                                      , 0x0034, 0x003e, 0x0041, 0x008e, 0x007a, 0x00b2, 0x0066, 0x000000b2, 0x013a, 0x000000bf, 0x00bf, 0x013e, 0x000000c1, 0x00c1, 0x0143, 0x000000c4, 0x00c4)
NTDLL(NtDeleteKey                                       , 0x0035, 0x003f, 0x0042, 0x008f, 0x007b, 0x00b3, 0x0067, 0x000000b3, 0x0139, 0x000000c0, 0x00c0, 0x013d, 0x000000c2, 0x00c2, 0x0142, 0x000000c5, 0x00c5)
NTDLL(NtDeleteObjectAuditAlarm                          , 0x0036, 0x0040, 0x0043, 0x0090, 0x007d, 0x00b4, 0x0068, 0x000000b4, 0x0138, 0x000000c1, 0x00c1, 0x013c, 0x000000c3, 0x00c3, 0x0141, 0x000000c6, 0x00c6)
NTDLL(NtDeletePrivateNamespace                          ,   NONE,   NONE,   NONE,   NONE, 0x007c, 0x00b5, 0x0069, 0x000000b5, 0x0137, 0x000000c2, 0x00c2, 0x013b, 0x000000c4, 0x00c4, 0x0140, 0x000000c7, 0x00c7)
NTDLL(NtDeleteValueKey                                  , 0x0037, 0x0041, 0x0044, 0x0091, 0x007e, 0x00b6, 0x006a, 0x000000b6, 0x0136, 0x000000c3, 0x00c3, 0x013a, 0x000000c5, 0x00c5, 0x013f, 0x000000c8, 0x00c8)
NTDLL(NtDeleteWnfStateData                              ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE, 0x0135, 0x000000c4, 0x00c4, 0x0139, 0x000000c6, 0x00c6, 0x013e, 0x000000c9, 0x00c9)
NTDLL(NtDeleteWnfStateName                              ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE, 0x0134, 0x000000c5, 0x00c5, 0x0138, 0x000000c7, 0x00c7, 0x013d, 0x000000ca, 0x00ca)
NTDLL(NtDeviceIoControlFile                             , 0x0038, 0x0042, 0x0045, 0x0004, 0x007f, 0x0004, 0x006b, 0x00000004, 0x0133, 0x001b0005, 0x0005, 0x0137, 0x001b0006, 0x0006, 0x013c, 0x001b0007, 0x0007)
NTDLL(NtDisableLastKnownGood                            ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE, 0x006c, 0x000000b7, 0x0132, 0x000000c6, 0x00c6, 0x0136, 0x000000c8, 0x00c8, 0x013b, 0x000000cb, 0x00cb)
NTDLL(NtDisplayString                                   , 0x0039, 0x0043, 0x0046, 0x0092, 0x0080, 0x00b7, 0x006d, 0x000000b8, 0x0131, 0x000000c7, 0x00c7, 0x0135, 0x000000c9, 0x00c9, 0x013a, 0x000000cc, 0x00cc)
NTDLL(NtDrawText                                        ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE, 0x006e, 0x000000b9, 0x0130, 0x000000c8, 0x00c8, 0x0134, 0x000000ca, 0x00ca, 0x0139, 0x000000cd, 0x00cd)
NTDLL(NtDuplicateObject                                 , 0x003a, 0x0044, 0x0047, 0x0039, 0x0081, 0x0039, 0x006f, 0x00000039, 0x012f, 0x0000003a, 0x003a, 0x0133, 0x0000003b, 0x003b, 0x0138, 0x0000003c, 0x003c)
NTDLL(NtDuplicateToken                                  , 0x003b, 0x0045, 0x0048, 0x003f, 0x0082, 0x003f, 0x0070, 0x0000003f, 0x012e, 0x00000040, 0x0040, 0x0132, 0x00000041, 0x0041, 0x0137, 0x00000042, 0x0042)
NTDLL(NtEnableLastKnownGood                             ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE, 0x0071, 0x000000ba, 0x012d, 0x000000c9, 0x00c9, 0x0131, 0x000000cb, 0x00cb, 0x0136, 0x000000ce, 0x00ce)
NTDLL(NtEnumerateBootEntries                            ,   NONE, 0x0046, 0x0049, 0x0093, 0x0083, 0x00b8, 0x0072, 0x000000bb, 0x012c, 0x000000ca, 0x00ca, 0x0130, 0x000000cc, 0x00cc, 0x0135, 0x000000cf, 0x00cf)
NTDLL(NtEnumerateDriverEntries                          ,   NONE,   NONE, 0x004a, 0x0094, 0x0084, 0x00b9, 0x0073, 0x000000bc, 0x012b, 0x000000cb, 0x00cb, 0x012f, 0x000000cd, 0x00cd, 0x0134, 0x000000d0, 0x00d0)
NTDLL(NtEnumerateKey                                    , 0x003c, 0x0047, 0x004b, 0x002f, 0x0085, 0x002f, 0x0074, 0x0000002f, 0x012a, 0x00000030, 0x0030, 0x012e, 0x00000031, 0x0031, 0x0133, 0x00000032, 0x0032)
NTDLL(NtEnumerateSystemEnvironmentValuesEx              ,   NONE, 0x0048, 0x004c, 0x0095, 0x0086, 0x00ba, 0x0075, 0x000000bd, 0x0129, 0x000000cc, 0x00cc, 0x012d, 0x000000ce, 0x00ce, 0x0132, 0x000000d1, 0x00d1)
NTDLL(NtEnumerateTransactionObject                      ,   NONE,   NONE,   NONE,   NONE, 0x0087, 0x00bb, 0x0076, 0x000000be, 0x0128, 0x000000cd, 0x00cd, 0x012c, 0x000000cf, 0x00cf, 0x0131, 0x000000d2, 0x00d2)
NTDLL(NtEnumerateValueKey                               , 0x003d, 0x0049, 0x004d, 0x0010, 0x0088, 0x0010, 0x0077, 0x00000010, 0x0127, 0x00000011, 0x0011, 0x012b, 0x00000012, 0x0012, 0x0130, 0x00000013, 0x0013)
NTDLL(NtExtendSection                                   , 0x003e, 0x004a, 0x004e, 0x0096, 0x0089, 0x00bc, 0x0078, 0x000000bf, 0x0126, 0x000000ce, 0x00ce, 0x012a, 0x000000d0, 0x00d0, 0x012f, 0x000000d3, 0x00d3)
NTDLL(NtFilterBootOption                                ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE, 0x0125, 0x000000cf, 0x00cf, 0x0129, 0x000000d1, 0x00d1, 0x012e, 0x000000d4, 0x00d4)
NTDLL(NtFilterToken                                     , 0x003f, 0x004b, 0x004f, 0x0097, 0x008a, 0x00bd, 0x0079, 0x000000c0, 0x0123, 0x000000d0, 0x00d0, 0x0127, 0x000000d2, 0x00d2, 0x012c, 0x000000d5, 0x00d5)
NTDLL(NtFilterTokenEx                                   ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE, 0x0124, 0x000000d1, 0x00d1, 0x0128, 0x000000d3, 0x00d3, 0x012d, 0x000000d6, 0x00d6)
NTDLL(NtFindAtom                                        , 0x0040, 0x004c, 0x0050, 0x0011, 0x008b, 0x0011, 0x007a, 0x00000011, 0x0122, 0x000a0012, 0x0012, 0x0126, 0x000a0013, 0x0013, 0x012b, 0x000a0014, 0x0014)
NTDLL(NtFlushBuffersFile                                , 0x0041, 0x004d, 0x0051, 0x0048, 0x008c, 0x0048, 0x007b, 0x00000048, 0x0120, 0x00000049, 0x0049, 0x0124, 0x0000004a, 0x004a, 0x0129, 0x0000004b, 0x004b)
NTDLL(NtFlushBuffersFileEx                              ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE, 0x0121, 0x000000d2, 0x00d2, 0x0125, 0x000000d4, 0x00d4, 0x012a, 0x000000d7, 0x00d7)
NTDLL(NtFlushInstallUILanguage                          ,   NONE,   NONE,   NONE,   NONE, 0x0183, 0x00be, 0x007c, 0x000000c1, 0x011f, 0x000000d3, 0x00d3, 0x0123, 0x000000d5, 0x00d5, 0x0128, 0x000000d8, 0x00d8)
NTDLL(NtFlushInstructionCache                           , 0x0042, 0x004e, 0x0052, 0x0098, 0x008d, 0x00bf, 0x007d, 0x000000c2, 0x011e, 0x000c00d4, 0x00d4, 0x0122, 0x000c00d6, 0x00d6, 0x0127, 0x000c00d9, 0x00d9)
NTDLL(NtFlushKey                                        , 0x0043, 0x004f, 0x0053, 0x0099, 0x008e, 0x00c0, 0x007e, 0x000000c3, 0x011d, 0x000300d5, 0x00d5, 0x0121, 0x000300d7, 0x00d7, 0x0126, 0x000300da, 0x00da)
NTDLL(NtFlushProcessWriteBuffers                        ,   NONE,   NONE,   NONE,   NONE, 0x008f, 0x00c1, 0x007f, 0x000000c4, 0x011c, 0x000000d6, 0x00d6, 0x0120, 0x000000d8, 0x00d8, 0x0125, 0x000000db, 0x00db)
NTDLL(NtFlushVirtualMemory                              , 0x0044, 0x0050, 0x0054, 0x009a, 0x0090, 0x00c2, 0x0080, 0x000000c5, 0x011b, 0x000000d7, 0x00d7, 0x011f, 0x000000d9, 0x00d9, 0x0124, 0x000000dc, 0x00dc)
NTDLL(NtFlushWriteBuffer                                , 0x0045, 0x0051, 0x0055, 0x009b, 0x0091, 0x00c3, 0x0081, 0x000000c6, 0x011a, 0x000100d8, 0x00d8, 0x011e, 0x000100da, 0x00da, 0x0123, 0x000100dd, 0x00dd)
NTDLL(NtFreeUserPhysicalPages                           , 0x0046, 0x0052, 0x0056, 0x009c, 0x0092, 0x00c4, 0x0082, 0x000000c7, 0x0119, 0x000000d9, 0x00d9, 0x011d, 0x000000db, 0x00db, 0x0122, 0x000000de, 0x00de)
NTDLL(NtFreeVirtualMemory                               , 0x0047, 0x0053, 0x0057, 0x001b, 0x0093, 0x001b, 0x0083, 0x0000001b, 0x0118, 0x0000001c, 0x001c, 0x011c, 0x0000001d, 0x001d, 0x0121, 0x0000001e, 0x001e)
NTDLL(NtFreezeRegistry                                  ,   NONE,   NONE,   NONE,   NONE, 0x0094, 0x00c5, 0x0084, 0x000000c8, 0x0117, 0x000000da, 0x00da, 0x011b, 0x000000dc, 0x00dc, 0x0120, 0x000000df, 0x00df)
NTDLL(NtFreezeTransactions                              ,   NONE,   NONE,   NONE,   NONE, 0x0095, 0x00c6, 0x0085, 0x000000c9, 0x0116, 0x000000db, 0x00db, 0x011a, 0x000000dd, 0x00dd, 0x011f, 0x000000e0, 0x00e0)
NTDLL(NtFsControlFile                                   , 0x0048, 0x0054, 0x0058, 0x0036, 0x0096, 0x0036, 0x0086, 0x00000036, 0x0115, 0x001b0037, 0x0037, 0x0119, 0x001b0038, 0x0038, 0x011e, 0x001b0039, 0x0039)
NTDLL(NtGetCachedSigningLevel                           ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE, 0x0114, 0x000000dc, 0x00dc, 0x0118, 0x000000de, 0x00de, 0x011d, 0x000000e1, 0x00e1)
NTDLL(NtGetCompleteWnfStateSubscription                 ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE,   NONE,       NONE,   NONE, 0x0117, 0x000000df, 0x00df, 0x011c, 0x000000e2, 0x00e2)
NTDLL(NtGetContextThread                                , 0x0049, 0x0055, 0x0059, 0x009d, 0x0097, 0x00c7, 0x0087, 0x000000ca, 0x0113, 0x000000dd, 0x00dd, 0x0116, 0x000000e0, 0x00e0, 0x011b, 0x000000e3, 0x00e3)
NTDLL(NtGetCurrentProcessorNumber                       ,   NONE,   NONE, 0x0126, 0x009e, 0x016d, 0x00c8, 0x0088, 0x000000cb, 0x0112, 0x001900de, 0x00de, 0x0115, 0x001900e1, 0x00e1, 0x011a, 0x001900e4, 0x00e4)
NTDLL(NtGetCurrentProcessorNumberEx                     ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE, 0x0119, 0x000000e5, 0x00e5)
NTDLL(NtGetDevicePowerState                             , 0x004a, 0x0056, 0x005a, 0x009f, 0x0098, 0x00c9, 0x0089, 0x000000cc, 0x0111, 0x000700df, 0x00df, 0x0114, 0x000700e2, 0x00e2, 0x0118, 0x000700e6, 0x00e6)
NTDLL(NtGetMUIRegistryInfo                              ,   NONE,   NONE,   NONE,   NONE, 0x0184, 0x00ca, 0x008a, 0x000000cd, 0x0110, 0x000000e0, 0x00e0, 0x0113, 0x000000e3, 0x00e3, 0x0117, 0x000000e7, 0x00e7)
NTDLL(NtGetNextProcess                                  ,   NONE,   NONE,   NONE,   NONE, 0x016f, 0x00cb, 0x008b, 0x000000ce, 0x010f, 0x000000e1, 0x00e1, 0x0112, 0x000000e4, 0x00e4, 0x0116, 0x000000e8, 0x00e8)
NTDLL(NtGetNextThread                                   ,   NONE,   NONE,   NONE,   NONE, 0x0170, 0x00cc, 0x008c, 0x000000cf, 0x010e, 0x000000e2, 0x00e2, 0x0111, 0x000000e5, 0x00e5, 0x0115, 0x000000e9, 0x00e9)
NTDLL(NtGetNlsSectionPtr                                ,   NONE,   NONE,   NONE,   NONE, 0x0099, 0x00cd, 0x008d, 0x000000d0, 0x010d, 0x000000e3, 0x00e3, 0x0110, 0x000000e6, 0x00e6, 0x0114, 0x000000ea, 0x00ea)
NTDLL(NtGetNotificationResourceManager                  ,   NONE,   NONE,   NONE,   NONE, 0x006d, 0x00ce, 0x008e, 0x000000d1, 0x010c, 0x000000e4, 0x00e4, 0x010f, 0x000000e7, 0x00e7, 0x0113, 0x000000eb, 0x00eb)
NTDLL(NtGetPlugPlayEvent                                , 0x004b, 0x0057, 0x005b, 0x00a0, 0x009a, 0x00cf, 0x008f, 0x000000d2,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE)
NTDLL(NtGetTickCount                                    , 0x004c,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE)
NTDLL(NtGetWriteWatch                                   , 0x004d, 0x0058, 0x005c, 0x00a1, 0x009b, 0x00d0, 0x0090, 0x000000d3, 0x010b, 0x000000e5, 0x00e5, 0x010e, 0x000000e8, 0x00e8, 0x0112, 0x000000ec, 0x00ec)
NTDLL(NtImpersonateAnonymousToken                       , 0x004e, 0x0059, 0x005d, 0x00a2, 0x009c, 0x00d1, 0x0091, 0x000000d4, 0x010a, 0x000300e6, 0x00e6, 0x010d, 0x000300e9, 0x00e9, 0x0111, 0x000300ed, 0x00ed)
NTDLL(NtImpersonateClientOfPort                         , 0x004f, 0x005a, 0x005e, 0x001c, 0x009d, 0x001c, 0x0092, 0x0000001c, 0x0109, 0x0007001d, 0x001d, 0x010c, 0x0007001e, 0x001e, 0x0110, 0x0007001f, 0x001f)
NTDLL(NtImpersonateThread                               , 0x0050, 0x005b, 0x005f, 0x00a3, 0x009e, 0x00d2, 0x0093, 0x000000d5, 0x0108, 0x000000e7, 0x00e7, 0x010b, 0x000000ea, 0x00ea, 0x010f, 0x000000ee, 0x00ee)
NTDLL(NtInitializeNlsFiles                              ,   NONE,   NONE,   NONE,   NONE, 0x009f, 0x00d3, 0x0094, 0x000000d6, 0x0107, 0x000000e8, 0x00e8, 0x010a, 0x000000eb, 0x00eb, 0x010e, 0x000000ef, 0x00ef)
NTDLL(NtInitializeRegistry                              , 0x0051, 0x005c, 0x0060, 0x00a4, 0x00a0, 0x00d4, 0x0095, 0x000000d7, 0x0106, 0x000000e9, 0x00e9, 0x0109, 0x000000ec, 0x00ec, 0x010d, 0x000000f0, 0x00f0)
NTDLL(NtInitiatePowerAction                             , 0x0052, 0x005d, 0x0061, 0x00a5, 0x00a1, 0x00d5, 0x0096, 0x000000d8, 0x0105, 0x001100ea, 0x00ea, 0x0108, 0x001100ed, 0x00ed, 0x010c, 0x001100f1, 0x00f1)
NTDLL(NtIsProcessInJob                                  ,   NONE, 0x005e, 0x0062, 0x004c, 0x00a2, 0x004c, 0x0097, 0x0000004c, 0x0104, 0x0008004d, 0x004d, 0x0107, 0x0008004e, 0x004e, 0x010b, 0x0008004f, 0x004f)
NTDLL(NtIsSystemResumeAutomatic                         , 0x0053, 0x005f, 0x0063, 0x00a6, 0x00a3, 0x00d6, 0x0098, 0x000000d9, 0x0103, 0x000100eb, 0x00eb, 0x0106, 0x000100ee, 0x00ee, 0x010a, 0x000100f2, 0x00f2)
NTDLL(NtIsUILanguageComitted                            ,   NONE,   NONE,   NONE,   NONE, 0x0182, 0x00d7, 0x0099, 0x000000da, 0x0102, 0x000000ec, 0x00ec, 0x0105, 0x000000ef, 0x00ef, 0x0109, 0x000000f3, 0x00f3)
NTDLL(NtListenChannel                                   , 0x00f2,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE)
NTDLL(NtListenPort                                      , 0x0054, 0x0060, 0x0064, 0x00a7, 0x00a4, 0x00d8, 0x009a, 0x000000db, 0x0101, 0x000000ed, 0x00ed, 0x0104, 0x000000f0, 0x00f0, 0x0108, 0x000000f4, 0x00f4)
NTDLL(NtLoadDriver                                      , 0x0055, 0x0061, 0x0065, 0x00a8, 0x00a5, 0x00d9, 0x009b, 0x000000dc, 0x0100, 0x000000ee, 0x00ee, 0x0103, 0x000000f1, 0x00f1, 0x0107, 0x000000f5, 0x00f5)
NTDLL(NtLoadKey                                         , 0x0056, 0x0062, 0x0066, 0x00a9, 0x00a6, 0x00da, 0x009c, 0x000000dd, 0x00ff, 0x000000ef, 0x00ef, 0x0102, 0x000000f2, 0x00f2, 0x0106, 0x000000f6, 0x00f6)
NTDLL(NtLoadKey2                                        , 0x0057, 0x0063, 0x0067, 0x00aa, 0x00a7, 0x00db, 0x009d, 0x000000de, 0x00fe, 0x000000f0, 0x00f0, 0x0101, 0x000000f3, 0x00f3, 0x0105, 0x000000f7, 0x00f7)
NTDLL(NtLoadKeyEx                                       ,   NONE,   NONE, 0x0068, 0x00ab, 0x00a8, 0x00dc, 0x009e, 0x000000df, 0x00fd, 0x000000f1, 0x00f1, 0x0100, 0x000000f4, 0x00f4, 0x0104, 0x000000f8, 0x00f8)
NTDLL(NtLockFile                                        , 0x0058, 0x0064, 0x0069, 0x00ac, 0x00a9, 0x00dd, 0x009f, 0x000000e0, 0x00fc, 0x000000f2, 0x00f2, 0x00ff, 0x000000f5, 0x00f5, 0x0103, 0x000000f9, 0x00f9)
NTDLL(NtLockProductActivationKeys                       ,   NONE, 0x0065, 0x006a, 0x00ad, 0x00aa, 0x00de, 0x00a0, 0x000000e1, 0x00fb, 0x000500f3, 0x00f3, 0x00fe, 0x000500f6, 0x00f6, 0x0102, 0x000500fa, 0x00fa)
NTDLL(NtLockRegistryKey                                 ,   NONE, 0x0066, 0x006b, 0x00ae, 0x00ab, 0x00df, 0x00a1, 0x000000e2, 0x00fa, 0x000300f4, 0x00f4, 0x00fd, 0x000300f7, 0x00f7, 0x0101, 0x000300fb, 0x00fb)
NTDLL(NtLockVirtualMemory                               , 0x0059, 0x0067, 0x006c, 0x00af, 0x00ac, 0x00e0, 0x00a2, 0x000000e3, 0x00f9, 0x000000f5, 0x00f5, 0x00fc, 0x000000f8, 0x00f8, 0x0100, 0x000000fc, 0x00fc)
NTDLL(NtMakePermanentObject                             ,   NONE, 0x0068, 0x006d, 0x00b0, 0x00ad, 0x00e1, 0x00a3, 0x000000e4, 0x00f8, 0x000300f6, 0x00f6, 0x00fb, 0x000300f9, 0x00f9, 0x00ff, 0x000300fd, 0x00fd)
NTDLL(NtMakeTemporaryObject                             , 0x005a, 0x0069, 0x006e, 0x00b1, 0x00ae, 0x00e2, 0x00a4, 0x000000e5, 0x00f7, 0x000300f7, 0x00f7, 0x00fa, 0x000300fa, 0x00fa, 0x00fe, 0x000300fe, 0x00fe)
NTDLL(NtManagePartition                                 ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE, 0x00f7, 0x000000ff, 0x00ff)
NTDLL(NtMapCMFModule                                    ,   NONE,   NONE,   NONE,   NONE, 0x0181, 0x00e3, 0x00a5, 0x000000e6, 0x00f6, 0x000000f8, 0x00f8, 0x00f9, 0x000000fb, 0x00fb, 0x00fd, 0x00000100, 0x0100)
NTDLL(NtMapUserPhysicalPages                            , 0x005b, 0x006a, 0x006f, 0x00b2, 0x00af, 0x00e4, 0x00a6, 0x000000e7, 0x00f5, 0x000a00f9, 0x00f9, 0x00f8, 0x000a00fc, 0x00fc, 0x00fc, 0x000a0101, 0x0101)
NTDLL(NtMapUserPhysicalPagesScatter                     , 0x005c, 0x006b, 0x0070, 0x0000, 0x00b0, 0x0000, 0x00a7, 0x00000000, 0x00f4, 0x000a0001, 0x0001, 0x00f7, 0x000a0002, 0x0002, 0x00fb, 0x000a0003, 0x0003)
NTDLL(NtMapViewOfSection                                , 0x005d, 0x006c, 0x0071, 0x0025, 0x00b1, 0x0025, 0x00a8, 0x00000025, 0x00f3, 0x00000026, 0x0026, 0x00f6, 0x00000027, 0x0027, 0x00fa, 0x00000028, 0x0028)
NTDLL(NtModifyBootEntry                                 ,   NONE, 0x006d, 0x0072, 0x00b3, 0x00b2, 0x00e5, 0x00a9, 0x000000e8, 0x00f2, 0x000000fa, 0x00fa, 0x00f5, 0x000000fd, 0x00fd, 0x00f9, 0x00000102, 0x0102)
NTDLL(NtModifyDriverEntry                               ,   NONE,   NONE, 0x0073, 0x00b4, 0x00b3, 0x00e6, 0x00aa, 0x000000e9, 0x00f1, 0x000000fb, 0x00fb, 0x00f4, 0x000000fe, 0x00fe, 0x00f8, 0x00000103, 0x0103)
NTDLL(NtNotifyChangeDirectoryFile                       , 0x005e, 0x006e, 0x0074, 0x00b5, 0x00b4, 0x00e7, 0x00ab, 0x000000ea, 0x00f0, 0x000000fc, 0x00fc, 0x00f3, 0x000000ff, 0x00ff, 0x00f6, 0x00000104, 0x0104)
NTDLL(NtNotifyChangeKey                                 , 0x005f, 0x006f, 0x0075, 0x00b6, 0x00b5, 0x00e8, 0x00ac, 0x000000eb, 0x00ef, 0x000000fd, 0x00fd, 0x00f2, 0x00000100, 0x0100, 0x00f5, 0x00000105, 0x0105)
NTDLL(NtNotifyChangeMultipleKeys                        , 0x0060, 0x0070, 0x0076, 0x00b7, 0x00b6, 0x00e9, 0x00ad, 0x000000ec, 0x00ee, 0x000000fe, 0x00fe, 0x00f1, 0x00000101, 0x0101, 0x00f4, 0x00000106, 0x0106)
NTDLL(NtNotifyChangeSession                             ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE, 0x00ae, 0x000000ed, 0x00ed, 0x000000ff, 0x00ff, 0x00f0, 0x00000102, 0x0102, 0x00f3, 0x00000107, 0x0107)
NTDLL(NtOpenChannel                                     , 0x00f3,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE)
NTDLL(NtOpenDirectoryObject                             , 0x0061, 0x0071, 0x0077, 0x0055, 0x00b7, 0x0055, 0x00af, 0x00000055, 0x00ec, 0x00000056, 0x0056, 0x00ef, 0x00000057, 0x0057, 0x00f2, 0x00000058, 0x0058)
NTDLL(NtOpenEnlistment                                  ,   NONE,   NONE,   NONE,   NONE, 0x0070, 0x00ea, 0x00b0, 0x000000ee, 0x00eb, 0x00000100, 0x0100, 0x00ee, 0x00000103, 0x0103, 0x00f1, 0x00000108, 0x0108)
NTDLL(NtOpenEvent                                       , 0x0062, 0x0072, 0x0078, 0x003d, 0x00b8, 0x003d, 0x00b1, 0x0000003d, 0x00ea, 0x0000003e, 0x003e, 0x00ed, 0x0000003f, 0x003f, 0x00f0, 0x00000040, 0x0040)
NTDLL(NtOpenEventPair                                   , 0x0063, 0x0073, 0x0079, 0x00b8, 0x00b9, 0x00eb, 0x00b2, 0x000000ef, 0x00e9, 0x00000101, 0x0101, 0x00ec, 0x00000104, 0x0104, 0x00ef, 0x00000109, 0x0109)
NTDLL(NtOpenFile                                        , 0x0064, 0x0074, 0x007a, 0x0030, 0x00ba, 0x0030, 0x00b3, 0x00000030, 0x00e8, 0x00000031, 0x0031, 0x00eb, 0x00000032, 0x0032, 0x00ee, 0x00000033, 0x0033)
NTDLL(NtOpenIoCompletion                                , 0x0065, 0x0075, 0x007b, 0x00b9, 0x00bb, 0x00ec, 0x00b4, 0x000000f0, 0x00e7, 0x00000102, 0x0102, 0x00ea, 0x00000105, 0x0105, 0x00ed, 0x0000010a, 0x010a)
NTDLL(NtOpenJobObject                                   , 0x0066, 0x0076, 0x007c, 0x00ba, 0x00bc, 0x00ed, 0x00b5, 0x000000f1, 0x00e6, 0x00000103, 0x0103, 0x00e9, 0x00000106, 0x0106, 0x00ec, 0x0000010b, 0x010b)
NTDLL(NtOpenKey                                         , 0x0067, 0x0077, 0x007d, 0x000f, 0x00bd, 0x000f, 0x00b6, 0x0000000f, 0x00e5, 0x00000010, 0x0010, 0x00e8, 0x00000011, 0x0011, 0x00eb, 0x00000012, 0x0012)
NTDLL(NtOpenKeyEx                                       ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE, 0x00b7, 0x000000f2, 0x00e3, 0x00000104, 0x0104, 0x00e6, 0x00000107, 0x0107, 0x00e9, 0x0000010c, 0x010c)
NTDLL(NtOpenKeyTransacted                               ,   NONE,   NONE,   NONE,   NONE, 0x00be, 0x00ee, 0x00b9, 0x000000f3, 0x00e2, 0x00000105, 0x0105, 0x00e5, 0x00000108, 0x0108, 0x00e8, 0x0000010d, 0x010d)
NTDLL(NtOpenKeyTransactedEx                             ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE, 0x00ba, 0x000000f4, 0x00e1, 0x00000106, 0x0106, 0x00e4, 0x00000109, 0x0109, 0x00e7, 0x0000010e, 0x010e)
NTDLL(NtOpenKeyedEvent                                  ,   NONE, 0x0118, 0x0122, 0x00bb, 0x0169, 0x00ef, 0x00b8, 0x000000f5, 0x00e4, 0x00000107, 0x0107, 0x00e7, 0x0000010a, 0x010a, 0x00ea, 0x0000010f, 0x010f)
NTDLL(NtOpenMutant                                      , 0x0068, 0x0078, 0x007e, 0x00bc, 0x00bf, 0x00f0, 0x00bb, 0x000000f6, 0x00e0, 0x00000108, 0x0108, 0x00e3, 0x0000010b, 0x010b, 0x00e6, 0x00000110, 0x0110)
NTDLL(NtOpenObjectAuditAlarm                            , 0x0069, 0x0079, 0x007f, 0x00bd, 0x00c1, 0x00f1, 0x00bc, 0x000000f7, 0x00df, 0x00000109, 0x0109, 0x00e2, 0x0000010c, 0x010c, 0x00e5, 0x00000111, 0x0111)
NTDLL(NtOpenPartition                                   ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE, 0x00e0, 0x00000112, 0x0112)
NTDLL(NtOpenPrivateNamespace                            ,   NONE,   NONE,   NONE,   NONE, 0x00c0, 0x00f2, 0x00bd, 0x000000f8, 0x00de, 0x0000010a, 0x010a, 0x00e1, 0x0000010d, 0x010d, 0x00e4, 0x00000113, 0x0113)
NTDLL(NtOpenProcess                                     , 0x006a, 0x007a, 0x0080, 0x0023, 0x00c2, 0x0023, 0x00be, 0x00000023, 0x00dd, 0x00000024, 0x0024, 0x00e0, 0x00000025, 0x0025, 0x00e3, 0x00000026, 0x0026)
NTDLL(NtOpenProcessToken                                , 0x006b, 0x007b, 0x0081, 0x00be, 0x00c3, 0x00f3, 0x00bf, 0x000000f9, 0x00dc, 0x0000010b, 0x010b, 0x00df, 0x0000010e, 0x010e, 0x00e2, 0x00000114, 0x0114)
NTDLL(NtOpenProcessTokenEx                              ,   NONE, 0x007c, 0x0082, 0x002d, 0x00c4, 0x002d, 0x00c0, 0x0000002d, 0x00db, 0x0000002e, 0x002e, 0x00de, 0x0000002f, 0x002f, 0x00e1, 0x00000030, 0x0030)
NTDLL(NtOpenResourceManager                             ,   NONE,   NONE,   NONE,   NONE, 0x006c, 0x00f4, 0x00c1, 0x000000fa, 0x00da, 0x0000010c, 0x010c, 0x00dd, 0x0000010f, 0x010f, 0x00df, 0x00000115, 0x0115)
NTDLL(NtOpenSection                                     , 0x006c, 0x007d, 0x0083, 0x0034, 0x00c5, 0x0034, 0x00c2, 0x00000034, 0x00d9, 0x00000035, 0x0035, 0x00dc, 0x00000036, 0x0036, 0x00de, 0x00000037, 0x0037)
NTDLL(NtOpenSemaphore                                   , 0x006d, 0x007e, 0x0084, 0x00bf, 0x00c6, 0x00f5, 0x00c3, 0x000000fb, 0x00d8, 0x0000010d, 0x010d, 0x00db, 0x00000110, 0x0110, 0x00dd, 0x00000116, 0x0116)
NTDLL(NtOpenSession                                     ,   NONE,   NONE,   NONE,   NONE, 0x00c7, 0x00f6, 0x00c4, 0x000000fc, 0x00d7, 0x0000010e, 0x010e, 0x00da, 0x00000111, 0x0111, 0x00dc, 0x00000117, 0x0117)
NTDLL(NtOpenSymbolicLinkObject                          , 0x006e, 0x007f, 0x0085, 0x00c0, 0x00c8, 0x00f7, 0x00c5, 0x000000fd, 0x00d6, 0x0000010f, 0x010f, 0x00d9, 0x00000112, 0x0112, 0x00db, 0x00000118, 0x0118)
NTDLL(NtOpenThread                                      , 0x006f, 0x0080, 0x0086, 0x00c1, 0x00c9, 0x00f8, 0x00c6, 0x000000fe, 0x00d5, 0x00000110, 0x0110, 0x00d8, 0x00000113, 0x0113, 0x00da, 0x00000119, 0x0119)
NTDLL(NtOpenThreadToken                                 , 0x0070, 0x0081, 0x0087, 0x0021, 0x00ca, 0x0021, 0x00c7, 0x00000021, 0x00d4, 0x00000022, 0x0022, 0x00d7, 0x00000023, 0x0023, 0x00d9, 0x00000024, 0x0024)
NTDLL(NtOpenThreadTokenEx                               ,   NONE, 0x0082, 0x0088, 0x002c, 0x00cb, 0x002c, 0x00c8, 0x0000002c, 0x00d3, 0x0000002d, 0x002d, 0x00d6, 0x0000002e, 0x002e, 0x00d8, 0x0000002f, 0x002f)
NTDLL(NtOpenTimer                                       , 0x0071, 0x0083, 0x0089, 0x00c2, 0x00cc, 0x00f9, 0x00c9, 0x000000ff, 0x00d2, 0x00000111, 0x0111, 0x00d5, 0x00000114, 0x0114, 0x00d7, 0x0000011a, 0x011a)
NTDLL(NtOpenTransaction                                 ,   NONE,   NONE,   NONE,   NONE, 0x0052, 0x00fa, 0x00ca, 0x00000100, 0x00d1, 0x00000112, 0x0112, 0x00d4, 0x00000115, 0x0115, 0x00d6, 0x0000011b, 0x011b)
NTDLL(NtOpenTransactionManager                          ,   NONE,   NONE,   NONE,   NONE, 0x0065, 0x00fb, 0x00cb, 0x00000101, 0x00d0, 0x00000113, 0x0113, 0x00d3, 0x00000116, 0x0116, 0x00d5, 0x0000011c, 0x011c)
NTDLL(NtPlugPlayControl                                 , 0x0072, 0x0084, 0x008a, 0x00c3, 0x00cd, 0x00fc, 0x00cc, 0x00000102, 0x00cf, 0x00000114, 0x0114, 0x00d2, 0x00000117, 0x0117, 0x00d4, 0x0000011d, 0x011d)
NTDLL(NtPowerInformation                                , 0x0073, 0x0085, 0x008b, 0x005c, 0x00ce, 0x005c, 0x00cd, 0x0000005c, 0x00ce, 0x0000005d, 0x005d, 0x00d1, 0x0000005e, 0x005e, 0x00d3, 0x0000005f, 0x005f)
NTDLL(NtPrePrepareComplete                              ,   NONE,   NONE,   NONE,   NONE, 0x005d, 0x00fd, 0x00d0, 0x00000103, 0x00cb, 0x00000115, 0x0115, 0x00ce, 0x00000118, 0x0118, 0x00d0, 0x0000011e, 0x011e)
NTDLL(NtPrePrepareEnlistment                            ,   NONE,   NONE,   NONE,   NONE, 0x0055, 0x00fe, 0x00d1, 0x00000104, 0x00ca, 0x00000116, 0x0116, 0x00cd, 0x00000119, 0x0119, 0x00cf, 0x0000011f, 0x011f)
NTDLL(NtPrepareComplete                                 ,   NONE,   NONE,   NONE,   NONE, 0x005e, 0x00ff, 0x00ce, 0x00000105, 0x00cd, 0x00000117, 0x0117, 0x00d0, 0x0000011a, 0x011a, 0x00d2, 0x00000120, 0x0120)
NTDLL(NtPrepareEnlistment                               ,   NONE,   NONE,   NONE,   NONE, 0x0056, 0x0100, 0x00cf, 0x00000106, 0x00cc, 0x00000118, 0x0118, 0x00cf, 0x0000011b, 0x011b, 0x00d1, 0x00000121, 0x0121)
NTDLL(NtPrivilegeCheck                                  , 0x0074, 0x0086, 0x008c, 0x00c4, 0x00cf, 0x0101, 0x00d2, 0x00000107, 0x00c8, 0x000c0119, 0x0119, 0x00cb, 0x000c011c, 0x011c, 0x00cd, 0x000c0122, 0x0122)
NTDLL(NtPrivilegeObjectAuditAlarm                       , 0x0076, 0x0087, 0x008d, 0x00c5, 0x00d0, 0x0102, 0x00d4, 0x00000108, 0x00c6, 0x0000011a, 0x011a, 0x00c9, 0x0000011d, 0x011d, 0x00cb, 0x00000123, 0x0123)
NTDLL(NtPrivilegedServiceAuditAlarm                     , 0x0075, 0x0088, 0x008e, 0x00c6, 0x00d1, 0x0103, 0x00d3, 0x00000109, 0x00c7, 0x0000011b, 0x011b, 0x00ca, 0x0000011e, 0x011e, 0x00cc, 0x00000124, 0x0124)
NTDLL(NtPropagationComplete                             ,   NONE,   NONE,   NONE,   NONE, 0x0175, 0x0104, 0x00d5, 0x0000010a, 0x00c5, 0x0000011c, 0x011c, 0x00c8, 0x0000011f, 0x011f, 0x00ca, 0x00000125, 0x0125)
NTDLL(NtPropagationFailed                               ,   NONE,   NONE,   NONE,   NONE, 0x0176, 0x0105, 0x00d6, 0x0000010b, 0x00c4, 0x0000011d, 0x011d, 0x00c7, 0x00000120, 0x0120, 0x00c9, 0x00000126, 0x0126)
NTDLL(NtProtectVirtualMemory                            , 0x0077, 0x0089, 0x008f, 0x004d, 0x00d2, 0x004d, 0x00d7, 0x0000004d, 0x00c3, 0x0000004e, 0x004e, 0x00c6, 0x0000004f, 0x004f, 0x00c8, 0x00000050, 0x0050)
NTDLL(NtPulseEvent                                      , 0x0078, 0x008a, 0x0090, 0x00c7, 0x00d3, 0x0106, 0x00d8, 0x0000010c, 0x00c2, 0x0007011e, 0x011e, 0x00c5, 0x00070121, 0x0121, 0x00c7, 0x00070127, 0x0127)
NTDLL(NtQueryAttributesFile                             , 0x007a, 0x008b, 0x0091, 0x003a, 0x00d4, 0x003a, 0x00d9, 0x0000003a, 0x00c1, 0x0000003b, 0x003b, 0x00c4, 0x0000003c, 0x003c, 0x00c6, 0x0000003d, 0x003d)
NTDLL(NtQueryBootEntryOrder                             ,   NONE, 0x008c, 0x0092, 0x00c8, 0x00d5, 0x0107, 0x00da, 0x0000010d, 0x00c0, 0x0000011f, 0x011f, 0x00c3, 0x00000122, 0x0122, 0x00c5, 0x00000128, 0x0128)
NTDLL(NtQueryBootOptions                                ,   NONE, 0x008d, 0x0093, 0x00c9, 0x00d6, 0x0108, 0x00db, 0x0000010e, 0x00bf, 0x00000120, 0x0120, 0x00c2, 0x00000123, 0x0123, 0x00c4, 0x00000129, 0x0129)
NTDLL(NtQueryDebugFilterState                           ,   NONE, 0x008e, 0x0094, 0x00ca, 0x00d7, 0x0109, 0x00dc, 0x0000010f, 0x00be, 0x00050121, 0x0121, 0x00c1, 0x00050124, 0x0124, 0x00c3, 0x0005012a, 0x012a)
NTDLL(NtQueryDefaultLocale                              , 0x007b, 0x008f, 0x0095, 0x0012, 0x00d8, 0x0012, 0x00dd, 0x00000012, 0x00bd, 0x00050013, 0x0013, 0x00c0, 0x00050014, 0x0014, 0x00c2, 0x00050015, 0x0015)
NTDLL(NtQueryDefaultUILanguage                          , 0x007c, 0x0090, 0x0096, 0x0041, 0x00d9, 0x0041, 0x00de, 0x00000041, 0x00bc, 0x00040042, 0x0042, 0x00bf, 0x00040043, 0x0043, 0x00c1, 0x00040044, 0x0044)
NTDLL(NtQueryDirectoryFile                              , 0x007d, 0x0091, 0x0097, 0x0032, 0x00da, 0x0032, 0x00df, 0x00000032, 0x00bb, 0x00000033, 0x0033, 0x00be, 0x00000034, 0x0034, 0x00c0, 0x00000035, 0x0035)
NTDLL(NtQueryDirectoryObject                            , 0x007e, 0x0092, 0x0098, 0x00cb, 0x00db, 0x010a, 0x00e0, 0x00000110, 0x00ba, 0x00000122, 0x0122, 0x00bd, 0x00000125, 0x0125, 0x00bf, 0x0000012b, 0x012b)
NTDLL(NtQueryDriverEntryOrder                           ,   NONE,   NONE, 0x0099, 0x00cc, 0x00dc, 0x010b, 0x00e1, 0x00000111, 0x00b9, 0x00000123, 0x0123, 0x00bc, 0x00000126, 0x0126, 0x00be, 0x0000012c, 0x012c)
NTDLL(NtQueryEaFile                                     , 0x007f, 0x0093, 0x009a, 0x00cd, 0x00dd, 0x010c, 0x00e2, 0x00000112, 0x00b8, 0x00000124, 0x0124, 0x00bb, 0x00000127, 0x0127, 0x00bd, 0x0000012d, 0x012d)
NTDLL(NtQueryEvent                                      , 0x0080, 0x0094, 0x009b, 0x0053, 0x00de, 0x0053, 0x00e3, 0x00000053, 0x00b7, 0x00000054, 0x0054, 0x00ba, 0x00000055, 0x0055, 0x00bc, 0x00000056, 0x0056)
NTDLL(NtQueryFullAttributesFile                         , 0x0081, 0x0095, 0x009c, 0x00ce, 0x00df, 0x010d, 0x00e4, 0x00000113, 0x00b6, 0x00000125, 0x0125, 0x00b9, 0x00000128, 0x0128, 0x00bb, 0x0000012e, 0x012e)
NTDLL(NtQueryInformationAtom                            , 0x0079, 0x0096, 0x009d, 0x00cf, 0x00e0, 0x010e, 0x00e5, 0x00000114, 0x00b5, 0x00000126, 0x0126, 0x00b8, 0x00000129, 0x0129, 0x00ba, 0x0000012f, 0x012f)
NTDLL(NtQueryInformationEnlistment                      ,   NONE,   NONE,   NONE,   NONE, 0x0072, 0x010f, 0x00e6, 0x00000115, 0x00b4, 0x00000127, 0x0127, 0x00b7, 0x0000012a, 0x012a, 0x00b9, 0x00000130, 0x0130)
NTDLL(NtQueryInformationFile                            , 0x0082, 0x0097, 0x009e, 0x000e, 0x00e1, 0x000e, 0x00e7, 0x0000000e, 0x00b3, 0x0000000f, 0x000f, 0x00b6, 0x00000010, 0x0010, 0x00b8, 0x00000011, 0x0011)
NTDLL(NtQueryInformationJobObject                       , 0x0083, 0x0098, 0x009f, 0x00d0, 0x00e2, 0x0110, 0x00e8, 0x00000116, 0x00b2, 0x00000128, 0x0128, 0x00b5, 0x0000012b, 0x012b, 0x00b7, 0x00000131, 0x0131)
NTDLL(NtQueryInformationPort                            , 0x0085, 0x0099, 0x00a0, 0x00d1, 0x00e3, 0x0111, 0x00e9, 0x00000117, 0x00b1, 0x00000129, 0x0129, 0x00b4, 0x0000012c, 0x012c, 0x00b6, 0x00000132, 0x0132)
NTDLL(NtQueryInformationProcess                         , 0x0086, 0x009a, 0x00a1, 0x0016, 0x00e4, 0x0016, 0x00ea, 0x00000016, 0x00b0, 0x00000017, 0x0017, 0x00b3, 0x00000018, 0x0018, 0x00b5,       NONE, 0x0019)
NTDLL(NtQueryInformationResourceManager                 ,   NONE,   NONE,   NONE,   NONE, 0x006e, 0x0112, 0x00eb, 0x00000118, 0x00af, 0x0000012a, 0x012a, 0x00b2, 0x0000012d, 0x012d, 0x00b4, 0x00000133, 0x0133)
NTDLL(NtQueryInformationThread                          , 0x0087, 0x009b, 0x00a2, 0x0022, 0x00e5, 0x0022, 0x00ec, 0x00000022, 0x00ae, 0x00000023, 0x0023, 0x00b1, 0x00000024, 0x0024, 0x00b3, 0x00000025, 0x0025)
NTDLL(NtQueryInformationToken                           , 0x0088, 0x009c, 0x00a3, 0x001e, 0x00e6, 0x001e, 0x00ed, 0x0000001e, 0x00ad, 0x0000001f, 0x001f, 0x00b0, 0x00000020, 0x0020, 0x00b2, 0x00000021, 0x0021)
NTDLL(NtQueryInformationTransaction                     ,   NONE,   NONE,   NONE,   NONE, 0x0053, 0x0113, 0x00ee, 0x00000119, 0x00ac, 0x0000012b, 0x012b, 0x00af, 0x0000012e, 0x012e, 0x00b1, 0x00000134, 0x0134)
NTDLL(NtQueryInformationTransactionManager              ,   NONE,   NONE,   NONE,   NONE, 0x0054, 0x0114, 0x00ef, 0x0000011a, 0x00ab, 0x0000012c, 0x012c, 0x00ae, 0x0000012f, 0x012f, 0x00b0, 0x00000135, 0x0135)
NTDLL(NtQueryInformationWorkerFactory                   ,   NONE,   NONE,   NONE,   NONE, 0x017b, 0x0115, 0x00f0, 0x0000011b, 0x00aa, 0x0000012d, 0x012d, 0x00ad, 0x00000130, 0x0130, 0x00af, 0x00000136, 0x0136)
NTDLL(NtQueryInstallUILanguage                          , 0x0089, 0x009d, 0x00a4, 0x00d2, 0x00e7, 0x0116, 0x00f1, 0x0000011c, 0x00a9, 0x0004012e, 0x012e, 0x00ac, 0x00040131, 0x0131, 0x00ae, 0x00040137, 0x0137)
NTDLL(NtQueryIntervalProfile                            , 0x008a, 0x009e, 0x00a5, 0x00d3, 0x00e8, 0x0117, 0x00f2, 0x0000011d, 0x00a8, 0x0005012f, 0x012f, 0x00ab, 0x00050132, 0x0132, 0x00ad, 0x00050138, 0x0138)
NTDLL(NtQueryIoCompletion                               , 0x0084, 0x009f, 0x00a6, 0x00d4, 0x00e9, 0x0118, 0x00f3, 0x0000011e, 0x00a7, 0x00000130, 0x0130, 0x00aa, 0x00000133, 0x0133, 0x00ac, 0x00000139, 0x0139)
NTDLL(NtQueryKey                                        , 0x008b, 0x00a0, 0x00a7, 0x0013, 0x00ea, 0x0013, 0x00f4, 0x00000013, 0x00a6, 0x00000014, 0x0014, 0x00a9, 0x00000015, 0x0015, 0x00ab, 0x00000016, 0x0016)
NTDLL(NtQueryLicenseValue                               ,   NONE,   NONE,   NONE,   NONE, 0x0180, 0x0119, 0x00f5, 0x0000011f, 0x00a5, 0x00000131, 0x0131, 0x00a8, 0x00000134, 0x0134, 0x00aa, 0x0000013a, 0x013a)
NTDLL(NtQueryMultipleValueKey                           , 0x008c, 0x00a1, 0x00a8, 0x00d5, 0x00eb, 0x011a, 0x00f6, 0x00000120, 0x00a4, 0x00000132, 0x0132, 0x00a7, 0x00000135, 0x0135, 0x00a9, 0x0000013b, 0x013b)
NTDLL(NtQueryMutant                                     , 0x008d, 0x00a2, 0x00a9, 0x00d6, 0x00ec, 0x011b, 0x00f7, 0x00000121, 0x00a3, 0x00000133, 0x0133, 0x00a6, 0x00000136, 0x0136, 0x00a8, 0x0000013c, 0x013c)
NTDLL(NtQueryObject                                     , 0x008e, 0x00a3, 0x00aa, 0x000d, 0x00ed, 0x000d, 0x00f8, 0x0000000d, 0x00a2, 0x0000000e, 0x000e, 0x00a5, 0x0000000f, 0x000f, 0x00a7, 0x00000010, 0x0010)
NTDLL(NtQueryOpenSubKeys                                , 0x008f, 0x00a4, 0x00ab, 0x00d7, 0x00ee, 0x011c, 0x00f9, 0x00000122, 0x00a1, 0x00000134, 0x0134, 0x00a4, 0x00000137, 0x0137, 0x00a6, 0x0000013d, 0x013d)
NTDLL(NtQueryOpenSubKeysEx                              ,   NONE,   NONE, 0x00ac, 0x00d8, 0x00ef, 0x011d, 0x00fa, 0x00000123, 0x00a0, 0x00000135, 0x0135, 0x00a3, 0x00000138, 0x0138, 0x00a5, 0x0000013e, 0x013e)
NTDLL(NtQueryPerformanceCounter                         , 0x0090, 0x00a5, 0x00ad, 0x002e, 0x00f0, 0x002e, 0x00fb, 0x0000002e, 0x009f, 0x0005002f, 0x002f, 0x00a2, 0x00050030, 0x0030, 0x00a4, 0x00050031, 0x0031)
NTDLL(NtQueryPortInformationProcess                     ,   NONE, 0x011b, 0x0125, 0x00d9, 0x016c, 0x011e, 0x00fc, 0x00000124, 0x009e, 0x00010136, 0x0136, 0x00a1, 0x00010139, 0x0139, 0x00a3, 0x0001013f, 0x013f)
NTDLL(NtQueryQuotaInformationFile                       , 0x0091, 0x00a6, 0x00ae, 0x00da, 0x00f1, 0x011f, 0x00fd, 0x00000125, 0x009d, 0x00000137, 0x0137, 0x00a0, 0x0000013a, 0x013a, 0x00a2, 0x00000140, 0x0140)
NTDLL(NtQuerySection                                    , 0x0092, 0x00a7, 0x00af, 0x004e, 0x00f2, 0x004e, 0x00fe, 0x0000004e, 0x009c, 0x0000004f, 0x004f, 0x009f, 0x00000050, 0x0050, 0x00a1, 0x00000051, 0x0051)
NTDLL(NtQuerySecurityAttributesToken                    ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE, 0x00ff, 0x00000126, 0x009b, 0x00000138, 0x0138, 0x009e, 0x0000013b, 0x013b, 0x00a0, 0x00000141, 0x0141)
NTDLL(NtQuerySecurityObject                             , 0x0093, 0x00a8, 0x00b0, 0x00db, 0x00f3, 0x0120, 0x0100, 0x00000127, 0x009a, 0x00000139, 0x0139, 0x009d, 0x0000013c, 0x013c, 0x009f, 0x00000142, 0x0142)
NTDLL(NtQuerySemaphore                                  , 0x0094, 0x00a9, 0x00b1, 0x00dc, 0x00f4, 0x0121, 0x0101, 0x00000128, 0x0099, 0x0000013a, 0x013a, 0x009c, 0x0000013d, 0x013d, 0x009e, 0x00000143, 0x0143)
NTDLL(NtQuerySymbolicLinkObject                         , 0x0095, 0x00aa, 0x00b2, 0x00dd, 0x00f5, 0x0122, 0x0102, 0x00000129, 0x0098, 0x0000013b, 0x013b, 0x009b, 0x0000013e, 0x013e, 0x009d, 0x00000144, 0x0144)
NTDLL(NtQuerySystemEnvironmentValue                     , 0x0096, 0x00ab, 0x00b3, 0x00de, 0x00f6, 0x0123, 0x0103, 0x0000012a, 0x0097, 0x0000013c, 0x013c, 0x009a, 0x0000013f, 0x013f, 0x009c, 0x00000145, 0x0145)
NTDLL(NtQuerySystemEnvironmentValueEx                   ,   NONE, 0x00ac, 0x00b4, 0x00df, 0x00f7, 0x0124, 0x0104, 0x0000012b, 0x0096, 0x0000013d, 0x013d, 0x0099, 0x00000140, 0x0140, 0x009b, 0x00000146, 0x0146)
NTDLL(NtQuerySystemInformation                          , 0x0097, 0x00ad, 0x00b5, 0x0033, 0x00f8, 0x0033, 0x0105, 0x00000033, 0x0095, 0x00000034, 0x0034, 0x0098, 0x00000035, 0x0035, 0x009a, 0x00000036, 0x0036)
NTDLL(NtQuerySystemInformationEx                        ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE, 0x0106, 0x0000012c, 0x0094, 0x0000013e, 0x013e, 0x0097, 0x00000141, 0x0141, 0x0099, 0x00000147, 0x0147)
NTDLL(NtQuerySystemTime                                 , 0x0098, 0x00ae, 0x00b6, 0x0057, 0x00f9, 0x0057, 0x0107, 0x00000057, 0x0093, 0x00180058,   NONE, 0x0096, 0x00180059,   NONE, 0x0098, 0x0018005a,   NONE)
NTDLL(NtQueryTimer                                      , 0x0099, 0x00af, 0x00b7, 0x0035, 0x00fa, 0x0035, 0x0108, 0x00000035, 0x0092, 0x00000036, 0x0036, 0x0095, 0x00000037, 0x0037, 0x0097, 0x00000038, 0x0038)
NTDLL(NtQueryTimerResolution                            , 0x009a, 0x00b0, 0x00b8, 0x00e0, 0x00fb, 0x0125, 0x0109, 0x0000012d, 0x0091, 0x000a013f, 0x013f, 0x0094, 0x000a0142, 0x0142, 0x0096, 0x000a0148, 0x0148)
NTDLL(NtQueryValueKey                                   , 0x009b, 0x00b1, 0x00b9, 0x0014, 0x00fc, 0x0014, 0x010a, 0x00000014, 0x0090, 0x00000015, 0x0015, 0x0093, 0x00000016, 0x0016, 0x0095, 0x00000017, 0x0017)
NTDLL(NtQueryVirtualMemory                              , 0x009c, 0x00b2, 0x00ba, 0x0020, 0x00fd, 0x0020, 0x010b, 0x00000020, 0x008f, 0x00000021, 0x0021, 0x0092, 0x00000022, 0x0022, 0x0094, 0x00000023, 0x0023)
NTDLL(NtQueryVolumeInformationFile                      , 0x009d, 0x00b3, 0x00bb, 0x0046, 0x00fe, 0x0046, 0x010c, 0x00000046, 0x008e, 0x00000047, 0x0047, 0x0091, 0x00000048, 0x0048, 0x0093, 0x00000049, 0x0049)
NTDLL(NtQueryWnfStateData                               ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE, 0x008c, 0x00000140, 0x0140, 0x008f, 0x00000143, 0x0143, 0x0091, 0x00000149, 0x0149)
NTDLL(NtQueryWnfStateNameInformation                    ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE, 0x008d, 0x00000141, 0x0141, 0x0090, 0x00000144, 0x0144, 0x0092, 0x0000014a, 0x014a)
NTDLL(NtQueueApcThread                                  , 0x009e, 0x00b4, 0x00bc, 0x0042, 0x00ff, 0x0042, 0x010d, 0x00000042, 0x008b, 0x00000043, 0x0043, 0x008e, 0x00000044, 0x0044, 0x0090, 0x00000045, 0x0045)
NTDLL(NtQueueApcThreadEx                                ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE, 0x010e, 0x0000012e, 0x008a, 0x00000142, 0x0142, 0x008d, 0x00000145, 0x0145, 0x008f, 0x0000014b, 0x014b)
NTDLL(NtRaiseException                                  , 0x009f, 0x00b5, 0x00bd, 0x00e1, 0x0100, 0x0126, 0x010f, 0x0000012f, 0x0089, 0x00000143, 0x0143, 0x008c, 0x00000146, 0x0146, 0x008e, 0x0000014c, 0x014c)
NTDLL(NtRaiseHardError                                  , 0x00a0, 0x00b6, 0x00be, 0x00e2, 0x0101, 0x0127, 0x0110, 0x00000130, 0x0088, 0x00000144, 0x0144, 0x008b, 0x00000147, 0x0147, 0x008d, 0x0000014d, 0x014d)
NTDLL(NtReadFile                                        , 0x00a1, 0x00b7, 0x00bf, 0x0003, 0x0102, 0x0003, 0x0111, 0x00000003, 0x0087, 0x001a0004, 0x0004, 0x008a, 0x001a0005, 0x0005, 0x008c, 0x001a0006, 0x0006)
NTDLL(NtReadFileScatter                                 , 0x00a2, 0x00b8, 0x00c0, 0x002b, 0x0103, 0x002b, 0x0112, 0x0000002b, 0x0086, 0x001a002c, 0x002c, 0x0089, 0x001a002d, 0x002d, 0x008b, 0x001a002e, 0x002e)
NTDLL(NtReadOnlyEnlistment                              ,   NONE,   NONE,   NONE,   NONE, 0x0058, 0x0128, 0x0113, 0x00000131, 0x0085, 0x00000145, 0x0145, 0x0088, 0x00000148, 0x0148, 0x008a, 0x0000014e, 0x014e)
NTDLL(NtReadRequestData                                 , 0x00a3, 0x00b9, 0x00c1, 0x0051, 0x0104, 0x0051, 0x0114, 0x00000051, 0x0084, 0x00000052, 0x0052, 0x0087, 0x00000053, 0x0053, 0x0089, 0x00000054, 0x0054)
NTDLL(NtReadVirtualMemory                               , 0x00a4, 0x00ba, 0x00c2, 0x003c, 0x0105, 0x003c, 0x0115, 0x0000003c, 0x0083, 0x0000003d, 0x003d, 0x0086, 0x0000003e, 0x003e, 0x0088, 0x0000003f, 0x003f)
NTDLL(NtRecoverEnlistment                               ,   NONE,   NONE,   NONE,   NONE, 0x0068, 0x0129, 0x0116, 0x00000132, 0x0082, 0x00000146, 0x0146, 0x0085, 0x00000149, 0x0149, 0x0087, 0x0000014f, 0x014f)
NTDLL(NtRecoverResourceManager                          ,   NONE,   NONE,   NONE,   NONE, 0x0069, 0x012a, 0x0117, 0x00000133, 0x0081, 0x00000147, 0x0147, 0x0084, 0x0000014a, 0x014a, 0x0086, 0x00000150, 0x0150)
NTDLL(NtRecoverTransactionManager                       ,   NONE,   NONE,   NONE,   NONE, 0x006a, 0x012b, 0x0118, 0x00000134, 0x0080, 0x00000148, 0x0148, 0x0083, 0x0000014b, 0x014b, 0x0085, 0x00000151, 0x0151)
NTDLL(NtRegisterProtocolAddressInformation              ,   NONE,   NONE,   NONE,   NONE, 0x0174, 0x012c, 0x0119, 0x00000135, 0x007f, 0x00000149, 0x0149, 0x0082, 0x0000014c, 0x014c, 0x0084, 0x00000152, 0x0152)
NTDLL(NtRegisterThreadTerminatePort                     , 0x00a5, 0x00bb, 0x00c3, 0x00e3, 0x0106, 0x012d, 0x011a, 0x00000136, 0x007e, 0x0003014a, 0x014a, 0x0081, 0x0003014d, 0x014d, 0x0083, 0x00030153, 0x0153)
NTDLL(NtReleaseCMFViewOwnership                         ,   NONE,   NONE,   NONE,   NONE, 0x0186, 0x012e,   NONE,       NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE)
NTDLL(NtReleaseKeyedEvent                               ,   NONE, 0x0119, 0x0123, 0x00e4, 0x016a, 0x012f, 0x011b, 0x00000137, 0x007d, 0x0014014b, 0x014b, 0x0080, 0x0014014e, 0x014e, 0x0082, 0x00140154, 0x0154)
NTDLL(NtReleaseMutant                                   , 0x00a6, 0x00bc, 0x00c4, 0x001d, 0x0107, 0x001d, 0x011c, 0x0000001d, 0x007c, 0x0007001e, 0x001e, 0x007f, 0x0007001f, 0x001f, 0x0081, 0x00070020, 0x0020)
NTDLL(NtReleaseSemaphore                                , 0x00a7, 0x00bd, 0x00c5, 0x0007, 0x0108, 0x0007, 0x011d, 0x00000007, 0x007b, 0x000c0008, 0x0008, 0x007e, 0x000c0009, 0x0009, 0x0080, 0x000c000a, 0x000a)
NTDLL(NtReleaseWorkerFactoryWorker                      ,   NONE,   NONE,   NONE,   NONE, 0x0178, 0x0130, 0x011e, 0x00000138, 0x007a, 0x0000014c, 0x014c, 0x007d, 0x0003014f, 0x014f, 0x007f, 0x00030155, 0x0155)
NTDLL(NtRemoveIoCompletion                              , 0x00a8, 0x00be, 0x00c6, 0x0006, 0x0109, 0x0006, 0x011f, 0x00000006, 0x0079, 0x001c0007, 0x0007, 0x007c, 0x001c0008, 0x0008, 0x007e, 0x001c0009, 0x0009)
NTDLL(NtRemoveIoCompletionEx                            ,   NONE,   NONE,   NONE,   NONE, 0x0173, 0x0131, 0x0120, 0x00000139, 0x0078, 0x0000014d, 0x014d, 0x007b, 0x00000150, 0x0150, 0x007d, 0x00000156, 0x0156)
NTDLL(NtRemoveProcessDebug                              ,   NONE, 0x00bf, 0x00c7, 0x00e5, 0x010a, 0x0132, 0x0121, 0x0000013a, 0x0077, 0x0008014e, 0x014e, 0x007a, 0x00080151, 0x0151, 0x007c, 0x00080157, 0x0157)
NTDLL(NtRenameKey                                       ,   NONE, 0x00c0, 0x00c8, 0x00e6, 0x010b, 0x0133, 0x0122, 0x0000013b, 0x0076, 0x0000014f, 0x014f, 0x0079, 0x00000152, 0x0152, 0x007b, 0x00000158, 0x0158)
NTDLL(NtRenameTransactionManager                        ,   NONE,   NONE,   NONE,   NONE, 0x0066, 0x0134, 0x0123, 0x0000013c, 0x0075, 0x00000150, 0x0150, 0x0078, 0x00000153, 0x0153, 0x007a, 0x00000159, 0x0159)
NTDLL(NtReplaceKey                                      , 0x00a9, 0x00c1, 0x00c9, 0x00e7, 0x010c, 0x0135, 0x0124, 0x0000013d, 0x0074, 0x00000151, 0x0151, 0x0077, 0x00000154, 0x0154, 0x0079, 0x0000015a, 0x015a)
NTDLL(NtReplacePartitionUnit                            ,   NONE,   NONE,   NONE,   NONE, 0x010d, 0x0136, 0x0125, 0x0000013e, 0x0073, 0x00000152, 0x0152, 0x0076, 0x00000155, 0x0155, 0x0078, 0x0000015b, 0x015b)
NTDLL(NtReplyPort                                       , 0x00aa, 0x00c2, 0x00ca, 0x0009, 0x010e, 0x0009, 0x0126, 0x00000009, 0x0072, 0x0000000a, 0x000a, 0x0075, 0x0000000b, 0x000b, 0x0077, 0x0000000c, 0x000c)
NTDLL(NtReplyWaitReceivePort                            , 0x00ab, 0x00c3, 0x00cb, 0x0008, 0x010f, 0x0008, 0x0127, 0x00000008, 0x0071, 0x00000009, 0x0009, 0x0074, 0x0000000a, 0x000a, 0x0076, 0x0000000b, 0x000b)
NTDLL(NtReplyWaitReceivePortEx                          , 0x00ac, 0x00c4, 0x00cc, 0x0028, 0x0110, 0x0028, 0x0128, 0x00000028, 0x0070, 0x00000029, 0x0029, 0x0073, 0x0000002a, 0x002a, 0x0075, 0x0000002b, 0x002b)
NTDLL(NtReplyWaitReplyPort                              , 0x00ad, 0x00c5, 0x00cd, 0x00e8, 0x0111, 0x0137, 0x0129, 0x0000013f, 0x006f, 0x00000153, 0x0153, 0x0072, 0x00000156, 0x0156, 0x0074, 0x0000015c, 0x015c)
NTDLL(NtReplyWaitSendChannel                            , 0x00f4,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE)
NTDLL(NtRequestDeviceWakeup                             , 0x00ae, 0x00c6, 0x00ce, 0x00e9, 0x0112, 0x0138,   NONE,       NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE)
NTDLL(NtRequestPort                                     , 0x00af, 0x00c7, 0x00cf, 0x00ea, 0x0113, 0x0139, 0x012a, 0x00000140, 0x006e, 0x00000154, 0x0154, 0x0071, 0x00000157, 0x0157, 0x0073, 0x0000015d, 0x015d)
NTDLL(NtRequestWaitReplyPort                            , 0x00b0, 0x00c8, 0x00d0, 0x001f, 0x0114, 0x001f, 0x012b, 0x0000001f, 0x006d, 0x00000020, 0x0020, 0x0070, 0x00000021, 0x0021, 0x0072, 0x00000022, 0x0022)
NTDLL(NtRequestWakeupLatency                            , 0x00b1, 0x00c9, 0x00d1, 0x00eb, 0x0115, 0x013a,   NONE,       NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE)
NTDLL(NtResetEvent                                      , 0x00b2, 0x00ca, 0x00d2, 0x00ec, 0x0116, 0x013b, 0x012c, 0x00000141, 0x006c, 0x00070155, 0x0155, 0x006f, 0x00070158, 0x0158, 0x0071, 0x0007015e, 0x015e)
NTDLL(NtResetWriteWatch                                 , 0x00b3, 0x00cb, 0x00d3, 0x00ed, 0x0117, 0x013c, 0x012d, 0x00000142, 0x006b, 0x000c0156, 0x0156, 0x006e, 0x000c0159, 0x0159, 0x0070, 0x000c015f, 0x015f)
NTDLL(NtRestoreKey                                      , 0x00b4, 0x00cc, 0x00d4, 0x00ee, 0x0118, 0x013d, 0x012e, 0x00000143, 0x006a, 0x00000157, 0x0157, 0x006d, 0x0000015a, 0x015a, 0x006f, 0x00000160, 0x0160)
NTDLL(NtResumeProcess                                   ,   NONE, 0x00cd, 0x00d5, 0x00ef, 0x0119, 0x013e, 0x012f, 0x00000144, 0x0069, 0x00030158, 0x0158, 0x006c, 0x0003015b, 0x015b, 0x006e, 0x00030161, 0x0161)
NTDLL(NtResumeThread                                    , 0x00b5, 0x00ce, 0x00d6, 0x004f, 0x011a, 0x004f, 0x0130, 0x0000004f, 0x0068, 0x00070050, 0x0050, 0x006b, 0x00070051, 0x0051, 0x006d, 0x00070052, 0x0052)
NTDLL(NtRevertContainerImpersonation                    ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE, 0x006c, 0x00000162, 0x0162)
NTDLL(NtRollbackComplete                                ,   NONE,   NONE,   NONE,   NONE, 0x0059, 0x013f, 0x0131, 0x00000145, 0x0067, 0x00000159, 0x0159, 0x006a, 0x0000015c, 0x015c, 0x006b, 0x00000163, 0x0163)
NTDLL(NtRollbackEnlistment                              ,   NONE,   NONE,   NONE,   NONE, 0x005a, 0x0140, 0x0132, 0x00000146, 0x0066, 0x0000015a, 0x015a, 0x0069, 0x0000015d, 0x015d, 0x006a, 0x00000164, 0x0164)
NTDLL(NtRollbackTransaction                             ,   NONE,   NONE,   NONE,   NONE, 0x005c, 0x0141, 0x0133, 0x00000147, 0x0065, 0x0000015b, 0x015b, 0x0068, 0x0000015e, 0x015e, 0x0069, 0x00000165, 0x0165)
NTDLL(NtRollforwardTransactionManager                   ,   NONE,   NONE,   NONE,   NONE, 0x0067, 0x0142, 0x0134, 0x00000148, 0x0064, 0x0000015c, 0x015c, 0x0067, 0x0000015f, 0x015f, 0x0068, 0x00000166, 0x0166)
NTDLL(NtSaveKey                                         , 0x00b6, 0x00cf, 0x00d7, 0x00f0, 0x011b, 0x0143, 0x0135, 0x00000149, 0x0063, 0x0000015d, 0x015d, 0x0066, 0x00080160, 0x0160, 0x0067, 0x00080167, 0x0167)
NTDLL(NtSaveKeyEx                                       ,   NONE, 0x00d0, 0x00d8, 0x00f1, 0x011c, 0x0144, 0x0136, 0x0000014a, 0x0062, 0x0000015e, 0x015e, 0x0065, 0x000e0161, 0x0161, 0x0066, 0x000e0168, 0x0168)
NTDLL(NtSaveMergedKeys                                  , 0x00b7, 0x00d1, 0x00d9, 0x00f2, 0x011d, 0x0145, 0x0137, 0x0000014b, 0x0061, 0x000b015f, 0x015f, 0x0064, 0x000b0162, 0x0162, 0x0065, 0x000b0169, 0x0169)
NTDLL(NtSecureConnectPort                               , 0x00b8, 0x00d2, 0x00da, 0x00f3, 0x011e, 0x0146, 0x0138, 0x0000014c, 0x0060, 0x00000160, 0x0160, 0x0063, 0x00000163, 0x0163, 0x0064, 0x0000016a, 0x016a)
NTDLL(NtSendWaitReplyChannel                            , 0x00f5,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE)
NTDLL(NtSerializeBoot                                   ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE, 0x0139, 0x0000014d, 0x005f, 0x00000161, 0x0161, 0x0062, 0x00000164, 0x0164, 0x0063, 0x0000016b, 0x016b)
NTDLL(NtSetBootEntryOrder                               ,   NONE, 0x00d3, 0x00db, 0x00f4, 0x011f, 0x0147, 0x013a, 0x0000014e, 0x005e, 0x00000162, 0x0162, 0x0061, 0x00000165, 0x0165, 0x0062, 0x0000016c, 0x016c)
NTDLL(NtSetBootOptions                                  ,   NONE, 0x00d4, 0x00dc, 0x00f5, 0x0120, 0x0148, 0x013b, 0x0000014f, 0x005d, 0x00000163, 0x0163, 0x0060, 0x00000166, 0x0166, 0x0061, 0x0000016d, 0x016d)
NTDLL(NtSetCachedSigningLevel                           ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE, 0x005c, 0x00000164, 0x0164, 0x005f, 0x00000167, 0x0167, 0x0060, 0x0000016e, 0x016e)
NTDLL(NtSetContextChannel                               , 0x00f6,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE)
NTDLL(NtSetContextThread                                , 0x00ba, 0x00d5, 0x00dd, 0x00f6, 0x0121, 0x0149, 0x013c, 0x00000150, 0x005b, 0x00000165, 0x0165, 0x005e, 0x00000168, 0x0168, 0x005f, 0x0000016f, 0x016f)
NTDLL(NtSetDebugFilterState                             ,   NONE, 0x00d6, 0x00de, 0x00f7, 0x0122, 0x014a, 0x013d, 0x00000151, 0x005a, 0x000a0166, 0x0166, 0x005d, 0x000a0169, 0x0169, 0x005e, 0x000a0170, 0x0170)
NTDLL(NtSetDefaultHardErrorPort                         , 0x00bb, 0x00d7, 0x00df, 0x00f8, 0x0123, 0x014b, 0x013e, 0x00000152, 0x0059, 0x00030167, 0x0167, 0x005c, 0x0003016a, 0x016a, 0x005d, 0x00030171, 0x0171)
NTDLL(NtSetDefaultLocale                                , 0x00bc, 0x00d8, 0x00e0, 0x00f9, 0x0124, 0x014c, 0x013f, 0x00000153, 0x0058, 0x00050168, 0x0168, 0x005b, 0x0005016b, 0x016b, 0x005c, 0x00050172, 0x0172)
NTDLL(NtSetDefaultUILanguage                            , 0x00bd, 0x00d9, 0x00e1, 0x00fa, 0x0125, 0x014d, 0x0140, 0x00000154, 0x0057, 0x00040169, 0x0169, 0x005a, 0x0004016c, 0x016c, 0x005b, 0x00040173, 0x0173)
NTDLL(NtSetDriverEntryOrder                             ,   NONE,   NONE, 0x00e2, 0x00fb, 0x0126, 0x014e, 0x0141, 0x00000155, 0x0056, 0x0000016a, 0x016a, 0x0059, 0x0000016d, 0x016d, 0x005a, 0x00000174, 0x0174)
NTDLL(NtSetEaFile                                       , 0x00be, 0x00da, 0x00e3, 0x00fc, 0x0127, 0x014f, 0x0142, 0x00000156, 0x0055, 0x0000016b, 0x016b, 0x0058, 0x0000016e, 0x016e, 0x0059, 0x00000175, 0x0175)
NTDLL(NtSetEvent                                        , 0x00bf, 0x00db, 0x00e4, 0x000b, 0x0128, 0x000b, 0x0143, 0x0000000b, 0x0054, 0x0007000c, 0x000c, 0x0057, 0x0007000d, 0x000d, 0x0058, 0x0007000e, 0x000e)
NTDLL(NtSetEventBoostPriority                           ,   NONE, 0x00dc, 0x00e5, 0x002a, 0x0129, 0x002a, 0x0144, 0x0000002a, 0x0053, 0x0003002b, 0x002b, 0x0056, 0x0003002c, 0x002c, 0x0057, 0x0003002d, 0x002d)
NTDLL(NtSetHighEventPair                                , 0x00c0, 0x00dd, 0x00e6, 0x00fd, 0x012a, 0x0150, 0x0145, 0x00000157, 0x0052, 0x0003016c, 0x016c, 0x0055, 0x0003016f, 0x016f, 0x0056, 0x00030176, 0x0176)
NTDLL(NtSetHighWaitLowEventPair                         , 0x00c1, 0x00de, 0x00e7, 0x00fe, 0x012b, 0x0151, 0x0146, 0x00000158, 0x0051, 0x0003016d, 0x016d, 0x0054, 0x00030170, 0x0170, 0x0055, 0x00030177, 0x0177)
NTDLL(NtSetIRTimer                                      ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE, 0x0040, 0x0000016e, 0x016e, 0x0041, 0x00070171, 0x0171, 0x0042, 0x00070178, 0x0178)
NTDLL(NtSetInformationDebugObject                       ,   NONE, 0x00df, 0x00e8, 0x00ff, 0x012c, 0x0152, 0x0147, 0x00000159, 0x0050, 0x0000016f, 0x016f, 0x0053, 0x00000172, 0x0172, 0x0054, 0x00000179, 0x0179)
NTDLL(NtSetInformationEnlistment                        ,   NONE,   NONE,   NONE,   NONE, 0x0071, 0x0153, 0x0148, 0x0000015a, 0x004f, 0x00000170, 0x0170, 0x0052, 0x00000173, 0x0173, 0x0053, 0x0000017a, 0x017a)
NTDLL(NtSetInformationFile                              , 0x00c2, 0x00e0, 0x00e9, 0x0024, 0x012d, 0x0024, 0x0149, 0x00000024, 0x004e, 0x00000025, 0x0025, 0x0051, 0x00000026, 0x0026, 0x0052, 0x00000027, 0x0027)
NTDLL(NtSetInformationJobObject                         , 0x00c3, 0x00e1, 0x00ea, 0x0100, 0x012e, 0x0154, 0x014a, 0x0000015b, 0x004d, 0x00000171, 0x0171, 0x0050, 0x00000174, 0x0174, 0x0051, 0x0000017b, 0x017b)
NTDLL(NtSetInformationKey                               , 0x00c4, 0x00e2, 0x00eb, 0x0101, 0x012f, 0x0155, 0x014b, 0x0000015c, 0x004c, 0x00000172, 0x0172, 0x004f, 0x00000175, 0x0175, 0x0050, 0x0000017c, 0x017c)
NTDLL(NtSetInformationObject                            , 0x00c5, 0x00e3, 0x00ec, 0x0059, 0x0130, 0x0059, 0x014c, 0x00000059, 0x004b, 0x0000005a, 0x005a, 0x004e, 0x0000005b, 0x005b, 0x004f, 0x0000005c, 0x005c)
NTDLL(NtSetInformationProcess                           , 0x00c6, 0x00e4, 0x00ed, 0x0019, 0x0131, 0x0019, 0x014d, 0x00000019, 0x004a, 0x0000001a, 0x001a, 0x004d, 0x0000001b, 0x001b, 0x004e, 0x0000001c, 0x001c)
NTDLL(NtSetInformationResourceManager                   ,   NONE,   NONE,   NONE,   NONE, 0x0063, 0x0156, 0x014e, 0x0000015d, 0x0049, 0x00000173, 0x0173, 0x004c, 0x00000176, 0x0176, 0x004d, 0x0000017d, 0x017d)
NTDLL(NtSetInformationSymbolicLink                      ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE, 0x01b8, 0x0000017e, 0x017e)
NTDLL(NtSetInformationThread                            , 0x00c7, 0x00e5, 0x00ee, 0x000a, 0x0132, 0x000a, 0x014f, 0x0000000a, 0x0048, 0x0000000b, 0x000b, 0x004b, 0x0000000c, 0x000c, 0x004c, 0x0000000d, 0x000d)
NTDLL(NtSetInformationToken                             , 0x00c8, 0x00e6, 0x00ef, 0x0102, 0x0133, 0x0157, 0x0150, 0x0000015e, 0x0047, 0x00000174, 0x0174, 0x004a, 0x00000177, 0x0177, 0x004b, 0x0000017f, 0x017f)
NTDLL(NtSetInformationTransaction                       ,   NONE,   NONE,   NONE,   NONE, 0x0061, 0x0158, 0x0151, 0x0000015f, 0x0046, 0x00000175, 0x0175, 0x0049, 0x00000178, 0x0178, 0x004a, 0x00000180, 0x0180)
NTDLL(NtSetInformationTransactionManager                ,   NONE,   NONE,   NONE,   NONE, 0x0062, 0x0159, 0x0152, 0x00000160, 0x0045, 0x00000176, 0x0176, 0x0048, 0x00000179, 0x0179, 0x0049, 0x00000181, 0x0181)
NTDLL(NtSetInformationVirtualMemory                     ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE, 0x00c9, 0x00000177, 0x0177, 0x00cc, 0x0000017a, 0x017a, 0x00ce, 0x00000182, 0x0182)
NTDLL(NtSetInformationWorkerFactory                     ,   NONE,   NONE,   NONE,   NONE, 0x017a, 0x015a, 0x0153, 0x00000161, 0x0044, 0x00000178, 0x0178, 0x0047, 0x0000017b, 0x017b, 0x0048, 0x00000183, 0x0183)
NTDLL(NtSetIntervalProfile                              , 0x00c9, 0x00e7, 0x00f0, 0x0103, 0x0134, 0x015b, 0x0154, 0x00000162, 0x0043, 0x00050179, 0x0179, 0x0046, 0x0005017c, 0x017c, 0x0047, 0x00050184, 0x0184)
NTDLL(NtSetIoCompletion                                 , 0x00b9, 0x00e8, 0x00f1, 0x0104, 0x0135, 0x015c, 0x0155, 0x00000163, 0x0042, 0x0000017a, 0x017a, 0x0045, 0x0000017d, 0x017d, 0x0046, 0x00000185, 0x0185)
NTDLL(NtSetIoCompletionEx                               ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE, 0x0156, 0x00000164, 0x0041, 0x0000017b, 0x017b, 0x0044, 0x0000017e, 0x017e, 0x0045, 0x00000186, 0x0186)
NTDLL(NtSetLdtEntries                                   , 0x00ca, 0x00e9, 0x00f2, 0x0105, 0x0136, 0x015d, 0x0157, 0x00000165, 0x003f, 0x0000017c, 0x017c, 0x0040, 0x0000017f, 0x017f, 0x0041, 0x00000187, 0x0187)
NTDLL(NtSetLowEventPair                                 , 0x00cb, 0x00ea, 0x00f3, 0x0106, 0x0137, 0x015e, 0x0158, 0x00000166, 0x003e, 0x0003017d, 0x017d, 0x003f, 0x00030180, 0x0180, 0x0040, 0x00030188, 0x0188)
NTDLL(NtSetLowWaitHighEventPair                         , 0x00cc, 0x00eb, 0x00f4, 0x0107, 0x0138, 0x015f, 0x0159, 0x00000167, 0x003d, 0x0003017e, 0x017e, 0x003e, 0x00030181, 0x0181, 0x003f, 0x00030189, 0x0189)
NTDLL(NtSetQuotaInformationFile                         , 0x00cd, 0x00ec, 0x00f5, 0x0108, 0x0139, 0x0160, 0x015a, 0x00000168, 0x003c, 0x0000017f, 0x017f, 0x003d, 0x00000182, 0x0182, 0x003e, 0x0000018a, 0x018a)
NTDLL(NtSetSecurityObject                               , 0x00ce, 0x00ed, 0x00f6, 0x0109, 0x013a, 0x0161, 0x015b, 0x00000169, 0x003b, 0x00000180, 0x0180, 0x003c, 0x00000183, 0x0183, 0x003d, 0x0000018b, 0x018b)
NTDLL(NtSetSystemEnvironmentValue                       , 0x00cf, 0x00ee, 0x00f7, 0x010a, 0x013b, 0x0162, 0x015c, 0x0000016a, 0x003a, 0x00000181, 0x0181, 0x003b, 0x00000184, 0x0184, 0x003c, 0x0000018c, 0x018c)
NTDLL(NtSetSystemEnvironmentValueEx                     ,   NONE, 0x00ef, 0x00f8, 0x010b, 0x013c, 0x0163, 0x015d, 0x0000016b, 0x0039, 0x00000182, 0x0182, 0x003a, 0x00000185, 0x0185, 0x003b, 0x0000018d, 0x018d)
NTDLL(NtSetSystemInformation                            , 0x00d0, 0x00f0, 0x00f9, 0x010c, 0x013d, 0x0164, 0x015e, 0x0000016c, 0x0038, 0x00000183, 0x0183, 0x0039, 0x00000186, 0x0186, 0x003a, 0x0000018e, 0x018e)
NTDLL(NtSetSystemPowerState                             , 0x00d1, 0x00f1, 0x00fa, 0x010d, 0x013e, 0x0165, 0x015f, 0x0000016d, 0x0037, 0x00000184, 0x0184, 0x0038, 0x00000187, 0x0187, 0x0039, 0x0000018f, 0x018f)
NTDLL(NtSetSystemTime                                   , 0x00d2, 0x00f2, 0x00fb, 0x010e, 0x013f, 0x0166, 0x0160, 0x0000016e, 0x0036, 0x00050185, 0x0185, 0x0037, 0x00050188, 0x0188, 0x0038, 0x00050190, 0x0190)
NTDLL(NtSetThreadExecutionState                         , 0x00d3, 0x00f3, 0x00fc, 0x010f, 0x0140, 0x0167, 0x0161, 0x0000016f, 0x0035, 0x00050186, 0x0186, 0x0036, 0x00050189, 0x0189, 0x0037, 0x00050191, 0x0191)
NTDLL(NtSetTimer                                        , 0x00d4, 0x00f4, 0x00fd, 0x005f, 0x0141, 0x005f, 0x0162, 0x0000005f, 0x0034, 0x00000060, 0x0060, 0x0035, 0x00000061, 0x0061, 0x0036, 0x00000062, 0x0062)
NTDLL(NtSetTimer2                                       ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE,   NONE,       NONE,   NONE, 0x0042, 0x0000018a, 0x018a, 0x0043, 0x00000192, 0x0192)
NTDLL(NtSetTimerEx                                      ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE, 0x0163, 0x00000170, 0x0033, 0x00000187, 0x0187, 0x0034, 0x0000018b, 0x018b, 0x0035, 0x00000193, 0x0193)
NTDLL(NtSetTimerResolution                              , 0x00d5, 0x00f5, 0x00fe, 0x0110, 0x0142, 0x0168, 0x0164, 0x00000171, 0x0032, 0x000a0188, 0x0188, 0x0033, 0x000a018c, 0x018c, 0x0034, 0x000a0194, 0x0194)
NTDLL(NtSetUuidSeed                                     , 0x00d6, 0x00f6, 0x00ff, 0x0111, 0x0143, 0x0169, 0x0165, 0x00000172, 0x0031, 0x00040189, 0x0189, 0x0032, 0x0004018d, 0x018d, 0x0033, 0x00040195, 0x0195)
NTDLL(NtSetValueKey                                     , 0x00d7, 0x00f7, 0x0100, 0x005d, 0x0144, 0x005d, 0x0166, 0x0000005d, 0x0030, 0x0000005e, 0x005e, 0x0031, 0x0000005f, 0x005f, 0x0032, 0x00000060, 0x0060)
NTDLL(NtSetVolumeInformationFile                        , 0x00d8, 0x00f8, 0x0101, 0x0112, 0x0145, 0x016a, 0x0167, 0x00000173, 0x002f, 0x0000018a, 0x018a, 0x0030, 0x0000018e, 0x018e, 0x0031, 0x00000196, 0x0196)
NTDLL(NtSetWnfProcessNotificationEvent                  ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE,   NONE,       NONE,   NONE, 0x002f, 0x0000018f, 0x018f, 0x0030, 0x00000197, 0x0197)
NTDLL(NtShutdownSystem                                  , 0x00d9, 0x00f9, 0x0102, 0x0113, 0x0146, 0x016b, 0x0168, 0x00000174, 0x002e, 0x0004018b, 0x018b, 0x002e, 0x00040190, 0x0190, 0x002f, 0x00040198, 0x0198)
NTDLL(NtShutdownWorkerFactory                           ,   NONE,   NONE,   NONE,   NONE, 0x017d, 0x016c, 0x0169, 0x00000175, 0x002d, 0x0000018c, 0x018c, 0x002d, 0x00000191, 0x0191, 0x002e, 0x00000199, 0x0199)
NTDLL(NtSignalAndWaitForSingleObject                    , 0x00da, 0x00fa, 0x0103, 0x0114, 0x0147, 0x016d, 0x016a, 0x00000176, 0x002c, 0x0013018d, 0x018d, 0x002c, 0x00130192, 0x0192, 0x002d, 0x0013019a, 0x019a)
NTDLL(NtSinglePhaseReject                               ,   NONE,   NONE,   NONE,   NONE, 0x0060, 0x016e, 0x016b, 0x00000177, 0x002b, 0x0000018e, 0x018e, 0x002b, 0x00000193, 0x0193, 0x002c, 0x0000019b, 0x019b)
NTDLL(NtStartProfile                                    , 0x00db, 0x00fb, 0x0104, 0x0115, 0x0148, 0x016f, 0x016c, 0x00000178, 0x002a, 0x0003018f, 0x018f, 0x002a, 0x00030194, 0x0194, 0x002b, 0x0003019c, 0x019c)
NTDLL(NtStopProfile                                     , 0x00dc, 0x00fc, 0x0105, 0x0116, 0x0149, 0x0170, 0x016d, 0x00000179, 0x0029, 0x00030190, 0x0190, 0x0029, 0x00030195, 0x0195, 0x002a, 0x0003019d, 0x019d)
NTDLL(NtSubscribeWnfStateChange                         ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE, 0x0028, 0x00000191, 0x0191, 0x0028, 0x00000196, 0x0196, 0x0029, 0x0000019e, 0x019e)
NTDLL(NtSuspendProcess                                  ,   NONE, 0x00fd, 0x0106, 0x0117, 0x014a, 0x0171, 0x016e, 0x0000017a, 0x0027, 0x00030192, 0x0192, 0x0027, 0x00030197, 0x0197, 0x0028, 0x0003019f, 0x019f)
NTDLL(NtSuspendThread                                   , 0x00dd, 0x00fe, 0x0107, 0x0118, 0x014b, 0x0172, 0x016f, 0x0000017b, 0x0026, 0x00070193, 0x0193, 0x0026, 0x00070198, 0x0198, 0x0027, 0x000701a0, 0x01a0)
NTDLL(NtSystemDebugControl                              , 0x00de, 0x00ff, 0x0108, 0x0119, 0x014c, 0x0173, 0x0170, 0x0000017c, 0x0025, 0x00000194, 0x0194, 0x0025, 0x00000199, 0x0199, 0x0026, 0x000001a1, 0x01a1)
NTDLL(NtTerminateJobObject                              , 0x00df, 0x0100, 0x0109, 0x011a, 0x014d, 0x0174, 0x0171, 0x0000017d, 0x0024, 0x00070195, 0x0195, 0x0024, 0x0007019a, 0x019a, 0x0025, 0x000701a2, 0x01a2)
NTDLL(NtTerminateProcess                                , 0x00e0, 0x0101, 0x010a, 0x0029, 0x014e, 0x0029, 0x0172, 0x00000029, 0x0023, 0x0000002a, 0x002a, 0x0023, 0x0007002b, 0x002b, 0x0024, 0x0007002c, 0x002c)
NTDLL(NtTerminateThread                                 , 0x00e1, 0x0102, 0x010b, 0x0050, 0x014f, 0x0050, 0x0173, 0x00000050, 0x0022, 0x00000051, 0x0051, 0x0022, 0x00070052, 0x0052, 0x0023, 0x00070053, 0x0053)
NTDLL(NtTestAlert                                       , 0x00e2, 0x0103, 0x010c, 0x011b, 0x0150, 0x0175, 0x0174, 0x0000017e, 0x0021, 0x00020196, 0x0196, 0x0021, 0x0002019b, 0x019b, 0x0022, 0x000201a3, 0x01a3)
NTDLL(NtThawRegistry                                    ,   NONE,   NONE,   NONE,   NONE, 0x0151, 0x0176, 0x0175, 0x0000017f, 0x0020, 0x00000197, 0x0197, 0x0020, 0x0000019c, 0x019c, 0x0021, 0x000001a4, 0x01a4)
NTDLL(NtThawTransactions                                ,   NONE,   NONE,   NONE,   NONE, 0x0152, 0x0177, 0x0176, 0x00000180, 0x001f, 0x00000198, 0x0198, 0x001f, 0x0000019d, 0x019d, 0x0020, 0x000001a5, 0x01a5)
NTDLL(NtTraceControl                                    ,   NONE,   NONE,   NONE,   NONE, 0x0154, 0x0178, 0x0177, 0x00000181, 0x001e, 0x00000199, 0x0199, 0x001e, 0x0000019e, 0x019e, 0x001f, 0x000001a6, 0x01a6)
NTDLL(NtTraceEvent                                      ,   NONE, 0x0104, 0x010d, 0x005b, 0x0153, 0x005b, 0x0178, 0x0000005b, 0x001d, 0x0000005c, 0x005c, 0x001d, 0x0000005d, 0x005d, 0x001e, 0x0000005e, 0x005e)
NTDLL(NtTranslateFilePath                               ,   NONE, 0x0105, 0x010e, 0x011c, 0x0155, 0x0179, 0x0179, 0x00000182, 0x001c, 0x0011019a, 0x019a, 0x001c, 0x0011019f, 0x019f, 0x001d, 0x001101a7, 0x01a7)
NTDLL(NtUmsThreadYield                                  ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE, 0x017a, 0x00000183, 0x001b, 0x0000019b, 0x019b, 0x001b, 0x000001a0, 0x01a0, 0x001c, 0x000001a8, 0x01a8)
NTDLL(NtUnloadDriver                                    , 0x00e3, 0x0106, 0x010f, 0x011d, 0x0156, 0x017a, 0x017b, 0x00000184, 0x001a, 0x0000019c, 0x019c, 0x001a, 0x000001a1, 0x01a1, 0x001b, 0x000001a9, 0x01a9)
NTDLL(NtUnloadKey                                       , 0x00e4, 0x0107, 0x0110, 0x011e, 0x0157, 0x017b, 0x017c, 0x00000185, 0x0019, 0x0000019d, 0x019d, 0x0019, 0x000001a2, 0x01a2, 0x001a, 0x000001aa, 0x01aa)
NTDLL(NtUnloadKey2                                      ,   NONE,   NONE, 0x0111, 0x011f, 0x0158, 0x017c, 0x017d, 0x00000186, 0x0018, 0x0000019e, 0x019e, 0x0018, 0x000001a3, 0x01a3, 0x0019, 0x000001ab, 0x01ab)
NTDLL(NtUnloadKeyEx                                     ,   NONE, 0x0108, 0x0112, 0x0120, 0x0159, 0x017d, 0x017e, 0x00000187, 0x0017, 0x0000019f, 0x019f, 0x0017, 0x000001a4, 0x01a4, 0x0018, 0x000001ac, 0x01ac)
NTDLL(NtUnlockFile                                      , 0x00e5, 0x0109, 0x0113, 0x0121, 0x015a, 0x017e, 0x017f, 0x00000188, 0x0016, 0x000001a0, 0x01a0, 0x0016, 0x000001a5, 0x01a5, 0x0017, 0x000001ad, 0x01ad)
NTDLL(NtUnlockVirtualMemory                             , 0x00e6, 0x010a, 0x0114, 0x0122, 0x015b, 0x017f, 0x0180, 0x00000189, 0x0015, 0x000001a1, 0x01a1, 0x0015, 0x000001a6, 0x01a6, 0x0016, 0x000001ae, 0x01ae)
NTDLL(NtUnmapViewOfSection                              , 0x00e7, 0x010b, 0x0115, 0x0027, 0x015c, 0x0027, 0x0181, 0x00000027, 0x0013, 0x00000028, 0x0028, 0x0013, 0x00000029, 0x0029, 0x0014, 0x0000002a, 0x002a)
NTDLL(NtUnmapViewOfSectionEx                            ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE, 0x0014, 0x000001a2, 0x01a2, 0x0014, 0x000001a7, 0x01a7, 0x0015, 0x000001af, 0x01af)
NTDLL(NtUnsubscribeWnfStateChange                       ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE, 0x0011, 0x000001a3, 0x01a3, 0x0011, 0x000001a8, 0x01a8, 0x0012, 0x000001b0, 0x01b0)
NTDLL(NtUpdateWnfStateData                              ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE, 0x0012, 0x000001a4, 0x01a4, 0x0012, 0x000001a9, 0x01a9, 0x0013, 0x000001b1, 0x01b1)
NTDLL(NtVdmControl                                      , 0x00e8, 0x010c, 0x0116, 0x0123, 0x015d, 0x0180, 0x0182, 0x0000018a, 0x0010, 0x000001a5, 0x01a5, 0x0010, 0x000001aa, 0x01aa, 0x0011, 0x000001b2, 0x01b2)
NTDLL(NtWaitForAlertByThreadId                          ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE, 0x000f, 0x000601a6, 0x01a6, 0x000f, 0x000601ab, 0x01ab, 0x0010, 0x000601b3, 0x01b3)
NTDLL(NtWaitForDebugEvent                               ,   NONE, 0x010d, 0x0117, 0x0124, 0x015e, 0x0181, 0x0183, 0x0000018b, 0x000e, 0x000001a7, 0x01a7, 0x000e, 0x000001ac, 0x01ac, 0x000f, 0x000001b4, 0x01b4)
NTDLL(NtWaitForKeyedEvent                               ,   NONE, 0x011a, 0x0124, 0x0125, 0x016b, 0x0182, 0x0184, 0x0000018c, 0x000d, 0x001501a8, 0x01a8, 0x000d, 0x001501ad, 0x01ad, 0x000e, 0x001501b5, 0x01b5)
NTDLL(NtWaitForMultipleObjects                          , 0x00e9, 0x010e, 0x0118, 0x0058, 0x015f, 0x0058, 0x0185, 0x00000058, 0x000c, 0x001d0059, 0x0059, 0x000c, 0x001d005a, 0x005a, 0x000d, 0x001d005b, 0x005b)
NTDLL(NtWaitForMultipleObjects32                        ,   NONE,   NONE, 0x0127, 0x0017, 0x016e, 0x0017, 0x0186, 0x00000017, 0x000b, 0x001e0018, 0x0018, 0x000b, 0x001e0019, 0x0019, 0x000c, 0x001e001a, 0x001a)
NTDLL(NtWaitForSingleObject                             , 0x00ea, 0x010f, 0x0119, 0x0001, 0x0160, 0x0001, 0x0187, 0x00000001, 0x000a, 0x000d0002, 0x0002, 0x000a, 0x000d0003, 0x0003, 0x000b, 0x000d0004, 0x0004)
NTDLL(NtWaitForWnfNotifications                         ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE, 0x0009, 0x000001a9, 0x01a9,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE)
NTDLL(NtWaitForWorkViaWorkerFactory                     ,   NONE,   NONE,   NONE,   NONE, 0x0179, 0x0183, 0x0188, 0x0000018d, 0x0008, 0x000001aa, 0x01aa, 0x0009, 0x000001ae, 0x01ae, 0x000a, 0x000001b6, 0x01b6)
NTDLL(NtWaitHighEventPair                               , 0x00eb, 0x0110, 0x011a, 0x0126, 0x0161, 0x0184, 0x0189, 0x0000018e, 0x0007, 0x000301ab, 0x01ab, 0x0008, 0x000301af, 0x01af, 0x0009, 0x000301b7, 0x01b7)
NTDLL(NtWaitLowEventPair                                , 0x00ec, 0x0111, 0x011b, 0x0127, 0x0162, 0x0185, 0x018a, 0x0000018f, 0x0006, 0x000301ac, 0x01ac, 0x0007, 0x000301b0, 0x01b0, 0x0008, 0x000301b8, 0x01b8)
NTDLL(NtWorkerFactoryWorkerReady                        ,   NONE,   NONE,   NONE,   NONE, 0x017c, 0x0186, 0x018b, 0x00000190, 0x0000, 0x00000000, 0x0000, 0x0000, 0x00030000, 0x0000, 0x0001, 0x00030001, 0x0001)
NTDLL(NtWow64AllocateVirtualMemory64                    ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE,   NONE, 0x000001bb,   NONE,   NONE, 0x000001bf,   NONE,   NONE, 0x000001c7,   NONE)
NTDLL(NtWow64CallFunction64                             ,   NONE,   NONE,   NONE,   NONE,   NONE, 0x0197,   NONE, 0x000001a3,   NONE, 0x000001bf,   NONE,   NONE, 0x000001c3,   NONE,   NONE, 0x000001ca,   NONE)
NTDLL(NtWow64CsrAllocateCaptureBuffer                   ,   NONE,   NONE,   NONE, 0x012c,   NONE, 0x018a,   NONE, 0x00000194,   NONE, 0x000001b0,   NONE,   NONE, 0x000001b4,   NONE,   NONE, 0x000001bc,   NONE)
NTDLL(NtWow64CsrAllocateMessagePointer                  ,   NONE,   NONE,   NONE, 0x012e,   NONE, 0x018c,   NONE, 0x00000196,   NONE, 0x000001b2,   NONE,   NONE, 0x000001b6,   NONE,   NONE, 0x000001be,   NONE)
NTDLL(NtWow64CsrCaptureMessageBuffer                    ,   NONE,   NONE,   NONE, 0x012f,   NONE, 0x018d,   NONE, 0x00000197,   NONE, 0x000001b3,   NONE,   NONE, 0x000001b7,   NONE,   NONE, 0x000001bf,   NONE)
NTDLL(NtWow64CsrCaptureMessageString                    ,   NONE,   NONE,   NONE, 0x0130,   NONE, 0x018e,   NONE, 0x00000198,   NONE, 0x000001b4,   NONE,   NONE, 0x000001b8,   NONE,   NONE, 0x000001c0,   NONE)
NTDLL(NtWow64CsrClientCallServer                        ,   NONE,   NONE,   NONE, 0x012b,   NONE, 0x0189,   NONE, 0x00000193,   NONE, 0x000001af,   NONE,   NONE, 0x000001b3,   NONE,   NONE, 0x000001bb,   NONE)
NTDLL(NtWow64CsrClientConnectToServer                   ,   NONE,   NONE,   NONE, 0x0128,   NONE, 0x0187,   NONE, 0x00000191,   NONE, 0x000001ad,   NONE,   NONE, 0x000001b1,   NONE,   NONE, 0x000001b9,   NONE)
NTDLL(NtWow64CsrFreeCaptureBuffer                       ,   NONE,   NONE,   NONE, 0x012d,   NONE, 0x018b,   NONE, 0x00000195,   NONE, 0x000001b1,   NONE,   NONE, 0x000001b5,   NONE,   NONE, 0x000001bd,   NONE)
NTDLL(NtWow64CsrGetProcessId                            ,   NONE,   NONE,   NONE, 0x0132,   NONE, 0x018f,   NONE, 0x00000199,   NONE, 0x000001b5,   NONE,   NONE, 0x000001b9,   NONE,   NONE, 0x000001c1,   NONE)
NTDLL(NtWow64CsrIdentifyAlertableThread                 ,   NONE,   NONE,   NONE, 0x012a,   NONE, 0x0188,   NONE, 0x00000192,   NONE, 0x000001ae,   NONE,   NONE, 0x000001b2,   NONE,   NONE, 0x000001ba,   NONE)
NTDLL(NtWow64CsrNewThread                               ,   NONE,   NONE,   NONE, 0x0129,   NONE,   NONE,   NONE,       NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE)
NTDLL(NtWow64CsrSetPriorityClass                        ,   NONE,   NONE,   NONE, 0x0131,   NONE,   NONE,   NONE,       NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE)
NTDLL(NtWow64CsrVerifyRegion                            ,   NONE,   NONE,   NONE,   NONE,   NONE, 0x0190,   NONE, 0x0000019a,   NONE, 0x000001b6,   NONE,   NONE, 0x000001ba,   NONE,   NONE, 0x000001c2,   NONE)
NTDLL(NtWow64DebuggerCall                               ,   NONE,   NONE,   NONE, 0x0133,   NONE, 0x0191,   NONE, 0x0000019b,   NONE, 0x000001b7,   NONE,   NONE, 0x000001bb,   NONE,   NONE, 0x000001c3,   NONE)
NTDLL(NtWow64GetCurrentProcessorNumberEx                ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE, 0x0000019c,   NONE, 0x000001b8,   NONE,   NONE, 0x000001bc,   NONE,   NONE, 0x000001c4,   NONE)
NTDLL(NtWow64GetNativeSystemInformation                 ,   NONE,   NONE,   NONE, 0x0134,   NONE, 0x0192,   NONE, 0x0000019d,   NONE, 0x000001b9,   NONE,   NONE, 0x000001bd,   NONE,   NONE, 0x000001c5,   NONE)
NTDLL(NtWow64InterlockedPopEntrySList                   ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE, 0x0000019e,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE)
NTDLL(NtWow64IsProcessorFeaturePresent                  ,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,   NONE,       NONE,   NONE,       NONE,   NONE,   NONE,       NONE,   NONE,   NONE, 0x000001cb,   NONE)
NTDLL(NtWow64QueryInformationProcess64                  ,   NONE,   NONE,   NONE, 0x0135,   NONE, 0x0193,   NONE, 0x0000019f,   NONE, 0x000001ba,   NONE,   NONE, 0x000001be,   NONE,   NONE, 0x000001c6,   NONE)
NTDLL(NtWow64QueryVirtualMemory64                       ,   NONE,   NONE,   NONE, 0x0137,   NONE, 0x0196,   NONE, 0x000001a2,   NONE, 0x000001be,   NONE,   NONE, 0x000001c2,   NONE,   NONE,       NONE,   NONE)
NTDLL(NtWow64ReadVirtualMemory64                        ,   NONE,   NONE,   NONE, 0x0136,   NONE, 0x0194,   NONE, 0x000001a0,   NONE, 0x000001bc,   NONE,   NONE, 0x000001c0,   NONE,   NONE, 0x000001c8,   NONE)
NTDLL(NtWow64WriteVirtualMemory64                       ,   NONE,   NONE,   NONE,   NONE,   NONE, 0x0195,   NONE, 0x000001a1,   NONE, 0x000001bd,   NONE,   NONE, 0x000001c1,   NONE,   NONE, 0x000001c9,   NONE)
NTDLL(NtWriteFile                                       , 0x00ed, 0x0112, 0x011c, 0x0005, 0x0163, 0x0005, 0x018c, 0x00000005, 0x0005, 0x001a0006, 0x0006, 0x0006, 0x001a0007, 0x0007, 0x0007, 0x001a0008, 0x0008)
NTDLL(NtWriteFileGather                                 , 0x00ee, 0x0113, 0x011d, 0x0018, 0x0164, 0x0018, 0x018d, 0x00000018, 0x0004, 0x001a0019, 0x0019, 0x0005, 0x001a001a, 0x001a, 0x0006, 0x001a001b, 0x001b)
NTDLL(NtWriteRequestData                                , 0x00ef, 0x0114, 0x011e, 0x0054, 0x0165, 0x0054, 0x018e, 0x00000054, 0x0003, 0x00000055, 0x0055, 0x0004, 0x00000056, 0x0056, 0x0005, 0x00000057, 0x0057)
NTDLL(NtWriteVirtualMemory                              , 0x00f0, 0x0115, 0x011f, 0x0037, 0x0166, 0x0037, 0x018f, 0x00000037, 0x0002, 0x00000038, 0x0038, 0x0003, 0x00000039, 0x0039, 0x0004, 0x0000003a, 0x003a)
NTDLL(NtYieldExecution                                  , 0x00f7, 0x0116, 0x0120, 0x0043, 0x0167, 0x0043, 0x0190, 0x00000043, 0x0001, 0x00010044, 0x0044, 0x0002, 0x00010045, 0x0045, 0x0003, 0x00010046, 0x0046)
